/* Copyright (c) 2000-2001 ActiveState SRL.
   See the file LICENSE.txt for licensing information. */

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

// Base class for controllers.  Subclasses need only implement
// is_<command>_enabled() and do_<command>() functions for each
// command to get basic controller functionality.

// If a command does not have an is_<command>_enabled() function,
// it will always be enabled.

// If controllers need to change the response to supportsCommand() for
// particular commands (as in checkbox commands), they can implement
// is_<command>_supported() for those commands.

function Controller() {}

Controller.prototype.supportsCommand = function(command) {
    var result;
    var query = "is_" + command + "_supported";
    if (query in this) {
        result = this[query]();
    } else {
        var doer = "do_" + command;
        result = doer in this;
    }
    return result;
}

Controller.prototype.isCommandEnabled = function(command) {
    var result;
    var query = "is_" + command + "_enabled";
    if (query in this) {
        result = this[query]();
    } else {
        var doer = "do_" + command;
        result = doer in this;
    }
    return result;
}

Controller.prototype.doCommand = function(command) {
    var func = "do_" + command;
    if (func in this) {
        this[func]();
    }
}

Controller.prototype.onEvent = function(event) {
}
