/* Copyright (c) 2000-2002 ActiveState SRL.
   See the file LICENSE.txt for licensing information. */

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * file_window_functions.js
 * 
 * file open/save type functions.  This requires the use of an observer.
 * If a XUL file utilizes this file, it should shut down the observer when
 * it closes!!!!
 */


var fpOpen = null;
var fpWatch = null;
var fpSaveAs = null;
var fpSaveAsLang = null; // The language last used by the Save As... dialog
var fpOpenProject = null;
var fpAddToProject = null;
var fpOpenTemplate = null;
var fpSaveAsTemplate = null;

// Shouldn't this be magically picked up from koGlobalOverlay.js?  *confused*
var globalPrefService = Components.classes["@activestate.com/koPrefService;1"].
        getService(Components.interfaces.koIPrefService);


// if the file associations change, we need to rebuild the filters
// unfortunately this means we need to rebuild the file pickers and
// loose the last selected filter state
function FileAssociationPrefObserver() {
    this.observerSvc = Components.classes["@mozilla.org/observer-service;1"].
        getService(Components.interfaces.nsIObserverService);
    globalPrefService.prefs.AddObserver(this);
}

FileAssociationPrefObserver.prototype.Observe = function(subject, topic,
                                                         data) {
    if (data == "fileAssociations") {
        // force affected filepickers to be re-created on demand
        fpOpen = null;
        fpWatch = null;
        fpSaveAs = null;
    } 
}

FileAssociationPrefObserver.prototype.destructor = function() {
    // remove the observer!
    if (globalPrefService && globalPrefService.prefs) {
        globalPrefService.prefs.RemoveObserver(this);
    }
}

var fileAssociationPrefObserver = new FileAssociationPrefObserver();

function _append_all_filter(fp) {
    fp.appendFilters(Components.interfaces.nsIFilePicker.filterAll);
}

function _append_other_filters(fp,lang) {
    // This assumes that source files and 'all files' have been appended
    // first -- these are the 'leftovers'. Special case for
    // dlg_saveLocalFileAs which may pass a language to be put at the top
    // of the list.
    
    var registry = Components.classes["@activestate.com/koLanguageRegistryService;1"].
        getService(Components.interfaces.koILanguageRegistryService);
  
    //var languages = _string_array_from_enumerator(registry.languages);
    var count = new Object();
    var temp = new Object();
    registry.getLanguages(temp,count);
    var languages = temp.value;
    var tempTitles = new Array();
    var tempFilters = new Array();
    var tempTitle = '';
    var tempFilter = '';
    
    for (var i = 0; i < languages.length; i++) {
        //var filters = _string_array_from_enumerator(
        //                            registry.getFiltersForLanguage(languages[i]));
        registry.getFiltersForLanguage(languages[i], temp, count);
        var filters = temp.value;
        var filterTitle = languages[i] + ' Files';
        if (lang)  {
            if (languages[i] != lang) {
                tempTitles[i] = filterTitle;
                tempFilters[i] = filters;
            } else { // for top of list
                tempTitle = languages[i];
                tempFilter = filters;
            }
        } else {
            _append_filter(fp, filterTitle, filters);
        }
    }
  
    if (lang)  {
        _append_filter(fp, tempTitle, tempFilter);
        for (var j=0; j < tempTitles.length; j++)  {
            if (typeof(tempTitles[j]) != 'undefined' && tempTitles[j]) {
                _append_filter(fp, tempTitles[j], tempFilters[j]);
            }
        }  
    }
    _append_filter(fp, 'Web Files', ['*.xul', '*.xml', '*.css', '*.html']);
    _append_filter(fp, 'Komodo Project Files', ['*.kpf']);
}

function _append_filter(fp, title, types) {
    // If no filters, Windows gets upset and truncates the list 
    // (see http://bugs.activestate.com/show_bug.cgi?id=18815)
    if (types.length==0) return;
    var hackedTitle = title;
    if (!isLinux) {
        //XXX Linux adds the types to the title automatically; Windows doesn't.
        hackedTitle += ' (' + types.join(',') + ')';
    }
    fp.appendFilter(hackedTitle, types.join('; '));
}

function _string_array_from_enumerator(enumerator, separator) {
    var koStrings = simpleEnumeratorToArray(enumerator, 
                                            Components.interfaces.koIString);
    var result = [];
    for (var i = 0; i < koStrings.length; i++) {
        result.push(koStrings[i].value);
    }
    return result;
}

function _openFile_init(filters, language) {
    var fp = Components.classes["@mozilla.org/filepicker;1"]
                       .createInstance(Components.interfaces.nsIFilePicker);

    fp.init(window, "Open File",
            Components.interfaces.nsIFilePicker.modeOpen);

    // if filters is not defined, this is a normal file open dialog
    // and we will put the all files filter first (unless a specific
    // language is prefered), otherwise
    // we are opening a specific type of file, so we will put
    // all files last
    if (typeof(filters)=="undefined" || filters == null) {
        if (language) {
            // Specified language at the top of the filter list.
            _append_other_filters(fp, language);
            _append_all_filter(fp);
        } else {
            // All files at the top of the filter list.
            _append_all_filter(fp);
            _append_other_filters(fp);
        }
    } else {
        // filters = [['description',['.ext1','.ext2'...]],...]
        for (var i = 0; i < filters.length; i++) {
            var filter = filters[i];
            if (filter.length == 2) _append_filter(fp, filter[0], filter[1]);
        }
        _append_all_filter(fp);
    }
    return fp;
}

function _openFile_complete(fp, defFileName /* = undefined */) {
    // If I have a default filename, use it.
    var defaultDirString;
    var osPathService = Components.classes["@activestate.com/koOsPath;1"]
                                    .getService();
    if (defFileName) {
        try {
            var leafName = osPathService.basename(defFileName);
            fp.defaultString = leafName;
            defaultDirString = osPathService.dirname(defFileName);
        } catch (e) { // bad path probably.
            logError("Failed to set the default filename: " + e);
        }
    } else {
        // If there's a current view which corresponds to a local file, use its directory
        // as the base directory for the openDialog

        var v = gLastFocusedViewManager.getCurrentView();
        
        if (v && v.url && v.url.indexOf('file:') == 0) {
            // Local File
            defaultDirString = osPathService.dirname(URLToLocalFile(v.url).path);
        } else {
            // Use and MRU for the directory. (Late windows versions have this, but WTH
            // Pull up the default directory. Yay! What a time saver!
            defaultDirString = globalPrefService.prefs.getStringPref("mruDirectory");
        }
    }
  
    // If we haven't set the default dir yet, don't try to change
    // the dir of the dialog; the Linux filepicker doesn't handle
    // a directory of "." correctly.
    if (defaultDirString && defaultDirString != ".") {
        try {
            var defaultDir = Components.classes["@mozilla.org/file/local;1"]
                             .createInstance(Components.interfaces.nsILocalFile);
            defaultDir.initWithPath(defaultDirString);
            fp.displayDirectory = defaultDir;
        } catch (e) {
            // bad path - ignore it.
        }
    }
    
    if (fp.show() == Components.interfaces.nsIFilePicker.returnOK) {
        // don't return fp.file -- let the caller access
        // url and local-file properties of fp
    
        // Remember this directory for the next file open operation.
        leafName = fp.file.leafName;
        var fullName = fp.file.path;
        var dirName = fullName.substring(0, fullName.length - leafName.length - 1);
        globalPrefService.prefs.setStringPref("mruDirectory", dirName);
        return fp;
    }
    return null;
}

function dlg_openFileInDir (path) {
    var fp = _openFile_init();
    var defaultDir = Components.classes["@mozilla.org/file/local;1"]
                        .createInstance(Components.interfaces.nsILocalFile);
    defaultDir.initWithPath(path);
    fp.displayDirectory = defaultDir;
    if (fp.show() == Components.interfaces.nsIFilePicker.returnOK) {
        return fp;
    }
    return null;
}


function dlg_openFile (/* array */ filters) {
    // filters = [['type title',['.ext1','.ext2',...]],...]
    var fp = fpOpen;
    if (fp == null || typeof(filters) != "undefined") { 
        fp = _openFile_init(filters);
        if (fpOpen == null && (typeof(filters)=="undefined"||filters==null)) {
            fpOpen = fp;
        }
    }
    return _openFile_complete(fp);
}

function dlg_openFileOfLanguage(language, defaultFilename) {
    var fp = _openFile_init(null, language);
    return _openFile_complete(fp, defaultFilename);
}

function dlg_openWatchFile () {
    if (fpWatch == null) {
        fpWatch = Components.classes["@mozilla.org/filepicker;1"]
                            .createInstance(Components.interfaces.nsIFilePicker);
        fpWatch.init(window, "Watch File",
                     Components.interfaces.nsIFilePicker.modeOpen);
        _append_all_filter(fpWatch);
        _append_other_filters(fpWatch);
    }
    if (fpWatch.show() == Components.interfaces.nsIFilePicker.returnOK) {
        // don't return fpWatch.file -- let the caller access
        // url and local-file properties of fpWatch
        return fpWatch; 
    }
    return null;
}

function dlg_saveLocalFileAsFromLanguage(lang, defaultString, url) {
    if (fpSaveAs == null || fpSaveAsLang != lang) {
        var mode = Components.interfaces.nsIFilePicker.modeSave
        fpSaveAs = Components.classes["@mozilla.org/filepicker;1"]
                             .createInstance(Components.interfaces.nsIFilePicker);
        fpSaveAs.init (window, "Save File As", mode);
        if (lang)  {
            _append_other_filters(fpSaveAs, lang);
            _append_all_filter(fpSaveAs);
        }
        else  {
            _append_all_filter(fpSaveAs);
            _append_other_filters(fpSaveAs);
        }
        fpSaveAsLang = lang;
    }
  
    fpSaveAs.defaultString = defaultString; 
  
    // Open the filepicker with the directory set to the location of the
    // file we are saving.  Don't bother if this is not a local file.
    if (url.match(/^file:\/\/\/.*/)) {
        var os = Components.classes["@activestate.com/koOs;1"].getService();
        var dirname = os.path.dirname(URLToLocalFile(url).path);
        var displayDirectory = Components.classes["@mozilla.org/file/local;1"]
                                         .createInstance(Components.interfaces.nsILocalFile);
        displayDirectory.initWithPath(dirname);
        fpSaveAs.displayDirectory = displayDirectory;
    }
          
    var fpResult = fpSaveAs.show();
    if (fpResult == Components.interfaces.nsIFilePicker.returnOK 
        || fpResult == Components.interfaces.nsIFilePicker.returnReplace) {
        return fpSaveAs;
    } else /* fpSaveAs.show() == Components.interfaces.nsIFilePicker.returnCancel */ {
        return null;
    }
}

function dlg_saveLocalFileAs(view /* = NULL */) {
    var lang = view.py_inst.language;
    return dlg_saveLocalFileAsFromLanguage(lang, view.leafName, view.url);
}

function dlg_saveFileAsTemplate(view /* = NULL */) {
    var lang = view.py_inst.language;
    if (fpSaveAsTemplate == null) {
        var mode = Components.interfaces.nsIFilePicker.modeSave;
        fpSaveAsTemplate = Components.classes["@mozilla.org/filepicker;1"]
                                     .createInstance(Components.interfaces.nsIFilePicker);
        fpSaveAsTemplate.init(window, "Save File As Template", mode);
        if (lang)  {
            _append_other_filters(fpSaveAsTemplate, lang);
            _append_all_filter(fpSaveAsTemplate);
        } else  {
            _append_all_filter(fpSaveAsTemplate);
            _append_other_filters(fpSaveAsTemplate);
        }
    }
  
    fpSaveAsTemplate.defaultString = view.leafName;
  
    // Open the filepicker with the directory set to the location of the user's templates
    var os = Components.classes["@activestate.com/koOs;1"].getService();
    var templateMgr = Components.classes["@activestate.com/koTemplateManager;1"]
                                .getService(Components.interfaces.koITemplateManager);
    var dirname = templateMgr.getUserTemplateDirectory();
    var displayDirectory = Components.classes["@mozilla.org/file/local;1"]
                           .createInstance(Components.interfaces.nsILocalFile);
    displayDirectory.initWithPath(dirname);
    fpSaveAsTemplate.displayDirectory = displayDirectory;
  
    fpResult = fpSaveAsTemplate.show();
    if (fpResult == Components.interfaces.nsIFilePicker.returnOK 
        || fpResult == Components.interfaces.nsIFilePicker.returnReplace)
    {
        return fpSaveAsTemplate;
    } else /* fpSaveAsTemplate.show() == Components.interfaces.nsIFilePicker.returnCancel */ {
        return null;
    }
}

function dlg_chooseLocalFileItem (startInDir) {
    var filePicker = null;
    var displayDirectory;
    if (typeof(startInDir) == 'undefined' || startInDir == null) {
        if (fpAddToProject == null) {
            fpAddToProject = Components.classes["@mozilla.org/filepicker;1"]
                                       .createInstance(Components.interfaces.nsIFilePicker);
            fpAddToProject.init (window, "Add File to Project",
                                 Components.interfaces.nsIFilePicker.modeOpen);
            _append_all_filter(fpAddToProject);
            _append_other_filters(fpAddToProject);
        }
        filePicker = fpAddToProject;
    } else {
        filePicker = Components.classes["@mozilla.org/filepicker;1"]
                               .createInstance(Components.interfaces.nsIFilePicker);
        filePicker.init (window, "Add File to Project",
                         Components.interfaces.nsIFilePicker.modeOpen);
        _append_all_filter(filePicker);
        _append_other_filters(filePicker);
        displayDirectory = Components.classes["@mozilla.org/file/local;1"]
                                     .createInstance(Components.interfaces.nsILocalFile);
        displayDirectory.initWithPath(startInDir);
        filePicker.displayDirectory = displayDirectory;
    }
    if (filePicker && filePicker.show() == Components.interfaces.nsIFilePicker.returnOK) {
        // don't return fpAddToProject.file -- let the caller access
        // url and local-file properties of fpAddToProject
        return filePicker; 
    }
    return null;
}

function dlg_openPRJFile () {
    if (fpOpenProject == null) {
        var fpOpenProject = Components.classes["@mozilla.org/filepicker;1"]
                            .createInstance(Components.interfaces.nsIFilePicker);
        fpOpenProject.init(window, "Open Project",
                           Components.interfaces.nsIFilePicker.modeOpen);
        fpOpenProject.appendFilter('Komodo Project Files', '*.kpf');
    }
    if (fpOpenProject.show() == Components.interfaces.nsIFilePicker.returnOK) {
        return fpOpenProject;
    }
    return null;
}

function dlg_openTemplate () {
    if (fpOpenTemplate == null) {
        fpOpenTemplate = Components.classes["@mozilla.org/filepicker;1"]
                         .createInstance(Components.interfaces.nsIFilePicker);
        fpOpenTemplate.init(window, "Open Template",
                            Components.interfaces.nsIFilePicker.modeOpen);
        fpOpenTemplate.appendFilter('All Files', '*.*');
    }
    var templateMgr = Components.classes["@activestate.com/koTemplateManager;1"]
                      .getService(Components.interfaces.koITemplateManager);
    var dirname = templateMgr.getUserTemplateDirectory();
    var displayDirectory = Components.classes["@mozilla.org/file/local;1"]
                           .createInstance(Components.interfaces.nsILocalFile);
    displayDirectory.initWithPath(dirname);
    fpOpenTemplate.displayDirectory = displayDirectory;
    if (fpOpenTemplate.show() == Components.interfaces.nsIFilePicker.returnOK) {
        return fpOpenTemplate;
    }
    return null;
}

// Bring up the file selection dialog.
//  "mode" is one of the Components.interfaces.nsIFilePicker.mode*
//      constants.
//  "title" is the title the file picker dialog will use.
//  "defaultDirectory" is the directory in which the picker will start.
//  "defaultFile" is ...? (I don't know what it is. nsIFilePicker.idl says
//      'what is this?')
//  "filetype" and "fileextension" define a file type entry in the picker's
//      'File of type' filter field that will be added to all the standard
//      file filter types and will be selected as default.
function dlg_fileSelectViaDialog(mode, title, defaultDirectory,
                                 defaultFile, filetype, fileextension)
{
    try {
        var fp = Components.classes["@mozilla.org/filepicker;1"]
                           .createInstance(Components.interfaces.nsIFilePicker);
        fp.init(window, title, mode);
        if (defaultDirectory) {
            var localFileObj = Components.classes["@mozilla.org/file/local;1"]
                               .createInstance(Components.interfaces.nsILocalFile);
            localFileObj.initWithPath(defaultDirectory);
            fp.displayDirectory = localFileObj;
        }
        if (defaultFile) {
            fp.defaultString = defaultFile;
        }
        fp.appendFilter(filetype, fileextension);
        fp.appendFilters(Components.interfaces.nsIFilePicker.filterAll);
        fpResult = fp.show()
        if (fpResult == Components.interfaces.nsIFilePicker.returnOK
            || (mode == Components.interfaces.nsIFilePicker.modeSave 
                && fpResult == Components.interfaces.nsIFilePicker.returnReplace)) {
            return fp;
        } else {
            return null;
        }
    } catch (e) {
        logError("Can't browse for a file (" + e +")");
    }
    return null;
}

function filewinfuncs_forceExtension(fileObj, extension) {
    if (fileObj.url.slice(-extension.length) != extension) {
        fileObj.url += extension;
        fileObj.path += extension;
        fileObj.leafName += extension;
    }
}
