/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * Functions for accessing common and relatively generic dialogs (both
 * ActiveState's and Netscape's).
 *
 * Summary of Dialogs:
 *  queryStringDialog - A dialog to present the user with a textbox in which to
 *      specify a string.
 *  yesNoCancel
 *  yesNoDialog
 *  confirmOkDialog
 *  confirmActionDialog
 *  alertDialog
 *  promptDialog
 *  dlg_openExeFile 
 *  dlg_get_URL_andName
 *  doOpenURL
 *  BrowseForDir
 *  BrowseForFile
 *  authDialog
 *  multiFileDialog
 *
 * NOTE:
 *  Only purely generic stuff should go in here at all, since it is utilized by
 *  many XUL files!
 *
 * TODO:
 *  - Improve the summary documentation above.
 *  - Use a consistent naming scheme for all the public functions here.
 *  - Move to a more consistent API.
 *  - Purge the unnecessary ones. E.g., queryStringDialog is a better version
 *    of promptDialog.
 */



//---- globals

// Save the file pickers for efficiency and so that the last filter used is
// remembered.
var _gExeFilePicker = null;

//XXX Psychotically, this is used in a few modules. This dep. should be remove
//    post-2.3.



var isLinux = true;




//---- public methods

//precondition: defaultAction is one of "Yes", "No" "Cancel"
function yesNoCancel(prompt, defaultAction)
{
    var obj = new Object ();
    obj.prompt = prompt;
    obj.res = defaultAction;
    var response = window.openDialog(
            "chrome://komodo/content/commdlg/yesnocancel.xul",
            "_blank",
            "centerscreen,chrome,modal,titlebar",
            obj);
    return obj.res;
}


function yesNoDialog(prompt, defaultAction)
{
    var obj = new Object ();
    obj.prompt = prompt;
    obj.res = defaultAction;
    var hideCancel = true;
    var response = window.openDialog(
            "chrome://komodo/content/commdlg/yesnocancel.xul",
            "_blank",
            "centerscreen,chrome,modal,titlebar",
            obj,
            hideCancel);
    return obj.res;
}


function confirmOkDialog(
    prompt,         // Text explaining what the user should confirm
    defaultAction,  // "OK" or "Cancel"
    dontAskMePref)  // boolean pref -- if true, return "OK"
{
    var globalPrefs = Components.classes['@activestate.com/koPrefService;1'].
                      getService(Components.interfaces.koIPrefService).prefs;
    if (dontAskMePref && globalPrefs.getBooleanPref(dontAskMePref)) {
        // the string pref is not null.
        return "OK"
    }
    var obj = new Object ();
    obj.prompt = prompt;
    obj.actions = ['OK'];
    obj.defaultAction  = defaultAction;
    obj.cancelLabel = "Cancel";
    obj.checkboxLabel = "Don't show me this again.";
    var response = window.openDialog(
            "chrome://komodo/content/commdlg/confirmDialog.xul",
            "_blank",
            "centerscreen,chrome,modal,titlebar",
            obj);
    if (obj.action && obj.action != "Cancel" && obj.dontShowAgain) {
        globalPrefs.setBooleanPref(dontAskMePref, 1);
    }
    return obj.action;
}


function confirmActionDialog(
    prompt,             // Text explaining what the user should confirm
    actions,            // List of text labels in order e.g. ['OK']
    defaultAction,      // Typically "OK" or "Cancel"
    cancelLabel,        // Offer a "Cancel" button, which is special -- if false, no
                        // Cancel button
    pickedActionPref)   // the action which should be used in the "don't show
                        // me this again" case.
{
    // If pickedActionPref is the empty string, do not show the "don't show me
    // this again" checkbox.
    
    // If the _value_ of the pref named by "pickedActionPref" is a non-empty
    // string in the global prefs which is one of the allowed actions, return
    // its value.
    
    // If the user cancels, return the empty string and leave the pref alone.
    
    // If the user picks an action, return it.
    
    // If the user picks an action and selects the "Don't show me this again"
    // checkbox, set the pref to the action before returning it.
    
    var globalPrefs = Components.classes['@activestate.com/koPrefService;1'].
                      getService(Components.interfaces.koIPrefService).prefs;
    if (pickedActionPref && globalPrefs.getStringPref(pickedActionPref)) {
        // the string pref is not null.
        return globalPrefs.getStringPref(pickedActionPref);
    }
    var obj = new Object ();
    obj.prompt = prompt;
    obj.actions = actions;
    obj.defaultAction  = defaultAction;
    obj.cancelLabel = cancelLabel;
    if (pickedActionPref) {
        obj.checkboxLabel = "Don't show me this again.";
    }
    var closeTrail;
    if (cancelLabel == '') {
        closeTrail = ',close=no';
    } else {
        closeTrail = '';
    }
    var response = window.openDialog(
            "chrome://komodo/content/commdlg/confirmDialog.xul",
            "_blank",
            "centerscreen,chrome,modal,titlebar" + closeTrail,
            obj);
    if (obj.action && obj.action != cancelLabel && obj.dontShowAgain) {
        globalPrefs.setStringPref(pickedActionPref, obj.action);
    }
    return obj.action;
}


// XXX this is becoming very overloaded, we need to clean it up
function alertDialog(prompt,
                     okIsDefault, hideCancel,
                     okLabel, cancelLabel,
                     text, checkboxLabel)
{
    var obj = new Object ();
    // "prompt" is the string displayed in the dialog. (must be defined)
    obj.prompt = prompt;
    obj.res = null;  // response
    // "okIsDefault" indicates that <Return> will fire the "Ok" button.
    // (defaults to true)
    if (typeof(okIsDefault) == "undefined") {
        obj.okIsDefault = true;
    } else {
        obj.okIsDefault = okIsDefault;
    }
    // "hideCancel" indicates if the <<Cancel>> button should be shown
    // (defaults to true, if "okIsDefault" is false this must be
    // true)
    if (!obj.okIsDefault) {
        obj.hideCancel = false;
    } else {
        if (typeof(hideCancel) == "undefined") {
            obj.hideCancel = true;
        } else {
            obj.hideCancel = hideCancel;
        }
    }
    // "okLabel" and "cancelLabel" are the button text strings
    if (typeof(okLabel) == "undefined" || !okLabel) {
        obj.okLabel = "OK";
    } else {
        obj.okLabel = okLabel;
    }
    if (typeof(cancelLabel) == "undefined" || !cancelLabel) {
        obj.cancelLabel = "Cancel";
    } else {
        obj.cancelLabel = cancelLabel;
    }
    if (typeof(checkboxLabel) == "undefined" || !checkboxLabel) {
        obj.checkboxLabel = null;
    } else {
        obj.checkboxLabel = checkboxLabel;
    }
    // "text" is an optional string to display in a multi-line textbox in the
    // dialog.
    if (typeof(text) == "undefined" || !text) {
        obj.text = null;
    } else {
        obj.text = text;
    }

    window.openDialog("chrome://komodo/content/commdlg/okcancel.xul",
                      "_blank",
                      "centerscreen,chrome,modal,titlebar",
                      obj);

    // Return the result, which is the text of the button clicked
    if (!checkboxLabel)
        return obj.res.res;    
    else
        return obj.res;
}


function promptDialog(prompt, defaultResponse, okLabel, cancelLabel, multiline)
{
    if (typeof(multiline) == 'undefined') {
        multiline = false;
    }
    var obj = new Object();
    obj.prompt = prompt;
    obj.defaultResponse = defaultResponse;
    obj.okLabel = okLabel;
    obj.cancelLabel = cancelLabel;
    obj.multiline = multiline;
    var opts = "chrome,modal,titlebar";
    if (multiline) {
        opts += ",resizable=yes";
    }
    window.openDialog("chrome://komodo/content/commdlg/prompt.xul",
                      "_blank",
                      opts,
                      obj);
    return obj.res;
}


// A dialog to query the user for a string in a textbox.
//
// All arguments can be left blank or specified as null to get a default value.
//  "title" is the dialog title.
//  "prompt" is text to place before the input textbox.
//  "label" is a label to place on the textbox.
//  "value" is a default value for the textbox.
//  "mruName" can be specified (a string) to get an MRU on the text box. The
//      value of the string is the namespace for the MRU.
//  "validator" is a callable object to validate the current value when the
//      user presses "OK". It is called with the current value as the only
//      argument.  If the function returns false the "OK" is ignored.
//
// It returns null if the dialog was cancelled. Otherwise it returns the value
// entered by the user.
//
function queryStringDialog(title, prompt, label, value, mruName, validator)
{
    if (typeof(title) == 'undefined') title = null;
    if (typeof(prompt) == 'undefined') prompt = null;
    if (typeof(label) == 'undefined') label = null;
    if (typeof(value) == 'undefined') value = null;
    if (typeof(mruName) == 'undefined') mruName = null;
    if (typeof(validator) == 'undefined') validator = null;

    var obj = new Object();
    obj.title = title;
    obj.prompt = prompt;
    obj.label = label;
    obj.value = value;
    obj.mruName = mruName;
    obj.validator = validator;
    window.openDialog("chrome://komodo/content/commdlg/queryString.xul",
                      "_blank",
                      "chrome,modal,titlebar",
                      obj);
    if (obj.retval == "OK") {
        return obj.value;
    } else {
        return null;
    }
}


function dlg_openExeFile()
{
    if (_gExeFilePicker == null) {
        _gExeFilePicker = Components.classes["@mozilla.org/filepicker;1"].
                          createInstance(Components.interfaces.nsIFilePicker);
        _gExeFilePicker.init(window, "Open File",
                             Components.interfaces.nsIFilePicker.modeOpen);




        _gExeFilePicker.appendFilter('All Files', '*');

    }
    if (_gExeFilePicker.show() == Components.interfaces.nsIFilePicker.returnOK) {
        // Don't return _gExeFilePicker.file -- let the caller access url and
        // local-file properties of _gExeFilePicker.
        return _gExeFilePicker; 
    }
    return null;
}


function dlg_get_URL_andName(filePicker)
{
    this.leafName = filePicker.file.leafName;
    this.fullName = filePicker.file.path;

    // Create a URL from the file path.
    this.url = LocalFileToURL(this.fullName);

    // Fix the url. File urls are horribly broken in Mozilla,
    // so they hacked together this fixup service.
    this.url = fixupUrl(this.url);
}


function doOpenURL(prompt)
{
    obj = new Object();
    obj.prompt = prompt;
    obj.defaultResponse = "";
    obj.okLabel = "OK";
    obj.cancelLabel = "Cancel";
    obj.checkboxLabel = "Use WebDAV";
    window.openDialog("chrome://komodo/content/commdlg/promptWithCheckbox.xul",
                      "_blank",
                      "chrome,modal,titlebar",
                      obj);
    url = obj.res;
    if (url) {
        u = String(url);
        if (obj.checkbox && u.substring(0, 7) == 'http://') {
            url = "dav://" + u.substring(7, u.length);
        }
    }
    return url;
}


function BrowseForDir(current_dir)
{
    var nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"].
             createInstance(nsIFilePicker);

    fp.init(window, "Select Folder", nsIFilePicker.modeGetFolder);
    if (current_dir) {
        try {
            var localFileObj = Components.classes["@mozilla.org/file/local;1"].
                               createInstance().
                               QueryInterface(Components.interfaces.nsILocalFile);
            localFileObj.initWithPath(current_dir);
            fp.displayDirectory = localFileObj;
        } catch(e) {
            ; // let the default directory remain.
            logError("Could not set the default directory for the "+
                     "directory picker to '" + current_dir + "': " +
                     e + "\n");
        }
    }
    if (fp.show() == nsIFilePicker.returnOK) {
        var path = fp.file.path;









        return path;
    }
    return null;
}


function BrowseForFile(current, filters) {
    var nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"].
             createInstance(nsIFilePicker);

    fp.init (window, "Select File", nsIFilePicker.modeOpen);

    if (filters) {
        for (var i = 0; i < filters.length; i++) {
            var filter = filters[i];
            if (filter.length == 2) {
                var hackedTitle = filter[0];





                fp.appendFilter(hackedTitle, filter[1].join('; '));
            }
        }
    }

    if (current) {
        try {
            var localFileObj = Components.classes["@mozilla.org/file/local;1"].
                               createInstance().
                               QueryInterface(Components.interfaces.nsILocalFile);
            localFileObj.initWithPath(current);
            fp.displayDirectory = localFileObj;
        } catch(e) {
            ; // let the default directory remain.
            logError("Could not set the default directory for the file "+
                     "picker to '" + current + "': " + e + "\n");
        }
    }
    if (fp.show() == nsIFilePicker.returnOK) {
        return fp.file.path;
    }
    return null;
}


function authDialog(title, message, loginname, allowAnonymous, allowPersist) {
    /* NOTE: if you change this usage, also change it in asJSDialogProxy.js */
    if (typeof(allowPersist)=="undefined") allowPersist = true;
    if (typeof(allowAnonymous)=="undefined") allowAnonymous = false;
    var obj = new Object ();
    obj.title = title;
    obj.message = message;
    obj.user = loginname;
    obj.allowAnonymous = allowAnonymous;
    obj.allowPersist = allowPersist;
    openWindowUniqueInstance("chrome://komodo/content/commdlg/authenticate.xul",
                      "komodo_authenticate",
                      "chrome,dialog,modal=yes",
                      obj);

    // Return the full object so caller can check and use whatever values they
    // wish.
    return obj;
}

function multiFileDialog(title, prompt, urls, okcancel, dontAskMePref,
                         allowNone,
                         sizingPrefix,
                         returnUnselected,
                         makeLeafname)
// If allowNone is true, then the checkbox is enabled when _no_ files are selected
// furthermore, in those cases, string prefs are used to store "All" or "None".
// If the user cancels the dialog (either by hitting Escape or the Cancel button),
//   null is returned.
// In every other case, a list of strings (a possibly empty subset of 'urls') is returned.

// sizingPrefix is a string that will be used for dialog sizing preferences (if nothing is set, then
// no sizing memory occurs.

// returnUnselected -- if true, then the returned objects will be those which are _not_ selected.
// if not specified, then the value of false is used.
{
    var globalPrefs = Components.classes['@activestate.com/koPrefService;1'].
                      getService(Components.interfaces.koIPrefService).prefs;
    var pref_value;
    if (typeof(okcancel)=='undefined') okcancel = true;
    if (typeof(dontAskMePref)=='undefined') dontAskMePref = null;
    if (typeof(allowNone)=='undefined') allowNone = false;
    if (typeof(sizingPrefix)=='undefined') sizingPrefix = null;
    if (typeof(returnUnselected)=='undefined') returnUnselected = false;
    if (typeof(makeLeafname)=='undefined') makeLeafname = true;
    if (dontAskMePref) {
        if (! allowNone) {
            pref_value = globalPrefs.getBooleanPref(dontAskMePref);
            if (pref_value) {
                return urls;
            }
        } else {
            // probably a string pref
            pref_value = globalPrefs.getStringPref(dontAskMePref);
            if (pref_value == 'All') {
                return urls;
            } else if (pref_value == 'None') {
                return [];
            }
        }
    }
    var obj = new Object();
    obj.URLs = urls;
    obj.okcancel = okcancel;
    obj.title = title;
    obj.prompt = prompt;
    obj.dontAskMePref = dontAskMePref;
    obj.allowNone = allowNone;
    obj.dimensionPrefPrefix = sizingPrefix;
    obj.returnUnselected = returnUnselected;
    obj.makeLeafname = makeLeafname;
    openWindowUniqueInstance("chrome://komodo/content/editor/confirmClose.xul", 
                                "komodo_closeConfirm",
                                "chrome,close=no,modal,resizable=yes",
                                obj);
    if (!obj || obj.ret == false) {
        // user pressed ESC or cancel
        return null;
    }
    if (obj.dontAskAgain) {
        if (allowNone) {
            globalPrefs.setStringPref(dontAskMePref, obj.dontAskAgain);
        } else {
            globalPrefs.setBooleanPref(dontAskMePref, true);
        }
    }
    return obj.URLs;
}


