/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * stackviewer.js
 * Handle Komodo Debug stack variables mornitoring -- main part
 *
 */

// Komodo stack viewer

// Keycodes
const VK_TAB = 9;
const VK_ENTER = 13;
const VK_ESC = 27;
const VK_DEL = 46;
const VK_F2 = 113
const obStack = "callstack";
const obWatched = "watch";


function StackViewerEventist(stackviewer) {
    this.stackviewer = stackviewer;
}
StackViewerEventist.prototype = {
    onStateChange : function(state, state_extra) {
        // Even though the event may slightly lag the real state of the debugger, it should
        // always catch up.  If we use the debuggers real state, we are likely to see
        // the same state twice (ie, BREAK and BREAK in succession) causing updates
        // to happen twice.
        //    dump("State change - event  says " + state + "and debugger says " + myDebugger.state + "\n");
        this.stackviewer.update_stack(state)
    }
};

var stackViewer = null; // global stackviewer class
function StackViewer() {
    this.myDebugger = Components.classes["@activestate.com/debugger;1"].getService(Components.interfaces.koIDebugger);
    this.stackstate = Components.classes["@activestate.com/stack;1"].createInstance(Components.interfaces.koIStack);
    this.current_frame = null;
    this.current_frame_index = 0;
    this.obLocal = "locals";
    this.obGlobal = "globals";
    this.Stack = document.getElementById(obStack);
    this.exceptionText = null;

    // set up the outliners
    this.localOutliner = document.getElementById("localview");
    this.globalOutliner = document.getElementById("globalview");
    this.watchedOutliner = document.getElementById("watchedview");
    this.localOutliner.onLoad(new variableOutlinerView(this, this.obLocal));
    this.globalOutliner.onLoad(new variableOutlinerView(this, this.obGlobal));
    this.watchedOutliner.onLoad(new variableOutlinerView(this, obWatched));

    this.needToUpdateOnNextShowing = 1;
    this.dbgcookie = this.myDebugger.addListener( new StackViewerEventist(this) );
    
    this.update_stack(this.myDebugger.state);
}
StackViewer.prototype = {
    
    unload: function() {
        if (this.myDebugger) {
            this.myDebugger.removeListener(this.dbgcookie);
            this.myDebugger = this.dbgcookie = null;
        }
    },

    // Called when output tabs selected to determine whether to update stack and variables
    check_stack: function(tabview)  {
        if (this.myDebugger)  {
            if (tabview == 'vars')  {
                if (this.needToUpdateOnNextShowing) {
                    this.update_stack(this.myDebugger.state);
                }
            }
        }
    },
    
    update_stack: function(newState) {
        var frame;
        this.needToUpdateOnNextShowing = 0;

        if (newState==Components.interfaces.koIDebugger.DBG_STATE_STOPPED) {
            this._clear_stack();
            this._clear_var_panes();
            //clear_watched_vars();// replaced with...
            //this.watchedOutliner.view.clearList();
            this.current_frame = null;
            this.current_frame_index = 0;
            this.update_watched_variables();
            return;
        }
        if (newState!=Components.interfaces.koIDebugger.DBG_STATE_BREAK) {
            return;
        }
        if (document.getElementById('output_tab').getAttribute('selected') == "true")
        {
            //dump("Stack and Vars Tab not showing, no need for update\n");
            this.needToUpdateOnNextShowing = 1;
            return;
        }

        var temp = new Object();
        this.myDebugger.enumStackFrames(temp, new Object());
        var stack = temp.value;
        this._clear_stack();
        for (var frame_index = 0; frame_index < stack.length; frame_index++) {
            frame = stack[frame_index].QueryInterface(Components.interfaces.koIDebugStackFrame);
            this._add_frame(frame, frame_index);
            if (frame_index == this.stackstate.current_frame_index) {
                this.current_frame = frame;
                this.current_frame_index = this.stackstate.current_frame_index;
                this._highlight_current_frame();
                this.setTabNames();
                // only add the variables for the current stack
                this.update_variables();
            }
        }
        // always update the watched variables
        this.update_watched_variables();
    },
    
    setTabNames: function() {  
        var element;
        // XXX language specifics need to be removed!
        if(this.current_frame.language == "Perl") {
            element = document.getElementById("global-text");
            element.setAttribute('label', 'Registers');
            element = document.getElementById("local-text");
            element.setAttribute('label', 'Proximity');
            this.obLocal = "Proximity";
            this.obGlobal = "Registers";
            this.localOutliner.view.setContext(this.obLocal);
            this.globalOutliner.view.setContext(this.obLocal);
        } else { // let any other language have defaults!
            element = document.getElementById("global-text");
            element.setAttribute('label', 'Globals');
            element = document.getElementById("local-text");
            element.setAttribute('label', 'Locals');
            this.obLocal = "locals";
            this.obGlobal = "globals";
            this.localOutliner.view.setContext(this.obLocal);
            this.globalOutliner.view.setContext(this.obLocal);
        }
    },
    
    update_for_stack_change: function() {
        // do it when user change to the different stack
        var temp = new Object();
        if (this.myDebugger.state != Components.interfaces.koIDebugger.DBG_STATE_BREAK) return;
        this.myDebugger.enumStackFrames(temp, new Object());
        var stack = temp.value;
        this.current_frame_index = this.stackstate.current_frame_index;
        this.current_frame = stack[this.current_frame_index].QueryInterface(Components.interfaces.koIDebugStackFrame);
        this.update_variables();
        this.update_watched_variables();
        // update the margin markers
        show_stack_position(this.current_frame, this.current_frame_index);
    },

    update_for_tab_change: function() {
        if (!this.myDebugger || !this.current_frame ||
            this.myDebugger.state == Components.interfaces.koIDebugger.DBG_STATE_STOPPED) return;
        // when we are ready to optimize variable tabs, uncomment
        this.update_variables();
    },

    update_variables: function() {
        // update information displayed about variables defined in the
        // current stack frame
        this._clear_var_panes();
        
        // XXX optimization, we need a better way to do this so we can have
        // arbitrary context tabs
        var tab = document.getElementById("varTabbox").selectedTab;
        var currentContext = this.obLocal;
        if (tab.id == "global-text") currentContext = this.obGlobal;
        
        var contexts = new Array();
        // If someone can figure out why we have to do this magic to do a
        // get() of an array, feel free to fix it!  The same pattern
        // occurs other places as well.
        this.current_frame.getContextNames(contexts, new Object());
        var context;
        for (var i = 0; i < contexts.value.length; i++) {
            context = contexts.value[i];
            
            // we'll only update the visible context
            if (currentContext != context) continue;
            
            var temp = new Object();
            this.current_frame.enumContext(context, temp, new Object());
            var debugProps = temp.value;
            
            if (context == this.obLocal) {
                this.localOutliner.view.buildList(debugProps, this.current_frame_index);
            } else {
                this.globalOutliner.view.buildList(debugProps, this.current_frame_index);
            }
            break;
        }
    },

    update_watched_variables: function() {
        // prevent updating when not debugging
        //if (this.myDebugger.state != Components.interfaces.koIDebugger.DBG_STATE_BREAK) return;
        var watchedVarNames = new Array();
        var watchedVarValues = new Array();
        this.stackstate.getWatchedVars(this.current_frame_index, watchedVarNames, watchedVarValues, new Object());
        // add/replace all entries in the stackstate
        // XXX this should be examined, why do we do this when we just retreived the variables?
        // this is necessary because we may add watched vars while not debugging, so we have to
        // retreive them and add the properties.
        this.watchedOutliner.view.clearList();
        if (this.myDebugger.state != Components.interfaces.koIDebugger.DBG_STATE_STOPPED) {
            for (var i = 0; i < watchedVarNames.value.length; i++) {
                var watched_expr = watchedVarNames.value[i];
                var variable = watchedVarValues.value[i];
                this.stackstate.addDebugProperty(obWatched, variable.fullName, variable);
            }
            if (watchedVarValues.value.length > 0) {
                this.watchedOutliner.view.buildList(watchedVarValues.value, this.current_frame_index);
            }
        } else if (watchedVarNames.value.length > 0) {
            // we're not debugging , but show the list anyway
            this.watchedOutliner.view.buildList(watchedVarNames.value, -1);
        }
    },
    
    _add_frame: function(frame, frame_index) {
        var desc = frame.description;
        var fullname = obStack + desc;
        
        // check if the stack description already exists?
        if(document.getElementById(fullname)) return;
        // otherwise add the stack into the pane
        var newEntry = this._make_stack_node(frame_index+1, fullname, desc);
        this.Stack.appendChild(newEntry);
    },

    _make_stack_node: function(level, fullname, xvalue) {
        var item = document.createElement('menuitem');
        item.setAttribute('id', fullname);
        item.setAttribute('label', xvalue);
        item.setAttribute('oncommand','stackViewer.selectStackFrame('+level+')');
        item.setAttribute('flex','1');
        return item;
    },

    _highlight_current_frame: function() {
        var stacklist = document.getElementById('stacklist');
        stacklist.selectedItem = this.Stack.firstChild;
    },
    
    selectStackFrame: function(level)  {
        // XXX Do we REALY have to go back to the debugger for this?  We have
        // the stack frames already, why not just get the filename from that?
        
        var temp = new Object();
        this.myDebugger.enumStackFrames(temp, new Object());
        var stack = temp.value;
        for (var frame_index = 0; frame_index < stack.length && frame_index < level; frame_index++) {
            var frame = stack[frame_index].QueryInterface(Components.interfaces.koIDebugStackFrame);
            this.current_frame = frame;
            this.stackstate.current_frame_index = frame_index;
        }
        
        var v = viewManager.doFileOpenAtLine(this.current_frame.uri, this.current_frame.lineNumber);
        
        // XXX attempt to open input file
        // this is temporary until new views work is done, will only effect ActiveXSLT debugging
        if (this.current_frame.inputURI != undefined && this.current_frame.inputURI != "") {
            //dump("INPUT FILE: "+current_frame.inputURI+" LINE: "+current_frame.inputLineNumber+"\n");
            if (v._inputViewManager) {
                v.showInput();
                v._inputViewManager.doFileOpenAtLine(this.current_frame.inputURI,this.current_frame.inputLineNumber);
                v._scimoz.ensureVisibleEnforcePolicy(this.current_frame.lineNumber);
            } else {
                //dump("NO INPUT FILE PANE!\n");
            }
        }
        
        this.update_for_stack_change();
    },

    _clear_stack: function() {
        if(this.Stack.childNodes.length > 0) {
            for( var i=this.Stack.childNodes.length-1 ; i>= 0 ; i--){ 
                this.Stack.removeChild(this.Stack.childNodes[i]);
            }
        }
        document.getElementById('stacklist').setAttribute('label','');
    },
    
    _clear_var_panes: function() {
        this.localOutliner.view.clearList();
        this.globalOutliner.view.clearList();
    },
    
    
    ///////////////////////////////////////////////////////////////////////////////////////////////////
    // functions to edit values
    addWatchedVar: function(expression) {
        this.stackstate.addWatchedVarName(expression);
        var watched_val = "";
        if (this.current_frame) {
            watched_val = this.current_frame.getExpression(expression, 10);
            this.stackstate.addDebugProperty(obWatched, expression, watched_val);
        }
        this.update_watched_variables();
    },
    
    removeWatchedVar: function(expression) {
        this.stackstate.removeWatchedVarName(expression);
        if (this.current_frame) {
            this.stackstate.removeDebugProperty(obWatched, expression);
        }
        this.update_watched_variables();
    },
    
    valueChanged: function(obcontext, fullname, text_inputed) {
        var succeeded = this._tryToSetValue(obcontext, fullname, text_inputed);
        if (!succeeded) {
            // do nothing -- the variable doesn't exist.
            alertDialog("Error setting value: " + this.exceptionText);
        } else {
            this.update_for_stack_change();
        }
        document.getElementById("scin").focus();
    },

    // attempt to set the value of an existing entry
    _tryToSetValue: function(obcontext, fullname, text_inputed) {
        var variable;
        var succeeded = false;
        
        try {
            // outer catch: does the variable even exist?
            variable = this.stackstate.lookupDebugProperty(obcontext, fullname);
            // inner catch: is the entry an l-value?
            if(variable) {
                try {
                    var got_error = new Object();
                    variable.SetValueAsString(text_inputed, got_error);
                    this.exceptionText = got_error.value;
                    succeeded = (this.exceptionText==null);
                } catch (e) {
                    this.exceptionText = "unexpected error setting value";
                }
            } else {
                this.exceptionText = "No variable named '" + fullname + "' exists.";
            }
        } catch(e) {
            this.exceptionText = "No variable named '" + fullname + "' exists.";
        }
        return succeeded;
    },

    addVariable: function(event, id) {
        var outliner = document.getElementById(id);
        outliner.newVar();
    },
    delVariable: function(event, id) {
        var outliner = document.getElementById(id);
        outliner.deleteVar();
    },
    notused: function() {
        // Just a placeholder
        logError("This shouldn't be called");
    }
};


function stackOnLoad() {
    stackViewer = new StackViewer();
}

function stackOnUnload() {
    stackViewer.unload();
    stackViewer = null;
}







