/* Copyright (c) 2000-2002 ActiveState SRL.
   See the file LICENSE.txt for licensing information. */

// Handle drag-drop of files onto the main Workspace window.

function ProcessDragOver(event) {
    var dragService = Components.classes["@mozilla.org/widget/dragservice;1"].
        getService(Components.interfaces.nsIDragService);
    var dragSession = dragService.getCurrentSession();
    if ( !dragSession ) {
        //dump("No DRAG SESSION!\n");
        return false;
    }
    
    var isOkToDrop = false;
    if (dragSession.isDataFlavorSupported("text/unicode") ||
        dragSession.isDataFlavorSupported("text/x-moz-url") ||
        dragSession.isDataFlavorSupported("application/x-moz-file")

        || dragSession.isDataFlavorSupported("TEXT")
        // XXX for a later release, scintilla needs work in this area
        //|| dragSession.isDataFlavorSupported("COMPOUND_TEXT")
        //|| dragSession.isDataFlavorSupported("STRING")
        //|| dragSession.isDataFlavorSupported("UTF-8")

        ) {
        isOkToDrop = true;
    }
    event.cancelBubble = true;
    event.preventBubble();
    dragSession.canDrop = isOkToDrop;
    return isOkToDrop;
}

function unpackDroppedData(dataObj, flavor, ret) {
    var os = Components.classes["@activestate.com/koOs;1"].
          getService(Components.interfaces.koIOs);
    var infoService = Components.classes['@activestate.com/koInfoService;1'].
            getService(Components.interfaces.koIInfoService);

    var fname = null;
    var url = null;
    ret.text = null;
    ret.isURL = false;
    ret.isFileURL = false;
    ret.isFTPURL= false;
    ret.isFileOrDir = false;  // true for both file:// and ftp://
    ret.isRemoteFileOrDir = false; // true for ftp:// and koremote://
    ret.isExistingFile = false;
    ret.isDebuggerFile = false;
    ret.isDir = false;
    switch (flavor) {
    case "application/x-moz-file":
        //dump("We're in x-moz-file?!??\n");
        try { // XXX don't know when this is true ...
            ret.text = dataObj.value.QueryInterface(Components.interfaces.nsIFile).target;
        } catch(e) {
            return;
        }
        ret.isFileOrDir = true;
        ret.isFileURL = true;
        ret.isExistingFile = os.path.exists(ret.text);
        //dump('type was application/x-moz-file\n');
        return;
        break;
    default: // we try this as a url irregardless of type
        try {
            ret.text = dataObj.value.QueryInterface(Components.interfaces.nsISupportsWString).data;
        } catch(e) {
            //dump("not support WSTRING\n");
            /* it's not nsISupportsWString */
            return;
        }
        //dump('dropped data is '+ret.text+'\n');
        var origdata = ret.text;
        if (ret.text.search("://") == -1) { // Disqualify things that don't come close to looking like URLs
            if (infoService.platform.match(/^linux/)) {
                if (ret.text.match(/^file:\/[\w|\.]/)) {
                    // unix file:/home.... url from drag/drop
                    ret.text = ret.text.slice(5);
                } else {
                    // XXX LocalFileToURL below will convert ANYTHING
                    // on linux, so we have to be strict about what we
                    // allow through here.
                    return;
                }
            }
            //dump('dropped data is '+ret.text+'\n');
            try {
                url = LocalFileToURL(origdata);
                ret.text = url; // fall through, magic presto it's now a URL
            } catch (e) {
                //dump('this is not a url: '+ret.text+'\n');
                return;
            }
        }
        //dump('dropped URL is '+ret.text+'\n');
        ret.isURL = true;
        if (ret.text.search('kodebugger://') == 0) {
            ret.isDebuggerFile = true;
            return;
        } else
        if (ret.text.search('file://') == 0) {
            ret.isFileOrDir = true;
            ret.isFileURL = true;
            try {
                //dump("before URLToLocalFile\n");
                var path = URLToLocalFile(ret.text).path;
                //dump("after URLToLocalFile: "+path+"\n");
                ret.isExistingFile = os.path.exists(path);
                //dump("after exists test\n");
                if (ret.isExistingFile) {
                    ret.isDir = os.path.isdir(path);
                }
            } catch (e) {
                ret.isExistingFile = false;
            }
            return;
        } else
        if (ret.text.search('ftp://') == 0) {
            ret.isFileOrDir = true;
            ret.isFTPURL = true;
            res.isRemoteFileOrDir = true;
            // XXX should find out if it is a file on one of the servers we know about?
            // XXX should find out if it's a dir (hard?)
            return;
        } else
        if (ret.text.search('koremote://') == 0) {
            ret.isFileOrDir = true;
            ret.isRemoteFileOrDir = true;
            return;
        } else
        if (ret.text.search('https?://') == 0) {
            return;
        }
        // not a url after all
        ret.text = origdata;
        ret.isURL = false;
    break;
    }
}

function ProcessDragDrop(event) {
    //dump('ProcessDragDrop start\n');
    event.cancelBubble = true;
    var desc;
    var dragService = Components.classes["@mozilla.org/widget/dragservice;1"].getService(Components.interfaces.nsIDragService);
  
    var dragSession = dragService.getCurrentSession();
    if (!dragSession) {
        logError("Could not get the drag session!");
        //dump('ProcessDragDrop no drag session\n');
        return null;
    }
    var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
  
    trans.addDataFlavor("application/x-moz-file");
    trans.addDataFlavor("text/x-moz-url");
    trans.addDataFlavor("text/unicode");

    trans.addDataFlavor("TEXT");
    // XXX for a later release, scintilla needs work in this area
    //trans.addDataFlavor("STRING");
    //trans.addDataFlavor("COMPOUND_TEXT");
    //trans.addDataFlavor("UTF-8");

    
    var num_items = 0;
    for (var i = 0; i < dragSession.numDropItems; ++i) {
        dragSession.getData(trans, i);
        var dataObj = new Object();
        var bestFlavor = new Object();
        var len = new Object();
        try {
            trans.getAnyTransferData(bestFlavor, dataObj, len);
        } catch (e) {
            continue;
        }

        desc = new Object();
        unpackDroppedData(dataObj,bestFlavor.value, desc)
        desc.dragAction = dragSession.dragAction;
        //dump('ProcessDragDrop data:'+desc.text+'\n');
        if (desc.isFileURL || desc.isFTPURL) {
            //dump('ProcessDragDrop opening files\n');
            doOpenPath(desc.text);
            num_items++;
        }
    } // foreach drag item
    if (num_items < 1) {
        return desc;
    }
    return null;
} 

