    /* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

/* XXX split editor support, see scintillaview.js for details
function EditorViewManager(tabBoxId, mysci_widget, sci_widget_split)
{
    TabViewManager.apply(this, [tabBoxId, mysci_widget, sci_widget_split]);
}
*/

function EditorViewManager(tabBoxId, mysci_widget)
{
    TabViewManager.apply(this, [tabBoxId, mysci_widget]);
}

// superclass: TabViewManager
EditorViewManager.prototype = new TabViewManager();
EditorViewManager.prototype.constructor = EditorViewManager;

EditorViewManager.prototype._createScintillaView = function() {
    return new EditorScintillaView(this, this._sci_widget);
}

EditorViewManager.prototype._hideScintilla = function() {
  koRequestUpdateCommands("buffer-visible");
  // send buffer-focus at this point.  This event
  // is generally sent by view.activate(), but as we
  // dont have a new view to activate, we handle this here.
  koRequestUpdateCommands("buffer-focus");

  //var browserBox = document.getElementById('browser-box');
  //browserBox.setAttribute('collapsed', 'false');
  
  GettingStarted_Show();
  var workDeck = document.getElementById('workspace-deck');
  workDeck.setAttribute('index', '0');

  // Set the focus to the komodo dragon so that the scrollwheel works
  // in the getting started page
  document.getElementById("komodopix").focus();
}

EditorViewManager.prototype._showScintilla = function() {
  koRequestUpdateCommands("buffer-visible");
  
  var workDeck = document.getElementById('workspace-deck');
  workDeck.setAttribute('index', '2');  // 1 is print buffer so skip that

  // needs to be collapsed or it will constrain the editor panel
  //var browserBox = document.getElementById('browser-box');
  //browserBox.setAttribute('collapsed', 'true');

  // Set focus to scintilla.
  this.setFocus();  
}

EditorViewManager.prototype.doDirtyDocsExist = function() {
    // Find out whether there are any dirty views.
    var dirtyDocsExist = false;
    for(var viewURL in this._views) {
        var view = this._views[viewURL];
        //XXX views need to be redesigned so that I don't have to reach inside to do this
        dirtyDocsExist |= view.isDirty() || (view._inputViewManager && view._inputViewManager.doDirtyDocsExist());
    }
    return dirtyDocsExist;
}

EditorViewManager.prototype.doSaveAll = function() {
  // walk the views for things we have in memory
  // for any that aren't associated, raise them as
  // a possibility.

  var initialView = this.getCurrentView();

  if (initialView == null) {
    return true; // not an error -- if saveAll is called before any views are opened, this will be true.
  }

  for(viewURL in this._views) {
    //dump(flattenJSObject(viewURL)+"\n");
    var view = this._views[viewURL];
    //XXX views need to be redesigned so that I don't have to reach inside to do this
    var childDirtyDocsExist = view._inputViewManager && view._inputViewManager.doDirtyDocsExist;
    if (view.isDirty() || childDirtyDocsExist) {

      //XXX koEditorWrapper always acts on the current view,
      //so we must switch to it here or it will save the contents
      //of the foreground buffer on top of the file at view.url!

      this.setCurrentView(view);

      if (!childDirtyDocsExist || view._inputViewManager.doSaveAll()) {
      	if (!view.url) {
        	// ignore failure - user may choose to cancel saving this
        	// file, but want to save the rest
        	// (indeed, MarkH believes these shouldnt be prompted!)
        	view.doSave();
      	} else {
        	if (!view.doSave()) {
          		return false;
        	}
      	}
      }
    }
  }

  // If we are already there, setCurrentView is smart :-)
  this.setCurrentView(initialView);
  return true;
}

 
EditorViewManager.prototype.destroyView = function(view) {
    TabViewManager.prototype.destroyView.apply(this, [view]);
}
