    /* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

function MenuListViewManager(menuListId, mysci_widget) {
    if (arguments.length == 0) return;
    ViewManager.apply(this);
    this._sci_widget = mysci_widget;
    this._scimoz = mysci_widget.ISciMoz;
    
    this._scimoz_eventist = new eventist(this);
    
    this._inputSplitter = document.getElementById('input-splitter');
    this._inputBox = document.getElementById('input-vbox');
    this._inputScin = document.getElementById('input-scin');
    this._inputBar = document.getElementById('inputbar-inner');
    this._inputBarButton = document.getElementById('inputBufferCloseButton');
    this._inputOuterBox = document.getElementById('input-outerbox');
        
    this._hideScintilla();
    
    this.editorItemsController = new EditorItemsController(this);
    this._fileItemsController = new FileItemsController(this);

    this._menuList = document.getElementById(menuListId);
    this._menuPopup = document.getElementById(menuListId+"-popup");
    this._currentview = null;
    
}

// superclass: ViewManager
MenuListViewManager.prototype = new ViewManager();
MenuListViewManager.prototype.constructor = MenuListViewManager;

// view manager destructor, be sure we dereference everything we use
MenuListViewManager.prototype.destructor = function(event) {
    this.editorItemsController.destructor();
    this.editorItemsController = null;
    
    this._fileItemsController.destructor();
    this._fileItemsController = null;
    
    //this._scimoz.unhookEvents( this._scimoz_eventist );
    this._sci_widget.scimoz_eventist = null;
    this._scimoz_eventist.destructor();
    this._scimoz_eventist = null;
    ViewManager.prototype.destructor.apply(this, []);
}


// interface to the tab widget
MenuListViewManager.prototype.doMenuPick = function(event) {
    var menuIndex = event.rangeOffset - 1;
    this.setCurrentView(this._views[menuIndex]);
    this.setFocus();
}

MenuListViewManager.prototype.setFocus = function() {
    this._sci_widget.focus();
    // this is a bit of a hack due to the delayed reflow.  The "focus()" method
    // above never makes it to SciMoz - so we set the focus property on SciMoz directly.
    this._scimoz.focus = true;
}

MenuListViewManager.prototype.findMenuItemByURL = function(url) {
    var item;
    for(var i=0; i<this._menuPopup.childNodes.length; i++) {
        item = this._menuPopup.childNodes[i];
        if(item.getAttribute("url") == url) {
            return item;
        }
    }
    return null;
}

MenuListViewManager.prototype.removeMenuItemByURL = function(url) {
    // find index for url
    var item = this.findMenuItemByURL(url);
    if (item) {
        document.getElementById('inputbar-status-img').removeAttribute('src');
        document.getElementById('inputbar-action-img').removeAttribute('src');
        this._menuPopup.removeChild(item);
        return true;
    }
    return false;
}

MenuListViewManager.prototype.addMenuItem = function(url) {
    if (typeof(url) == "undefined" || url == "") return false;
    var item = document.createElement('menuitem');
    item.setAttribute('url', url);
    item.setAttribute('label', displayNameFromURL(url));
    item.setAttribute('oncommand','this.parentNode.manager.doFileOpen("'+url+'");this.parentNode.manager.updateViewIcon();');
    return this._menuPopup.appendChild(item);
}

MenuListViewManager.prototype.updateViewIcon = function(view) {
    if (typeof(view)=='undefined') {
        view = this.getCurrentView();
    }
    var statusimg = document.getElementById('inputbar-status-img');
    var actionimg = document.getElementById('inputbar-action-img');
    var pKeys = {};
    var pValues = {};
    view.item = new partWrapper(null,0);
    gFileStatusSvc.getFileStatus(view.url, pKeys, new Object(), pValues, new Object());
    view.item.url = view.url;
    for (index in pKeys.value) {
        updateStatusItem(view.item, pKeys.value[index], pValues.value[index]);
    }
    this.updateFileStatusIcon(view.item,actionimg,statusimg);
}


MenuListViewManager.prototype.updateStatusIcons = function() {
    // we only update the currently selected url
    var view = this.getCurrentView();
    if (view) {
        var menuitem = this.findMenuItemByURL(view.url);
        this.updateViewIcon(menuitem);
    }
}


MenuListViewManager.prototype.selectMenuItemByURL = function(url) {
}

MenuListViewManager.prototype.renameMenuItem = function(oldname, newname) {
    var item = this.findMenuItemByURL(oldname);
    if (!item) return false;
    var newitem = this.findMenuItemByURL(newname);
    if (newitem) return false;
    item.setAttribute('url', newname);
    item.setAttribute('label', displayNameFromURL(newname));
    item.setAttribute('oncommand','this.parentNode.manager.doFileOpen("'+newname+'");this.parentNode.manager.updateViewIcon(this);');
    return true;
}

// main view manager interface

MenuListViewManager.prototype.getCurrentView = function() {
    return this._currentview;
}

MenuListViewManager.prototype.getNextView = function(/*bool*/ rotate /*=true*/) {
    if (typeof(rotate)=="undefined") rotate = true;
      if (this._currentview) {
          var item;
          for(var i=0; i<this._menuPopup.childNodes.length; i++) {
              item = this._menuPopup.childNodes[i];
              if(item.getAttribute("url") == this._currentview.url) {
                  if (i==this._menuPopup.childNodes.length-1) {
                      item = this._menuPopup.childNodes[0];
                  } else {
                      item = this._menuPopup.childNodes[i+1];
                  }
                  if (item) {
                      return this._views[item.getAttribute("url")];
                  }
              }
          }
     }
    return this._currentview;
}

MenuListViewManager.prototype.getPreviousView = function(/*bool*/ rotate /*=true*/) {
    if (typeof(rotate)=="undefined") rotate = true;
    if (this._currentview) {
        var item;
        for(var i=this._menuPopup.childNodes.length-1; i>=0 ; i--) {
            item = this._menuPopup.childNodes[i];
            if(item.getAttribute("url") == this._currentview.url) {
                if (i==0) {
                    item = this._menuPopup.childNodes[this._menuPopup.childNodes.length-1];
                } else {
                    item = this._menuPopup.childNodes[i-1];
                }
                if (item) {
                    return this._views[item.getAttribute("url")];
                }
            }
        }
    }
    return this._currentview;
}

MenuListViewManager.prototype.setCurrentView = function (view, force) {
    ViewManager.prototype.setCurrentView.apply(this, [view, force]);
    this._currentview = view;
    this._setMenuText();
}

MenuListViewManager.prototype.displayNameFromURL = function(url) {
    // XXX this should have been in displayNameFromURL, but
    // that function is being used for many things other than a display name!
    if (url.match(/^kodebugger:\/\/\w+\//)) {
        // don't return kodebugger urls for display
        return url.replace(/^kodebugger:\/\/\w+\//,'')
    }
    return displayNameFromURL(url);
}

MenuListViewManager.prototype._setMenuText = function() {
    var v = this.getCurrentView();
    if (v) {
        this._menuList.label = this.displayNameFromURL(v.url);
        var menuitem = this.findMenuItemByURL(v.url);
        this.updateViewIcon(v);
    } else {
    	this._menuList.label = "";
        document.getElementById('inputbar-status-img').removeAttribute('src');
        document.getElementById('inputbar-action-img').removeAttribute('src');
    }
}

MenuListViewManager.prototype.activate = function() {
    var v = this.getCurrentView();
    if (v) {
        this._showScintilla();
        this.setCurrentView(v, true);
    } else {
        this._hideScintilla();
        this._setMenuText();
    }
    this._sci_widget.scimoz_eventist = this._scimoz_eventist;
    this._scimoz.hookEvents( this._scimoz_eventist );
    this.observerSvc.AddObserver(this, "SciMoz:FileDrop");
    this.observerSvc.AddObserver(this, "file_status_updated");

    //XXX can't append, because koGo*ScintillaCommands (in
    //koGlobalOverlay.js) assume the koScintillaController comes last
    this._sci_widget.controllers.insertControllerAt(0, this.editorItemsController);
    this._sci_widget.controllers.insertControllerAt(0, this._fileItemsController);

    this._menuList.appendChild(this._menuPopup);
    this._menuPopup.removeAttribute('menugenerated');
    this._menuPopup.manager = this;
    this._sci_widget.viewManager = this;
}

MenuListViewManager.prototype.deactivate = function() {
    var view = this.getCurrentView();
    if (view) {
        view.deactivate();
    }
    this.observerSvc.RemoveObserver(this, "SciMoz:FileDrop");
    this.observerSvc.RemoveObserver(this, "file_status_updated");
    this._scimoz.unhookEvents( this._scimoz_eventist );
    this._sci_widget.scimoz_eventist = null;
    this._sci_widget.controllers.removeController(this.editorItemsController);
    this._sci_widget.controllers.removeController(this._fileItemsController);
    this._menuPopup.manager = null;
}

MenuListViewManager.prototype.destroyView = function(view) {
    // remove the menu item from the DOM tree.
    this.removeMenuItemByURL(view.url);
    
    delete this._views[view.url];

    view.destructor();
    view = null;

    if (this.getCurrentView() == null) {
        this._hideScintilla();
    }
    this._setMenuText();
    return true;
}

MenuListViewManager.prototype._hideScintilla = function() {
  this._inputSplitter.setAttribute('collapsed', 'true');
  if (this._inputSplitter.getAttribute('state') == 'open')
      this._inputSplitter.setAttribute('state','collapsed');  
  this._inputBarButton.setAttribute('disabled', 'true');
  //this._inputOuterBox.removeAttribute('flex');
  this._inputOuterBox.setAttribute('collapsed', 'true');
}

MenuListViewManager.prototype._showScintilla = function() {
  this._inputSplitter.removeAttribute('collapsed');
  if (this._inputSplitter.getAttribute('state') == 'collapsed')
      this._inputSplitter.setAttribute('state','open');
  this._inputBarButton.removeAttribute('disabled');
  this._inputOuterBox.removeAttribute('collapsed');
  //this._inputOuterBox.setAttribute('flex', '1');
}
 
MenuListViewManager.prototype._createNewView = function(type, url, leafName, language) {
    // deactivate any current view
    window.setCursor("wait");
    var oldview = null;
    var currentView = this.getCurrentView();
    if (currentView != null) {
        oldview = currentView;
        currentView.deactivate();
    }

    var newView = null;
    switch (type) {
    case "scintilla":
        newView = new ScintillaView(this, this._sci_widget, url, leafName, language);
        if (url && leafName) {
            newView.initWithURL(url, leafName);
        }
        else {
            logDebug("MenuListViewManager._createNewView only supports URI-based initialization.");
        }
        
        // Create a new tab for the view and append it to the tab box.
        this.addMenuItem(newView.url);
        this.selectMenuItemByURL(newView.url);
	
        this._views[newView.url] = newView;
        this._currentview = newView;

        if (!newView.loadURI(language)) {
            if (oldview) {
                this._currentview = oldview;
                this.selectMenuItemByURL(this._currentview);
                oldview.activate();
            }
            var errmsg = newView.errmsg;
            this.destroyView(newView);
            var prompt = "Unable to load " + displayNameFromURL(url) + ": " +
                         errmsg;
            window.setCursor("auto");
            alertDialog(prompt);
            return null;
        }
        break;
    default:
        window.setCursor("auto");
        throw Components.results.NS_ERROR_UNEXPECTED;
    }
    if (newView == null) {
        if (oldview) oldview.activate();
        window.setCursor("auto");
        return null;
    }
    
    this._showScintilla();

    if (url) newView.item.url = url;

    newView.activate();
    updateTitlebar(newView);
    window.setCursor("auto");
    return newView;
}

MenuListViewManager.prototype.createNewViewWithURI = function(type, URI, leafName) {
    return this._createNewView(type, URI, leafName, null)
}

MenuListViewManager.prototype.renameView = function(view, url, leafName) {

    this.renameMenuItem(view.url, url, leafName);

    delete this._views[view.url];	
    view.url = url;
    view.leafName = leafName;
    view.untitled = false;
    view.item.url = url;
    this._views[view.url] = view;
}

// Change the appearance of a view's tab, etc. if it is dirty.
    
MenuListViewManager.prototype.indicateIfDirty = function(view) {
    var item = this.findMenuItemByURL(view.url);
    if(item) {
        if (view.isDirty()) {
            addStyle(item, "text-decoration", "underline");
        }
        else {
            addStyle(item, "text-decoration", "none");
        }
    }
}

MenuListViewManager.prototype.doFileNew = function(language) {
	//Check the preference to see if one exists
	var globalPrefService = Components.classes["@activestate.com/koPrefService;1"].
	    getService(Components.interfaces.koIPrefService);
	var globalPrefs = globalPrefService.prefs;
	var newType = globalPrefs.getStringPref("fileDefaultNew");
	//dump ("newType = " + newType + "\n");
	var lang = language;

	if (newType != null && language == 'Default') {  // && not the default
	    lang = newType;
	}

	// Create a new view object.
    return this.createNewView("scintilla", null, null, lang);
}

MenuListViewManager.prototype.selectView = function(view) {
    var menuitem = this.findMenuItemByURL(view.url);
    this._menuList.label = menuitem.getAttribute('label');
    this.updateViewIcon(view);
}

MenuListViewManager.prototype.Observe = function (subject ,topic, fname) {
    if (topic == 'file_status_updated') {
        this.updateStatusIcons();
        return;
    } else if (topic == 'SciMoz:FileDrop') {
        var scimoz = subject.QueryInterface(Components.interfaces.ISciMoz);
        var v = this.getCurrentView();
        if (v._scimoz == scimoz) {
          doOpenPath(fname,this);
        }
    }
}


