    /* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

// XXX split editor support, see EditorViewManager.js for details
// function TabViewManager(tabBoxId, mysci_widget, sci_widget_split)
function TabViewManager(tabBoxId, mysci_widget)
{
    if (arguments.length == 0) return;

    ViewManager.apply(this);
    this._sci_widget = mysci_widget;
    this._sci_widget.viewManager = this;
    this._sci_widget.scimoz_eventist = new eventist(this);
    this._scimoz = this._sci_widget.ISciMoz;
    this._scimoz.hookEvents( this._sci_widget.scimoz_eventist );

    // can not hide scintilla yet, as the controllers need to be added first,
    // but the controllers require this to be initialized.    
    this._tabBox = document.getElementById(tabBoxId);
    this._tabBox.tabManager = this;

    /* XXX split editor support, see scintillaview.js for details
    this.sci_widget_split = null;
    if (typeof(sci_widget_split) != 'undefined' && sci_widget_split) {
        this.sci_widget_split = sci_widget_split;
        this.splitViewManager = new ScintillaViewManager('splitView', this.sci_widget_split);
    }
    */
    
    // Observe the dirtyness of the view
    // this.observerSvc set in parent class
    this.observerSvc.AddObserver(this, 'dirtyness');
    this.observerSvc.AddObserver(this, "SciMoz:FileDrop");    
    this.observerSvc.AddObserver(this, 'file_status_updated');

    this._views = [];

    // For "recent" buffer switching.
    this._recentUrls = new Array();
    this._inBufferSwitchingSession = false;
}

// superclass: ViewManager
TabViewManager.prototype = new ViewManager();
TabViewManager.prototype.constructor = TabViewManager;

// interface to the tab widget
TabViewManager.prototype.doTabPick = function(event) {
    var tabId = event.target.getAttribute('id');
    this.setCurrentView(this._views[tabId]);
    this.setFocus();
    this.doViewTabSwitch();
}

TabViewManager.prototype.doViewTabSwitch = function(event) {
    var view = this.getCurrentView();
    var language = view.py_inst.language;
    var htab = document.getElementById('view_html_tab');
    var stab = document.getElementById('view_source_tab');
    var vtabs = document.getElementById('view_tabs');
    // XXX until we get xml stylesheet working right, only HTML
    if (language == 'HTML'/* || language == 'XML'*/) {
        vtabs.removeAttribute('collapsed');
        if (htab.selected) viewAsHTML();
    } else {
        vtabs.setAttribute('collapsed',true);
        vtabs.selectedTab = stab;
        // reset the tab switch
        doViewAsHTMLTabSwitch(stab);
    }
}

TabViewManager.prototype.destructor = function() {
    this._scimoz.unhookEvents( this._sci_widget.scimoz_eventist );
    this.observerSvc.RemoveObserver(this, 'dirtyness');
    this.observerSvc.RemoveObserver(this, "SciMoz:FileDrop");
    this.observerSvc.RemoveObserver(this, "file_status_updated");
    ViewManager.prototype.destructor.apply(this, []);
}

TabViewManager.prototype.setFocus = function() {
    this._sci_widget.focus();
    // this is a bit of a hack due to the delayed reflow.  The "focus()" method
    // above never makes it to SciMoz - so we set the focus property on SciMoz directly.
    this._scimoz.focus = true;
}

// main view manager interface

TabViewManager.prototype.getCurrentView = function() {
    var currentTab = this._tabBox.currentTab;
    var currentTabId = this._tabBox.getTabId(currentTab);

    if(currentTabId && this._views[currentTabId]) {
        return this._views[currentTabId];
    }
    else {
        return null;
    }
}

TabViewManager.prototype.getNextView = function(/*bool*/ rotate /*=true*/) {
    if (typeof(rotate)=="undefined") rotate = true;
    
    if (this._tabBox.canNextTab()) {
        var nextTabIndex = this._tabBox.getNextTab();
        var nextTabId = this._tabBox.getTabId(nextTabIndex);
        return this._views[nextTabId];
    } else if (!rotate && this._tabBox.canPrevTab()) {
        var prevTabIndex = this._tabBox.getPrevTab();
        var prevTabId = this._tabBox.getTabId(prevTabIndex);
        return this._views[prevTabId];
    } else {
        // Try and get the first.
        var firstTabId = this._tabBox.getTabId(0);
        if (firstTabId)
            return this._views[firstTabId];
    }
    // Cant find anything better - return the current view 
    // (which may _still_ be NULL!)
    return this.getCurrentView();
}

TabViewManager.prototype.getPreviousView = function(/*bool*/ rotate /*=true*/) {
    if (typeof(rotate)=="undefined") rotate = true;
    if(this._tabBox.canPrevTab()) {
        var prevTabIndex = this._tabBox.getPrevTab();
        var prevTabId = this._tabBox.getTabId(prevTabIndex);
        return this._views[prevTabId];
    } else if (!rotate && this._tabBox.canNextTab()) {
        var nextTabIndex = this._tabBox.getNextTab();
        var nextTabId = this._tabBox.getTabId(nextTabIndex);
        return this._views[nextTabId];
    } else {
        // Try and get the last.
        var lastTabId = this._tabBox.getTabIdByIndex(this._tabBox.tabs.length-1);
        if (lastTabId)
            return this._views[lastTabId];
    }
    // Cant find anything better - return the current view 
    // (which may _still_ be NULL!)
    return this.getCurrentView();
}

TabViewManager.prototype.destroyView = function(view) {
    // remove the tab from the DOM tree.
    this._tabBox.removeTab(view.url);
    this.updateWindowList(view, 'close');
    
    delete this._views[view.url];
    view.destructor();
    view = null;

    var separator = document.getElementById('window-menu-separator');
    if (this.getCurrentView() == null) {
        this._hideScintilla();
        separator.setAttribute('collapsed', 'true');
    }
    return true;
}

TabViewManager.prototype._createScintillaView = function() {
    return new ScintillaView(this, this._sci_widget);
}

TabViewManager.prototype._createNewView = function(type, url, leafName, language, fileTemplate) {
    // perfTimer("TabViewManager.createNewView", "start", true);
    doViewAsHTMLTabSwitch(document.getElementById('view_source_tab'));
    window.setCursor("wait");
    // deactivate any current view
    var oldview = null;
    var currentView = this.getCurrentView();
    if (currentView != null) {
        oldview = currentView;
        currentView.deactivate();
    }

    var newView = null;
    switch (type) {
    case "scintilla":
        newView = this._createScintillaView();
        if (!url && !fileTemplate && language) {
            newView.initWithLanguage(language);
        }
        else if (url) {
            newView.initWithURL(url, leafName);
        } else if (fileTemplate) {
            newView.initWithFileTemplate(fileTemplate);
        } else
        {
            logDebug("Cannot create view with given arguments\n\n");
        }            
        // create the tab in advance so we can set the current view
        this._tabBox.appendTab(newView.url, newView.leafName);
        this._views[newView.url] = newView;
        this.selectView(newView);

        if ((url || fileTemplate)&& !newView.loadURI(language)) {
            if (oldview) { 
                this.selectView(oldview);
                oldview.activate();
            }
            var errmsg = newView.errmsg;
            this.destroyView(newView);

            window.setCursor("auto");
            var prompt;
            if (fileTemplate) {
                prompt = "Unable to load template \"" + fileTemplate.name + "\": " + errmsg;
                alertDialog(prompt, true, true, 'OK');
            }                
            else {
                prompt = "Unable to load \"" + displayNameFromURL(url) + "\": " + errmsg;
                alertDialog(prompt, true, true, 'OK');
            }
            
            return null;
        }
        break;
    default:
        window.setCursor("auto");
        throw Components.results.NS_ERROR_UNEXPECTED;
    }
    if (newView == null) {
        if (oldview) oldview.activate();
        window.setCursor("auto");
        return null;
    }

    this._showScintilla();
    if (url) newView.item.url = url;

    newView.activate();
    
    if (url) {
        this._tabBox.setTabStatus(url, newView.item);
    }
    
    updateTitlebar(newView);
    this.updateWindowList(newView, 'open');
    // perfTimer("TabViewManager.createNewView", "done");
    window.setCursor("auto");
    return newView;
}

TabViewManager.prototype.createNewViewWithURI = function(type, URI, leafName) {
    return this._createNewView(type, URI, leafName, null, null);
}    

TabViewManager.prototype.createNewViewWithFileTemplate = function(type, template) {
    return this._createNewView(type, null, null, null, template);
}
    
TabViewManager.prototype.createNewViewWithLanguage = function(type, language) {
    return this._createNewView(type, null, null, language, null);
}

TabViewManager.prototype.renameView = function(view, url, leafName) {
    this._renameTab(view.url, url, leafName);
    var oldurl = view.url;

    delete this._views[view.url];	
    view.url = url;
    view.item.url = url;
    view.leafName = leafName;
    view.untitled = false;
    this.updateWindowList(view, 'rename', oldurl);
    this._views[view.url] = view;
    this.setMostRecentView(view);
}

TabViewManager.prototype._renameTab = function(oldId, url, leafName) {
    // Change the tab's title _and_ it's underlying id.
    this._tabBox.setTabTitle(oldId, leafName+" ");
    this._tabBox.setTabId(oldId, url);
}
 
// Change the appearance of a view's tab, etc. if it is dirty.
 
TabViewManager.prototype.indicateIfDirty = function(view) {
    var tabtext = this._tabBox.getTabTitle(view.url)
    if(tabtext) {
        if (view.isDirty()) {
            tabtext = tabtext.replace(/\s$/, "*");
            this._tabBox.setTabTitle(view.url, tabtext);
        }
        else {
            tabtext = tabtext.replace(/\*$/, "");
            tabtext = tabtext.replace(/(\s)+$/, "");
            this._tabBox.setTabTitle(view.url, tabtext+" ");
        }
        updateTitlebar(view);
    }
}

TabViewManager.prototype.doFileNew = function(language) {
    //Check the preference to see if one exists
    var globalPrefService = Components.classes["@activestate.com/koPrefService;1"].
        getService(Components.interfaces.koIPrefService);
    var globalPrefs = globalPrefService.prefs;
    var newType = globalPrefs.getStringPref("fileDefaultNew");
    var lang = language;

    if (newType != null && language == 'Default') {  // && not the default
        lang = newType;
    }

    // Create a new tab object.
    return this.createNewViewWithLanguage("scintilla", lang);
}

TabViewManager.prototype.doNewFileTemplate = function(fileTemplate) {
    return this.createNewViewWithFileTemplate("scintilla", fileTemplate);
}

// This updates the list in the Window menu
TabViewManager.prototype.updateWindowList = function(view, action, oldurl) {
    var separator = document.getElementById('window-menu-separator');
    var popup = document.getElementById('window-popup');
    var menuitem;
    var checkeditem;
    var fileName;
    // clear out checked items first
    var items = popup.getElementsByAttribute('data', 'fileItem');
    if (items.length == 0)
        separator.removeAttribute('collapsed');  // renable this for first item
    var i = 0, j = 0;
    for (i=0;i<items.length;i++)  {
        items[i].setAttribute('checked', 'false');
        items[i].setAttribute('index', i);
    }
    // id = full url to avoid duplication
    // append if open, the order is chronological right now
    // remove if close - no need to rebuild all the items, rename if rename!
    switch (action) {
        case 'open':
            menuitem = document.createElement('menuitem');
            menuitem.setAttribute('data', 'fileItem');
            menuitem.setAttribute('id', view.url);
            menuitem.setAttribute("label", (i + 1) + " " + makeLeafName(view.url));
            if (i+1 <= 9)
                menuitem.setAttribute("accesskey", i+1);
            else if (i+1 == 10)
                menuitem.setAttribute("accesskey", "0");
            menuitem.setAttribute('oncommand', 'doFileOpen(this.id);');
            menuitem.setAttribute('type', 'checkbox');
            menuitem.setAttribute('checked', 'true');
            popup.appendChild(menuitem);
            break;
        case 'close':
            menuitem = popup.getElementsByAttribute('id', view.url)[0];
            if (menuitem)  {
                var itemindex = menuitem.getAttribute('index');
                for (j=itemindex;j<items.length;j++)  { // redo accesskeys before removal
                    if (items[j].getAttribute('index') <= 9)
                        items[j].setAttribute("accesskey", items[j].getAttribute('index'));
                    else if (items[j].getAttribute('index') == 10)
                        items[j].setAttribute("accesskey", "0");
                    items[j].setAttribute('label', j + " " + makeLeafName(items[j].getAttribute('id')));
                }
                if (items[itemindex].nextSibling)
                    items[itemindex].nextSibling.setAttribute('checked', 'true');
                else if (items[itemindex].previousSibling.nodeName != 'menuseparator')
                    items[itemindex].previousSibling.setAttribute('checked', 'true');
                popup.removeChild(menuitem);
            }
            break;
        case 'existing':
            menuitem = popup.getElementsByAttribute('id', view.url)[0];
            if (menuitem)
                menuitem.setAttribute('checked', 'true');
            break;
        case 'rename':
            menuitem = popup.getElementsByAttribute('id', oldurl)[0];
            menuitem.setAttribute('id', view.url);
            var label = menuitem.getAttribute('label')
            menuitem.setAttribute("label", label.slice(0, label.indexOf(' ')) + " " + makeLeafName(view.url));
            menuitem.setAttribute('checked', 'true');
            break;
    }
}

TabViewManager.prototype._hideScintilla = function() {
}

TabViewManager.prototype._showScintilla = function() {
}

TabViewManager.prototype.activate = function() {
    this._sci_widget.viewManager = this;
}
  

TabViewManager.prototype.selectView = function(view) {
    this._tabBox.selectTab(this._tabBox.getTabIndexById(view.url));
    this.setMostRecentView(view);
    this.doViewTabSwitch();
    /* XXX split editor support, see scintillaview.js for details
    if(this.sci_widget_split) this.splitViewManager.view.clone(view);
    */
}

TabViewManager.prototype.Observe = function (subject ,topic, fname) {
    if (topic == 'SciMoz:FileDrop') {
        var scimoz = subject.QueryInterface(Components.interfaces.ISciMoz);
        var v = this.getCurrentView();
        if (v && v._scimoz == scimoz) {
          doOpenPath(fname,this);
        }
    }
    else if (topic == 'dirtyness') {
        var view = this.getCurrentView();
        if (view) this.indicateIfDirty(view);
    }
    else if (topic == 'file_status_updated') {
        this.updateStatusIcons();
    }
}

TabViewManager.prototype.updateStatusIcons = function() {
    // go through our files and update their icons and SCC status information
    for (var url in this._views) {
        this._tabBox.setTabStatus(url, this._views[url].item);
    }
}

// Recently-visited views:
// The ViewManager maintains a stack of recently visited lists. This
// information can be used to switch to recent views. The semantics of the
// following methods are as follows:
// - .getNextMostRecentView() and .getNextLeastRecentView() are used by the
//   buffer switching commands to know which views to switch to.
// - .enterRecentSwitchingSession() and .exitRecentSwitchingSession() are
//   called by the buffer swiching commands to help .setMostRecentView() know
//   how to behave (the "recent views" stack is NOT updated during such a
//   session). Note that .enter...() may be called multiple times for one
//   .exit...() call.
// - .setMostRecentView() is called by the view manager's .selectView() and can
//   be called directly by the buffer switching commands for proper session
//   handling.

TabViewManager.prototype.getNextMostRecentView = function() {
    // Find the current view in the "recentViews" stack and return the
    // preceding view.
    var currIndex = null;
    var currUrl = this.getCurrentView().url;
    for (i in this._recentUrls) {
        i = Number(i);
        if (this._recentUrls[i] == currUrl) {
            currIndex = i;
            break;
        }
    }
    var index = (currIndex + 1) % this._recentUrls.length;

    var url = this._recentUrls[index];
    var view = this._views[url];
    if (view == undefined) {
        // The "recent urls" list is out of date. Remove this entry and try
        // again.
        var newStack = new Array();
        for (i in this._recentUrls) {
            i = Number(i);
            if (this._recentUrls[i] != url) {
                newStack[newStack.length] = this._recentUrls[i];
            }
        }
        this._recentUrls = newStack;
        //dump("TabViewManager.getNextMostRecentView(): curr=" + currUrl +
        //     ", recents list is out of date, remove " + url + "\n");
        return this.getNextMostRecentView();
    } else {
        //dump("TabViewManager.getNextMostRecentView(): curr=" + currUrl +
        //     ", url=" + url + "\n");
        return view;
    }
}

TabViewManager.prototype.getNextLeastRecentView = function() {
    // Find the current view in the "recent views" stack and return the
    // succeeding view.
    var currIndex = null;
    var currUrl = this.getCurrentView().url;
    for (i in this._recentUrls) {
        i = Number(i);
        if (this._recentUrls[i] == currUrl) {
            currIndex = i;
            break;
        }
    }
    // XXX JavaScript get's modulus wrong for negative values, so add the
    //     .length to ensure the value is not negative.
    var index = (currIndex - 1 + this._recentUrls.length) %
                this._recentUrls.length;

    var url = this._recentUrls[index];
    var view = this._views[url];
    if (view == undefined) {
        // The "recent urls" list is out of date. Remove this entry and try
        // again.
        var newStack = new Array();
        for (i in this._recentUrls) {
            i = Number(i);
            if (this._recentUrls[i] != url) {
                newStack[newStack.length] = this._recentUrls[i];
            }
        }
        this._recentUrls = newStack;
        //dump("TabViewManager.getNextLeastRecentView(): curr=" + currUrl +
        //     ", recents list is out of date, remove " + url + "\n");
        return this.getNextLeastRecentView();
    } else {
        //dump("TabViewManager.getNextLeastRecentView(): curr=" + currUrl +
        //     ", url=" + url + "\n");
        return view;
    }
}

TabViewManager.prototype.enterBufferSwitchingSession = function() {
    //dump("TabViewManager.enterBufferSwitchingSession()\n");
    this._inBufferSwitchingSession = true;
}

TabViewManager.prototype.exitBufferSwitchingSession = function() {
    //dump("TabViewManager.exitBufferSwitchingSession()\n");
    this._inBufferSwitchingSession = false;
}

TabViewManager.prototype.setMostRecentView = function(view) {
    // Bring the given view to front of the "recent views" stack (note: it may
    // not yet be *in* the stack) *unless* we are currently in a buffer
    // switching session (e.g. press <Ctrl> and hold, hit <Tab> a few times,
    // release <Ctrl>).  Have to do the "unless" checking in here because
    // .selectView() calls this method.
    if (! this._inBufferSwitchingSession) {
        var newStack = new Array();
        newStack[0] = view.url;
        for (i in this._recentUrls) {
            i = Number(i);
            var url = this._recentUrls[i];
            if (url != view.url) {
                newStack[newStack.length] = url;
            }
        }
        this._recentUrls = newStack;
        //dump("TabViewManager.setMostRecentView(" + view.url +
        //     "): new stack: " + this._recentUrls + "\n");
    }
    else {
        //dump("TabViewManager.setMostRecentView(" + view.url +
        //     "): ignored because in buffer switched session.\n");
    }
}


