/* Copyright (c) 2000-2002 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

// Note that in general, we use dbg_ui_state instead of the debugger state
// whenever we must handle "running" _and_ "debugging" - when running
// (but not debugging) only dbg_ui_state will have changed.

// Note that we can never use the "current file's extension" or other
// simple rules to determine "can we debug".  *ANY* file may have a 
// valid invocation setup.  So for now, this code assumes that
// ALL files are debuggable - this will mean that ALL files prompt
// for an invocation, which the user may cancel if they made
// an error.

function debuggerStepMakesSense() {
   if (!dbgd || !dbgd.dbg)
      return true;
   var state = dbgd.dbg_ui_state;
   if (state == DBG_STATE_BREAK)
      return true;
    return (state == DBG_STATE_STOPPED);
}

function koGoUpdateDebuggerCommands(commandset) {
  var childNodes = commandset.childNodes;
  debuggerItemsController.beginBatchUpdate();
  var length = childNodes.length;
  for (var i = 0; i < length; i++) {
    var commandID = childNodes[i].id;
    koGoUpdateCommand(commandID, debuggerItemsController);
  }
  debuggerItemsController.endBatchUpdate();
}

var koCtrlKeyForDebuggerStart; // see below.

// Execute a "start debugger" command - 
// the "Control" key switches whether to show the dialog
// or not, with the default being set by the dbgShowDialog pref
function koGoDoDebuggerStartCommand(command, ctrlKey)
{
    koCtrlKeyForDebuggerStart = ctrlKey;
    return koGoDoCommand(command);
}

function DebuggerItemsController() {}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
DebuggerItemsController.prototype = new Controller();
DebuggerItemsController.prototype.constructor = DebuggerItemsController;

DebuggerItemsController.prototype.beginBatchUpdate = function() {
        this.bHaveView = viewManager.getCurrentView(); // global Editor view manager always!
        this.bCanStep = debuggerStepMakesSense();
        if (dbgd && dbgd.dbg)
            this.dbg_state = dbgd.dbg.state
        else
            this.dbg_state = DBG_STATE_STOPPED
        this.inBatchUpdate = true;
}
DebuggerItemsController.prototype.endBatchUpdate = function() {
        this.inBatchUpdate = false;
}

DebuggerItemsController.prototype.is_cmd_dbgStepIn_enabled = function () {
    return this.bCanStep;
}

DebuggerItemsController.prototype.do_cmd_dbgStepIn = function() {
    return do_step(1, koCtrlKeyForDebuggerStart);
}

DebuggerItemsController.prototype.is_cmd_dbgStepOver_enabled = function () {
    return this.bCanStep;
}

DebuggerItemsController.prototype.do_cmd_dbgStepOver = function() {
    return do_step(2, koCtrlKeyForDebuggerStart);
}

DebuggerItemsController.prototype.is_cmd_dbgStepOut_enabled = function () {
    return this.bCanStep;
}

DebuggerItemsController.prototype.do_cmd_dbgStepOut = function() {
    return do_step(3, koCtrlKeyForDebuggerStart);
}

DebuggerItemsController.prototype.is_cmd_dbgStop_enabled = function () {
    return dbgd.dbg_ui_state != DBG_STATE_STOPPED;
}

DebuggerItemsController.prototype.do_cmd_dbgStop = function() {
      return terminate_debugger();
}
DebuggerItemsController.prototype.is_cmd_dbgStepOut_enabled = function () {
    return this.bCanStep;
}
DebuggerItemsController.prototype.is_cmd_dbgGo_enabled = function () {
    return this.bCanStep;
}
DebuggerItemsController.prototype.do_cmd_dbgGo = function() {
    return do_step(4, koCtrlKeyForDebuggerStart);
}

DebuggerItemsController.prototype.is_cmd_dbgShowCurrentStatement_enabled = function () {
    return this.dbg_state == DBG_STATE_BREAK;
}
DebuggerItemsController.prototype.do_cmd_dbgShowCurrentStatement = function() {
      show_current_statement();
}
DebuggerItemsController.prototype .is_cmd_dbgBreakpointToggle_enabled = function () {
    return true;
}
DebuggerItemsController.prototype.do_cmd_dbgBreakpointToggle = function() {
      return do_toggle_breakpoint ();
}
DebuggerItemsController.prototype .is_cmd_dbgBreakpointClearAll_enabled = function () {
    return true;
}
DebuggerItemsController.prototype.do_cmd_dbgBreakpointClearAll = function() {
      return do_clear_breakpoints ();
}
DebuggerItemsController.prototype .is_cmd_dbgListen_enabled = function () {
    return this.dbg_state == DBG_STATE_STOPPED;
}
DebuggerItemsController.prototype.do_cmd_dbgListen = function() {
    if (listener_manager.isListening()) return false;
    return listener_manager.startListening(0, true);
}
// supported handles the greyed out of the menu item
DebuggerItemsController.prototype.is_cmd_debuggerListener_supported = function() {
    dump("is_cmd_debuggerListener_supported\n");
    return this.dbg_state == DBG_STATE_STOPPED;
}
// enabled is the checkmark
DebuggerItemsController.prototype.is_cmd_debuggerListener_enabled = function() {
    // pref listenForDebugger
    dump("is_cmd_debuggerListener_enabled\n");
    return listener_manager.isListening();
}
// called on changing the state of the menu checkmark
DebuggerItemsController.prototype.do_cmd_debuggerListener = function() {
    dump("do_cmd_debuggerListener\n");
    // listenfordebugger pref is for starting listening when komodo starts
    if (listener_manager.isListening()) {
        listener_manager.stopListening();
        globalPrefs.setBooleanPref("listenForDebugger",false);
    } else {
        listener_manager.startListening(0, false);
        globalPrefs.setBooleanPref("listenForDebugger",true);
    }
}
