/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * editor-debugger.js
 * Set up the buffer- and window-manager components.
 */

//---- globals

var _gObserverSvc = null;
var _gDebuggerObserver = null;

const DBG_STATE_STOPPED = Components.interfaces.koIDebugger.DBG_STATE_STOPPED;
const DBG_STATE_STARTING = Components.interfaces.koIDebugger.DBG_STATE_STARTING;
const DBG_STATE_BREAK = Components.interfaces.koIDebugger.DBG_STATE_BREAK;
const DBG_STATE_RUNNING = Components.interfaces.koIDebugger.DBG_STATE_RUNNING;
const DBG_STATE_STOPPING = Components.interfaces.koIDebugger.DBG_STATE_STOPPING;

const INVOKE_RUN = Components.interfaces.koIInvocation.INVOKE_RUN;
const INVOKE_DEBUG = Components.interfaces.koIInvocation.INVOKE_DEBUG;
const INVOKE_BUILD = Components.interfaces.koIInvocation.INVOKE_BUILD;

var komodoQuitting = false;

// Debugger Data
var dbgd = null;

// markerOrdering: the lower the number, the higher the marker's z-order
var mo = null; 



//---- editor-debugger.js's observer
// This observer handles any notification pertinent to the UI side of
// debugging stuff in Komodo (i.e. stuff that editor-debugger.js deals with).
//
function _DebuggerObserver() {}
_DebuggerObserver.prototype.Observe = function(subject, topic, data)
{
    switch (topic) {
    case 'invocation_terminated':
        // XXX "data" is null for now.
        //var exitCode = data;  // XXX can "data" be an int here???
        terminate_debugger();
        break;
    }
}



//---- module routines

function is_local_host(host_name) {
    return host_name == "localhost" || host_name == "127.0.0.1";
}


function debugger_onLoad()
{
    _gObserverSvc = Components.classes["@mozilla.org/observer-service;1"].
                   getService(Components.interfaces.nsIObserverService);
    _gDebuggerObserver = new _DebuggerObserver();
    _gObserverSvc.AddObserver(_gDebuggerObserver, "invocation_terminated");

    dbgd = new Object();
    dbgd.dbg = null;
    dbgd.dbg_ui_state = DBG_STATE_STOPPED;
    dbgd.terminal = null;
    dbgd.have_init_scimoz = false;
    dbgd.listener_cookie = null;
    dbgd.debugger_start_in_progress = false;
    dbgd.invocation = null;
    dbgd.debugFileName = "";
    dbgd.debugInputFileName = "";

    dbgd.currentmarker = new Object();
    dbgd.currentmarker.file = null;
    dbgd.currentmarker.line = null;
    dbgd.currentmarker.ifile = null;
    dbgd.currentmarker.iline = null;

    mo = new Object (); 
    mo.clb = 5; // current line background color
    mo.bm = 4; // bookmark
    mo.cl = 3; // current line
    mo.wb = 2; // working (ie, enabled) breakpoint
    mo.fb = 1; // false breakpoint
}


function debugger_onUnLoad()
{
    _gObserverSvc.RemoveObserver(_gDebuggerObserver, "invocation_terminated");
    _gDebuggerObserver = null;
}



////////////////////////////////////////////////////////////////
// Debugger initialization/termination related functions
////////////////////////////////////////////////////////////////
function debuggerEventist() {}
debuggerEventist.prototype = {
  onStateChange : function(state, state_extra) {
    // *sigh* - threads are too fast! The debugger may send a number of 
    // state requests before the first has fired.  Thus, the state we are 
    // notified of may not be the state we actually are!
    // So - ignore the state sent, and just query the current state.

    //XXX if Komodo is shutting down, we don't care what the 
    // debugger state is, and trying to exec change_debugger_ui_state()
    // may cause trouble, since some objects may already be released
    if (!komodoQuitting) {
      change_debugger_ui_state(dbgd.dbg.state);
    }
  }
};


// Get the koIDebugger interface, creating and initializing if necessary.
function get_debugger_interface () {
  if (dbgd.dbg == null) {
    //window.style.cursor = "wait"
    var clz = Components.classes["@activestate.com/debugger;1"];
    dbgd.dbg = clz.getService(Components.interfaces.koIDebugger);
    if (dbgd.listener_cookie != null)
        logWarning("Adding new debugger listener, but already appear to have one!");
    dbgd.listener_cookie = dbgd.dbg.addListener( new debuggerEventist() );
  }
  return dbgd.dbg;
}


function terminate_debugger () {
  var invocation = dbgd.invocation; // incase it goes away as part of .stop()
  if (dbgd && dbgd.dbg && dbgd.dbg.state != DBG_STATE_STOPPED)
      dbgd.dbg.stop();
  else
      change_debugger_ui_state(DBG_STATE_STOPPED);
  if (invocation) {
    try {
      invocation.terminate(99,1);
    } catch(e) {
      ; // already dead
    }
  }
}


////////////////////////////////////////////////////////////////
// State related functions 
////////////////////////////////////////////////////////////////

var stateNames = ['DBG_STATE_STOPPED', 'DBG_STATE_BREAK', 'DBG_STATE_RUNNING', 'DBG_STATE_STARTING', 'DBG_STATE_STOPPING'];
var viewAtDebuggerStart;
var lineAtDebuggerStart;
var currentPosAtDebuggerStart;
var anchorAtDebuggerStart;

function change_debugger_ui_state(new_state) {
  var view = viewManager.getCurrentView(); //NOT the current focused, but the current Editor View
  var scimoz = null;
  var i_scimoz = null;
  var iv = null; // input view
  if (view) {
    scimoz = view._scimoz;
    if (view._inputViewManager) {
      iv = view._inputViewManager.getCurrentView();
      if (iv) i_scimoz = iv._scimoz;
    }
  }
  
  var old_state = dbgd.dbg_ui_state;
  //logDebug("Change_debugger_ui_state: " + stateNames[dbgd.dbg_ui_state] + " => " + stateNames[new_state] + "\n");
  
  // The only thing that goes wrong with this optimization is that we
  // may miss a "running" state if we hit 2 "break" states quickly in
  // succession.  As this doesnt think we have moved, the breakpoint
  // stays at the original position.
  // Also can not afford to miss a stop (even though we never should)
  // So only skip intermediate "run" states.
  if (dbgd.dbg_ui_state == new_state &&
      new_state != DBG_STATE_BREAK && new_state != DBG_STATE_STOPPED) {
//    dump("skipping state update with " + new_state + "\n");
    koRequestUpdateCommands("debug");
    return;
  }
  var f5_item = document.getElementById("menu_dbgGo");
  if (new_state == DBG_STATE_STOPPED) {
    f5_item.setAttribute("label", gKomodoBundle.getString( "dbg_start" ));
  } else if (new_state == DBG_STATE_BREAK) {
    var t = gKomodoBundle.getString( "dbg_continue" );
    f5_item.setAttribute("label", t);
  }

  if (dbgd.terminal) {
    dbgd.terminal.acceptInput = new_state == DBG_STATE_RUNNING;
  }
  // Remember the state now.
  dbgd.dbg_ui_state = new_state;
  switch(new_state) {
    case DBG_STATE_STOPPED:
        StatusBar_AddMessage("Debugging session has ended.", "debug",
                             3000, false);
        if (view) {
            view.deleteAllMarkers(mo.cl);
            view.deleteAllMarkers(mo.clb);
        }
        if (iv) {
            iv.deleteAllMarkers(mo.cl);
            iv.deleteAllMarkers(mo.clb);
        }
        if (dbgd && dbgd.terminal)
            dbgd.terminal.clearInput();
        OutputWindow_DebuggingFinished();

        if (dbgd && dbgd.dbg) {
            if (dbgd.listener_cookie != null) {
                dbgd.dbg.removeListener(dbgd.listener_cookie);
                dbgd.listener_cookie = null;
            }
            dbgd.dbg = null;
            dbgd.invocation = null;
            listener_manager.stopListening();
        }
        updateTitlebar(view);
        SetBusyCursor(window, false);
        break;

    case DBG_STATE_STARTING:
        StatusBar_AddMessage("Debugger is starting...", "debug", 0, false);
        viewAtDebuggerStart = view;
        lineAtDebuggerStart = view._scimoz.lineFromPosition(view._scimoz.currentPos);
        anchorAtDebuggerStart = view._scimoz.anchor;
        currentPosAtDebuggerStart = view._scimoz.currentPos;
        break;
        
    case DBG_STATE_RUNNING:
        StatusBar_AddMessage("Debugger is running...", "debug", 0, false);
        if (view) {
            view.deleteAllMarkers(mo.cl);
            view.deleteAllMarkers(mo.clb);
        }
        if (iv) {
            iv.deleteAllMarkers(mo.cl);
            iv.deleteAllMarkers(mo.clb);
        }
        updateTitlebar(view);
        break;

    case DBG_STATE_BREAK:
        // Move to the location of the break.
        // XXX - later another debugger listener (such as the project
        // manager) should probably do this.

        var dbg = get_debugger_interface();
        
        var temp = new Object();
        dbg.enumStackFrames(temp, new Object());
        var stack = temp.value;
        var stack_frame = stack[0].QueryInterface(Components.interfaces.koIDebugStackFrame);
        
        // get the new view for the stack frame, otherwise breakpoints and title bar
        // do not get updated correctly
        view = show_stack_position(stack_frame, 0);
        if (view._inputViewManager) {
          iv = view._inputViewManager.getCurrentView();
        }
        
        var breakLocation = "Break at " + displayNameFromURL(stack_frame.uri)
                            + ", line " + stack_frame.lineNumber + ".";
        StatusBar_AddMessage(breakLocation, "debug", 0, false);
        updateTitlebar(view);
        break;

    case DBG_STATE_STOPPING:
        StatusBar_AddMessage("Debugger is stopping...", "debug", 0, false);
        break;

  } // end switch(new_state)
  // Breakpoints are always updated
  // (and currently need to be done after we change locations)
  update_breakpoints(view);
  if (iv) update_breakpoints(iv,view); // update breakpoints for input view
  koRequestUpdateCommands("debug");
  // logDebug("Change_debugger_ui_state OUT : " + stateNames[dbgd.dbg_ui_state] + " (really) " + stateNames[dbgd.dbg.state] + "\n");

  if (new_state == DBG_STATE_STOPPED) {
    try {
        // If the view that we started with still exists (wasn't closed), get back to it.
        if (viewAtDebuggerStart.py_inst) { // XXX is there a better marker?
            viewManager.setCurrentView(viewAtDebuggerStart);
            viewAtDebuggerStart._scimoz.gotoLine(lineAtDebuggerStart);
            viewAtDebuggerStart._scimoz.anchor = anchorAtDebuggerStart;
            viewAtDebuggerStart._scimoz.currentPos = currentPosAtDebuggerStart;
        }
    } catch (e) {
        // none of the above really matters.
        ;
    }
  }
}

function show_current_statement() {
    var dbg = get_debugger_interface();
    
    var temp = new Object();
    dbg.enumStackFrames(temp, new Object());
    var stack = temp.value;
    var stack_frame = stack[0].QueryInterface(Components.interfaces.koIDebugStackFrame);
    
    // logDebug("uri is '" + stack_frame.uri + "', file is " +
    // stack_frame.filename + ", line " + stack_frame.lineNumber + "\n");
    show_stack_position(stack_frame, 0); 
}

function reset_stack_position(view) {
    var fv = viewManager.getCurrentView();
    if (fv && !view != fv && fv._inputViewManager) {
        if (view == fv._inputViewManager.getCurrentView()) {
            /* when we reset the parent pane, this pane
               will get reset also */
            return;
        }
    }
    
    view.deleteAllMarkers(mo.cl);
    view.deleteAllMarkers(mo.clb);
    var iv = null;
    if (typeof(view._inputViewmanager) != 'undefined' &&
       view._inputViewmanager) {
        iv = view._inputViewManager.getCurrentView();
        if (iv) {
            iv.deleteAllMarkers(mo.cl);
            iv.deleteAllMarkers(mo.clb);
        }
    }
    if (dbgd.dbg_ui_state == DBG_STATE_STOPPED) return;
    if (dbgd.currentmarker.file == view.url) {
        view.addMarker(dbgd.currentmarker.line, mo.cl);
        view.addMarker(dbgd.currentmarker.line, mo.clb);
        if (dbgd.currentmarker.ifile && iv && iv.url == dbgd.currentmarker.ifile) {
            iv.addMarker(dbgd.currentmarker.iline, mo.cl);
            iv.addMarker(dbgd.currentmarker.iline, mo.clb);
        }
    }
}

function show_stack_position(stackFrame, frameNumber) {
    // If we're doing an eval in Perl, it isn't a real position.
    // So we aren't going to show it, and the perl debugger will
    // keep going as well.
    
    if (stackFrame.filename.match(/\(eval\s+\d+\)\[/)) {
      // Whatever we were doing before we hit the eval, keep doing it.
      // XXX set up a temporary buffer to hold the contents of the eval
      if (dbgd.lastCommand) {
        dbg.resume(dbgd.lastCommand);
      }
      return null;
    }

    dbgd.currentmarker.file = null;
    dbgd.currentmarker.line = null;
    dbgd.currentmarker.ifile = null;
    dbgd.currentmarker.iline = null;
    
  // XXX Fixme : need to do smarter koResource stuff here.
  // Just attempt to open the URI.  If it is already open, then fine!
  var v = viewManager.doFileOpenAtLine(stackFrame.uri, stackFrame.lineNumber);
  if (v) {
    var linecolor = hexColorToLong(globalPrefs.getStringPref('debuggerLineColorBackLite'));
    var linecolorlight = hexColorToLong(globalPrefs.getStringPref('debuggerLineColorBack'));
  
        v.deleteAllMarkers(mo.cl);
        v.deleteAllMarkers(mo.clb);

    if (frameNumber != 0) {
        v._scimoz.markerSetBack (mo.clb, linecolor);
    } else {
        v._scimoz.markerSetBack (mo.clb, linecolorlight);
    }

      v.addMarker(stackFrame.lineNumber-1, mo.cl);
      v.addMarker(stackFrame.lineNumber-1, mo.clb);
      dbgd.currentmarker.file = v.url;
      dbgd.currentmarker.line = stackFrame.lineNumber-1;

      // XXX attempt to open input file
      // this is temporary until new views work is done, will only effect ActiveXSLT debugging
      if (stackFrame.inputURI != undefined && stackFrame.inputURI != "") {
            //dump("INPUT FILE: "+stackFrame.inputURI+" LINE: "+stackFrame.inputLineNumber+"\n");
            if (v._inputViewManager) {
                var iv = v._inputViewManager.doFileOpenAtLine(stackFrame.inputURI,stackFrame.inputLineNumber);
                if (iv) {
                    v.showInput();
                    iv.deleteAllMarkers(mo.cl);
                    iv.deleteAllMarkers(mo.clb);

                    if (frameNumber != 0) {
                        iv._scimoz.markerSetBack (mo.clb, linecolor);
                    } else {
                        iv._scimoz.markerSetBack (mo.clb, linecolorlight);
                    }

                    iv.addMarker(stackFrame.inputLineNumber-1, mo.cl);
                    iv.addMarker(stackFrame.inputLineNumber-1, mo.clb);
                    v.manager.setFocus();
                    dbgd.currentmarker.ifile = iv.url;
                    dbgd.currentmarker.iline = stackFrame.inputLineNumber-1;
                }
            } else {
                //dump("NO INPUT FILE PANE!\n");
            }
      }
  }
  return v;
}

function find_bp(dbg, url, lineNo) {
    var ienum = dbg.enumBreakpoints(url);
    var bp;
    while (ienum.hasMoreElements()) {
        bp = ienum.getNext();
        bp = bp.QueryInterface(Components.interfaces.koIBreakpoint);
        if (bp.lineNumber == lineNo) {
            return bp;
        }
    }
    return null;
}


function update_breakpoints(view,pv) {
    // Synchronizes the breakpoints view of the world for the debugger and the view
    var v = view;
    // no point in updating breakpoints if there are no open files
    if (v == null) {
        return;
    }
    if (typeof(pv)=='undefined') pv=null;
    
    // if our vew is the inputview, and we didn't get
    // the parent, get it now
    var fv = viewManager.getCurrentView();
    if (fv && !pv && v != fv && fv._inputViewManager) {
        if (v == fv._inputViewManager.getCurrentView()) {
            pv = fv;
        }
    }
    
    var dbg = get_debugger_interface();

    // reach into debugger for full name
    var url = view.url;

    var marker_mask = 1 << mo.wb | 1 << mo.fb;
    var lineNo = -1;
    var bp, markertype;
    // Make sure that all of the breakpoints in scimoz exist in the debugger
    while (true) {
        lineNo = view.nextMarker(lineNo+1, marker_mask);
        if (lineNo < 0)
            break;
        //breakpoints.appendLongPref(lineNo);
        bp = find_bp(dbg, url, lineNo+1);
        if (!bp) {
            var lang = pv?pv.py_inst.language:v.py_inst.language;
            dbg.addBreakpoint(lang, v.url, lineNo+1);
        } else {
            // make sure that the debugger breakpoint state determines
            // the viewpoint breakpoint type (red vs. green)
            markertype = view.getMarker(lineNo);
            var markerPending = ! (markertype & (1 << mo.wb));
            if (bp.state == Components.interfaces.koIBreakpoint.BREAKPOINT_DELETED) {
                bp.state = Components.interfaces.koIBreakpoint.BREAKPOINT_ENABLED;
                view.addMarker(lineNo, (dbg.state == DBG_STATE_BREAK)?mo.wb:mo.fb);
            } else if (bp.state == Components.interfaces.koIBreakpoint.BREAKPOINT_DISABLED) {
                bp.state = Components.interfaces.koIBreakpoint.BREAKPOINT_DELETED;
                view.deleteMarker(lineNo, markerPending?mo.fb:mo.wb);
            } else if (bp.pending && ! markerPending) {
                view.deleteMarker(lineNo, mo.wb);
                view.addMarker(lineNo, mo.fb);
            }
            else if (! bp.pending && markerPending) {
                view.deleteMarker(lineNo, mo.fb);
                view.addMarker(lineNo, mo.wb);
            }
        }
    }
    // make sure that all of the breakpoints in the debugger correspond to scimoz
    // breakpoints
    var ienum = dbg.enumBreakpoints(url);
    while (ienum.hasMoreElements()) {
        bp = ienum.getNext();
        bp = bp.QueryInterface(Components.interfaces.koIBreakpoint);
        markertype = view.getMarker(bp.lineNumber-1);
        if (! (markertype & ( 1 << mo.wb)) &&
            ! (markertype & ( 1 << mo.fb))) {
            bp.state = Components.interfaces.koIBreakpoint.BREAKPOINT_DELETED;
        }
    }
}

function deleteBreakpointList(url) {
    var dbg = get_debugger_interface();
    var ienum = dbg.enumBreakpoints(url);
    var bp;
    while (ienum.hasMoreElements()) {
        bp = ienum.getNext();
        bp = bp.QueryInterface(Components.interfaces.koIBreakpoint);
        bp.state = Components.interfaces.koIBreakpoint.BREAKPOINT_DELETED;
    }
}

function toggle_breakpoint_at_line(view, lineClicked) {
    if (!view) {
        logError("toggle-breakpoints: internal error: can't access the current buffer");
        return;
    }
    var dbg = get_debugger_interface();
    var bp = find_bp(dbg, view.url, lineClicked+1);
    var markertype = view.getMarker(lineClicked);
    if (markertype & (1 << mo.wb)) {
        view.deleteMarker(lineClicked, mo.wb);
        if (bp) bp.state = Components.interfaces.koIBreakpoint.BREAKPOINT_DELETED;
    }
    if (markertype & (1 << mo.fb)) {
        view.deleteMarker(lineClicked, mo.fb);
        if (bp) bp.state = Components.interfaces.koIBreakpoint.BREAKPOINT_DELETED;
    }
    if (! (markertype & (1 << mo.wb)) &&
        ! (markertype & (1 << mo.fb))) {
        var dbg = get_debugger_interface();
        if (bp) bp.state = Components.interfaces.koIBreakpoint.BREAKPOINT_ENABLED;
        if (dbg.state == DBG_STATE_BREAK) {
            view.addMarker(lineClicked, mo.wb);
        } else {
            view.addMarker(lineClicked, mo.fb);
        }
    }
    update_breakpoints(view);
}


function do_clear_breakpoints () {
    var vm = getFocusedViewManager();
    if (!vm || vm == outputViewManager) return 0;
    var v = vm.getCurrentView();
    if (!v) return 0;
    marker_mask = 1 << mo.wb | 1 << mo.fb;
    lineNo = -1;
    while (true) {
        lineNo = v.nextMarker(lineNo+1, marker_mask);
        if (lineNo < 0)
            break;
        v.deleteMarker(lineNo, mo.fb);
        v.deleteMarker(lineNo, mo.wb);
    }
    deleteBreakpointList(v.url);
    return 1;
}

// This is used for internal use: passing arbitrary commands
// to the debugger, before instituting the functionality into
// regular UI elements.


////////////////////////////////////////////////////////////////
// UI callback do_* event handlers.
////////////////////////////////////////////////////////////////

function do_toggle_breakpoint () {
    var vm = getFocusedViewManager();
    if (!vm || vm == outputViewManager) return;
    var v = vm.getCurrentView();
    if (!v) return;
  
    var lineClicked = v._scimoz.lineFromPosition(v._scimoz.currentPos);
    toggle_breakpoint_at_line(v, lineClicked);
}

function do_step(how, ctrlKey) {
  if (typeof(ctrlKey) == 'undefined') ctrlKey = 0;
  var skipDialog;
  var skipDialogPref = globalPrefs.getBooleanPref('debugSkipDialog');
  if (skipDialogPref) {
    skipDialog = ! ctrlKey;
  } else {
    skipDialog = ctrlKey;
  }
  if (!check_start_debugger(how, skipDialog)) {
    //logWarning("do_step: check_start_debugger failed, either because user cancelled dialog or because something is wrong w/ debugger engine registration\n");
    return;
  }
  var dbg = get_debugger_interface();
  dbgd.lastCommand = how;
  var bp = null;
  
  var view = viewManager.getCurrentView();
  if (view) update_breakpoints(view);
  var iv = null;
  if (view._inputViewManager) {
    iv = view._inputViewManager.getCurrentView();
    if (iv) update_breakpoints(iv,view);
  }

  var v = getFocusedView();
    
  // this handles 'run-to-cursor' capabilitiy.  It should always
  // be dependent on the ctrl key being pressed.  The idea is that
  // we set a temporary breakpoint on the line where the cursor is,
  // then we change our step type to GO.  when we come out of
  // dbg.resume below, we unset the temporary breakpoint.
  if (v && ctrlKey &&
      how == Components.interfaces.koIDebugger.RESUME_STEP_OVER) {
        // set breakpoint
        var currentline = v._scimoz.lineFromPosition(v._scimoz.currentPos) + 1;
        bp = dbg.addBreakpoint(view.py_inst.language, v.url, currentline);
        how = Components.interfaces.koIDebugger.RESUME_GO
        if (bp == null) {
            alertDialog("Can not stop at this location.");
            return;
        }
  }
  
  if (dbg.state == DBG_STATE_BREAK) {
    try {
        dbg.resume(how);
    } catch(e) {
        logDebug("XXX Caught exception on dbg.resume\n");
    }
  }
  
  // clean up the temporary breakpoint that was added for
  // run-to-cursor
  if (bp && bp.state == Components.interfaces.koIBreakpoint.BREAKPOINT_ENABLED) {
        bp.state = Components.interfaces.koIBreakpoint.BREAKPOINT_DELETED;
  }
}

function OutputWindow_DragDrop() {}
OutputWindow_DragDrop.prototype = {
    Observe : function (subject ,topic, fname) {
        if (topic == 'SciMoz:FileDrop') {
            var scimoz = subject.QueryInterface(Components.interfaces.ISciMoz);
            if (document.getElementById("redirect-scin").ISciMoz == scimoz) {
              doOpenPath(fname);
            }
        }
    }
}


function OutputWindow_Init() {
    var sw = document.getElementById("redirect-scin");
    if (!dbgd.have_init_scimoz) {
        dbgd.have_init_scimoz = true;
        sw.init();

        // Setup the style for the output window using the user's text settings
        sw.ISciMoz.lexer = sw.ISciMoz.SCLEX_NULL;
        var styleApplier = Components.classes['@activestate.com/koScintillaStyleApplier;1']
                    .createInstance(Components.interfaces.koIScintillaStyleApplier);
        
        // Force the font to be fixed width by making a preference item with that
        // single pref
        var fakeViewPrefs = Components.classes['@activestate.com/koPreferenceSet;1']
            .createInstance(Components.interfaces.koIPreferenceSet);
        fakeViewPrefs.setBooleanPref('editUseFixedFont', true)

        // Set the style and apply it            
        styleApplier.init(sw.ISciMoz, 'Text', fakeViewPrefs)
        styleApplier.applyStyles();
        // Add a listener for drag/drop operations.
        dbgd.drag_and_drop_listener = new OutputWindow_DragDrop();
        var observerSvc = Components.classes["@mozilla.org/observer-service;1"].
            getService(Components.interfaces.nsIObserverService);
        // This observer is never removed, but it doesn't matter - very light-weight and no cycles.
        observerSvc.AddObserver(dbgd.drag_and_drop_listener, "SciMoz:FileDrop");    
    }
}

// url may be null indicating no redirection for this run.
function OutputWindow_DebuggingStarting(show_terminal, emulate_cgi) {
    // XXX - Hack alert.  Mozilla may reuse our plugins.  Ensure that our
    // main scimoz widget is made visible before us.
    var swstack = document.getElementById("redirect-stack");
    var sw = document.getElementById("redirect-scin");
    var blank = document.getElementById("redirect-blank");
    if (show_terminal) {
        blank.setAttribute("src","about:blank");
        swstack.setAttribute("index", "1");
        OutputWindow_Init();
        if (!dbgd.terminal) {
          dbgd.terminal = Components.classes['@activestate.com/KoDebuggerTerminal;1']
            .createInstance(Components.interfaces.koIDebuggerTerminal);
          dbgd.terminal.init(sw.ISciMoz, "debugio", sw.scratch_document, "TEXT", null);
          dbgd.terminal.acceptInput = !emulate_cgi;
          dbgd.terminal.emulateCGI = emulate_cgi;
          dbgd.terminalEvents = dbgd.terminal.QueryInterface(Components.interfaces.ISciMozEvents);
          sw.ISciMoz.hookEvents(dbgd.terminalEvents);
          sw.controllers.insertControllerAt(0, dbgd.terminal.controller);
          dbgd.terminal.clear();
        }

      } else {
//        blank.removeAttribute("style");
//        blank.setAttribute("src", "chrome://komodo/locale/debugger/blankDebuggerOutputWindowDebugging.html");
        swstack.setAttribute("index", "0");
    }
}

var gOpeningDialog = false;

function OutputWindow_DebuggingFinished() {
    var sw = document.getElementById("redirect-scin");
    if (typeof(dbgd.terminal != 'undefined') && dbgd.terminal) {
        sw.controllers.removeController(dbgd.terminal.controller);
    }
    dbgd.terminal = null;
}

// Get the "current" invocation to use, creating an appropriate one of necessary.
// Need some smarts here:
// A "project" and a "view" optionally have invocation data
// If the current view has a set, we use them.
// If not, look if the project has one, and use them.
// Depending on requirements, each project/file may have multiple
// invocations stored - so may need to pass multiple invocations to the
// dialog.
function _do_get_invocation(skipDialog) {
    // MUST get the MAIN viewManagers current view for debugging
    var curView =  viewManager.getCurrentView();
    
    var prefs = getEffectivePrefsForFile(curView ? curView.url:null);
    var langName = curView ? curView.py_inst.language : null;

    dbgd.debugFileName = "";
    dbgd.debugInputFileName = "";
    try {
        dbgd.debugFileName = curView ? URLToLocalFile(curView.url).path : "";
        // XXX a better way to handle input views is necessary!
        if (langName == "XSLT") {
            var iv = curView._inputViewManager.getCurrentView();
            if (iv) dbgd.debugInputFileName = curView._inputViewManager.displayNameFromURL(iv.url);
        }        
      } catch (e) {
        // Invalid URL - Quite likely a "kodebugger" URL!
        // that is fine - we still allow them to invoke something, just don't
        // default the filename.
      }

    // Locate an invocation to use.
    var invocation = null;
    var def_invocation_name = langName;
    // If we have no current view, show the "Command" invocation rather than
    // the "unrecognized file type" one
    if (curView == null)
        langName = "Batch";
    // XXX .bat is associated with Batch, maybe that should change?
    if (langName == "Batch") 
        def_invocation_name = "Command";
    
    // Create a default invocation for this view.
    if (langName && def_invocation_name) {
        var invocation_svc = Components.classes["@activestate.com/InvocationRegistryService;1"].getService();
        var info = invocation_svc.getInvocationInformation(def_invocation_name);
        if (info)
            try {
                invocation = Components.classes[info.contractid].createInstance();
            } catch (e) {
                logError("Failed to create invocation '" + info.contractid + "' - " + e + "\n");
                invocation = null;
            }
        if (invocation) {
            var catOb = new Object();
            var instOb = new Object();
            invocation.getDefaultPreferences(catOb, instOb);
            invocation.currentCategoryPreferences = catOb.value;
            invocation.currentInstancePreferences = instOb.value;
            var storedPrefs,root;
            
            // clear any global prefs for the filename
            // to prevent pref chaining the wrong filename
            if (globalPrefs.hasPref("Invocations")) {
                root = globalPrefs.getPref("Invocations");
                if (root.hasPref(invocation.name)) {
                    storedPrefs = root.getPref(invocation.name);
                    storedPrefs.setStringPref("filename","");
                    storedPrefs.setStringPref("inputfile","");
                }
            }
            
            var iprefs = invocation.currentInstancePreferences;
            // Update the invocation preferences from the global preference set.
            if (prefs.hasPref("Invocations")) {
                root = prefs.getPref("Invocations");
                if (root.hasPref(invocation.name)) {
                    storedPrefs = root.getPref(invocation.name);
                    iprefs.update( storedPrefs );
                }
            }
            
            
            // Setup other default values.
            if (iprefs.hasStringPref("filename") &&
                dbgd.debugFileName &&
                !iprefs.getStringPref("filename")) {
                iprefs.setStringPref("filename", dbgd.debugFileName);
            }
            if (iprefs.hasStringPref("inputfile") &&
                dbgd.debugInputFileName &&
                !iprefs.getStringPref("inputfile")) {
                iprefs.setStringPref("inputfile", dbgd.debugInputFileName);
            }
        }
    }

    // If I don't have an invocation, or it is not valid, I must show the dialog.
    // XXX - it would be nice if this preference was per invocation, but this begs
    // the problem - how to enable it per invocation if that invocation's dialog isn't
    // shown.  Must wait for an "Invocation Preferences" UI element on the file/project itself
    if (skipDialog && (!invocation || !invocation.validate(INVOKE_DEBUG, new Object(), new Object())))
        skipDialog = false;

    if (!skipDialog) {
        var resp = new Object ();
        resp.prefs = prefs;
        resp.invocation = invocation;
        resp.invoke_type = INVOKE_DEBUG;
        resp.res = "";
        // Check to see if there's a debugger dialog being opened
        // Not quite sure why this happens only for this dialog, but without
        // this guard, hitting F10 and F11 simultaneously results in a crash.
        if (gOpeningDialog) {
            return null;
        }
        gOpeningDialog = true;
        window.openDialog
                 ("chrome://komodo/content/pref/invocations/debugging-properties.xul",
                  "Komodo",
                  "chrome,dialog=yes,close=yes,dependent=yes,modal=yes",
                  resp );
        if (resp.res != "ok" || resp.invocation==null) {
            //logDebug("User cancelled debugging args dialog");
            return null;
        }
        invocation = resp.invocation;
        // And save the invocation properties back.
        if (!prefs.hasPrefHere("Invocations")) {
            var newPrefs = Components.classes["@activestate.com/koPreferenceSet;1"].createInstance();
            newPrefs.id = "Invocations";
            prefs.setPref( newPrefs.id, newPrefs);
        }
        var prefRoot = prefs.getPref("Invocations");
        prefRoot.parent = prefs;
        // XXX - this is really a workaround for bug 18218, and should probably die,
        // using invocation.instancePreferences
        var invocation_prefs;
        if (prefRoot.hasPref(invocation.name)) {
            invocation_prefs = prefRoot.getPref(invocation.name);
            invocation_prefs.parent = prefRoot;
            invocation_prefs.update(invocation.persistableInstancePreferences);
        } else {
            invocation_prefs = invocation.persistableInstancePreferences
            invocation_prefs.parent = prefRoot
            prefRoot.setPref(invocation.name, invocation_prefs);
        }
    }
    if (!invocation) {
        logError("Shouldn't have managed to get this far without an invocation or the dialog being cancelled");
        return null;
    }
    return invocation;
}

// Called by the listener when it has a debugger connection.
function on_debugger_connection(language, host, port, cookie)
{
    var invocation = dbgd.invocation;
    var step_type = dbgd.initial_step_type;
    dbgd.initial_step_type = Components.interfaces.koIDebugger.RESUME_STEP_IN;

    var dbg = dbgd.dbg;
    // XXX - move to caller - pass the prefset
    // Create a preference set for the debugger.
    prefset = Components.classes["@activestate.com/koPreferenceSet;1"].createInstance();
    prefset.setStringPref("debugger.listener-reponse.cookie", cookie);
    prefset.setStringPref("debugger.language", language);
    prefset.setStringPref("debugger.listener-reponse.host", host);
    prefset.setLongPref("debugger.listener-reponse.port", port);
    var stdin = null;
    var stdout = null;
    var stderr = null;
    var show_terminal;
    var emulate_cgi = false;
    var debugger_redirect = false;
    if (!is_local_host(host)){
        // remote connection - redirect IO via the debugger.
        debugger_redirect = true;
        show_terminal = true;
    } else {
        // local connection - if we have an invocation, then use the "use-console"
        // preference - otherwise treat as remote debug.
        if (invocation) {
            // hook up invocation preferences, and use them for debug session.
            invocation.currentCategoryPreferences.parent = prefset;
            prefset = invocation.currentInstancePreferences;
            debugger_redirect = false;
            show_terminal = !prefset.hasBooleanPref("use-console") || !prefset.getBooleanPref("use-console");
            emulate_cgi = !prefset.hasBooleanPref("sim-cgi") || !prefset.getBooleanPref("sim-cgi");
        } else {
            // local, but not via invocation - treat as remote.
            debugger_redirect = true;
            show_terminal = true;
        }
    }
    OutputWindow_DebuggingStarting(show_terminal, emulate_cgi);
    if (debugger_redirect) {
        if (!show_terminal) logError("Logic Error - redirecting debugger output, but not showing terminal!");
        stdin = dbgd.terminal.stdin;
        stdout = dbgd.terminal.stdout;
        stderr = dbgd.terminal.stderr;
    }
    dbg.startDebugging(step_type, prefset, stdin, stdout, stderr);
}

function _do_execute_invocation(invocation, invoke_type, new_debugger_state) {
    var use_console = invocation.currentInstancePreferences.getBooleanPref("use-console");
    var emulate_cgi = invocation.currentInstancePreferences.hasPref("sim-cgi") ?
            invocation.currentInstancePreferences.getBooleanPref("sim-cgi") :
            false;

    OutputWindow_DebuggingStarting(!use_console, emulate_cgi);
    if (!use_console) {
        invocation.stdin = dbgd.terminal.stdin;
        invocation.stdout = dbgd.terminal.stdout;
        invocation.stderr = dbgd.terminal.stderr;
    }
    // and fire off the process.
    var rc;
    try {    
        invocation.invoke(invoke_type);
        if (new_debugger_state != null)
            change_debugger_ui_state(new_debugger_state)
        rc = true;
    } catch (e) {
        change_debugger_ui_state(DBG_STATE_STOPPED);
        rc = false;
        logError("There was an unexpected error starting the invocation:" + e);
    }
    return rc;
}

function _do_start_debugger(how, skipDialog) {
    var dbg = get_debugger_interface();
    if (!OptionallySaveDirtyFiles('Save Modified Files Before Debugging', "save_on_debug_action")) {
        return false;
    }
    var invocation = _do_get_invocation(skipDialog);
    if (invocation == null)
        return false;
    dbgd.invocation = invocation;
    dbgd.initial_step_type = how;
    var invoke_type;
    // XXX - maybe we should abort here, telling the user
    supports_debug = (invocation.supportedInvokeTypes & INVOKE_DEBUG) != 0;
    if (!supports_debug) {
        logDebug("This invocation appears to not support debugging - not starting the listener");
        invoke_type = INVOKE_RUN;
        change_debugger_ui_state(DBG_STATE_STARTING);
    } else {
        invoke_type = INVOKE_DEBUG;
        dbg.state = DBG_STATE_STARTING;
    }
    
    // start the listener, and allow it 10 seconds before it shows itself.
    // XXX when persistent listening is done, this needs to be removed
    if (supports_debug)
        listener_manager.startListening(10000, true);

    var new_state = supports_debug ? null : DBG_STATE_RUNNING;
    // and fire off the process.
    var ok =  _do_execute_invocation(invocation, invoke_type, new_state)

    if (!ok) {
        if (supports_debug) {
            try {
                listener_manager.stopListening();
            } catch (e) {
                logError("Error stopping the debugger listener after debugger failure:" + e);
            }
        }
        alertDialog("There was an unexpected error starting the debugger.");
        return false;
    }
    return true;
}

function check_start_debugger(how, skipDialog){
    if (listener_manager.isListening()) {
        // our controllers don't have time to update properly if you
        // click very fast.  So it is possible to be already listening
        // when we get here.  Fail silently.
        return false;
    }
  var dbg = get_debugger_interface();
  if (dbg.state == DBG_STATE_STOPPED || dbg.state == DBG_STATE_STARTING) {
        if (dbgd.dbg_ui_state != DBG_STATE_STOPPED) {
            logDebug("** Detected potential re-entrancy (ui_state is " + dbgd.dbg_ui_state + ") - ignoring debugger request\n");
            return 1; // pretend it worked OK - presumably another debug session _is_ starting correctly and the user is impatient
        }
    return _do_start_debugger(how, skipDialog);
  }
  // return 1, not 0, since everything's working.
  return 1;
}

// Test code
function doStandaloneInvocationTest() {
   var obj = new Object ();
   var response = window.openDialog
         ("chrome://komodo/content/test/test_standalone_invocation.xul",
          "Komodo",
          "chrome,dialog=yes,close=yes,dependent=yes,modal=yes",
          obj)

   var command = obj.command;
   var params = obj.params;

    if (!command)
       return;
    var invocation = Components.classes["@activestate.com/koInvocation;1?type=Command"].createInstance();
    var catOb = new Object();
    var instOb = new Object();
    invocation.getDefaultPreferences(catOb, instOb);
    invocation.currentCategoryPreferences = catOb.value;
    invocation.currentInstancePreferences = instOb.value;
    var iprefs = invocation.currentInstancePreferences;
    iprefs.setStringPref("filename", command);
    iprefs.setStringPref("params",  params);
    iprefs.setBooleanPref("use-console", false);

    var error_pref = new Object();
    var error_text = new Object()
    if (!invocation.validate(Components.interfaces.koIInvocation.INVOKE_RUN, error_pref, error_text)) {
        alert("The invocation is not valid - " + error_text.value);
        return;
    }
    _do_execute_invocation(invocation, Components.interfaces.koIInvocation.INVOKE_RUN, Components.interfaces.koIDebugger.DBG_STATE_RUNNING);
}
