/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

// Methods used by Scintilla context menu

// global

function koScinContextMenu(xulMenu) {
    this.target     = null;
    this.menu       = null;
    // Distinguish between main and input buffers
    var view = getFocusedView();
    this.scintilla = view._sci_widget;
    this.scimoz = view._scimoz;
    this.isDebugFile = true; // XXX - fix this (or just delete it!)
        
    //if (!SCN_MARGINCLICK)
    	this.shouldDisplay = true;
    //else
    	//this.shouldDisplay = false;

    var sci_widget = document.getElementById(this.scintilla.id);
    this.clickX = sci_widget.getAttribute("_lastx");
    this.clickY = sci_widget.getAttribute("_lasty");
    
    // Initialize new menu.
    this.initMenu(xulMenu);
}

// Prototype for koScinContextMenu "class."
koScinContextMenu.prototype = {
    // onDestroy is empty at the moment.
    onDestroy : function () {
    },
    // Initialize context menu.
    initMenu : function (popup) {
        // Save menu.
        this.menu = popup;

        // Get contextual info.
        this.setTarget( this.scintilla );

        this.seltext = this.scimoz.selText;

        // Initialize (disable/remove) menu items.
        this.initItems();
    },
    initItems : function () {
        this.initCommonItems();
        this.initLanguageItems();
        this.initDebugItems();
    },
    initCommonItems : function () {
        //this.showItem( "scin-context-selectall", true );
    },
    initLanguageItems : function () {
        this.showItem( "scin-context-cookbook", this.seltext != "" );
        this.showItem( "scin-context-makeSnippet", this.seltext != "" );
    },       
    initDebugItems : function () {
        this.showItem( "scin-context-breakpoint", this.isDebugFile );
        // Remove separator if file is not 'debuggable'!
        this.showItem( "context-sep-debug", this.isDebugFile || this.seltext != "" );
    },    
    // Set various context menu attributes based on what part of Mozilla is clicked on
    // line number bar, expand/collapse bar, foo bar or content area
    setTarget : function ( node ) {
        // Initialize contextual info.
        this.onLineNums    = false;
        this.onFoldBar = false;
        this.onBar   = false;
    
        // Remember the node that was clicked.
        this.target = node;
    
        if ( this.target.nodeType == 1 && this.target.nodeName == 'scintilla' ) {  // we have a scintilla
        }
    },
    
    doBreakpoint : function ()  {
        var v = getFocusedView();
        if (!v) return;
      
        var lineClicked = this.getClickLine();
        toggle_breakpoint_at_line(v, lineClicked);        
    },

    doCookbook : function ()  {
        submitRecipe();
    },    

    makeSnippet : function ()  {
        var sci_widget = document.getElementById("scin");
        var scimoz = sci_widget.ISciMoz;
        try {
            var encodingSvc = Components.classes["@activestate.com/koEncodingServices;1"].
                                getService(Components.interfaces.koIEncodingServices);
            var unicodetext = encodingSvc.unicode(scimoz.selText, sci_widget.encoding, 'strict');
        } catch(ex) {
            logError(ex);
            var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                getService(Components.interfaces.koILastErrorService);
            if (lastErrorSvc.getLastErrorCode() == Components.results.NS_ERROR_INVALID_ARG) {
                alertDialog("The text being dragged into the snippet can't fit in the current encoding.  You must" +
                            " change the file's encoding first. [" +  + lastErrorSvc.getLastErrorMessage() + ']')
            }
            return null;
        }
        AddSnippetFromTextToToolbox(unicodetext);
    },

    launchProperties : function ()  {
        doFilePrefs('properties');
    },    
    
    // Misc Helper Functions //
    
    getClickLine : function ()  {
        return this.getLineNumber(this.scimoz, this.clickX, this.clickY);
    },
  
    getLineNumber : function (widget, x, y) {
        var sm = widget;
        var pos = sm.positionFromPoint(x, y);
        if (pos > 0 && pos < sm.length) {
            var lineNo = sm.lineFromPosition(pos);
            return lineNo;
        }
        return null;
    },
    
    // Utilities //

    // Create instance of component given contractId and iid (as string).
    createInstance : function ( contractId, iidName ) {
        var iid = Components.interfaces[ iidName ];
        return Components.classes[ contractId ].createInstance( iid );
    },
    // Get service given contractId and iid (as string).
    getService : function ( contractId, iidName ) {
        var iid = Components.interfaces[ iidName ];
        return Components.classes[ contractId ].getService( iid );
    },
    // Show/hide one item (specified via name or the item element itself).
    showItem : function ( itemOrId, show ) {
        var item = null;
        if ( itemOrId.constructor == String ) {
            // Argument specifies item id.
            item = document.getElementById( itemOrId );
        } else {
            // Argument is the item itself.
            item = itemOrId;
        }
        if ( item ) {
            var styleIn = item.getAttribute( "style" );
            var styleOut = styleIn;
            if ( show ) {
                // Remove style="display:none;".
                styleOut = styleOut.replace( "display:none;", "" );

            } else {
                // Set style="display:none;".
                if ( styleOut.indexOf( "display:none;" ) == -1 ) {
                    // Add style the first time we need to.
                    styleOut += "display:none;";
                }
            }
            // Only set style if it's different.
            if ( styleIn != styleOut ) {
                item.setAttribute( "style", styleOut );
            }
        }
    },
    // Set given attribute of specified context-menu item.  If the
    // value is null, then it removes the attribute (which works
    // nicely for the disabled attribute).
    setItemAttr : function ( id, attr, val ) {
        var elem = document.getElementById( id );
        if ( elem ) {
            if ( val == null ) {
                // null indicates attr should be removed.
                elem.removeAttribute( attr );
            } else {
                // Set attr=val.
                elem.setAttribute( attr, val );
            }
        }
    },
    // Set context menu attribute according to like attribute of another node
    // (such as a broadcaster).
    setItemAttrFromNode : function ( item_id, attr, other_id ) {
        var elem = document.getElementById( other_id );
        if ( elem && elem.getAttribute( attr ) == "true" ) {
            this.setItemAttr( item_id, attr, "true" );
        } else {
            this.setItemAttr( item_id, attr, null );
        }
    }       

};        
