    /* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */


/*
 ****************************************************************
 * View functions
 * 
 * This code hides all concepts of "tabs", and presents a simple "view" interface.
 */

// These globals should only be used by the view functions.
// None of the view user's (eg, the file operations stuff)
// should need to use these.

// The untitled document counter.
var gUntitledCount = 0;

function getNextUntitledName( base /*= "Untitled" */, extension /* = '' */) {
    if (!base) {
        base = "Untitled";
    }

    var rc = base + (++gUntitledCount);
    if (extension) {
        rc += extension;
    }
    return rc;
}

var globalCounter = 1;  // used to generate IDs for viewsin scintillaview.js

// called from menu's built in kombo.js
function doViewAs(language) {
    var v = getFocusedView();
    if (v) v.viewAs(language);
}

function UpdateViewAsLanguage(popup) {
    var v = getFocusedView();
    var language = v.py_inst.language;
    var list = popup.childNodes[0];
    for (var i = 0; i < list.childNodes.length; i++) {
        if (list.childNodes[i].getAttribute('label') == language) {
            list.childNodes[i].setAttribute('type', 'checkbox');
            list.childNodes[i].setAttribute('checked', 'true');
        } else {
            list.childNodes[i].removeAttribute('checked');
        }
    }
}

// this might return NULL
function getFocusedView() {
    var vm = getFocusedViewManager();
    if (vm) return vm.getCurrentView();
    return viewManager.getCurrentView();
}

function editorHasFocus() {
    // returns true if the focus is in either the main scintilla (whether in the
    // html:embed or the <scintilla> element or in the XSLT input pane scintilla
    var focusedElement = top.document.commandDispatcher.focusedElement;
    var scin = top.document.getElementById('scin');
    var input_scin = top.document.getElementById('input-scin');
    return (childOf(focusedElement, scin) || childOf(focusedElement, input_scin));
}

function getFocusedScintilla() {
    // return the <scintilla> which has focus, or null if neither 
    // the main nor the XSLT input pane scintilla has focus.
    var focusedElement = top.document.commandDispatcher.focusedElement
    var scin = top.document.getElementById('scin');
    if (childOf(focusedElement, scin)) {
        return scin;
    }
    var input_scin = top.document.getElementById('input-scin');
    if (childOf(focusedElement, input_scin)) {
        return input_scin;
    }
    return null;
}

function projectHasFocus() {
    return childOf(top.document.commandDispatcher.focusedElement,
                   top.document.getElementById('projectviewbox'));
}

function toolboxHasFocus() {
    return childOf(top.document.commandDispatcher.focusedElement,
                   top.document.getElementById('toolboxviewbox'));
}

function childOf(child, target) {
    // Returns true if 'child' is a descendant of 'target'
    if (!child || !target) return false;
    while (child) {
        if (child == target) {
            return true;
        }
        child = child.parentNode;
    }
    return false;
}

// this ALWAYS returns a view manager
function getFocusedViewManager() {
    var focusedElement = top.document.commandDispatcher.focusedElement;

    // XXX hack alert
    // seems the latest mozilla is returning the html element inside scintilla, rather than scintilla itself.
    if (focusedElement) {
    
        var vm = null;
        if (focusedElement.tagName=="scintilla" &&
            typeof(focusedElement.viewManager) != "undefined") {
            vm = focusedElement.viewManager;
        } else if ("tagName" in focusedElement.parentNode && 
		   focusedElement.parentNode.tagName=="scintilla" &&
                   typeof(focusedElement.parentNode.viewManager) != "undefined") {
            vm = focusedElement.parentNode.viewManager;
        }

        if (vm) {
            return vm;
        }
        
    }
    // if no focused view, always return the current view for the editor.
    return viewManager;
}

// checkDiskFiles is only called from the window's focus handler,
// located in komodo.xul
var gInCheckDiskFiles = false;
function checkDiskFiles() {
    if (gInCheckDiskFiles) return true;

    var globalPrefService = Components.classes["@activestate.com/koPrefService;1"].
            getService(Components.interfaces.koIPrefService);
    var checkDisk = (globalPrefService.prefs.hasBooleanPref("checkDiskFile") &&
        globalPrefService.prefs.getBooleanPref("checkDiskFile"));

    if (!checkDisk) return true;
    gInCheckDiskFiles = true;
    // checks open files and projects for dirtiness
    try {
        var currentvm = getFocusedViewManager();
        var currentview = currentvm.getCurrentView();

        var changedViews = {};
        var removedViews = {};
        var changed = [];
        var gone = [];
        var conflicted = [];
        var view, url, i, j, done, change;
        
        // get all opened files that have changed
        for (var vm in gViewManagerList) {
            var views = gViewManagerList[vm].getViews();
            for (url in views) {
                if (gFileStatusSvc.hasFileChanged(url)!=0) {
                    // the line above will detect some changes like mode changes
                    // which should not trigger this dialog...
                    change = views[url].isDiskFileDirty();
                    // so we use the view's isDiskFileDirty which only picks up
                    // on _content_ changes or mtime changes.
                    if (change == 1) {
                        if (views[url].isDirty()) {
                            // This a "dirty buffer
                            conflicted.push(URLToLocalFile(url).path);
                        } else {
                            changedViews[url] = views[url];
                            changed.push(url);
                        }
                    } else if (change == -1 && !views[url].isDirty()) {
                        removedViews[url] = views[url];
                        gone.push(url);
                    }
                }
            }
        }
        
        // handle files that have changed on disk
        var selected;
        if (changed.length > 0) {
            selected = multiFileDialog('Files Change',
                        'Some files that are open have changed on '+
                        'disk, do you want to reload these files?  ',
                        changed, true,
                        "dontask_changed_files");
            for (j in changed) {
                url = changed[j];
                done = false;
                view = changedViews[url];
                view._observedFileChange = 1;
                for (i in selected) {
                    if (selected[i] == url) {
                        done = true;
                        view.manager.setCurrentView(view);
                        view.revert();
                    }
                }
                if (!done) {
                    view.py_inst.diskFileDirty=1;
                    koRequestUpdateCommands("save");
                    view.manager.indicateIfDirty(view);
                }
                view._observedFileChange = 0;
                gObserverSvc.Notify(null,'file_changed',url)
            }
        }
        
        // handle files that have been removed from disk
        if (gone.length > 0) {
            for (j in gone) {
                url = gone[j];
                done = false;
                view = removedViews[url];
                view._observedFileChange = 1;
                if (!done) {
                    view.py_inst.diskFileDirty=1;
                    koRequestUpdateCommands("save");
                    view.manager.indicateIfDirty(view);
                }
                view._observedFileChange = 0;
                gObserverSvc.Notify(null,'file_changed',url)
            }
        }
        
        // get all opened projects that have changed
        var projects = gProjectManager._projects;
        var project;
        changed = [];
        gone = [];
        changedViews = [];
        removedViews = [];
        for (i = 0; i < projects.length; i++) {
            url = projects[i].url
            change = gFileStatusSvc.hasFileChanged(url);
            if (change == 1) {
                if (projects[i].isDirty) {
                    // The project is dirty -- don't risk data loss.
                    conflicted.push(URLToLocalFile(url).path);
                } else {
                    changedViews.push(i);
                    changed.push(url);
                }
            } else if (change == -1 && !projects[i].part.isDirty) {
                removedViews.push(i);
                gone.push(url);
            }
        }

        // handle projects that have changed on disk
        if (changed.length > 0) {
            selected = multiFileDialog('Projects Changed',
                        'Some projects that are open have changed on disk, '+
                        'do you want to reload these projects?',
                        changed, true,
                        "dontask_changed_projects");
            for (j in changedViews) {
                url = projects[j].url;
                done = false;
                for (i in selected) {
                    if (selected[i] == url) {
                        gProjectManager.closeProjectEvenIfDirty(j);
                        gProjectManager.loadProject(url)
                        done = true;
                    }
                }
                if (!done) {
                    projects[j].part.isDirty = true;
                }
                gObserverSvc.Notify(null,'file_changed',url)
            }
        }
        
        // handle projects that have been removed from disk
        if (gone.length > 0) {
            for (j in removedViews) {
                url = projects[j].url;
                done = false;
                if (!done) {
                    projects[j].part.isDirty = true;
                }
                gObserverSvc.Notify(null,'file_changed',url)
            }
        }
        if (conflicted.length > 0) {
            alertDialog('The following files and projects have changed on disk '+
                        'and are modified -- either revert them or save your changes:\n\n' + 
                        conflicted.join('\n'));
        }
        // reset the the last used manager and view!
        if (currentvm && currentvm != getFocusedViewManager()) {
            currentvm.setFocus();
        }
        if (currentview && currentview != currentvm.getCurrentView()) {
            currentvm.setCurrentView(currentview);
        }
    } catch(e) {
        logError(e);
    }
    gObserverSvc.Notify(null,'file_update_now',url)
    gInCheckDiskFiles = false;
    return true;
}


/**
 * take the text from the debugger output scintilla, put it into the iframe, and switch
 */
var outputTmpFile=null;
              
function renderOutputAsHTML() {
    var swstack = document.getElementById("redirect-stack");
    var outputIndex = parseInt(swstack.getAttribute("index"));
    var scout = document.getElementById("redirect-scin").ISciMoz;
    var browser = document.getElementById("redirect-blank");
    var flags = nsIWebNavigation.LOAD_FLAGS_BYPASS_PROXY | nsIWebNavigation.LOAD_FLAGS_BYPASS_CACHE;
    if (outputTmpFile) {
        delete outputTmpFile;
        outputTmpFile = null;
    }
    if (outputIndex == 0) {
        outputIndex = 1;
        OutputWindow_Init()
        swstack.setAttribute("index", outputIndex);
        // force reload of blank
        //browser.webNavigation.loadURI('about:blank', flags);
    } else {
        var filename = dbgd.debugFileName;
        var basedir = null;
        if (filename) basedir = os.path.dirname(filename);
        
        // get the text from scintilla, stuff it into the iframe, and switch the deck
        var text = scout.text;
        if (text) {
            var tmpFileSvc = Components.classes["@activestate.com/koTempFileFactory;1"].
                getService(Components.interfaces.koITempFileFactory);

            if (basedir) {
                outputTmpFile = tmpFileSvc.MakeTempFileInDir(basedir,".html","w");
            } else {
                outputTmpFile = tmpFileSvc.MakeTempFile(".html","w");
            }

            outputTmpFile.puts(text);
            outputTmpFile.flush();
            outputTmpFile.close();
            var outputURL = outputTmpFile.file.URL;
            browser.webNavigation.loadURI(outputURL, flags);
            outputIndex = 0;
            swstack.setAttribute("index", outputIndex);
        }
    }
}


/**
 * view the text scintilla editor as html
 */
 var gViewHTMLTab = false;
function doViewAsHTMLTabSwitch(tab) {
    if (tab.id == 'view_html_tab') {
        if (gViewHTMLTab) {
            document.getElementById('html-view-editor').focus();
            return;
        }
        gViewHTMLTab = true;
        viewAsHTML();
        document.getElementById('html-view-editor').focus();
    } else if (gViewHTMLTab) {
        gViewHTMLTab = false;
        viewManager.setFocus();
        document.getElementById("html-view-editor").removeAttribute('src');
    } else {
        viewManager.setFocus();
    }
}

function viewAsHTML() {
    var view = viewManager.getCurrentView();
    if (!view) return;
    
    var browser = document.getElementById("html-view-editor");
    var flags = nsIWebNavigation.LOAD_FLAGS_BYPASS_PROXY | nsIWebNavigation.LOAD_FLAGS_BYPASS_CACHE;
    if (outputTmpFile) {
        delete outputTmpFile;
        outputTmpFile = null;
    }
    
    // get the text from scintilla, stuff it into the iframe, and switch the deck
    var text = view._scimoz.text;
    var filename = null;
    if (view.url) {
        try {
            filename = URLToLocalFile(view.url).path;
        } catch (e) {
            // This fails if the file doesn't exist!
            ;
        }
    } 
    
    // put the temp file in the basedir for the url
    var basedir = null;
    // os defined in koGlobalOverlay.js
    if (filename) basedir = os.path.dirname(filename);
    
    if (text) {
        var tmpFileSvc = Components.classes["@activestate.com/koTempFileFactory;1"].
            getService(Components.interfaces.koITempFileFactory);
        if (basedir) {
            outputTmpFile = tmpFileSvc.MakeTempFileInDir(basedir,".html","w");
        } else {
            outputTmpFile = tmpFileSvc.MakeTempFile(".html","w");
        }
        outputTmpFile.puts(text);
        outputTmpFile.flush();
        outputTmpFile.close();
        var outputURL = outputTmpFile.file.URL;
        browser.webNavigation.loadURI(outputURL, flags);
    }
}

function OptionallySaveDirtyFiles(prompt, pref) {
    // This function asks the user about any files that are dirty, and saves them as requested.
    // it returns false if the user cancels, true otherwise.
    
    // 'prompt' is a string
    // 'pref' is a pref that's used for the "don't ask me again" checkbox

    // XXX this does not account for input panes: dirty input docs may not be saved -- need to check!
    
    var result = true;

    // If there are, then ask the user if she wants to save changes to the
    // documents before starting the debugger.
    var dirtyViews = viewManager.getDirtyURLs(new Object());
    var allDirtyURLs = dirtyViews.slice();







    if (allDirtyURLs.length == 0) return true;

    var selectedURLs = multiFileDialog(prompt,
                    'Some file changes have not been saved.'+
                    'Do you want to save these files?',
                    allDirtyURLs, true,
                    pref,
                    true ); // allow "none" as a don't ask me again pref.
    if (selectedURLs == null) return false;
    // keep track of the current view -- we'll want to go back to it.
    var currentViewBefore = viewManager.getCurrentView();
    // Save the selected ones
    // Save the URLs that the user selected.
    var view, url, found;
    for (i = 0; i < selectedURLs.length; i++) {
        url = selectedURLs[i];
        found = false;
        for (j = 0; j < dirtyViews.length; j++) {
            if (dirtyViews[j] == url) {
                found = true;
                if (! viewManager.ConfirmedSave(url)) {
                    return false;  // error or something.
                }
                break;
            }
        }













    }
    viewManager.setCurrentView(currentViewBefore);
    return result;
}



