/* Copyright (c) 2000-2002 ActiveState SRL.
   See the file LICENSE.txt for licensing information. */

// The Getting Started page.
// TODO:
// - bug: remove one or a few files, File->Recent Files-><one of them>
//    result: cmd_mruFile0 has no controller
//   still a bug?


//---- globals

var dlg = new Object();  // shortcuts to DOM node in the XUL.

var gPrefSvc = null;
var gUrllib = null;
var gTimeSvc = null;
var gOsPathSvc = null;
var gOsSvc = null;
var gLastErrorSvc = null;
var gObserverSvc = null;
var gKoDirSvc = null;
var gInfoSvc = null;
var gFeatureStatusSvc = null;

var gGSObserver = null;
var gGSPrefObserver = null;
var gDateFormat = null;

// Flags used to delay updating the MRU tables when not visible.
var gUpdateFileMRUWhenVisible = false;
var gUpdateProjectMRUWhenVisible = false;

// Indicates if the feature status has been requested for the features listed
// in the "Feature Control Panel".
var gRequestedFeatureStatuses = false;



//---- internal support routines and classes

function _isToday(secs)
{
    // Return true iff the given 'secs' (seconds since the epoch) is today.
    var timeTuple = gTimeSvc.localtime(secs, new Object());
    var date = gTimeSvc.strftime("%Y %m %d", timeTuple.length, timeTuple);

    var secsNow = gTimeSvc.time();
    var timeTupleNow = gTimeSvc.localtime(secsNow, new Object());
    var dateNow = gTimeSvc.strftime("%Y %m %d", timeTupleNow.length,
                                    timeTupleNow);

    return date == dateNow;
}


function _getLastModifiedMessage(url)
{
    // Return a pretty string describing the last modified time of the given
    // URL if available.
    var msg = "";
    if (url.search(/^(file\:)/) != -1) {
        var fname = displayNameFromURL(url);
        var exists = false;
        try {
            exists = gOsPathSvc.exists(fname)
        } catch(e) {
            // if we are not in a UTF locale, and the filename has
            // characters outside the locale characterset, we will
            // get an exception
            return "Unknown";
        }
        if (exists) {
            var mode = new Object();
            var ino = new Object();
            var nlink = new Object();
            var uid = new Object();
            var gid = new Object();
            var size = new Object();
            var atime = new Object();
            var mtime = new Object();
            var ctime = new Object();
            try {
                gOsSvc.stat(fname, mode, ino, ino, nlink, uid, gid, size,
                            atime, mtime, ctime);
            } catch (ex) {
                msg = "Error: " + gLastErrorSvc.getLastErrorMessage();
                logError("Error stating '"+fname+"': "+ex);
            }
            if (! msg) {
                var timeTuple = gTimeSvc.localtime(mtime.value, new Object());
                if (_isToday(mtime.value)) {
                    msg = "Today at " + gTimeSvc.strftime("%H:%M:%S", 
                                                          timeTuple.length, 
                                                          timeTuple);
                } else {
                    msg = gTimeSvc.strftime(gDateFormat, timeTuple.length,
                                            timeTuple);
                }
            }
        } else {
            msg = "Does not exist";
        }

    } else if (url.search(/^(koremote\:)/) != -1)  {
        msg = "Could not determine (Remote File)";
    } else  {
        msg = "Could not determine";
    }
    return msg;
}

function _addMRURow(mru, index, url)
{
    // Add the given project/file URL to the MRU table.
    //    "mru" is either "project" or "file", indicating which MRU.
    //    "index" is the zero-based index of the URL in the MRU.
    //    "url" is the URL to add.
    //
    // Add the equivalent of the following XUL:
    //  <row equalsize="always">
    //    <hbox style="background-color: #eaeaea;" flex="1" align="center"
    //            pack="start">
    //        <checkbox id="gs-project-mru-checkbox-0" checked="false"
    //                  oncommand="GettingStarted_ToggleCheckbox('project');");
    //                  _command="cmd_mruProject1" _index="0"/>
    //        <image src="chrome://komodo/skin/gettingstarted/images/project_icon_16x16.png"
    //                style="padding-left: 4px;"/>
    //        <button label="PROJECTNAME" class="button-link"
    //                onclick="koGoDoCommand('cmd_mruProject1');"
    //                style="cursor: pointer;" crop="middle"
    //                tooltiptext="FULL_PATH_TO_PROJECT"
    //                tooltip="aTooltip"/>
    //    </hbox>
    //    <hbox style="background-color: #eaeaea;" flex="1" align="center"
    //            pack="start">
    //        <label class="text" value="TIMESTAMP"/>
    //    </hbox>
    //  </row>

    // Gather data.
    var color;
    if (index % 2) {
        color = "#f5f5f5";
    } else {
        color = "#eaeaea";
    }
    var fileName = gUrllib.unquote(makeLeafName(url));
    var filePath = gUrllib.unquote(displayNameFromURL(url));
    var cmdName;
    if (mru == "project") {
        cmdName = "cmd_mruProject" + (index);
    } else if (mru == "file") {
        cmdName = "cmd_mruFile" + (index);
    }
    
    // Build the XUL.
    var hbox, checkbox, image, button, label;
    var row = document.createElement("row");
    row.setAttribute("equalsize", "always");
    
    hbox = document.createElement("hbox");
    hbox.setAttribute("style", "background-color:"+color+";");
    hbox.setAttribute("flex", "1");
    hbox.setAttribute("align", "center");
    hbox.setAttribute("pack", "start");
    checkbox = document.createElement("checkbox");
    checkbox.setAttribute("id", "gs-"+mru+"-mru-checkbox-"+index);
    checkbox.setAttribute("checked", "false");
    checkbox.setAttribute("oncommand", 
                          "GettingStarted_ToggleCheckbox('"+mru+"');");
    checkbox.setAttribute("_command", cmdName); // used by the Open button
    checkbox.setAttribute("_index", index); // used by the Remove button
    hbox.appendChild(checkbox);
    image = document.createElement("image");
    image.setAttribute("src", "chrome://komodo/skin/gettingstarted/images/"+mru+"_icon_16x16.png");
    image.setAttribute("style", "padding-left: 4px;");
    hbox.appendChild(image);
    button = document.createElement("button");
    button.setAttribute("label", fileName);
    button.setAttribute("class", "button-link");
    button.setAttribute("oncommand", "koGoDoCommand('"+cmdName+"');");
    button.setAttribute("style", "cursor: pointer;");
    button.setAttribute("crop", "middle"); // XXX I don't think this works.
    button.setAttribute("tooltiptext", filePath);
    button.setAttribute("tooltip", "aTooltip");
    hbox.appendChild(button)
    row.appendChild(hbox);
    
    hbox = document.createElement("hbox");
    hbox.setAttribute("style", "background-color:"+color+";");
    hbox.setAttribute("flex", "1");
    hbox.setAttribute("align", "center");
    hbox.setAttribute("pack", "start");
    label = document.createElement("label");
    label.setAttribute("class", "text");
    label.setAttribute("value", _getLastModifiedMessage(url));
    hbox.appendChild(label);
    row.appendChild(hbox);

    if (mru == "project") {
        dlg.projectMRURows.appendChild(row);
    } else if (mru == "file") {
        dlg.fileMRURows.appendChild(row);
    }
}

function _addEmptyMRURow(mru)
{
    // Add a "No Recent Files/Projects" row.
    //    "mru" is either "project" or "file", indicating which MRU.
    //
    // One like this:
    //    <row equalsize="always">
    //        <hbox style="background-color: #eaeaea;" flex="1" align="center" pack="start">
    //            <label value=" No recent projects" class="text" style="font-weight:bold;"/>
    //        </hbox>
    //        <hbox style="background-color: #eaeaea;" flex="1" align="center" pack="start"/>
    //    </row>
    var hbox, label;
    var color = "#eaeaea";
    var row = document.createElement("row");
    row.setAttribute("equalsize", "always");
    
    hbox = document.createElement("hbox");
    hbox.setAttribute("style", "background-color:"+color+";");
    hbox.setAttribute("flex", "1");
    hbox.setAttribute("align", "center");
    hbox.setAttribute("pack", "start");
    label = document.createElement("label");
    label.setAttribute("class", "text");
    label.setAttribute("value", " No recent "+mru+"s");
    label.setAttribute("style", "font-weight: bold;");
    hbox.appendChild(label);
    row.appendChild(hbox);
    
    // Add the timestamp cell.
    hbox = document.createElement("hbox");
    hbox.setAttribute("style", "background-color:"+color+";");
    hbox.setAttribute("flex", "1");
    hbox.setAttribute("align", "center");
    hbox.setAttribute("pack", "start");
    row.appendChild(hbox);

    if (mru == "project") {
        dlg.projectMRURows.appendChild(row);
    } else if (mru == "file") {
        dlg.fileMRURows.appendChild(row);
    }
}


function _updateProjectMRUTable() {
    // Get the MRU info.
    var projectMRU = null;
    if (gPrefSvc.prefs.hasPref("mruProjectList")) {
        projectMRU = gPrefSvc.prefs.getPref("mruProjectList");
    }
    
    // Clear the table (don't clear the header row).
    while (dlg.projectMRURows.childNodes.length > 1)  {
        var row = dlg.projectMRURows.firstChild.nextSibling;
        dlg.projectMRURows.removeChild(row)
    }
    
    if (projectMRU && projectMRU.length > 0) {
        for (var i=0; i<projectMRU.length; i++) {
            var project = projectMRU.getStringPref(i);
            _addMRURow("project", i, project);
        }
        dlg.projectMRUSelectCheckbox.setAttribute("disabled", false);
    } else {
        _addEmptyMRURow("project");
        dlg.projectMRUSelectCheckbox.setAttribute("disabled", true);
    }
}


function _updateFileMRUTable() {
    // Get the MRU info.
    var fileMRU = null;
    if (gPrefSvc.prefs.hasPref("mruFileList")) {
        fileMRU = gPrefSvc.prefs.getPref("mruFileList");
    }
    
    // Clear the table (don't clear the header row).
    while (dlg.fileMRURows.childNodes.length > 1)  {
        var row = dlg.fileMRURows.firstChild.nextSibling;
        dlg.fileMRURows.removeChild(row)
    }
    
    if (fileMRU && fileMRU.length > 0) {
        for (var i=0; i<fileMRU.length; i++) {
            var file = fileMRU.getStringPref(i);
            _addMRURow("file", i, file);
        }
        dlg.fileMRUSelectCheckbox.setAttribute("disabled", false);
    } else {
        _addEmptyMRURow("file");
        dlg.fileMRUSelectCheckbox.setAttribute("disabled", true);
    }
}


function _toggleFileMRUSelections(checked)
{
    var checkboxes = dlg.fileMRURows.getElementsByTagName("checkbox");
    for (var i=0; i < checkboxes.length; i++) {
        checkboxes[i].setAttribute("checked", checked);
    }
}

function _toggleProjectMRUSelections(checked)
{
    var checkboxes = dlg.projectMRURows.getElementsByTagName("checkbox");
    for (var i=0; i < checkboxes.length; i++) {
        checkboxes[i].setAttribute("checked", checked);
    }
}


function _requestFeatureStatus(featureName /* ="*" */)
{
    // Make a request to determine the status of the given feature.
    //    "featureName" identifies the feature to get the status for. It
    //        defaults to all features.
    if (typeof(featureName) == "undefined" || !featureName) {
        featureName = "*";
    }
    var observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                      getService(Components.interfaces.nsIObserverService);
    
    var dummy = new Object();
    if (featureName == "*") {
        var allFeatureNames = ["Perl Debugging", "Perl Syntax Checking",
                               "PHP Debugging", "PHP Syntax Checking", 
                               "Python Debugging", "Python Syntax Checking", 
                               "Tcl Debugging", "Tcl Syntax Checking", 




                               ];
        for (var i=0; i < allFeatureNames.length; i++) {
            try {
                observerSvc.Notify(dummy, "feature_status_request",
                                   allFeatureNames[i]);
            } catch(e) {
                logError("Error requesting status for '" +
                         allFeatureNames[i] + "' feature:" + e);
            }
        }
    } else {
        try {
            observerSvc.Notify(dummy, "feature_status_request", featureName);
        } catch(e) {
            logError("Error requesting status for '" + featureName +
                     "' feature:" + e);
        }
    }
}


function _updateFeatureControlPanel(featureName, status, reason)
{
    // Update status in the feature control panel.
    //    "featureName" names the feature.
    //    "status" is a string describing the feature's status.
    //    "reason" (optional) is a reason describing why the feature is not
    //        functional.
    if (typeof(reason) == "undefined") {
        reason = null;
    }

    var widget = null;
    switch (featureName) {
    case "Perl Debugging":
        status = "Debugging: " + status;
        widget = dlg.perlDebuggingStatus;
        break;
    case "Perl Syntax Checking":
        status = "Syntax Checking: " + status;
        widget = dlg.perlSyntaxCheckingStatus;
        break;
    case "PHP Debugging":
        status = "Debugging: " + status;
        widget = dlg.phpDebuggingStatus;
        break;
    case "PHP Syntax Checking":
        status = "Syntax Checking: " + status;
        widget = dlg.phpSyntaxCheckingStatus;
        break;
    case "Python Debugging":
        status = "Debugging: " + status;
        widget = dlg.pythonDebuggingStatus;
        break;
    case "Python Syntax Checking":
        status = "Syntax Checking: " + status;
        widget = dlg.pythonSyntaxCheckingStatus;
        break;
    case "Tcl Debugging":
        status = "Debugging: " + status;
        widget = dlg.tclDebuggingStatus;
        break;
    case "Tcl Syntax Checking":
        status = "Syntax Checking: " + status;
        widget = dlg.tclSyntaxCheckingStatus;
        break;

























    }
    
    if (widget) {
        widget.setAttribute("value", status);
        if (reason) {
            widget.setAttribute("tooltip", "aTooltip");
            widget.setAttribute("tooltiptext", reason);
        } else {
            widget.removeAttribute("tooltip");
            widget.removeAttribute("tooltiptext");
        }
    }
}


function _updatePanelVisibility(panelId, prefId) {
    var panel = document.getElementById(panelId);
    var visible = gPrefSvc.prefs.getBooleanPref(prefId);
    if (visible && panel.hasAttribute('collapsed')) {
        panel.removeAttribute('collapsed');
        return;
    }
    if (!visible && !panel.hasAttribute('collapsed')) {
        panel.setAttribute('collapsed', 'true');
        return;
    }
}


function _GettingStarted_IsVisible()
{
    // Return true iff the getting started page is currently visible.
    //
    // Check if the 'index' of the main <deck> in komodo.xul is the
    // getting started page's index.
    var workspaceDeck = top.document.getElementById("workspace-deck");
    return workspaceDeck.getAttribute("index") == 0;
}


function _GSObserver () {};
_GSObserver.prototype.Observe = function(subject, topic, data)
{
    //dump("_GSObserver: observed '"+topic+"' notification: data='"+
    //     data+"'\n");

    switch (topic) {
    case "mru_changed":
        var prefName = data;
        // XXX This is inefficient if say, a number of files are opened at
        //     once. We will regenerate the file list for opened file which
        //     means, among other things, that we will stat every file in
        //     the MRU for every file open.
        var visible = _GettingStarted_IsVisible();
        if (prefName == "mruProjectList") {
            if (visible) {
                _updateProjectMRUTable();
            } else {
                gUpdateProjectMRUWhenVisible = true;
            }
        } else if (prefName == "mruFileList") {
            if (visible) {
                _updateFileMRUTable();
            } else {
                gUpdateFileMRUWhenVisible = true;
            }
        }
        break;
    case "feature_status_ready":
        var featureName = data;
        var featureStatus = subject;
        _updateFeatureControlPanel(featureName, featureStatus.status,
                                   featureStatus.reason);
        break;
    }
};

function _GSPrefObserver () {};
_GSPrefObserver.prototype.Observe = function(prefSet, prefSetID, prefName)
{
    //dump("XXX _GSPrefObserver: observed change in '"+prefName+"' pref\n");
    switch (prefName) {
    case 'showGettingStartedPanel':
        _updatePanelVisibility('gettingstarted-panel', prefName);
        _updatePanelVisibility('gettingstarted-separator', prefName);
        break;
    case 'showFeaturePanel':
        _updatePanelVisibility('feature-panel', prefName);
        _updatePanelVisibility('feature-separator', prefName);
        if (!gRequestedFeatureStatuses
            && gPrefSvc.prefs.getBooleanPref(prefName)) {
            _requestFeatureStatus();
            gRequestedFeatureStatuses = true;
        }
        break;
    case 'showMRUPanel':
        _updatePanelVisibility('mru-panel', prefName);
        _updatePanelVisibility('mru-separator', prefName);
        break;








    case "perlDefaultInterpreter":




        _requestFeatureStatus('Perl Debugging');
        _requestFeatureStatus('Perl Syntax Checking');
        break;
    case "phpDefaultInterpreter":
        _requestFeatureStatus('PHP Debugging');
        _requestFeatureStatus('PHP Syntax Checking');
    case "pythonDefaultInterpreter":



        _requestFeatureStatus('Python Debugging');
        _requestFeatureStatus('Python Syntax Checking');
    case "tclshDefaultInterpreter":
        _requestFeatureStatus('Tcl Syntax Checking');
        // fall through
    case "wishDefaultInterpreter":



        _requestFeatureStatus('Tcl Debugging');
        break;
    case "defaultDateFormat":
        gDateFormat = gPrefSvc.prefs.getStringPref("defaultDateFormat");
        _updateProjectMRUTable();
        _updateFileMRUTable();
        break;
    }
};



//---- public routines

function GettingStarted_OnLoad()
{
    gPrefSvc = Components.classes['@activestate.com/koPrefService;1'].
        getService(Components.interfaces.koIPrefService);
    gUrllib = Components.classes["@activestate.com/koUrllib;1"].
        getService(Components.interfaces.koIUrllib);
    gTimeSvc = Components.classes["@activestate.com/koTime;1"].
        getService(Components.interfaces.koITime);
    gOsSvc = Components.classes["@activestate.com/koOs;1"].
        getService(Components.interfaces.koIOs);
    gOsPathSvc = Components.classes["@activestate.com/koOsPath;1"].
        getService(Components.interfaces.koIOsPath);
    gLastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
        getService(Components.interfaces.koILastErrorService);
    gObserverSvc = Components.classes["@mozilla.org/observer-service;1"].
        getService(Components.interfaces.nsIObserverService);
    gKoDirSvc = Components.classes["@activestate.com/koDirs;1"].
        getService(Components.interfaces.koIDirs);
    gInfoSvc = Components.classes["@activestate.com/koInfoService;1"].
        getService(Components.interfaces.koIInfoService);
    gFeatureStatusSvc = Components.classes["@activestate.com/koFeatureStatusService;1"].
        getService(Components.interfaces.koIFeatureStatusService);
    gFeatureStatusSvc.initialize();

    dlg.projectMRURows = document.getElementById("gs-project-mru-rows");
    dlg.fileMRURows = document.getElementById("gs-file-mru-rows");
    dlg.projectMRUSelectCheckbox = document.getElementById("gs-project-mru-select-checkbox");
    dlg.fileMRUSelectCheckbox = document.getElementById("gs-file-mru-select-checkbox");
    dlg.openSelectedButton = document.getElementById("gs-open-selected-button");
    dlg.removeSelectedButton = document.getElementById("gs-remove-selected-button");
    dlg.productInfoLabel = document.getElementById("gs-product-info-label");
    dlg.perlDebuggingStatus = document.getElementById("gs-perl-debugging-status");
    dlg.perlSyntaxCheckingStatus = document.getElementById("gs-perl-syntax-checking-status");
    dlg.phpDebuggingStatus = document.getElementById("gs-php-debugging-status");
    dlg.phpSyntaxCheckingStatus = document.getElementById("gs-php-syntax-checking-status");
    dlg.pythonDebuggingStatus = document.getElementById("gs-python-debugging-status");
    dlg.pythonSyntaxCheckingStatus = document.getElementById("gs-python-syntax-checking-status");
    dlg.tclDebuggingStatus = document.getElementById("gs-tcl-debugging-status");
    dlg.tclSyntaxCheckingStatus = document.getElementById("gs-tcl-syntax-checking-status");









    gDateFormat = gPrefSvc.prefs.getStringPref("defaultDateFormat");

    // Fill in license and Komodo product type information.
    var info = "Komodo ";
    if (gInfoSvc.productType == "professional") {
        info += " Professional";
    } else if (gInfoSvc.productType == "personal") {
        info += " Personal";
    }
    info += ". " + gInfoSvc.licenseInfo;
    dlg.productInfoLabel.appendChild( document.createTextNode(info) );

    _updateProjectMRUTable();
    _updateFileMRUTable();
    Tip_OnLoad('gs');
    if (gPrefSvc.prefs.getBooleanPref("showFeaturePanel")) {
        // Don't bother getting info that will not be shown.
        _requestFeatureStatus();
        gRequestedFeatureStatuses = true;
    }

    _updatePanelVisibility('gettingstarted-panel', 'showGettingStartedPanel');
    _updatePanelVisibility('gettingstarted-separator', 'showGettingStartedPanel');
    _updatePanelVisibility('feature-panel', 'showFeaturePanel');
    _updatePanelVisibility('feature-separator', 'showFeaturePanel');
    _updatePanelVisibility('mru-panel', 'showMRUPanel');
    _updatePanelVisibility('mru-separator', 'showMRUPanel');

    // Setup observers for data changes.
    gGSObserver = new _GSObserver();
    gObserverSvc.AddObserver(gGSObserver, 'mru_changed');
    gObserverSvc.AddObserver(gGSObserver, 'feature_status_ready');

    gGSPrefObserver = new _GSPrefObserver();
    gPrefSvc.prefs.AddObserver(gGSPrefObserver);

    GettingStarted_Show();
}

function GettingStarted_OnUnload()
{
    // Shutdown observers.
    gPrefSvc.prefs.RemoveObserver(gGSPrefObserver);
    gGSPrefObserver = null

    gObserverSvc.RemoveObserver(gGSObserver, 'feature_status_ready');
    gObserverSvc.RemoveObserver(gGSObserver, 'mru_changed');
    gGSObserver = null;

    gFeatureStatusSvc.finalize();
}


function GettingStarted_SelectAllFiles()
{
    var checkboxes = dlg.fileMRURows.getElementsByTagName("checkbox");
    for (var i=0; i < checkboxes.length; i++) {
        checkboxes[i].setAttribute("checked", true);
    }
}


function GettingStarted_OpenAllSelected()
{
    var cmds = new Array();
    var i;
    var checkboxes = dlg.projectMRURows.getElementsByTagName("checkbox");
    for (i=0; i < checkboxes.length; i++) {
        // skip header row's checkbox
        if (checkboxes[i].hasAttribute("_command") && checkboxes[i].checked) {
            cmds.push( checkboxes[i].getAttribute("_command") );
        }
    }
    _toggleProjectMRUSelections(false);

    checkboxes = dlg.fileMRURows.getElementsByTagName("checkbox");
    for (i=0; i < checkboxes.length; i++) {
        // skip header row's checkbox
        if (checkboxes[i].hasAttribute("_command") && checkboxes[i].checked) {
            cmds.push( checkboxes[i].getAttribute("_command") );
        }
    }
    _toggleFileMRUSelections(false);
    
    if (cmds.length) {
        for (i=0; i < cmds.length; i++) {
            koGoDoCommand(cmds[i]);
        }
    }
}


function GettingStarted_RemoveAllSelected()
{
    // Remove the selected files and projects from their respective MRUs.
    // - Delete in reverse order so indeces are correct.
    var i;
    var indeces = new Array();
    var checkboxes = dlg.projectMRURows.getElementsByTagName("checkbox");
    for (i = checkboxes.length-1; i >= 0; i--) {
        if (checkboxes[i].hasAttribute("_index") && checkboxes[i].checked) {
            indeces.push( checkboxes[i].getAttribute("_index") );
        }
    }
    _toggleProjectMRUSelections(false);
    for (i=0; i < indeces.length; i++) {
        MRU_del("mruProjectList", indeces[i]);
    }

    indeces = new Array();
    checkboxes = dlg.fileMRURows.getElementsByTagName("checkbox");
    for (i = checkboxes.length-1; i >= 0; i--) {
        if (checkboxes[i].hasAttribute("_index") && checkboxes[i].checked) {
            indeces.push( checkboxes[i].getAttribute("_index") );
        }
    }
    _toggleFileMRUSelections(false);
    for (i=0; i < indeces.length; i++) {
        MRU_del("mruFileList", indeces[i]);
    }
    
    GettingStarted_UpdateMRUButtons();
}


function GettingStarted_ClearMRU(mru) {
    // Verify and then clear the given MRU.
    //    "mru" is either "project" or "file", indicating which MRU.
    var prompt = "Are you sure you want to clear the list of recent "+
                 mru+"s?";
    var answer = alertDialog(prompt, true, false, "Ok", "Cancel")
    if (answer == "Ok") {
        if (mru == "project") {
            MRU_reset("mruProjectList");
        } else if (mru == "file") {
            MRU_reset("mruFileList");
        }
    }
}


function GettingStarted_ToggleMRUSelection(mru)
{
    // Toggle checked or unchecked all checkboxes for the given mru.
    //    "mru" is either "project" or "file", indicating which MRU.
    if (mru == "project") {
        _toggleProjectMRUSelections(dlg.projectMRUSelectCheckbox.checked);
    } else if (mru == "file") {
        _toggleFileMRUSelections(dlg.fileMRUSelectCheckbox.checked);
    }
    GettingStarted_UpdateMRUButtons();
}

function GettingStarted_ToggleCheckbox(mru)
{
    // Do the necessary updates when a user (un)checks the given checkbox.
    //    "mru" is either "project" or "file", indicating which MRU.
    //    "index" is the index in the MRU table.
    var groupSelectCheckbox = document.getElementById("gs-"+mru+"-mru-select-checkbox");
    groupSelectCheckbox.setAttribute('checked', false);
    
    GettingStarted_UpdateMRUButtons();
}


function GettingStarted_UpdateMRUButtons()
{
    var noneChecked = true;
    var i;
    var checkboxes = dlg.projectMRURows.getElementsByTagName("checkbox");
    for (i=0; i < checkboxes.length; i++) {
        // Skip the "group-select" checkbox in the title row.
        if (checkboxes[i].hasAttribute("_command") && checkboxes[i].checked) {
            noneChecked = false;
            break;
        }
    }
    if (noneChecked) {
        checkboxes = dlg.fileMRURows.getElementsByTagName("checkbox");
        for (i=0; i < checkboxes.length; i++) {
            // Skip the "group-select" checkbox in the title row.
            if (checkboxes[i].hasAttribute("_command") && checkboxes[i].checked) {
                noneChecked = false;
                break;
            }
        }
    }

    dlg.openSelectedButton.setAttribute("disabled", noneChecked);
    dlg.removeSelectedButton.setAttribute("disabled", noneChecked);
}


function GettingStarted_OpenSampleProject() {
    // We convert to a URL first so we can extend the path portably without
    // any magic
    var sampleKpf = LocalFileToURL(gKoDirSvc.installDir);
    sampleKpf = sampleKpf + "samples/sample_project.kpf";
    doOpenProjectFromURL(sampleKpf);
}


function GettingStarted_Show() {
    // Should be called when the Getting Started page is being made visible.
    if (gUpdateProjectMRUWhenVisible) {
        _updateProjectMRUTable();
        gUpdateProjectMRUWhenVisible = false;
    } else {
        _updateFileMRUTable();
        gUpdateFileMRUWhenVisible = false;
    }
}


function GettingStarted_OpenHelp(url /* = null */) {
    var windowMgr = Components.classes["@mozilla.org/rdf/datasource;1?name=window-mediator"].
                    getService(Components.interfaces.nsIWindowMediator);
    var helpWindow = windowMgr.getMostRecentWindow('komodo_help');
    if (helpWindow) {
        // XXX it'd be nice to have a way to change the help page of an already
        // open window instead of closing and restarting it.
        helpWindow.close();
    }

    func_helpHelp(url);
}


var _boingInterval = null;
var _boingA, _boingV, _boingT, _boingC, _boingPxPerM, _boingD,
    _boingAvail_px, _boingRemainder_px, _boingD_px, _boingBouncedLastStep,
    _boingStep;
function _Boing(event) {
    if (!event.ctrlKey) return;

    _boingA = 9.81; // gravity
    _boingV = 0.0; // initial y velocity (positive is down)
    _boingT = 0.010; //time interval
    _boingC = 0.8; //coefficient of energy loss on _boingA bounce
    _boingPxPerM = 200.0;
    _boingD = 0.0;
    _boingAvail_px;
    _boingRemainder_px =0.0;
    _boingD_px = 0.0;
    _boingBouncedLastStep = 0;
    _boingStep = 0;

    if (_boingInterval != null) {
        window.clearInterval(_boingInterval);
        _boingInterval = null;
    }
    _boingInterval = window.setInterval('_BoingABit();', 10)
}

function _BoingABit() {
    if (_boingStep >= 500) {
        window.clearInterval(_boingInterval);
        _boingInterval = null;
        return
    }

    // how far to fall (or rise)
    _boingD = _boingV * _boingT;
    _boingD_px = _boingD * _boingPxPerM + _boingRemainder_px;

    // save remainder for next _boingStep (or could .round() to try to even out)
    if (_boingD_px < 0.0) { // workaround JS's broken modulus operator
        _boingRemainder_px = 1.0 - (Math.abs(_boingD_px) % 1.0);
    } else {
        _boingRemainder_px = _boingD_px % 1.0;
    }
    _boingD_px = Math.floor(_boingD_px);

    // handle _boingA bounce and move
    _boingAvail_px = screen.availHeight - (window.screenY + window.outerHeight)
    if (_boingD_px >= _boingAvail_px) { // bounce
        _boingV = -_boingV*_boingC;
        _boingD_px = _boingAvail_px - (_boingD_px-_boingAvail_px)*_boingC;
        if (_boingBouncedLastStep) {
            window.clearInterval(_boingInterval);
            _boingInterval = null;
            return;
        }
        _boingBouncedLastStep = 1;
    } else {
        _boingBouncedLastStep = 0;
    }

    //dump("_boingStep " + _boingStep + ": _boingV=" + _boingV + ", _boingD_px=" + _boingD_px + " _boingRemainder_px=" + _boingRemainder_px + '\n');
    window.moveBy(0, _boingD_px);

    _boingStep += 1;
    _boingV += _boingA * _boingT; // accelerate
}
