/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

// Komodo Help
//
// Arguments:
//  - "gHelpArg_StartPage" in the opener namespace is used to indicate the
//    HTML page which should be loaded into help.
//  - "gHelpArg_Align" indicates if the help window should be "align"ed.
//
// NOTE:
//  The appCore and browser associated stuff was lifted from navigator.js|xul


//-------- global variables

const nsIWebNavigation = Components.interfaces.nsIWebNavigation;
const nsIContentViewerEdit = Components.interfaces.nsIContentViewerEdit;
var sysUtils = Components.classes["@activestate.com/koSysUtils;1"].
               getService(Components.interfaces.koISysUtils);
var indexer = Components.classes['@activestate.com/koIndexer;1'].getService()

/*var gURLBar = null;
var gNavigatorBundle;
var gBrandBundle;
var gNavigatorRegionBundle;
var gBrandRegionBundle;*/

var pref = Components.classes["@mozilla.org/preferences;1"]
                     .getService(Components.interfaces.nsIPref);

var koPrefSvc = Components.classes['@activestate.com/koPrefService;1'].getService();
var globalPrefs = koPrefSvc.prefs;

var appCore = null;

//cached elements
var gBrowser = null;

// focused frame URL
//var gFocusedURL = null;

var infoSvc = Components.classes["@activestate.com/koInfoService;1"].
                getService(Components.interfaces.koIInfoService);
var koDirs = Components.classes["@activestate.com/koDirs;1"].
             getService(Components.interfaces.koIDirs);
var docdir = koDirs.mozBinDir +'/chrome/komodo/content/doc';
var komodoVersion = infoSvc.version;
var startPage;

// Komodo Help Categories (mapping of category name -> RDF datasource)
var gCategories = new Object();
gCategories["Komodo"] = "chrome://komodo/content/doc/komodo-toc.rdf";
gCategories["Web Services"] = "";  // determined at runtime
var _gDataSourcesCache = new Object();


//--------- functionality lifted from navigator.js|xul

function UpdateBackForwardButtons()
{
  var backBroadcaster = document.getElementById("canGoBack");
  var forwardBroadcaster = document.getElementById("canGoForward");
  var webNavigation = getWebNavigation();

  // Avoid setting attributes on broadcasters if the value hasn't changed!
  // Remember, guys, setting attributes on elements is expensive!  They
  // get inherited into anonymous content, broadcast to other widgets, etc.!
  // Don't do it if the value hasn't changed! - dwh

  var backDisabled = (backBroadcaster.getAttribute("disabled") == "true");
  var forwardDisabled = (forwardBroadcaster.getAttribute("disabled") == "true");

  if (backDisabled == webNavigation.canGoBack)
    backBroadcaster.setAttribute("disabled", !backDisabled);
  
  if (forwardDisabled == webNavigation.canGoForward)
    forwardBroadcaster.setAttribute("disabled", !forwardDisabled);
}

function getBrowser()
{
  return document.getElementById("help-content");
}

function getWebNavigation()
{
  try {
    return getBrowser().webNavigation;
  } catch (e) {
    return null;
  }
}

function getMarkupDocumentViewer()
{
  return getBrowser().markupDocumentViewer;
}

function BrowserBack()
{
  try {
    getWebNavigation().goBack();
  }
  catch(ex) {
  }
  UpdateBackForwardButtons();
}

function BrowserForward()
{
  try {
    getWebNavigation().goForward();
  }
  catch(ex) {
  }
  UpdateBackForwardButtons();
}

function BrowserStop()
{
  try {
    getWebNavigation().stop();
  }
  catch(ex) {
  }
}

function BrowserReallyReload(event)
{
  // Default is no flags.
  var reloadFlags = nsIWebNavigation.LOAD_FLAGS_NONE;
  // See if the event was a shift-click.
  if (event && event.shiftKey) {
    // Shift key means bypass proxy and cache.
    reloadFlags = nsIWebNavigation.LOAD_FLAGS_BYPASS_PROXY | nsIWebNavigation.LOAD_FLAGS_BYPASS_CACHE;
  }
  getWebNavigation().reload(reloadFlags);
}

function BrowserPrint()
{
  // using _content.print() until printing becomes scriptable on docShell
  try {
    _content.print();
  }
  catch(ex) {
    // Pressing cancel is expressed as an NS_ERROR_FAILURE return value,
    // causing an exception to be thrown which we catch here.
    // Unfortunately this will also consume helpful failures, so add a
    // dump(e); // if you need to debug
  }
}

function loadEventHandlers(event)
{
  // Filter out events that are not about the document load we are interested in
  if (event.target == _content.document) {
    //UpdateBookmarksLastVisitedDate(event);
    //UpdateInternetSearchResults(event);
    checkForDirectoryListing();
    //getContentAreaFrameCount();
    //postURLToNativeWidget();
  }
}

function checkForDirectoryListing()
{
  if ( "HTTPIndex" in _content &&
       _content.HTTPIndex instanceof Components.interfaces.nsIHTTPIndex ) {
    _content.defaultCharacterset = getMarkupDocumentViewer().defaultCharacterSet;
    _content.parentWindow = window;
  }
}

function UpdateBookmarksLastVisitedDate(event)
{
  // XXX This somehow causes a big leak, back to the old way
  //     till we figure out why. See bug 61886.
  // var url = getWebNavigation().currentURI.spec;
  var url = _content.location.href;
  if (url) {
    // if the URL is bookmarked, update its "Last Visited" date
    var bmks = Components.classes["@mozilla.org/browser/bookmarks-service;1"]
                         .getService(Components.interfaces.nsIBookmarksService);

    bmks.UpdateBookmarkLastVisitedDate(url, _content.document.characterSet);
  }
}


function _UpdateSidebarButtons()
{
    var hideButtonWidget = document.getElementById("helpHideSidebarButton");
    var showButtonWidget = document.getElementById("helpShowSidebarButton");
    var splitterWidget = document.getElementById("helpsidebar-splitter");
    var state = splitterWidget.getAttribute("state");

    if (state == "collapsed") {
        hideButtonWidget.setAttribute("collapsed", true);
        showButtonWidget.setAttribute("collapsed", false);
    } else {
        hideButtonWidget.setAttribute("collapsed", false);
        showButtonWidget.setAttribute("collapsed", true);
    }
}


function _Help_OnLoad()  {
    // Check for existing window
    const nsIWindowMediator = Components.interfaces.nsIWindowMediator;
    const MEDIATOR_CONTRACTID =
       "@mozilla.org/rdf/datasource;1?name=window-mediator";
    var windowManager = Components.classes[MEDIATOR_CONTRACTID].getService();
    var windowManagerInterface = windowManager.QueryInterface( Components.interfaces.nsIWindowMediator);
    var newWindow = windowManager.getMostRecentWindow('komodo_help');
    var topWindow = windowManagerInterface.getMostRecentWindow( null ); // all windows

    // try to get a handle on an already open komodo workspace
    var enumerator = windowManagerInterface.getEnumerator('komodo_help');
    var firstWindow = windowManagerInterface.convertISupportsToDOMWindow ( enumerator.getNext() );
    var iWindow = firstWindow; // ;-)
    while ( iWindow && iWindow != newWindow && enumerator.hasMoreElements() )
        iWindow = windowManagerInterface.convertISupportsToDOMWindow ( enumerator.getNext() );

    var desiredWindow = firstWindow;
    if ( enumerator.hasMoreElements()  && desiredWindow != newWindow )  {
        desiredWindow = windowManagerInterface.convertISupportsToDOMWindow ( enumerator.getNext() );
    }

    if (desiredWindow != newWindow)  {
        // Help already open - desiredWindow
        logDebug("STARTUP: calling an existing Help");
        if (opener.gHelpArg_StartPage != undefined) {
            desiredWindow._Help_LoadContent(opener.gHelpArg_StartPage);
        } else {
            desiredWindow._Help_Home();
        }
        if (opener.gHelpArg_Align) {
            desiredWindow._Help_Align();
        }
        desiredWindow.focus();
        _Help_Init();
        window.close();
    }
    else {
        logDebug("STARTUP: creating a new Help Window");
        _Help_Init();
    }
}

function _Help_Init() {
    // Initialize the Komodo Help window

    var webNavigation;
    var contentViewerEdit;
    try {
      // Create the browser instance component.
      appCore = Components.classes["@mozilla.org/appshell/component/browser/instance;1"]
                          .createInstance(Components.interfaces.nsIBrowserInstance);
      if (!appCore)
        throw Components.results.NS_ERROR_FAILURE;

      webNavigation = getWebNavigation();
      if (!webNavigation)
        throw Components.results.NS_ERROR_FAILURE;
      contentViewerEdit = getContentViewerEdit();
      if (!contentViewerEdit)
        throw Components.results.NS_ERROR_FAILURE;
    } catch (e) {
      alert("Error creating Help browser instance");
      window.close(); // Give up.
      return;
    }

    // initialize observers and listeners
    window.XULBrowserWindow = new koBrowserStatusHandler();
    //window.buttonPrefListener = new nsButtonPrefListener();

    _content.appCore = appCore;

    // Initialize browser instance..
    appCore.setWebShellWindow(window);

    // Add a capturing event listener to the content area
    // (rjc note: not the entire window, otherwise we'll get sidebar pane loads too!)
    //  so we'll be notified when onloads complete.
    var contentArea = document.getElementById("helpappcontent");
    contentArea.addEventListener("load", loadEventHandlers, true);
    //contentArea.addEventListener("focus", contentAreaFrameFocus, true);

    // wire up session history before any possible progress notifications for back/forward button updating
    webNavigation.sessionHistory = Components.classes["@mozilla.org/browser/shistory;1"]
                                             .createInstance(Components.interfaces.nsISHistory);

    // hook up UI through progress listener
    var interfaceRequestor = getBrowser().docShell.QueryInterface(Components.interfaces.nsIInterfaceRequestor);
    var webProgress = interfaceRequestor.getInterface(Components.interfaces.nsIWebProgress);
    webProgress.addProgressListener(window.XULBrowserWindow);

    // XXXjag see bug 68662
    getBrowser().boxObject.setPropertyAsSupports("listenerkungfu", window.XULBrowserWindow);
    
    // tweak some preferences
    pref.SetBoolPref("signon.rememberSignons", false);

    _UpdateSidebarButtons();
        
    // move to right end of screen
    //XXX what is this code? I think this should be turfed.
    var width = document.documentElement.getAttribute("width");
    var height = document.documentElement.getAttribute("height");
    window.moveTo(screen.availWidth-width, (screen.availHeight-height)/2);

    _SetupDataSources();
        
    if (opener.gHelpArg_StartPage != undefined) {
        _Help_LoadContent(opener.gHelpArg_StartPage);
    } else {
        _Help_Home();
    }
    if (opener.gHelpArg_Align) {
        _Help_Align();
    }

    helpSetupOutliner(document.getElementById('help-outliner'), gHelpSearchResults);
}

function _Help_CreateIndices() {
    var cur_index_key = docdir + komodoVersion;
    var last_index_key = '';
    if (globalPrefs.hasStringPref('help_index_key')){
        last_index_key = globalPrefs.getStringPref('help_index_key');
    }
    if (cur_index_key == last_index_key) {
        indexer.load_index(); 
    } else {
        // We are upgrading to a new Komodo -- need to do a brand new index =(
        window.openDialog("chrome://komodo/content/help/helpIndex.xul",
                           'komodo_helpIndex',
                           'chrome,close,modal=yes,dialog=yes');        
        indexer.load_index();
        globalPrefs.setStringPref('help_index_key', cur_index_key);
    }
}    

function _getDataSource(filename) {
        var ds = Components
            .classes["@mozilla.org/rdf/datasource;1?name=xml-datasource"]
            .createInstance(Components.interfaces.nsIRDFDataSource);
            
        var remote =  ds.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
        try {
            // throws exception if already init'd
            remote.Init(filename)
        } catch (e) {
            ; // do nothing
        }
            
        if (!remote.loaded) {
            remote.Refresh(1);
        }
        return ds;
}


function _SetupDataSources()
{
    // Setup all the RDF datasource for the TOC tree.
    var tree = document.getElementById('help-toc-tree');
    var ds;
    var rdffile;

    // Add all datasources.
    for (category in gCategories) {
        if (category in _gDataSourcesCache) {
            ds = _gDataSourcesCache[category];
        } else {
            ds = null;
        }
        if (!ds) {
            if (category == "Web Services") {
                // If the user has any Web Services bookmarks then use the
                // RDF in their user data directory.
                var userdir = new String(koDirs.userDataDir);
                userdir = userdir.replace(/([\\\])/g, "/"); // for MSWindows
                rdffile = userdir+"/webservices/webservices-toc.rdf";
                var fileexists = 0;
                if (sysUtils.IsFile(rdffile)) {
                    rdffile = "file://"+rdffile;
                } else {
                    rdffile = "chrome://komodo/content/doc/webservices-toc.rdf";
                }
            } else {
                rdffile = gCategories[category];
            }
            if (rdffile != "") {
                try  {
                    ds = _getDataSource(rdffile);
                    _gDataSourcesCache[category] = ds;
                }
                catch (ex)  {
                    // unable to get the datasource
                    logError("Unable to get the help RDF datasource for category '"+category+"' using rdffile '"+rdffile+"'\n");
                }
            }
        }
        if (ds) {
            try {
                tree.database.AddDataSource(ds);
            } catch (ex) {
                // unable to add the data source
                logError("Unable to add the help RDF datasource: "+ds+"\n");
            }
        }
    }

    tree.builder.rebuild();
}


function _Help_CurrentTOCItem() {
    // Return the current Table of Contents XUL tree item for the
    // current help content url.
    var url = document.getElementById('help-content').getAttribute('src');
    var toc = document.getElementById('help-toc-tree');
}

function _Help_Locate() {
    // Locate the current URL in the current tree and open the tree to that
    // point. Return 1 if found, 0 if not.
    //XXX
}
function _Help_Previous() {
    //XXX
}
function _Help_Next() {
    //XXX
}

function _Help_Back() {
    BrowserBack();
}

function _Help_Forward() {
    BrowserForward();
}

function _Help_Stop() {
    BrowserStop();
}

// browser is the <browser> element
// rootSearchWindow is the window to constrain the search to (normally window._content)
// startSearchWindow is the frame to start searching (can be, and normally, rootSearchWindow)
function findInPage(browser, rootSearchWindow, startSearchWindow)
{
    var findInst = browser.webBrowserFind;
    // set up the find to search the focussedWindow, bounded by the content window.
    var findInFrames = findInst.QueryInterface(Components.interfaces.nsIWebBrowserFindInFrames);
    findInFrames.rootSearchFrame = rootSearchWindow;
    findInFrames.currentSearchFrame = startSearchWindow;
 
    // always search in frames for now. We could add a checkbox to the dialog for this.
    findInst.searchFrames = true;
   
    // is the dialog up already?
    if (window.findDialog)
        window.findDialog.focus();
    else
        window.findDialog = window.openDialog("chrome://komodo/content/help/helpFind.xul", "_blank", "chrome,resizable=no,dependent=yes", findInst);
}
 
function _Help_Find() {
    var focusedWindow = document.commandDispatcher.focusedWindow;
    if (!focusedWindow || focusedWindow == window)
        focusedWindow = window._content;

    findInPage(getBrowser(), window._content, focusedWindow)
}

function _Help_Refresh() {
    // refresh the current help content
    BrowserReallyReload(0);
//    var helpContent = document.getElementById('help-content');
//    var url = helpContent.getAttribute('src');
//    helpContent.setAttribute('src', url);
}

function _Help_Home() {
    // load "home" URL, this should probably be the ASPN home page
    //document.getElementById('help-content').setAttribute('src','chrome://komodo/content/doc/komodo/komodo-doc-top.html');
    _Help_LoadContent('chrome://komodo/content/doc/komodo/komodo-doc-top.html'); //XXX
}


function _Help_ASPN()
{
    // Load ASPN. If have Komodo license meta-data then use that to login
    // directly. If not (e.g. this is a Komodo trial) then we *could* look
    // for a "Subscription" license feature in the user's ActiveState
    // license but bailing on that.
    if (infoSvc.licensingType == "license") {
        return _Help_LoadContent(
            "http://aspn.ActiveState.com/ASPN/acl_users/komodo_validate?" +
            "serial="+infoSvc.licenseSerialNo+"&token="+infoSvc.licenseToken);
    } else {
        return _Help_LoadContent("http://aspn.ActiveState.com/ASPN");
    }
}


function _Help_Print() {
    BrowserPrint();
}

function _Help_LoadContent(url) {
    if (url && url != "about:blank") {
        loadURI(url);
    }
    _content.focus();
}

function _Help_LoadKomodoDoc(url) {
    //var helpContent = document.getElementById('help-content');
    var relUrl = new String(url);
    if (relUrl.search(/file:|http:/) < 0) {
        relUrl = 'chrome://komodo/content/doc/' + url;
    }
    var curUrl = _content.location.href;
    if (curUrl != relUrl)  {
        _Help_LoadContent(relUrl)
    }
}


function Help_Query(query) {
    // Query the Komodo Help service.
    //   "query" is a koIHelpQuery object
    //   returns a koIHelpResult object
    
    // get the Komodo Help service
    //XXX how to make a servicve rather than a regular component?
    var help = Components.classes["Komodo.Help"].createInstance();
    var ihelp = help.QueryInterface(Components.interfaces.koIHelp);

    return ihelp.query(query);
}

function Help_QueryBrief(query) {
    // Query the Komodo Help service.
    //   "query" is a koIHelpQuery object
    //   returns a brief help string

    var res = Help_Query(query);
    var brief = res.brief;
    return brief;
}


function _ShowSidebar()
{
    // Open the sidebar.
    var splitterWidget = document.getElementById("helpsidebar-splitter");
    splitterWidget.setAttribute("state", null);

    // Get the width of the sidebar box.
    var sidebarWidget = document.getElementById("helpsidebar-box");
    var sidebarWidth = sidebarWidget.boxObject.width;

    // "Remove" that section of the help window.
    window.outerWidth += sidebarWidth;
    window.screenX -= sidebarWidth;

    _UpdateSidebarButtons();
}


function _HideSidebar()
{
    // Get the width of the sidebar box.
    var sidebarWidget = document.getElementById("helpsidebar-box");
    var sidebarWidth = sidebarWidget.boxObject.width;

    // Close the sidebar.
    var splitterWidget = document.getElementById("helpsidebar-splitter");
    splitterWidget.setAttribute("state", "collapsed");

    // "Insert" that section of the help window.
    window.outerWidth -= sidebarWidth;
    window.screenX += sidebarWidth;

    _UpdateSidebarButtons();
}


function _Help_ToggleSidebar()
{
    // This is more than just collapsing or opening the sidebar splitter.
    // The result of toggling the sidebar should be to change the dimensions of
    // the Help window rather than the dimensions of the right pane (the
    // content)
    //XXX Apparently the "state" is either 'closed', 'collapsed', 'open', or
    //    ''. See http://swatter.activestate.com/lxr/source//mozilla/mailnews/base/resources/content/widgetglue.js#212
    //    for an example how to use it.
    var state = document.getElementById("helpsidebar-splitter").getAttribute("state");

    if (state == "collapsed") {
        _ShowSidebar();
    } else {
        _HideSidebar();
    }
}


function _Help_Align()
{
    // Align the Komodo window and the help window beside each other,
    // fullscreen, Komodo on the left/Help on the right, 67%/33% split.
    _HideSidebar();
    
    // Playing with screen width and height variables is risky (on Linux
    // esp. seeing as I don't believe that the WM decorations are
    // included in the Mozilla window sizing). So, hardcoded padding is
    // added on each side to attempt to improve the user experience.
    var leftPadding = 0;
    var rightPadding = 0;
    var topPadding = 0;
    var bottomPadding = 0;
    if (!infoSvc.isWindows) {
        leftPadding = 25;
        rightPadding = 25;
        topPadding = 50;
        bottomPadding = 100;
    }
    
    opener.moveTo(0+leftPadding, 0+topPadding);
    opener.outerWidth = (screen.availWidth * 67 / 100) - leftPadding;
    opener.outerHeight = screen.availHeight - topPadding - bottomPadding;
    
    window.moveTo(screen.availWidth * 67 / 100, 0+topPadding);
    window.outerWidth = (screen.availWidth * 33 / 100) - rightPadding;
    window.outerHeight = screen.availHeight - topPadding - bottomPadding;
}


function _Help_Shutdown()
{
  var browser = getBrowser();

  browser.boxObject.removeProperty("listenerkungfu");

  try {
    var interfaceRequestor = browser.docShell.QueryInterface(Components.interfaces.nsIInterfaceRequestor);
    var webProgress = interfaceRequestor.getInterface(Components.interfaces.nsIWebProgress);
    webProgress.removeProgressListener(window.XULBrowserWindow);
  } catch (ex) {
  }

  if (window.XULBrowserWindow)  {
      window.XULBrowserWindow.destroy();
      window.XULBrowserWindow = null;
  }

  try {
    // If bookmarks are dirty, flush 'em to disk
    var bmks = Components.classes["@mozilla.org/browser/bookmarks-service;1"]
                         .getService(Components.interfaces.nsIRDFRemoteDataSource);
    bmks.Flush();
  } catch (ex) {
  }

  try {
    // give history a chance at flushing to disk also
    var history = Components.classes["@mozilla.org/browser/global-history;1"]
                            .getService(Components.interfaces.nsIRDFRemoteDataSource);
    history.Flush();
  } catch (ex) {
  }

  // unregister us as a pref listener
  //pref.removeObserver(window.buttonPrefListener.domain,
                      //window.buttonPrefListener);

  // Close the app core.
  if (appCore)
    appCore.close();

  opener.focus();
}

function loadURI(uri)
{
  if (uri) {
    try {
      getWebNavigation().loadURI(uri, nsIWebNavigation.LOAD_FLAGS_NONE);
    } catch (e) {
      logError(e);
    }
  }
}

function openInBrowser(url) {
  // from koGlobalOverlay.js
  OpenUrlInDefaultBrowser(url);
  return true;
}

function getContentViewerEdit()
{
  try {
    return getBrowser().contentViewerEdit;
  } catch (e) {
    return null;
  }
}


// The view object for the match variables outliner widget.
var gHelpSearchResults = ({
    // nsIOutlinerView
    rowCount : 0,
    getRowProperties : function(i, prop) {
        // prop.AppendElement(this.smile);
    },
    getColumnProperties : function(index, prop) {},
    getCellProperties : function(index, prop) {},
    isContainer : function(index) {return false;},
    outliner : null,
    setOutliner : function(out) { this.outliner = out; },
    getCellText : function(i, col) {
         return this.titles[i];
    },
    performAction : function(action) {
    },
    cycleHeader : function(index) {
    },
    selectionChanged : function() {
        
    },
    isSeparator: function() { return false; },
    
    // Private stuff
    matchData : null,
    groupCount : 0,
    setRowCount : function(rowCount) {
        this.rowCount = rowCount;
        this.outliner.rowCountChanged(0, this.rowCount);
        this.outliner.invalidate();
    }
});
 
function helpSetupOutliner(outliner, v) {
    outliner.boxObject.QueryInterface(Components.interfaces.nsIOutlinerBoxObject).view = v;
}

function gHelpDoFind()
{
    var textbox = document.getElementById('htab-pattern');
    MRU_updateFromWidget(textbox);
    var searchstring = textbox.value;
    var i;
    indexer.load_index();
    var results = indexer.find(searchstring);
    if (results == '') {
        alertDialog("Search returned no results.");
        gHelpSearchResults.uris = Array(0);
        gHelpSearchResults.titles= Array(0);
        gHelpSearchResults.setRowCount(0);
        return;
    }
    var lines = results.split('\n');
    var uris = Array(lines.length);
    var titles = Array(lines.length);
    for (i = 0; i < lines.length; i++) {
        var parts = lines[i].split('\t');
        var fname = parts[0];
        var title = parts[1];
        if (fname.substr(0, docdir.length) == docdir) {
            // do magic w/ the stuff in the online docs.  
            uris[i] = 'chrome://komodo/content/doc' + fname.substr(docdir.length, fname.length-docdir.length);
        } else {
            uris[i] = fname;
        }
        titles[i] = title;
    }
    gHelpSearchResults.uris = uris;
    gHelpSearchResults.titles= titles;
    gHelpSearchResults.setRowCount(lines.length);
}    

function CheckDefaultKeys(event, onOK) {
    if (event.keyCode == 13) {
        // fire given function for "Enter" key
        onOK();
        event.preventDefault();
    }
}

function gHelpFindSelectionChanged()
{
  if (gHelpSearchResults.uris[gHelpSearchResults.selection.currentIndex]) {
    loadURI(gHelpSearchResults.uris[gHelpSearchResults.selection.currentIndex]);
  }
}    

function koHelpViewSource()
{
  var focusedWindow = document.commandDispatcher.focusedWindow;
  if (focusedWindow == window)
    focusedWindow = _content;

  //dump("focusedWindow = " + focusedWindow + "\n");
  if (focusedWindow)
    var docCharset = "charset=" + focusedWindow.document.characterSet;

  koHelpViewSourceOfURL(_content.location, docCharset);
}

function koHelpViewSourceOfURL(url, charset)
{
  // try to open a view-source window while inheriting the charset (if any)
  openDialog("chrome://navigator/content/viewSource.xul",
             "_blank",
             "scrollbars,resizable,chrome,dialog=no",
             url, charset);
}

function doHelpEnter ()  {
    var element = document.commandDispatcher.focusedElement;
    var elementId = element.getAttribute("id");
    if (element && element.nodeName == "button" && elementId == "help-searchBtn")
        gHelpDoFind();
}
