// koLogging.js

// JS interface to the logging system.

/* This should be available in all JS code in Komodo.

Usage:

    log = gLoggingMgr.getLogger(<logger_name>);
    log.debug(<message>);
    log.info(<message>);
    etc.

To turn on or off a logger, use:

    log.setLevel(gKO_LOG_DEBUG)

Loggers that we are using:

    "ko.performance": Where major bits of performance information go.
    "ko.performance.startup": Perf logging w.r.t. startup perf.
    
    "ko.process": information re: the process module
*/

// ONLY ONE GLOBAL VARIABLE:
var _gLoggingMgr;

var LOG_DEBUG = 10;
var LOG_INFO = 20;
var LOG_WARN = 30;
var LOG_ERROR = 40;
var LOG_CRITICAL = 50;


// Logger wrapper objects get notified of the log levels associated with
// particular loggers.  They only make the calls to the logging system if
// those logging calls will actually be processed.

// See the 'set_logger_level' notification information

function loggerWrapper(logger) {
    this.level = 0;
    this._logger = logger;
}

loggerWrapper.prototype.constructor = loggerWrapper;

loggerWrapper.prototype.setLevel = function(level) {
    //dump("Calling setLevel" + level + '\n');
    this._logger.setLevel(level);
}

loggerWrapper.prototype.getEffectiveLevel = function() {
    return this._logger.getEffectiveLevel();
}

loggerWrapper.prototype.debug= function(message) {
    if (this.level == 0) {
        this.level = this._logger.getEffectiveLevel();
    }
    if (this.level <= LOG_DEBUG) {
        this._logger.debug(message);
    }
}

loggerWrapper.prototype.info = function(message) {
    if (this.level == 0) {
        this.level = this._logger.getEffectiveLevel();
    }
    if (this.level <= LOG_INFO) {
        this._logger.info(message);
    }
}

loggerWrapper.prototype.warn = function(message) {
    if (this.level == 0) {
        this.level = this._logger.getEffectiveLevel();
    }
    if (this.level <= LOG_WARN) {
        this._logger.warn(message);
    }
}

loggerWrapper.prototype.error = function(message) {
    if (this.level == 0) {
        this.level = this._logger.getEffectiveLevel();
    }
    if (this.level <= LOG_ERROR) {
        this._logger.error(message);
    }
}

loggerWrapper.prototype.critical = function(message) {
    if (this.level == 0) {
        this.level = this._logger.getEffectiveLevel();
    }
    if (this.level <= LOG_CRITICAL) {
        this._logger.critical(message);
    }
}

function loggingMgr() {
    this.loggerMap = {}
    this.loggingSvc = Components.classes["@activestate.com/koLoggingService;1"].
                    getService(Components.interfaces.koILoggingService);
}

loggingMgr.prototype.constructor = loggingMgr;

loggingMgr.prototype.getLogger = function(logger_name) {
    if (logger_name in this.loggerMap) {
        return this.loggerMap[logger_name];
    }
    var logger = this.loggingSvc.getLogger(logger_name);
    //this.loggerMap[logger_name] = logger;
    try {
        this.loggerMap[logger_name] = new loggerWrapper(logger);
    } catch (e) {
        logError(e);
    }
    return this.loggerMap[logger_name];
}

loggingMgr.prototype.Observe = function(subject, topic, data) {
    var logger_name;
    logger_name = data.slice(0, data.indexOf(':'));
    var otherlogger, i;
    // We need to invalidate the logging levels of "derived" loggers.
    for (otherlogger  in this.loggerMap) {
        if (logger_name == '' || logger_name == otherlogger || otherlogger.indexOf(logger_name+'.') == 0) {
            this.loggerMap[otherlogger].level = 0;
        }
    }
    //var log_level;
    //log_level = Number(data.slice(data.indexOf(':')+1));
    //this.loggerMap[logger_name].log_level = log_level;
}

function logging_onload() {
    window._gLoggingMgr = new loggingMgr();
    var observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                         getService(Components.interfaces.nsIObserverService);
    observerSvc.AddObserver(window._gLoggingMgr, 'set_logger_level');
}

// Use this function to always get the logging manager
// (which may not be a global in your namespace)
function getLoggingMgr() {
    // Factory function which returns the "superglobal" loggingMgr instance
    var namespace = window;
    while ('opener' in namespace && namespace.opener) {
        namespace = namespace.opener;
    }
    return namespace._gLoggingMgr;
}



