/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

/*
 * The KoInitService initialisation must occur first thing in
 * order to do the proper initialisation that may be required
 * in any later module __init__'s.
 *
 * For now, this sets the python encoding to Latin-1 on Windows
 * in order to get around Unicode problems in os.*.
 */
var _gInitSvc = Components.classes["@activestate.com/koInitService;1"].
                getService(Components.interfaces.koIInitService);
try {
    _gInitSvc.setPlatformErrorMode();
    _gInitSvc.setEncoding();
    _gInitSvc.initProcessUtils();
    _gInitSvc.upgradeUserSettings();
} catch(e) {
    logError("STARTUP: Komodo Initialization Service failed.");
    _gInitSvc = null;
}


//---- globals

var progressbarSvc = null;

var globalPrefService = Components.classes["@activestate.com/koPrefService;1"].
                        getService(Components.interfaces.koIPrefService);
var globalPrefs = globalPrefService.prefs;

var infoSvc = Components.classes["@activestate.com/koInfoService;1"].
              getService(Components.interfaces.koIInfoService);
var gObserverSvc = null;

// Used by the Run Command system to maintain a list of spawned processes for
// proper Komodo shutdown (see run_functions.js).
var gRunProcesses = [];

var _gWMSvc = Components.classes["@mozilla.org/rdf/datasource;1?name=window-mediator"]
                        .getService(Components.interfaces.nsIWindowMediator);

var _gKomodoObserver = null;

// Flags used to defer (re)building of the MRU menus until necessary.
var _gNeedToUpdateFileMRUMenu = false;
var _gNeedToUpdateProjectMRUMenu = false;

var _gCommandmentSvc = Components.classes["@activestate.com/koCommandmentService;1"]
                                 .getService(Components.interfaces.koICommandmentService);

// Set to true in onload handler if aborting because don't have valid
// license.
var _gAbortBecauseInvalidLicense = false;


//---- internal support routines

function _updateMRUMenu(prefName)
{
    // Update a MRU menu popup under the file menu.
    //    "prefName" indicate which MRU menu to update.
    var popupId, menuitemIdBase, cmdBase;
    if (prefName == "mruProjectList") {
        popupId = "popup_mruProjects";
        menuitemIdBase = "menu_mruProject";
        cmdBase = "cmd_mruProject";
    } else if (prefName == "mruFileList") {
        popupId = "popup_mruFiles";
        menuitemIdBase = "menu_mruFile";
        cmdBase = "cmd_mruFile";
    } else {
        throw("Unexpected MRU menu to update: prefName='"+prefName+"'");
    }

    var menupopup = document.getElementById(popupId);
    var mruList = null;
    var menuitem;
    if (gPrefSvc.prefs.hasPref(prefName)) {
        mruList = gPrefSvc.prefs.getPref(prefName);
    }

    if (mruList && mruList.length) {
        // Wipe out existing menuitems:
        while (menupopup.firstChild) {
            menupopup.removeChild(menupopup.firstChild);
        }

        // Add a menuitem like the following for each entry in the MRU:
        //    <menuitem class="menuitem_mru"
        //              oncommand="koGoDoCommand('cmd_mruProject0');"
        //              label="URL_DISPLAY_NAME"/>
        var length = mruList.length;
        for (var i = 0; i < length; i++) {
            var url = mruList.getStringPref(i);
            menuitem = document.createElement("menuitem");
            // Mozilla does not handle duplicate accesskeys, so only putting
            // them on first 10.
            if ((i+1) <= 9) {
                menuitem.setAttribute("accesskey", "" + (i+1));
            } else if ((i+1) == 10) {
                menuitem.setAttribute("accesskey", "0");
            }
            menuitem.setAttribute("label", (i+1)+" "+displayNameFromURL(url));
            menuitem.setAttribute("class", "menuitem_mru");
            // XXX:HACK: For whatever reason, the "observes" attribute is
            // ignored when the menu item is inside  popup, so we call
            // koGoDoCommand directly. THIS IS NOT A GOOD THING!
            menuitem.setAttribute("oncommand",
                                  "koGoDoCommand('" + cmdBase + i + "')");
            menupopup.appendChild(menuitem);
        }
    }
    
    // MRU is empty or does not exist
    else {
        // Wipe out existing entries and put in an "Empty" one like this:
        //    <menuitem label="Empty" disabled="true"/>
        while (menupopup.firstChild) {
            menupopup.removeChild(menupopup.firstChild);
        }
        menuitem = document.createElement("menuitem");
        menuitem.setAttribute("label", "Empty");
        menuitem.setAttribute("disabled", true);
        menupopup.appendChild(menuitem);
    }
}



//---- public methods

/* komodo chrome entry point
 *
 *  - If there is no Komodo currently running, then launch the
 *      the Komodo splash screen and Komodo itself.
 *  - If there *is* a Komodo currently running, then get a handle
 *      on it and tell it to open the given files.
 *  - The latter now accomodates other open Komodo windows (Listen for .., Prefs..)
 *      If these are not found and focused Komodo will hang.
 */
function KomodoOnLoad() {
try {
    var errmsg = null;
    logging_onload();
    perfTimer("KomodoOnLoad", "Logging_initialized", 1);
    try {
        if (progressbarSvc == null) {
            progressbarSvc = Components.classes['@activestate.com/koProgressBarService;1'].
                             getService(Components.interfaces.koIProgressBarService);
            progressbarSvc.tick('Loading Main Window', 1);
        }
    } catch(ex) {
        errmsg = "There was an error setting up the startup progressbar."
                 + " Komodo is shutting down. Exception: " + ex;
        logError(errmsg);
        alertDialog(errmsg);
        window.close();
        return;
    }
    perfTimer("KomodoOnLoad", "ProgressBar Ticked");
    try {
        // Abort if do not have a valid license.
        // XXX Should only do this check for new Komodo's (i.e. not if this is
        //     just going to hand off to an already running Komodo).
        if (! infoSvc.licenseValid) {
            openWindowUniqueInstance("chrome://komodo/content/invalidlicense.xul",
                                     "komodo_invalid_license",
                                     "chrome,dialog,modal");
            _gAbortBecauseInvalidLicense = true;
            progressbarSvc.done();
            window.close();
            return;
        }
    } catch(ex) {
        errmsg = "There was an error checking and handling your Komodo license."
                 + " Komodo is shutting down. Exception: " + ex;
        logError(errmsg);
        alertDialog(errmsg);
        window.close();
        return;
    }
    perfTimer("KomodoOnLoad", "License Checked");
    try {  
        //logDebug("STARTUP: starting a new Komodo");
        // XXX What is this stuff. Is this for the old, obselete, and no
        //     longer used komodo-service.js command line arg stuff?
        var console = Components.classes["@mozilla.org/consoleservice;1"].
            getService(Components.interfaces.nsIConsoleService);
        var listener = Components.classes["@activestate.com/koConsoleListener;1"].
            createInstance(Components.interfaces.nsIConsoleListener);
        console.registerListener(listener);
        perfTimer("KomodoOnLoad", "Registered Console Listener");

        gObserverSvc = Components.classes["@mozilla.org/observer-service;1"].
                       getService(Components.interfaces.nsIObserverService);
        _gKomodoObserver = new _KomodoObserver();



        gObserverSvc.AddObserver(_gKomodoObserver, "mru_changed");
        gObserverSvc.AddObserver(_gKomodoObserver, "open_file");
        gObserverSvc.AddObserver(_gKomodoObserver, "quit");

        // Show 'Test' menu if developers build
        var testMenu = window.document.getElementById("testMenu");
        if (infoSvc.installerType == "") {
            addStyle(testMenu, 'display', 'block');
        } else {
            addStyle(testMenu, 'display', 'none');
        }

        // hard-code view MRU to 300 now, just for the sake of testing until we get this hooked with the UI.
        var viewMRU = globalPrefService.getPrefs("viewStateMRU");
        viewMRU.max_length = 300
        //dump("ViewState MRU currently has " + viewMRU.length + " items\n");

        MRU_initialize();
        perfTimer("KomodoOnLoad", "Initialized MRU");
        GettingStarted_OnLoad();
        perfTimer("KomodoOnLoad", "Initialized GettingStarted");
        progressbarSvc.tick('Loading Project Manager', 1);
        // From projectviewer.js
        projectManagerOnLoad();
        perfTimer("KomodoOnLoad", "Initialized Project Manager");
        // in runOutputWindow.js
        progressbarSvc.tick('Loading Command Output Window', 1);
        RunOutput_Init();
        perfTimer("KomodoOnLoad", "Initialized RunOutput");

        KomboOnLoad();
        perfTimer("KomodoOnLoad", "Initialized Kombo");
        
        _updateMRUMenu("mruProjectList");
        _updateMRUMenu("mruFileList");

        progressbarSvc.tick('Starting Debugger Engines', 10);
        get_debugger_interface(); // start debug engines!
        perfTimer("KomodoOnLoad", "Initialized Debugger");

        progressbarSvc.tick('Loading Find Results Tab', 1);
        FindResultsTab_OnLoad();
        perfTimer("KomodoOnLoad", "Initialized FindResultsTab");
    } catch(ex) {
        errmsg = "There was an error initializing Komodo's primary services."
                 + " Komodo is shutting down. Exception: " + ex;
        logError(errmsg);
        alertDialog(errmsg);
        window.close();
        return;
    }

    try {
        var templateMgr = Components.classes["@activestate.com/koTemplateManager;1"].getService();
        templateMgr.init();
    } catch(ex) {
        // This sucks but it's not enough reason to stop Komodo
        logError("Exception initializing template manager: "+ex)
    }
    perfTimer("KomodoOnLoad", "Initialized TemplateMgr");

    try {
        progressbarSvc.tick('Loading Keybinding Manager', 10);
        keybindingManagerOnLoad();
        perfTimer("KomodoOnLoad", "Initialized KeybindingMgr");
        progressbarSvc.tick('Loading Toolbox', 10);
        toolboxManagerOnLoad();
        perfTimer("KomodoOnLoad", "Initialized ToolboxMgr");

        _gCommandmentSvc.initialize();
        perfTimer("KomodoOnLoad", "Initialized CommandmentsSvc");

        _gInitSvc.finishInitialization();
        perfTimer("KomodoOnLoad", "Finished Initialization");

        // Possibly show the "Tip of the Day" dialog.
        if (globalPrefs.getBooleanPref("tipShowAtStartup")) {
            func_helpTipOfTheDay();
        }










    
        StatusBar_AddMessage("Ready", "", 0, 0);  // default status bar msg
        progressbarSvc.done();
    } catch(ex) {
        errmsg = "There was an error in the last stage of Komodo's initialization."
                 + " Komodo is shutting down. Exception: " + ex;
        logError(errmsg);
        alertDialog(errmsg);
        window.close();
        return;
    }
    var statusLog = getLoggingMgr().getLogger('status');
    statusLog.info("KomodoOnLoad_Done " + new Date().getTime());
    perfTimer("KomodoOnLoad", "done");
} catch (e) {
    logError(e);
    alertDialog(e);
}
}


/**
 * KomodoCanClose
 *
 * this function checks with the projectmanager, and the viewmanagers
 * to see if it's ok to shutdown
 */
function KomodoCanClose()
{
    // Check for other OPEN windows and close if there
    // This is expandable - just add your windowtype to the array
    var openWindow;
    // openedWindowTypes defined in koGlobalOverlay.js, with openWindowUniqueInstance
    for (var type in openedWindowTypes)
    {
        do {
            openWindow = _gWMSvc.getMostRecentWindow(type);
            if (openWindow) {
                openWindow.close();
            }
        } while (openWindow);
    }

    if (!Run_CanClose()) {
        return false;
    }

    if (!KomboCanClose()) {
        return false;
    }  

    if (!projectManagerCanUnload()) return false;
    if (!toolboxManagerCanUnload()) return false;

    return true;
}


function KomodoDoUnload()
{
    // If we aborted then do not have to unload/finalize things that were
    // not loaded/initialized.
    if (_gAbortBecauseInvalidLicense) return;
    
    _gCommandmentSvc.finalize();
    GettingStarted_OnUnload();







    KomboOnUnload();
    projectManagerOnUnload();
    toolboxManagerOnUnload();
    MRU_finalize();

    fileAssociationPrefObserver.destructor(); // free observer in commdlg/file_window_functions.js

    RunOutput_Fini();




    gObserverSvc.RemoveObserver(_gKomodoObserver, "mru_changed");
    gObserverSvc.RemoveObserver(_gKomodoObserver, "open_file");
    _gKomodoObserver = null;

    //final shutdown of various components, mainly debugger stuff.

    var globalPreferencesService =
      Components.classes["@activestate.com/koPrefService;1"].
    getService(Components.interfaces.koIPrefService);
    var globalPreferences = globalPreferencesService.prefs;

    // Now write out some global preferences.
    // The global preference service handles serializing the preferences
    // to disk at shutdown time.

    // Save Komodo window dimensions.
    // Workaround bug http://bugs.activestate.com/Komodo/show_bug.cgi?id=23180
    // - "screenX == screenY == -32000 seems to be a reliable signifier that
    //   Komodo is minimized (on Win2K at least).
    if (window.screenX == -32000 && window.screenY == -32000) {
        // skip saving dimensions
    } else {
        globalPreferences.setLongPref('screenX', window.screenX);
        globalPreferences.setLongPref('screenY', window.screenY);
        globalPreferences.setLongPref('outerHeight', window.outerHeight);
        globalPreferences.setLongPref('outerWidth', window.outerWidth);
    }

    // Save the view MRU (*sob* - this needs more thought -
    // really want to save all pref files known to the service at shutdown.
    globalPrefService.savePrefsState("viewStateMRU");

    // Shut down the global preferences service.
    globalPreferencesService.shutDown();
}

//XXX Mozilla's goQuitApplication() sends a "quit-application"
//XXX notification before tryToClose().  This notify causes nsPluginHostImpl
//XXX to unload all plugins, including Scintilla.  We call ProjectViewerCanClose
//XXX ourselves before passing control on to Moz.
function koGoQuitApplication () {
    if (!KomodoCanClose()) {
        return false;
    }
    return goQuitApplication();
}


function Komodo_SendCtrlKeyUpNotification()
{
    var observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                      getService(Components.interfaces.nsIObserverService);
    try {
        observerSvc.Notify(null, 'ctrl_keyup', null);
    } catch(e) {
        // Raises exception if no observers are registered. Just ignore that.
    }
}




















































































function doWatchLocalFile() {
    var fp = dlg_openWatchFile();
    if (fp == null) return;
    var file = new dlg_get_URL_andName(fp);
    openWindowMultipleInstance("chrome://komodo/content/tail/tail.xul",
                                            "komodo_tail",
                                            "chrome,resizable,scrollbars,dialog=no",
                                            file.url,
                                            window);
}

function doRxWindow() {
    openWindowUniqueInstance('chrome://komodo/content/rx/koRx.xul',
                                'komodo_rx',
                                'chrome,resizable,dialog=no,close=yes,dependent=no',
                                document.getElementById("scin").ISciMoz);
}


























// Creating an HTML file containing the keyboard shortcuts
function func_helpKeybindings() {
    
    var text = gKeybindingMgr.makeCurrentKeyBindingTable();
    var tmpFileSvc = Components.classes["@activestate.com/koTempFileFactory;1"].
        getService(Components.interfaces.koITempFileFactory);
    var outputTmpFile = tmpFileSvc.MakeTempFile(".html","w");

    outputTmpFile.puts(text);
    outputTmpFile.flush();
    outputTmpFile.close();
    var outputURL = outputTmpFile.file.URL;
    OpenUrlInDefaultBrowser(outputURL);
}


function Komodo_UpdateMRUMenuIfNecessary(mru)
{
    // (Re)build the identified MRU menu if necessary.
    //    "mru" is either "project" or "file", indicating which MRU menu
    //        to update.
    if (mru == "project" && _gNeedToUpdateProjectMRUMenu) {
        _updateMRUMenu("mruProjectList");
        _gNeedToUpdateProjectMRUMenu = false;
    } else if (mru == "file" && _gNeedToUpdateFileMRUMenu) {
        _updateMRUMenu("mruFileList");
        _gNeedToUpdateFileMRUMenu = false;
    }
}


//---- komodo.xul's observer
// This observer handles any notification pertinent to komodo.xul that are
// not handled by sepcific observers in other modules.
//
function _KomodoObserver() {}
_KomodoObserver.prototype.Observe = function(subject, topic, data)
{
    switch (topic) {
    case 'quit':
        window.setTimeout("koGoQuitApplication()", 0);
        break;
    case 'mru_changed':
        // Schedule update "File->Recent Files" and "File->Recent Projects"
        // menus.
        var prefName = data;
        if (prefName == "mruFileList") {
            _gNeedToUpdateFileMRUMenu = true;
        } else if (prefName == "mruProjectList") {
            _gNeedToUpdateProjectMRUMenu = true;
        }
        break;
    case 'open_file':
        var filePath = data;
        var pathSvc = Components.classes["@activestate.com/koOsPath;1"]
                      .getService(Components.interfaces.koIOsPath);
        var fileURL = LocalFileToURL(filePath);

        var isProjectFile;
        if (infoSvc.isWindows) {
            isProjectFile = filePath.slice(-4).toLowerCase() == '.kpf';
        } else {
            isProjectFile = filePath.slice(-4) == '.kpf';
        }
        var exists = pathSvc.exists(filePath);
        var answer;
        if (isProjectFile) {
            // Open Komodo project files as projects.
            if (exists) {
                doOpenProject(fileURL);
            } else {
                answer = yesNoDialog("'"+filePath+"' does not exist. " +
                    "Would you like to create it?", "Yes");
                if (answer == "Yes") {
                    doNewProject(fileURL);
                }
            }
        } else {
            if (exists) {
                doOpenPath(fileURL);
            } else {
                answer = yesNoDialog("'"+filePath+"' does not exist. " +
                    "Would you like to create it?", "Yes");
                if (answer == "Yes") {
                    // Don't currently have a convenient doNewFile(url)
                    // ViewManager method so create the file here and open it.
                    var sysUtils = Components.classes["@activestate.com/koSysUtils;1"]
                                   .createInstance(Components.interfaces.koISysUtils);
                    sysUtils.Touch(filePath);
                    doOpenPath(fileURL);
                }
            }
        }
        break;












    }
}


