/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

// start the extension
registeredExtensions[registeredExtensions.length] = new peFile();
var gSysUtilsSvc = Components.classes["@activestate.com/koSysUtils;1"].
              getService(Components.interfaces.koISysUtils);
var gLastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                    getService(Components.interfaces.koILastErrorService);

function peFile() {
    this.koLanguageRegistryService =
                Components.classes["@activestate.com/koLanguageRegistryService;1"].
                            getService(Components.interfaces.koILanguageRegistryService);
    this.OsPathSvc = Components.classes["@activestate.com/koOsPath;1"].
        getService(Components.interfaces.koIOsPath);
    this.observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                       getService(Components.interfaces.nsIObserverService);
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
peFile.prototype.constructor = peFile;

peFile.prototype.init = function() {
    // register our command handlers
    this.observerSvc.AddObserver(this, 'file_status');
    this.observerSvc.AddObserver(this, 'file_status_updated');

    extensionManager.setDatapoint('Date','mtime');
    extensionManager.setDatapoint('Size','size');
    extensionManager.setDatapoint('Status','status');
    extensionManager.setDatapoint('Rev','myRevision');
    extensionManager.setDatapoint('Depot Rev','depotRevision');
    extensionManager.setDatapoint('Action','action');
}

peFile.prototype.registerCommands = function() {
    extensionManager.registerCommand('cmd_dumpPartProperties', this);
    extensionManager.registerCommand('cmd_editFilePart', this);
    extensionManager.registerCommand('cmd_openInDir', this);
    extensionManager.registerCommand('cmd_makeDirectoryShortcutFromFile', this);
    extensionManager.registerCommand('cmd_makeDirectoryShortcut', this);
    extensionManager.registerCommand('cmd_makeDirectoryShortcut_Project', this);
    extensionManager.registerCommand("cmd_editDirectoryShortcut",this);
    extensionManager.registerCommand('cmd_convertToMacro', this);
    extensionManager.registerCommand('cmd_refreshStatus', this);
    extensionManager.registerCommand('cmd_editFileProperties', this);
    extensionManager.registerCommand('cmd_showUnsavedChanges', this);
    extensionManager.registerCommand('cmd_compareFiles', this);
    extensionManager.registerCommand('cmd_compareFileWith', this);
    extensionManager.registerCommand("cmd_exportItems",this);
}

peFile.prototype.registerEventHandlers = function() {
    extensionManager.addEventHandler(Components.interfaces.koIPart_file,'ondblclick',this);
    extensionManager.addEventHandler(Components.interfaces.koIDirectoryShortcut,'ondblclick',this);
    // we don't add a dblclick handler for Components.interfaces.koIPart_ProjectRef, since
    // it inherits from koIPart_file
    
    extensionManager.addEventHandler(Components.interfaces.koIPart_file,'onUpdatePart',this);
    extensionManager.addEventHandler(Components.interfaces.koIProject,'onUpdatePart',this);
    extensionManager.addEventHandler(Components.interfaces.koIPart_ProjectRef,'onUpdatePart',this);
    extensionManager.addEventHandler(Components.interfaces.koIDirectoryShortcut,'onUpdatePart',this);
}

peFile.prototype.registerMenus = function() {
    extensionManager.createMenuItem(Components.interfaces.koIPart_file,
                                    'Open File','cmd_editFilePart');
    extensionManager.createMenuItem(Components.interfaces.koIPart_file,
                                    'Make "Open..." Shortcut','cmd_makeDirectoryShortcutFromFile');
    extensionManager.createMenuItem(Components.interfaces.koIContainer,
                                    'Add "Open..." Shortcut...','cmd_makeDirectoryShortcut');
    extensionManager.createMenuItem(Components.interfaces.koIDirectoryShortcut,
                                    'Open...','cmd_openInDir');
    extensionManager.createMenuItem(Components.interfaces.koIDirectoryShortcut,
                                    'Properties...','cmd_editDirectoryShortcut');
    extensionManager.createMenuItem(Components.interfaces.koIPart_file,
                                    'Refresh Status','cmd_refreshStatus');
    extensionManager.createMenuItem(Components.interfaces.koIPart_file,
                                    'Save As Macro in Toolbox','cmd_convertToMacro');
    extensionManager.createMenuItem(Components.interfaces.koIPart_file, 
                                    'Properties', 'cmd_editFileProperties');
    extensionManager.createMenuItem(Components.interfaces.koIPart_file,
                                    'Rename','cmd_renamePart');
    extensionManager.createMenuItem(Components.interfaces.koIPart_file,
                                    'Show unsaved changes...','cmd_showUnsavedChanges');
    extensionManager.createMenuItem(Components.interfaces.koIPart_file,
                                    'Compare Files...','cmd_compareFiles');
    extensionManager.createMenuItem(Components.interfaces.koIPart_file,
                                    'Compare File With...','cmd_compareFileWith');
    extensionManager.createMenuItem(Components.interfaces.koIPart,
                                    'Export as Project File...','cmd_exportItems');

    // XXX debug info
    extensionManager.createMenuItem(Components.interfaces.koIPart,'Dump',
                                    'cmd_dumpPartProperties');
}

peFile.prototype.supportsCommand = function(command, item) {
    var items;
    switch (command) {
    case 'cmd_dumpPartProperties':
        // Only return true in developer builds
        return infoSvc.installerType == '';
    case 'cmd_makeDirectoryShortcut_Project':
        return gProjectManager.getCurrentProject() != null;
    case 'cmd_editFilePart':
    case 'cmd_makeDirectoryShortcut':
    case 'cmd_editDirectoryShortcut':
    case 'cmd_openInDir':
        return true;
    case 'cmd_makeDirectoryShortcutFromFile':
        items = gFocusedProjectView.getSelectedItems()
        return (items.length == 1 && items[0].type == 'file' && items[0].url.indexOf('file://') == 0);
    case 'cmd_convertToMacro':
        items = gFocusedProjectView.getSelectedItems()
        if (items.length != 1 || items[0].type != 'file') return false;
        var url = items[0].url;
        // gLanguageRegistry is initialized in peMacro.js
        var language = gLanguageRegistry.suggestLanguageForFile(url)
        return (language == 'JavaScript' || language == 'Perl' || language == 'Python'); 
    case 'cmd_refreshStatus':
        if (editorHasFocus()) {
            //dump("Scintilla has focus -- actions should be based on the current file\n");
            var view = viewManager.getCurrentView();
            if (!view) return false;
            item = view.item;
            if (! view.url) {
                // This is an unsaved file -- command doesn't apply.
                return false;
            }
            return true;
        } else {
            if (! projectHasFocus() && !toolboxHasFocus()) {
                //dump("Scintilla or project or toolbox -- none have focus -- returning early \n");
                return true;
            } else {
            // otherwise, look at the focused project view's selected parts.
                item = gFocusedProjectView.getSelectedItem();
                // item must be a file
                if (!item || !item.url) return false;
            }
            return true;
        }
        break;
    case 'cmd_editFileProperties':
        return true;
    case 'cmd_showUnsavedChanges':
        if (! editorHasFocus()) return false;
        view = viewManager.getCurrentView();
        if (! view) return false;
        return (view.isDirty() && view.url.indexOf('file://') == 0);
    case 'cmd_compareFiles':
        items = gFocusedProjectView.getSelectedItems()
        return (items.length == 2 && items[0].type == 'file' && items[1].type == 'file' &&
                items[0].url.indexOf('file://') == 0 &&
                items[1].url.indexOf('file://') == 0);
    case 'cmd_compareFileWith':
        items = gFocusedProjectView.getSelectedItems()
        return (items.length == 1 && items[0].type == 'file' && items[0].url.indexOf('file://') == 0);
    case 'cmd_exportItems':
        var items = gFocusedProjectView.getSelectedItems();
        // the following isn't great, but it deals with
        // http://bugs.activestate.com/Komodo/show_bug.cgi?id=23226
        // with minimal risk.
        if (items.length == 1 && items[0].type == 'project') return 0;
        return items.length > 0;
    default:
        break;
    }
    return false;
}

peFile.prototype.isCommandEnabled = peFile.prototype.supportsCommand;

peFile.prototype.doCommand = function(command) {
    var item, items = null;
    var i, fname, otherfile;
    var dirname = '';
    switch (command) {
    case 'cmd_editFilePart':
        // get the current selection, then open the file
        items = gFocusedProjectView.getSelectedItems();
        for (i = 0; i < items.length; i++) {
            if (items[i].type == 'file') viewManager.doFileOpen(items[i].url);
        }
        break;
    case 'cmd_makeDirectoryShortcutFromFile':
        var view, url;
        // find item, get url
        if (editorHasFocus()) {
            view = viewManager.getCurrentView();
            item = view.item;
            url = view.url;
        } else {
            item = gFocusedProjectView.getSelectedItem();
            if (! item) return;
            url = item.part.getStringAttribute('url');
        }
        var path = URLToLocalFile(url).path;
        dirname = this.OsPathSvc.dirname(path);
        // fall through, it's ok
    case 'cmd_makeDirectoryShortcut_Project':
        if (gProjectManager.getCurrentProject() == null) return;
        gProjectManager.viewMgr.focus();
    case 'cmd_makeDirectoryShortcut':
        if (!dirname) {
            dirname = BrowseForDir('');
            if (!dirname) return;
        }
        var dirshortcut = Components.classes["@activestate.com/koPart?type=DirectoryShortcut;1"]
            .createInstance(Components.interfaces.koIDirectoryShortcut);
        dirshortcut.value = dirname;
        if (dirname.length > 25) {
            dirname = dirname.substr(0, 10) + '...' + dirname.substr(dirname.length - 15, dirname.length)
        }
        dirshortcut.setStringAttribute('name', dirname);
        gFocusedProjectView.manager.addItem(dirshortcut);
        break;
    case 'cmd_refreshStatus':
        doRefreshStatus();
        break;
    case 'cmd_dumpPartProperties':
        item = gFocusedProjectView.getSelectedItem();
        if (item) item.dump();
        break;
    case 'cmd_convertToMacro':
        item = gFocusedProjectView.getSelectedItem();
        if (item) {
            makeMacroFromFileItem(item);
        }
        break;
    case 'cmd_editFileProperties':
        item = gFocusedProjectView.getSelectedItem();
        this._fileProperties(item);
        break;
    case 'cmd_openInDir':
        item = gFocusedProjectView.getSelectedItem();
        if (item) {
            OpenDirectoryShortcut(item.part)
        }
        break;
    case 'cmd_editDirectoryShortcut':
        this.editDirectoryShortcut(gFocusedProjectView.getSelectedItem());
        break;
    case 'cmd_showUnsavedChanges':
        // get the current selection, then open the file
        view = viewManager.getCurrentView();
        var changes = view.py_inst.getUnsavedChanges();
        openDiffWindow(changes, 'Unsaved changes for ' + displayNameFromURL(view.url));
        break;
    case 'cmd_compareFiles':
        items = gFocusedProjectView.getSelectedItems()
        fname = URLToLocalFile(items[0].url).path;
        otherfile = URLToLocalFile(items[1].url).path;
        this._showDiffs(fname, otherfile);
        return;
    case 'cmd_compareFileWith':
        items = gFocusedProjectView.getSelectedItems()
        fname = URLToLocalFile(items[0].url).path;
        if (isLinux) {
           filters = [['All Files', ['*']]];
        } else {
           filters = [['All Files', ['*.*']]];
        }
        otherfile = BrowseForFile(this.OsPathSvc.dirname(URLToLocalFile(items[0].url)), filters);
        if (otherfile) {
            this._showDiffs(fname, otherfile);
        }
        return;
    case "cmd_exportItems":
        var items = gFocusedProjectView.getSelectedItems();
        Toolbox_ExportItems(items);
        break;
    default:
        break;
    }
}

function openDiffWindow(text, title /* ="Diff Output" */)
{
    if (typeof(title) == "undefined") {
        title = "Diff Output";
    }
    var obj = new Object();
    obj.title = title;
    obj.text = text;
    obj.lexer = 'SCLEX_DIFF';
    obj.lexerName = 'Diff';
    obj.dimensionPrefPrefix = "sccDiffWin-";
    obj.processDiffs = true;
    openWindowUniqueInstance("chrome://komodo/content/editor/simplewindow.xul", 
                             "komodo_run", 
                             "chrome,dialog=no,resizable=yes,scrollbars,close=yes",
                             obj);
}

function _openDiffWindowForFiles(fname1, fname2) {
    window.setCursor("wait");
    try {
        var diff = null;
        try {
            diff = gSysUtilsSvc.diff_files(fname1, fname2);
        } catch (ex) {
            var msg = "Error calculating differences: " +
                      gLastErrorSvc.getLastErrorMessage();
            alertDialog(msg);
            return;
        }
        openDiffWindow(diff, "Comparison of '"+fname1+"' and '"+fname2+"'");
    } finally {
        window.setCursor('auto');
    }
}

peFile.prototype._showDiffs = function(fname1, fname2) {
    // difflib can be slow, so we do this in a timeout
    fname1 = fname1.replace("\\", "\\\\", 'g');
    fname2 = fname2.replace("\\", "\\\\", 'g');
    window.setTimeout("_openDiffWindowForFiles('"+fname1+"', '"+fname2+"');",
                      0);
}


peFile.prototype.ondblclick = function(item,event) {
    if (item.type == 'DirectoryShortcut') {
        OpenDirectoryShortcut(item.part);
    } else {
        doOpenPath(item.url);
    }
}

peFile.prototype.onUpdatePart = function(item,event)
{
    if (item.type == 'DirectoryShortcut') {
        item.renameable = true;
        item.iconstyle['DirectoryShortcut'] = 1;
        return;
    }

    var useprojecticon = false;
    try {
        useprojecticon = item.part.QueryInterface(Components.interfaces.koIProject) != null;
    } catch (e) {}    
    if (!useprojecticon) {
        try {
            useprojecticon = item.part.QueryInterface(Components.interfaces.koIPart_ProjectRef) != null;
        } catch (e) {}    
    }
    // event is null for onUpdatePart
    // get the file properties!
    if (useprojecticon) {
        item.iconstyle['project'] = 1;
        item.iconopenstyle['project'] = 1;
    } else {
        item.iconstyle['projectFile'] = 1;
    }
    item.properties['tooltiptext'] = displayNameFromURL(item.url);
    if (typeof(item.properties['filesize']) == 'undefined') {
        this.getDefaultFileProperties(item);
    }
    if (item.url) {
        var pKeys = {};
        var pValues = {};
        gFileStatusSvc.getFileStatus(item.url, pKeys, new Object(), pValues, new Object());
        for (var index = 0; index < pKeys.value.length; index++) {
            updateStatusItem(item, pKeys.value[index], pValues.value[index]);
        }
    }
}

peFile.prototype.getDefaultFileProperties = function(item)
{
    item.properties['size'] = 0;
    item.properties['read'] = true;
    item.properties['write'] = true;
    item.properties['ctime']  = "Not Available";
    item.properties['mtime']  = "Not Available";
    item.properties['atime']  = "Not Available";



    item.iconopenstyle['readwrite']=item.iconstyle['readwrite']=0;
    item.iconopenstyle['readonly']=item.iconstyle['readonly']=0;
}

// XXX duplicated from currentViewPrefs.js
function getDateTimeString(datetime) {
    // XXX not localization friendly
    var now = new Date();
    var months = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"];
    var mins = datetime.getMinutes();
    if (mins.toString().length == 1)  {
        mins = '0' + mins;
    }
    var hours = datetime.getHours();
    var fulltime;
    if (hours > 12)  {
        hours = hours-12;
        fulltime = hours.toString() + ':' + mins + " PM";
    }
    else  {
        fulltime = hours.toString() + ':' + mins + " AM";
    }
    var dtstring;
    if (datetime.getFullYear() == now.getFullYear() &&
        datetime.getMonth() == datetime.getMonth() &&
        now.getDate() == datetime.getDate()) {
            dtstring = fulltime;
    } else if (datetime.getFullYear() == now.getFullYear()) {
        dtstring = (datetime.getMonth()+1)+'/'+datetime.getDate()+'/'+datetime.getFullYear();
    } else {
        dtstring = (datetime.getMonth()+1)+'/'+datetime.getFullYear();
    }
    return dtstring;
}

function updateStatusItem (item, key, data) {
    //item.properties = {};
    if (typeof(key) == 'undefined') return;
    if (typeof(data) == 'undefined') data = '0';
    //dump('key: '+key+' data: '+data+' url: '+item.url+'\n');
    switch(key) {
        case 'missing':
        case 'readonly':
        case 'readwrite':
        case 'sync':
        case 'ok':
        case 'conflict':
            item.properties[key] = parseInt(data);
            item.iconopenstyle[key] = item.iconstyle[key] = parseInt(data);
            break;
        case 'action':
            //dump(item.url+" action "+data+"\n");
            item.iconopenstyle['edit'] = item.iconstyle['edit'] = 0;
            item.iconopenstyle['add'] = item.iconstyle['add'] = 0;
            item.iconopenstyle['delete'] = item.iconstyle['delete'] = 0;
            if (data) item.iconopenstyle[data] = item.iconstyle[data] = 1;
            item.properties[key] = data;
            break;
        default:
            item.properties[key] = data;
            break;
    }
    //dump('key: '+key+' data: ['+item.properties[key]+']\n');
}

function updateStatus (url, keys,values) {
    var item;
    var index;
    var i;
    // get the item from the url
    var items = findItemsByURL(url);
    //dump(items.length+' matching parts!\n');
    for (i=0; i < items.length; i++) {
        item = items[i];
        //this.getDefaultFileProperties(item);
        for (index = 0; index < keys.length; index++) {
            updateStatusItem(item, keys[index], values[index]);
        }
    }
}

function doRefreshStatus () {
    var urls = {};
    var url;
    var item, items;
    if (editorHasFocus()) {
        //dump("Scintilla has focus -- actions should be based on the current file\n");
        var view = viewManager.getCurrentView();
        item = view.item;
        if (! view.url) {
            // This is an unsaved file -- no SCC commands can apply.
            return;
        }
        item.url = view.url;
        urls[item.url]=1;
    } else {
        if (! projectHasFocus() && !toolboxHasFocus()) {
            //dump("Scintilla or project or toolbox -- none have focus -- returning early \n");
            return;
        } else {
            items = gFocusedProjectView.getSelectedItems();
            // otherwise, look at the focused project view's selected parts.
            for (i=0; i<items.length; i++) {
                items[i].updateUniquePaths(urls,true);
            }
        }
    }
    try {
        for (url in urls) {
            gObserverSvc.Notify(this,'file_status_now',url);
        }
    } catch(e) { /* noone listening, ignore error */ }
}

peFile.prototype._fileProperties = function(item)
{
    if (typeof(item.properties['missing']) != 'undefined' && item.properties['missing']) {
        alertDialog("The file " + item.name + " does not exist on disk at " + displayNameFromURL(item.url));
    } else {
        var obj = new Object();
        obj.item = item;
        obj.task = 'edit';
        openWindowMultipleInstance(
            "chrome://komodo/content/project/fileProperties.xul", 
            "komodo_fileproperties",
            "chrome,close=yes,dialog=no", obj);
    }
}

peFile.prototype.editDirectoryShortcut = function(item) {
    var obj = new Object();
    obj.item = item;
    obj.task = 'edit';
    obj.imgsrc = 'chrome://komodo/skin/images/open.png';
    obj.type = 'DirectoryShortcut';
    obj.prettytype = 'Directory Shortcut';
    openWindowMultipleInstance(
        "chrome://komodo/content/project/simplePartProperties.xul", 
        "komodo_DirectoryShortcutProperties",
        "chrome,close=yes,dependent=yes,resizable=yes", obj);
}

peFile.prototype.Observe = function (object, topic, url)
{
    switch (topic) {
        case "file_status":
            try {
                var pKeys = {};
                var pValues = {};
                gFileStatusSvc.getFileStatus(url, pKeys, new Object(), pValues, new Object());
                //dump('updating status for '+url+' with '+pKeys.value.length+' items\n');
                if (pKeys.value.length) {
                    updateStatus(url, pKeys.value, pValues.value);
                }
            } catch(e) {
                dump(e);
            }
            break;
        case "file_status_updated":
            for (var i=0; i < gPartManagers.length; i++) {
                gPartManagers[i].refreshView();
            }
            break;
        default:
            // ignore
            break;
    }
}

function OpenDirectoryShortcut(part) {
    var fp = dlg_openFileInDir(part.value);
    if (fp == null) return;
    var file = new dlg_get_URL_andName(fp);
    doOpenPath(file.url);
}
