/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

// start the extension
registeredExtensions[registeredExtensions.length] = new peFolder();

function peFolder() {
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
peFolder.prototype.constructor = peFolder;

peFolder.prototype.init = function() {
}

peFolder.prototype.registerCommands = function() {
    extensionManager.registerCommand('cmd_delete', this);
    extensionManager.registerCommand('cmd_renamePart', this);
    extensionManager.registerCommand('cmd_removePart', this);
    extensionManager.registerCommand('cmd_addFilePart', this);
    extensionManager.registerCommand('cmd_addRemoteFilePart', this);
    extensionManager.registerCommand('cmd_addFolderPart', this);
    extensionManager.registerCommand('cmd_importFromFS', this);
    
    extensionManager.registerCommand('cmd_addFilePart_Project', this);
    extensionManager.registerCommand('cmd_addRemoteFilePart_Project', this);
    extensionManager.registerCommand('cmd_addFolderPart_Project', this);
    extensionManager.registerCommand('cmd_importFromFS_Project', this);
}

peFolder.prototype.registerMenus = function() {
    extensionManager.createMenuItem(Components.interfaces.koIContainer,'Add New Folder...','cmd_addFolderPart');
    extensionManager.createMenuItem(Components.interfaces.koIContainer,'Add Existing File...','cmd_addFilePart');
    extensionManager.createMenuItem(Components.interfaces.koIContainer,'Add Existing Remote File...','cmd_addRemoteFilePart');
    
    extensionManager.createMenuItem(Components.interfaces.koIPart_folder,'Import from File System...','cmd_importFromFS');
    extensionManager.createMenuItem(Components.interfaces.koIPart_folder,'Refresh Status','cmd_refreshStatus');
    extensionManager.createMenuItem(Components.interfaces.koIPart_folder, 'Properties', 'cmd_editFileProperties');
    
    extensionManager.createMenuItem(Components.interfaces.koIPart,'Rename','cmd_renamePart');
    extensionManager.createMenuItem(Components.interfaces.koIPart,'Remove','cmd_removePart');
}

peFolder.prototype.registerEventHandlers = function() {
    extensionManager.addEventHandler(Components.interfaces.koIPart,'onUpdatePart',this);
}

peFolder.prototype.supportsCommand = function(command, item) {
    var items = gFocusedProjectView.getSelectedItems();
    var havemultiple = items.length > 1;
    switch (command) {
    case 'cmd_renamePart':
        // we only allow renaming parts that are not attached to a url,
        // such as a virtual folder
        if (!item)
            item = gFocusedProjectView.getSelectedItem();
        return !havemultiple && item && item.renameable;
    case 'cmd_removePart':
    case 'cmd_delete':
        // Make sure that the focus is in the project manager or the toolbox
        if (!projectHasFocus() && !toolboxHasFocus()) return false;
        // get the current selection, make sure it's not a project
        if (!item)
            item = gFocusedProjectView.getSelectedItem();
        if (item) {
            try {
                var i = item.part.QueryInterface(Components.interfaces.koIProject);
            } catch (e) {
                // if the interface is not supported, it is not a project, so return true.
                return true;
            }
        }
        return false;
    case 'cmd_addFilePart':
    case 'cmd_addRemoteFilePart':
    case 'cmd_addFolderPart':
        return true;
    case 'cmd_addFilePart_Project':
    case 'cmd_addRemoteFilePart_Project':
    case 'cmd_addFolderPart_Project':
    case "cmd_importFromFS_Project":
        return gProjectManager.getCurrentProject() != null;
    case "cmd_importFromFS":
        var ok =false;
        if (!item)
            item = gFocusedProjectView.getSelectedItem();
        if (item) {
            try {
                ok = item.part.QueryInterface(Components.interfaces.koIProject) != null;
            } catch (e) {}
            if (!ok) {
                try {
                    ok = item.part.QueryInterface(Components.interfaces.koIPart_folder) != null;
                } catch (e) {}
            }
        }
        return ok;
    default:
        break;
    }
    return false;
}

peFolder.prototype.isCommandEnabled = peFolder.prototype.supportsCommand;

peFolder.prototype.doCommand = function(command) {
    var item = null;
    switch (command) {
    case 'cmd_addFilePart_Project':
        gProjectManager.viewMgr.focus();
        item = gProjectManager.getCurrentProject();
        // fall through
    case 'cmd_addFilePart':
        if (!item) item = gFocusedProjectView.getSelectedItem();
        this.addFile(item);
        break;
    case 'cmd_addRemoteFilePart_Project':
        gProjectManager.viewMgr.focus();
        item = gProjectManager.getCurrentProject();
        // fall through
    case 'cmd_addRemoteFilePart':
        if (!item) item = gFocusedProjectView.getSelectedItem();
        this.addRemoteFile(item);
        break;
    case 'cmd_renamePart':
        item = gFocusedProjectView.getSelectedItem();
        if (item) {
            gFocusedProjectView.manager.renameItem(item);
        }
        break;
    case 'cmd_addFolderPart_Project':
        gProjectManager.viewMgr.focus();
        item = gProjectManager.getCurrentProject();
        // fall through
    case 'cmd_addFolderPart':
        if (!item) item = gFocusedProjectView.getSelectedItem();
        var fitem = new Object();
        fitem.name = 'New Folder';
        fitem.value = null;
        if (gFocusedProjectView.manager.renameItem(fitem)) {
            this.addFolder(fitem.name, item);
        }
        break;
    case 'cmd_removePart':
    case 'cmd_delete':
        var items = gFocusedProjectView.getSelectedItems();
        var havemultiple = (items.length > 1);
        if (havemultiple) {
            if (yesNoDialog("Do you want to remove the " + items.length + " items you have selected?", "Yes") != "Yes") return;
        }
        for (var i = 0; i < items.length; i++) {
            gFocusedProjectView.manager.removeItem(items[i], havemultiple);
        }        
        break;
    case "cmd_importFromFS_Project":
        gProjectManager.viewMgr.focus();
        item = gProjectManager.getCurrentProject();
    case "cmd_importFromFS":
        if (!item) item = gFocusedProjectView.getSelectedItem();
        if (!item) return;
        this.ImportFromFS(item);
        break;
    default:
        break;
    }    
}

peFolder.prototype._addFile = function(url, item)
{
    var index = -1;
    if (typeof(item)=='undefined' || !item)
        item = gFocusedProjectView.getSelectedItem();
    if (item) index = gFocusedProjectView.getIndexByPart(item);    
    var part;
// #if PRODUCT_TYPE == "professional"
    if (url.match(/\.ui$/i)) {
        part = Components.classes["@activestate.com/koPart?type=dialog;1"]
                                    .createInstance(Components.interfaces.koIPart_dialog);
    } else {
// #endif
        part = Components.classes["@activestate.com/koPart?type=file;1"]
                                    .createInstance(Components.interfaces.koIPart_file);
// #if PRODUCT_TYPE == "professional"
    }
// #endif
    part.setStringAttribute('url', url);
    part.setStringAttribute('name', makeLeafName(url));
    gFocusedProjectView.manager.addItem(part,index);
}

peFolder.prototype.onUpdatePart = function(item,event)
{
    // event is null for onUpdatePart
    var isProject = null;
    try {
        isProject = item.part.QueryInterface(Components.interfaces.koIProject);
    } catch (e) {    }
    
    item.renameable = !item.part.hasAttribute('url');
    
    if (item.isContainer && !isProject) {
        if (!item.url) {
            item.properties['tooltiptext'] = '';
            item.iconopenstyle['projectVirtualFolderOpen'] = 1;
            item.iconstyle['projectVirtualFolderClosed'] = 1;
        } else {
            item.properties['tooltiptext'] = displayNameFromURL(item.url);
            item.iconopenstyle['projectFolderOpen'] = 1;
            item.iconstyle['projectFolderClosed'] = 1;
        }
    }
}

peFolder.prototype.addFile = function(parent_item)
{
    //XXX todo: Support other resources 
    var path = null;
    var parentUrl = null;
    if (parent_item) {
        parentUrl = parent_item.url;
        if (parentUrl) {
            path = displayNameFromURL(parentUrl);
            if (path && !os.path.isdir(path)) {
                path = os.path.dirname(path);
                if (! os.path.isdir(path)) {
                    path = null;
                }
            }
        }
    }
    var fp = dlg_chooseLocalFileItem(path);
    if (fp == null) return false;
    var file = new dlg_get_URL_andName(fp);
    this._addFile(file.url, parent_item);
    return true;
}

peFolder.prototype.addRemoteFile = function(item)
{
    var list = doBrowseForFiles(); // in fileops.js
    if (list) {
        for (var i = 0; i < list.length; i++) {
            this._addFile(list[i], item);
        }
        return true;
    }
    return false;
}

peFolder.prototype.addFolder = function(name, item)
{
    if (typeof(name)=='undefined') name = 'New Folder';
    var index = -1;
    if (typeof(item) == 'undefined' || !item)
        item = gFocusedProjectView.getSelectedItem();
    if (item) index = gFocusedProjectView.getIndexByPart(item);
    var folder = Components.classes["@activestate.com/koPart?type=folder;1"]
                                    .createInstance(Components.interfaces.koIPart_folder);
    folder.name = name;
    gFocusedProjectView.manager.addItem(folder,index);
}

peFolder.prototype.ImportFromFS = function(item)
{
    if (typeof(item) == 'undefined' || !item) 
        item = gFocusedProjectView.getSelectedItem();
    if (!item) return false;

    var resp = new Object();
    resp.include = item.part.prefset.getStringPref("import_include_matches");
    resp.exclude = item.part.prefset.getStringPref("import_exclude_matches");
    resp.recursive= item.part.prefset.getBooleanPref("import_recursive");
    resp.importType= item.part.prefset.getStringPref("import_type");
    if (item.part.prefset.hasPrefHere("import_dirname")) {
        resp.dirname = item.part.prefset.getStringPref("import_dirname");
    } else {
        if (item.part.hasAttribute('url')) {
            // Is it a project or a folder?
            var url = item.part.getStringAttribute('url')
            var pathSvc = Components.classes["@activestate.com/koOsPath;1"]
                                    .getService(Components.interfaces.koIOsPath);
            resp.dirname = URLToLocalFile(url).path;
            if (item.part.type == 'project') {
                resp.dirname = pathSvc.dirname(resp.dirname );
            }
        } else {
            resp.dirname = os.getcwd();
        }
    }
    resp.res = '';
    resp.item = item;
    resp.global = window;
    
    openWindowUniqueInstance(
        "chrome://komodo/content/project/importFromFS.xul",
        'komodo_import',
        "chrome,dialog,modal",
        resp);
    if (resp.res != true) {
        return false;
    }

    if (resp.include != item.part.prefset.getStringPref("import_include_matches")) {
        item.part.prefset.setStringPref("import_include_matches", resp.include);
    }
    if (resp.exclude != item.part.prefset.getStringPref("import_exclude_matches")) {
        item.part.prefset.setStringPref("import_exclude_matches", resp.exclude);
    }
    if (resp.recursive != item.part.prefset.getBooleanPref("import_recursive")) {
        item.part.prefset.setBooleanPref("import_recursive", resp.recursive);
    }
    if (resp.importType != item.part.prefset.getStringPref("import_type")) {
        item.part.prefset.setStringPref("import_type", resp.importType);
    }
    if (resp.dirname != item.part.prefset.getStringPref("import_dirname")) {
        item.part.prefset.setStringPref("import_dirname", resp.dirname);
    }

    return true;
}

