/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

// start the extension
registeredExtensions[registeredExtensions.length] = new peSnippet();

var ANCHOR_MARKER = '!@#_anchor';
var CURRENTPOS_MARKER = '!@#_currentPos';

function peSnippet() {
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
peSnippet.prototype.constructor = peSnippet;

peSnippet.prototype.init = function() {
}

peSnippet.prototype.registerCommands = function() {
    extensionManager.registerCommand('cmd_insertSnippet', this);
    extensionManager.registerCommand('cmd_editSnippetProperties', this);
    extensionManager.registerCommand('cmd_addSnippet', this);
    extensionManager.registerCommand('cmd_addSnippet_Project', this);
}

peSnippet.prototype.registerEventHandlers = function() {
    extensionManager.addEventHandler(Components.interfaces.koIPart_snippet,'ondblclick',this);
    extensionManager.addEventHandler(Components.interfaces.koIPart_snippet, 'onUpdatePart',this);
}

peSnippet.prototype.registerMenus = function() {
    extensionManager.createMenuItem(Components.interfaces.koIPart_snippet,
                                    'Insert Snippet','cmd_insertSnippet');
    extensionManager.createMenuItem(Components.interfaces.koIPart_snippet, 
                                    'Properties', 'cmd_editSnippetProperties');
    extensionManager.createMenuItem(Components.interfaces.koIContainer,
                                    'Add New Snippet...', 'cmd_addSnippet');
}

peSnippet.prototype.ondblclick = function(item,event) {
    if (item.type != 'snippet') return;
    _insertSnippet(item.part)
}

peSnippet.prototype.supportsCommand = function(command, part) {
    var items = gFocusedProjectView.getSelectedItems();
    if (items.length > 1) return false;
    
    var view = gFocusedProjectView;
    if (!view) return false;
    //dump("supportsCommand for " + command +'\n');
    switch (command) {
    case 'cmd_insertSnippet':
        // XXX should just return true here, but the Extension manager isn't calling
        // isCommandEnabled, so we'll just hide the menu item for now. 
        if (!part)
            part = view.getSelectedItem();
        var vm = getFocusedViewManager();
        if (!vm) return false;
        //dump("vm.getCurrentView() = " + vm.getCurrentView() + '\n');
        if (!vm.getCurrentView()) return false;
        return true;
    case 'cmd_addSnippet_Project':
        return gProjectManager.getCurrentProject() != null;
    case 'cmd_editSnippetProperties':
    case 'cmd_addSnippet':
        return true;
    default:
        break;
    }
    return false;
}


peSnippet.prototype.isCommandEnabled = peSnippet.prototype.supportsCommand;

peSnippet.prototype.doCommand = function(command) {
    var item = gFocusedProjectView.getSelectedItem();
    switch (command) {
    case 'cmd_insertSnippet':
        _insertSnippet(item.part);
        break;
    case 'cmd_addSnippet_Project':
        gProjectManager.viewMgr.focus();
        item = gProjectManager.getCurrentProject();
        // fall through
    case 'cmd_addSnippet':
        this._addSnippet(item);
        break;
    case 'cmd_editSnippetProperties':
        this._snippetProperties(item);
        break;
    default:
        break;
    }    
}

peSnippet.prototype._snippetProperties = function(item)
{
    var obj = new Object();
    obj.item = item;
    obj.task = 'edit';
    openWindowMultipleInstance(
        "chrome://komodo/content/project/snippetProperties.xul", 
        "komodo_snippetproperties",
        "chrome,close=yes,dependent=yes,resizable=yes", obj);
}

peSnippet.prototype._addSnippet = function(item)
{
    var snippet = Components.classes['@activestate.com/koPart?type=snippet;1']
               .createInstance(Components.interfaces.koIPart_snippet);
    snippet.setStringAttribute('name', 'New Snippet');
    snippet.setStringAttribute('set_selection', 'false');
    snippet.setStringAttribute('indent_relative', 'false');
    snippet.value = '';
    var obj = new Object();
    obj.item = new partWrapper(snippet, 0);
    if (typeof(item)=='undefined' || !item)
        item = gFocusedProjectView.getSelectedItem();
    obj.parentitem = item;
    obj.gFocusedProjectView = gFocusedProjectView;
    obj.task = 'new';
    openWindowUniqueInstance(
        "chrome://komodo/content/project/snippetProperties.xul", 
        "komodo_snippetProperties",
        "chrome,close=yes,dependent=yes,resizable=yes", obj);
}


function _insertSnippet (snippet) { // a part
    
    // Snippet insertion is surprisingly tricky to do.
    // Snippets are stored with escaped newlines.
    // Snippets also store the selection within the snippet using line/column markers.
    // These are converted to index positions, inserted into the text using special
    // markers, the text is run through the interpolation service, indented
    // according to the local context if appropriate; the markers are removed and
    // the selection is set (if required) after the insertion.
    var vm = gLastFocusedViewManager;
    if (!vm) return;
    var view = vm.getCurrentView();
    if (!view) return;
    var scimoz = view._scimoz;
    var text, indent = '';
    scimoz.beginUndoAction();
    try {
        try {
            vm.setFocus();
            var eol = _getCurrentEOL();
            var setSelection = snippet.hasAttribute('set_selection') && snippet.getStringAttribute('set_selection') == 'true';
            var relativeIndent = snippet.hasAttribute('indent_relative') && snippet.getStringAttribute('indent_relative') == 'true';
            var viewData = Interpolation_getViewData(window);
            text = unescapeWhitespace(snippet.value, eol);
            var encodingSvc = Components.classes["@activestate.com/koEncodingServices;1"].
                                getService(Components.interfaces.koIEncodingServices);
            var encoding = view._sci_widget.encoding;
            try {
                text = encodingSvc.encode(text, encoding, 'strict');
            } catch (ex) {
                logError(ex);
                var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                    getService(Components.interfaces.koILastErrorService);
                alertDialog("The snippet being inserted can't be converted to the" +
                            " current file's encoding.  You must" +
                            " change the file's encoding first. [" + lastErrorSvc.getLastErrorMessage() + ']')
                return null;
            }
    
            if (scimoz.selText.length == 0 && text.match(/%\(?[wW]/) != null) {
                // There is no selection but there is a '%w', '%W', '%(w', or '%(W' in the snippet
                // Special case: select the word
                if (_IsWordCharacter(scimoz.getWCharAt(scimoz.currentPos-1))) {  // There is part of a word to our left
                    scimoz.wordLeft();
                }
                // Using several wordPartRights instead of one wordRight because the latter is whitespace swallowing.
                while (_IsWordCharacter(scimoz.getWCharAt(scimoz.currentPos))) {  // There is part of a word to our right
                    scimoz.wordPartRightExtend();
                }
            }
    
            var anchor = 0;
            var currentPos = 0;
            var line, i;
            var before_selection, selection, after_selection;
            var lines = text.split(eol);
    
            // Do the interpolation of special codes
            var istrings = Interpolate(window, [text], true,
                                       snippet.getStringAttribute("name"),
                                       viewData);
            text = istrings[0];
    
            if (relativeIndent) {
                // Do the indent
                var indentSize = scimoz.selectionStart - scimoz.positionFromLine(scimoz.lineFromPosition(scimoz.selectionStart))
                indent = '';
                var tabWidth;
                var useTabs = view.prefs.getBooleanPref("useTabs");
                for (i = 0; i < indentSize; i++) {
                    indent += ' ';
                }
                var tabequivalent = '';
                if (useTabs) {
                    tabWidth = view.prefs.getLongPref("tabWidth");
                    for (i = 0; i < tabWidth; i++) { 
                        tabequivalent += ' ';
                    }
                }
                var parts = text.split(eol);
                var splits, newindent, rest;
                for (i = 0; i < parts.length; i++) {
                    if (i != 0 && parts[i].length != 0 && parts[i] != "<!@#_end>" && parts[i] != "<!@#_start>") {  // Do not indent empty lines.
                        parts[i] = indent + parts[i];
                        if (useTabs) {
                            newindent = '';
                            // This is ugly but I don't know a better way to do this in JS.
                            rest = parts[i].replace(/^\s*/, '');
                            newindent = parts[i].slice(0, parts[i].length-rest.length);
                            newindent = newindent.replace(tabequivalent, '\t', 'g');
                            parts[i] = newindent + rest;
                        }
                    }
                }
                text = parts.join(eol);
            }
            anchor = text.indexOf(ANCHOR_MARKER);
            currentPos = text.indexOf(CURRENTPOS_MARKER);
            if (anchor != -1 && currentPos != -1) {
                if (anchor < currentPos) {
                    anchor = text.indexOf(ANCHOR_MARKER);
                    text = text.replace(ANCHOR_MARKER, '');
                    currentPos = text.indexOf(CURRENTPOS_MARKER);
                    text = text.replace(CURRENTPOS_MARKER, '');
                } else {
                    currentPos = text.indexOf(CURRENTPOS_MARKER);
                    text = text.replace(CURRENTPOS_MARKER, '');
                    anchor = text.indexOf(ANCHOR_MARKER);
                    text = text.replace(ANCHOR_MARKER, '');
                }
            } else {
                anchor = 0;
                currentPos = 0;
            }
            scimoz.replaceSel("");
            var oldInsertionPoint = scimoz.currentPos;
            scimoz.insertText(oldInsertionPoint, text);
            if (setSelection) {
                scimoz.anchor = oldInsertionPoint + anchor;
                scimoz.currentPos = oldInsertionPoint + currentPos;
            } else {
                // selection will be after snippet
                scimoz.anchor = scimoz.anchor + text.length;
                scimoz.currentPos = scimoz.anchor;
            }
        } catch (ex) {
            var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                                getService(Components.interfaces.koILastErrorService);
            var errno = lastErrorSvc.getLastErrorCode();
            if (errno == Components.results.NS_ERROR_ABORT) {
                // Command was cancelled.
            } else if (errno == Components.results.NS_ERROR_INVALID_ARG) {
                var errmsg = lastErrorSvc.getLastErrorMessage();
                alertDialog("Error inserting snippet:" + errmsg);
            } else {
                logError(ex);
                alertDialog("There was an unexpected error: " + ex);
            }
        }
    } finally {
        scimoz.endUndoAction();
    }
}

peSnippet.prototype.onUpdatePart = function(part,event)
{
    if (part.type == 'snippet') {
        part.renameable = true;
        part.iconstyle['snippet'] = 1;
    }
}

function _AddSnippetFromText(snippettext) {
    // Important: The snippettext MUST be a unicode string
    var escapedtext = escapeWhitespace(snippettext);
    escapedtext = escapedtext.replace('%', '%%', 'g');
    var snippet = Components.classes['@activestate.com/koPart?type=snippet;1'].createInstance(Components.interfaces.koIPart_snippet);
    snippet.type = 'snippet';
    snippet.setStringAttribute('name', snippetMakeDisplayName(snippettext));
    snippet.setStringAttribute('set_selection', 'true');
    snippet.setStringAttribute('indent_relative', 'false');
    escapedtext = ANCHOR_MARKER + escapedtext + CURRENTPOS_MARKER;
    snippet.value = escapedtext;
    return snippet;
}

function AddSnippetFromTextToToolbox(snippettext) {
    var snippet = _AddSnippetFromText(snippettext);
    toolboxMgr.addItem(snippet);
}

function AddSnippetFromTextAtIndex(snippettext,index) {
    var snippet = _AddSnippetFromText(snippettext);
    gFocusedProjectView.manager.addItem(snippet,index);
}

function AddSnippetFromText(snippettext) {
    AddSnippetFromTextAtIndex(snippet,-1);
}

/* Utility functions */

function _getCurrentEOL() {
    var vm = getFocusedViewManager()
    var view = vm.getCurrentView();
    if (!view) {
        // we're not looking at anything -- return '\n' just to return something
        logDebug("toolboxManager: Asked for EOL when no view present...");
        return '\n'
    }
    var eol;
    if (view.py_inst.new_line_endings == view.py_inst.EOL_LF) {
        eol = '\n';
    } else if (view.py_inst.new_line_endings == view.py_inst.EOL_CR) {
        eol = '\r';
    } else if (view.py_inst.new_line_endings == view.py_inst.EOL_CRLF) {
        eol = '\r\n';
    } else {
        eol = '\n';
        logError("toolboxManager: Unknown End of line")
    }
    return eol;
}

function snippetMakeDisplayName(text) {
    // Strip leading whitespace.
    text = text.replace( /^\s+/, "" );
    // Strip trailing whitespace.
    text = text.replace( /\s+$/, "" );
    if (text.length > 30) {
        text = text.substr(0, 20) + ' ... ' + text.substr(text.length - 10, text.length)
    }
    // Compress remaining whitespace.
    text = text.replace( /\s+/g, " " );
    return text
}

function Insert_NamedSnippet(name) {
    // 
    var part = toolboxMgr.findPartByAttributeValue('name', name);
    if (!part) {
        part = gProjectManager.findPartByAttributeValue('name', name);
    }
    if (part) {
        _insertSnippet(part);
    }
}



