/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

// Globals
function projectOutlinerView(outliner, manager) {
  partOutlinerView.apply(this, [outliner, manager]); // inherit from partOutlinerView, defined in partOutlinerView.js
  this.datapoints = {};
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
projectOutlinerView.prototype = new partOutlinerView();
projectOutlinerView.prototype.constructor = projectOutlinerView;

projectOutlinerView.prototype.getSelectedProjectName = function() {
    var selection = this.entriesList[this.mSelection.currentIndex];
    if (selection.type == 'project') {
        return selection.name;
    }
    return null;
}

projectOutlinerView.prototype.roundDigits =  function(n,d) {
    n = Math.round(n * 100) / 100;
    n = (n + 0.001) + '';
    return n.substring(0, n.indexOf('.') + d);
}

projectOutlinerView.prototype.getCellText = function(row, colID) {
    var res = "";
    if (row < this.entriesList.length) {
        var item = this.entriesList[row];
        if (typeof(item) == 'undefined') return res;
        //dump('get cell text for '+colID+'\n');
        switch (colID) {
        case 'Name':
            if (item.type == 'extensionfolder') {
                extension = item.part.getStringAttribute('extension')
                if (extension == '') {
                    res = "Files with no extension";
                } else {
                    res = extension+' files'
                }
            } else if (item.type =='languagefolder') {
                res = item.part.getStringAttribute('language') +' files'
            } else if (item.type == 'project' && item.part.isDirty) {
                res = item.name+"*";
            } else {
                res = item.name;
            }
            break;
        case 'Size':
            if (typeof(item.properties['Size']) != 'undefined') {
                res =item.properties['Size'];
            } else if (typeof(item.properties['size']) != 'undefined' &&
                       item.properties['size']) {
                var size = parseInt(item.properties['size']);
                if (size < 1024) {
                    size = size + ' bytes';
                } else if (size < 1024*1024) {
                    size = Math.ceil(size / 1024);
                    size = size + ' K';
                } else {
                    size = this.roundDigits(size / (1024 * 1024),2);
                    size = size + 'MB';
                }
                item.properties['Size'] = res = size;
            }
            break;
        case 'Date':
            if (typeof(item.properties['Date']) != 'undefined') {
                res = item.properties['Date'];
            } else if (typeof(item.properties['mtime']) != 'undefined' &&
                       item.properties['mtime']) {
                var date = new Date(parseInt(item.properties['mtime'])*1000);
                res = item.properties['Date'] = getDateTimeString(date);
            }
            break;
        default:
            var fieldname = this.datapoints[colID];
            if (typeof(item.properties[fieldname]) != 'undefined') {
                res = item.properties[fieldname];
            }
            break;
        }
    }
    //dump("projectOutlinerView.getCellText("+row+", "+colID+")=["+res+"]\n");
    return res;
}

projectOutlinerView.prototype._setProperties = function(stylearray, properties) {
    var done = false;
    for (var style in stylearray) {
        if (stylearray[style]) {
            //dump('setting style '+style+'\n');
            properties.AppendElement(this.atomService.getAtom(style));
            done=true;
        }
    }
    return done;
}

  /* void getCellProperties(in long row, in wstring colID, in nsISupportsArray properties); */
projectOutlinerView.prototype.getCellProperties = function(row, colID, properties) {
    // plugins set the properties
    if (row >= this.entriesList.length || typeof(this.entriesList[row]) == 'undefined') return;
    var item = this.entriesList[row];
    
    this._setProperties(item.styles, properties);

    if (colID != 'Name') return;

    var haveicon = false;
    // set up icons for the entry
    if (item.isContainer && item.isContainerOpen) {
        haveicon = this._setProperties(item.iconopenstyle, properties);
    } else {
        haveicon = this._setProperties(item.iconstyle, properties);
    }
    if (!haveicon) {
        // generic icons for files and folders if no style exists for the parts icon
        if (item.isContainer) {
            if (item.isContainerOpen) {
                if (!item.url) {
                    properties.AppendElement(this.atomService.getAtom("projectVirtualFolderOpen"));
                } else {
                    properties.AppendElement(this.atomService.getAtom("projectFolderOpen"));
                }
            } else {
                if (!item.url) {
                    properties.AppendElement(this.atomService.getAtom("projectVirtualFolderClosed"));
                } else {
                    properties.AppendElement(this.atomService.getAtom("projectFolderClosed"));
                }
            }
        } else {
            properties.AppendElement(this.atomService.getAtom("projectFile"));
        }
    }
}

projectOutlinerView.prototype.updateSortIndicators = function(sortId)
{
    var fieldname = null;
    for (var id in this.datapoints) {
        if (sortId == this.datapoints[id]) {
            fieldname = id;
            break;
        }
    }
    if (!fieldname) return;
    
    var currCol = this.manager.viewMgr.outliner.firstChild;
    var sortedColumn = currCol;
    // set the sort indicator on the column we are sorted by
    if (sortId) {
        while (sortedColumn) {
            if (sortedColumn.localName == "outlinercol" &&
               sortedColumn.getAttribute('id') == fieldname) break;
            sortedColumn = sortedColumn.nextSibling;
        }
        if (sortedColumn && sortedColumn.getAttribute('id') == fieldname) {
            var sortDirection = sortedColumn.getAttribute("sortDirection");
            if (sortDirection && sortDirection == "ascending") {
                sortedColumn.setAttribute("sortDirection", "descending");
            } else {
                sortedColumn.setAttribute("sortDirection", "ascending");
            }
        }
    }

    // remove the sort indicator from all the columns
    // except the one we are sorted by
    while (currCol) {
        while (currCol && currCol.localName != "outlinercol") {
            currCol = currCol.nextSibling;
        }
        if (currCol && (currCol != sortedColumn)) {
            currCol.removeAttribute("sortDirection");
        }
        if (currCol) {
            currCol = currCol.nextSibling;
        }
    }
}


var projectTooltipShowTimer = null;
var projectTooltipCloseTimer = null;
function cancelProjectTooltipTimer()
{
    if (projectTooltipShowTimer) window.clearTimeout(projectTooltipShowTimer);
    projectTooltipShowTimer = null;
    if (projectTooltipCloseTimer) window.clearTimeout(projectTooltipCloseTimer);
    projectTooltipCloseTimer = null;
}

function setProjectTooltipTimer(tooltipName, elName, x,y)
{
    if (projectTooltipCloseTimer) window.clearTimeout(projectTooltipCloseTimer);
    dump('showProjectTooltip("'+tooltipName+'", "'+elName+'",'+x+', '+y+');\n');
    projectTooltipShowTimer = window.setTimeout('showProjectTooltip("'+tooltipName+'", "'+elName+'",'+x+', '+y+');', 1000);
}

function closeProjectTooltip(tooltipName)
{
    cancelProjectTooltipTimer();
    document.getElementById(tooltipName).closePopup();
}

function showProjectTooltip(tooltipName, elName, x,y)
{
    document.getElementById(tooltipName).closePopup();
    var el = document.getElementById(elName).tooltipBaseElement;
    document.getElementById(tooltipName).openPopup(el, x, y, 'tooltip');
    projectTooltipCloseTimer =
        window.setTimeout('closeProjectTooltip("'+tooltipName+'");', 5000);
}
