/* Copyright (c) 2003 ActiveState SRL. 
   See the file LICENSE.txt for licensing information. */

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * Komodo's Snippet Properties and "Add Snippet" dialog.
 *
 */

var snippetname, snippetvalue, okButton, indentCheckbox, keybinding, gPart, gItem;
var shortcut_tab, snippet_tab, tabs, snippetnamelabel, applyButton;
var gCaretPos;
var gSetSelectionCheckbox;
var snippetViewManager;
var scin;
var encodingSvc = Components.classes["@activestate.com/koEncodingServices;1"].
                    getService(Components.interfaces.koIEncodingServices);
var ANCHOR_MARKER = '!@#_anchor';
var CURRENTPOS_MARKER = '!@#_currentPos';

function onLoad() {
    gItem = window.arguments[0].item;
    gPart = gItem.part;
    applyButton = document.getElementById('apply-button');

    if (window.arguments[0].task == 'new') {
        window.title = "Create New snippet";
        applyButton.setAttribute('collapsed', 'true');
    } else {
        window.title = "Snippet Properties";
    }

    tabs = document.getElementById('tabs');
    shortcut_tab = document.getElementById('shortcut_tab');
    snippet_tab = document.getElementById('snippet_tab');
    snippetname = document.getElementById('snippetname');
    snippetvalue = document.getElementById('snippetvalue');
    scin = snippetvalue.ISciMoz;
    snippetnamelabel = document.getElementById('snippetnamelabel');
    okButton = document.getElementById('ok');
    indentCheckbox = document.getElementById('indent_relative');
    gSetSelectionCheckbox = document.getElementById('set_selection');
    tabs.selectedTab = snippet_tab; // we may want to change this sometimes.
    
    keybinding = document.getElementById('keybindings');
    keybinding.gKeybindingMgr = opener.gKeybindingMgr;
    keybinding.part = gPart;

    snippetname.value = keybinding.commandParam = gPart.getStringAttribute('name');
    var text = gPart.value;
    text = opener.unescapeWhitespace(text, '\n');
    // Setup the regex entry boxes with appropriate styling.
    var styleApplier = Components.classes["@activestate.com/koScintillaStyleApplier;1"].createInstance();
    var fakeViewPrefs = Components.classes['@activestate.com/koPreferenceSet;1']
        .createInstance(Components.interfaces.koIPreferenceSet);
    fakeViewPrefs.setBooleanPref('editUseFixedFont', true)
    // Set up global settings like cursor color and current background color.
    opener.update_scintilla_global_pref(scin, 'cursorColor');
    opener.update_scintilla_global_pref(scin, 'currentLineColor');
    opener.update_scintilla_global_pref(scin, 'useCurrentLineColor');
    opener.update_scintilla_global_pref(scin, 'selectFore');
    opener.update_scintilla_global_pref(scin, 'useSelectFore');
    opener.update_scintilla_global_pref(scin, 'edgeColor');
    styleApplier.init(scin, 'Text', fakeViewPrefs);
    styleApplier.applyStyles()
    opener.scimoz_widget_init(snippetvalue, 1);

    text = encodingSvc.encode(text, 'utf-8', 'strict');
    scin.caretFore = 0x0000ff;
    scin.setSelBack(1, 11184895);
    scin.caretWidth = 2;
    gSetSelectionCheckbox.setAttribute('checked',
                                       gPart.getStringAttribute('indent_relative'));
    var anchor = text.indexOf(ANCHOR_MARKER);
    var currentPos = text.indexOf(CURRENTPOS_MARKER);
    if (anchor < currentPos) {
        anchor = text.indexOf(ANCHOR_MARKER);
        text = text.replace(ANCHOR_MARKER, '');
        currentPos = text.indexOf(CURRENTPOS_MARKER);
        text = text.replace(CURRENTPOS_MARKER, '');
    } else {
        currentPos = text.indexOf(CURRENTPOS_MARKER);
        text = text.replace(CURRENTPOS_MARKER, '');
        anchor = text.indexOf(ANCHOR_MARKER);
        text = text.replace(ANCHOR_MARKER, '');
    }
    scin.text = text;
    scin.anchor = anchor;
    scin.currentPos = currentPos;
    indentCheckbox.setAttribute('checked', gPart.getStringAttribute('indent_relative'));
    gSetSelectionCheckbox.setAttribute('checked', gPart.getStringAttribute('set_selection'));
    keybinding.init();
    keybinding.updateCurrentKey();
    
    snippetname.focus();
    scin.focus = true; // show the caret
    scin.caretPeriod = 0;
    snippetname.select();
    koDoSetOKCancel(onOk, onCancel);
    koCenterWindowOnScreen();
    updateOK();
}


function onOk()  {
    if (Apply()) {
        window.arguments[0].res = true;
        if (window.arguments[0].task == 'new') {
            var parentitem = window.arguments[0].parentitem;
            var item = window.arguments[0].item;
            var gFocusedProjectView = window.arguments[0].gFocusedProjectView;
            var index = gFocusedProjectView.getIndexByPart(parentitem);
            gFocusedProjectView.manager.addItem(item.part,index);
        }
        return true;
    }
    return false;
}


// Return true iff the current position and anchor position are valid.
//
// If the selection position(s) are to be maintain for snippet insertion
// they must not be within an interpolation code block because the final
// position is then ambiguous.
//
// If the positions are not valid an alert dialog is raised and the
// positions are automatically moved to appropriate valid positions.
//
function _validateSelection()
{
    var text = scin.text;
    var anchor = scin.anchor;
    var currentPos = scin.currentPos;
    var blockOffsets = GetBlockOffsets(text, true);

    // Detemine if the positions are valid. If not, determine appropriate
    // relocations for them.
    var i, start, end;
    var valid = true;
    for (i=0; i+1 < blockOffsets.length; i+=2) {
        start = blockOffsets[i];
        end = blockOffsets[i+1];
        if (start < currentPos && currentPos < end) {
            valid = false;
            if (currentPos < anchor) { // move back
                currentPos = start;
            } else if (currentPos == anchor) { // move both back
                currentPos = start;
                anchor = start;
            } else { // move forward
                currentPos = end;
            }
        }
        if (start < anchor && anchor < end) {
            valid = false;
            if (anchor < currentPos) { // move back
                anchor = start;
            } else if (anchor == currentPos) { // move both back
                anchor = start;
                currentPos = start;
            } else { // move forward
                anchor = end;
            }
        }
    }

    // If not valid, ask the user if should readjust.
    if (!valid) {
        var query = null;
        if (anchor == currentPos) {
            query = "The cursor is in the middle of a shortcut block. " +
                    "This is not allowed. Would you like Komodo to " +
                    "adjust the cursor position to an appropriate position?";
        } else {
            query = "The selection partially overlapped with a shortcut " +
                    "block. This is not allowed. Would you like Komodo to " +
                    "adjust the selection appropriatly?";
        }
        var readjust = confirmOkDialog(query, "OK",
                                       "dontask_readjust_snippet_selection");
        if (readjust == "OK") {
            scin.currentPos = currentPos;
            scin.anchor = anchor;
            valid = true;
        }
    }

    return valid;
}


// Apply the current changes and return true iff successful. Otherwise return
// false.
function Apply() {
    // Validate snippet data.
    if (gSetSelectionCheckbox.getAttribute('checked') == 'true') {
        var valid = _validateSelection();
        if (!valid) {
            return false;
        }
    }

    // Apply the keybinding.
    try {
        // The keybinding needs the commandparam to be set before the application would ever work.
        keybinding.commandParam = snippetname.value;
        var retval = keybinding.apply(); // This may return false if a keybinding is partially entered
        if (!retval) return retval;
    } catch (e) {
        logError(e);
        return false;
    }

    gItem.name = snippetname.value;
    opener.invalidateItem(gItem);
    gPart.setStringAttribute('name', snippetname.value);
    var indent_relative = indentCheckbox.getAttribute('checked') == 'true';
    if (indent_relative) {
        gPart.setStringAttribute('indent_relative', 'true');
    } else {
        gPart.setStringAttribute('indent_relative', 'false');
    }
    var setSelection = gSetSelectionCheckbox.getAttribute('checked') == 'true';
    if (setSelection) {
        gPart.setStringAttribute('set_selection', 'true');
    } else {
        gPart.setStringAttribute('set_selection', 'false');
        scin.documentEnd();
    }
    // Insert anchor/column markers
    var anchor = scin.anchor;
    var currentPos = scin.currentPos;
    var text = scin.text;
    if (anchor < currentPos) {
        text = text.slice(0, anchor) + ANCHOR_MARKER +
               text.slice(anchor, currentPos) + CURRENTPOS_MARKER +
               text.slice(currentPos);
    } else {
        text = text.slice(0, currentPos) + CURRENTPOS_MARKER +
               text.slice(currentPos, anchor) + ANCHOR_MARKER +
               text.slice(anchor);
    }
    gPart.value = opener.escapeWhitespace(encodingSvc.unicode(text, 'utf-8', 'strict')); 
    applyButton.setAttribute('disabled', 'true');
    return true;
}


function updateOK() {
    if (snippetname.value == '' || scin.text == '') {
        okButton.setAttribute('disabled', 'true');
        applyButton.setAttribute('disabled', 'true');
    } else {
        if (okButton.hasAttribute('disabled')) { 
            okButton.removeAttribute('disabled');
        }
        if (applyButton.hasAttribute('disabled')) { 
            applyButton.removeAttribute('disabled');
        }
    }
}


function onCancel()  {
    window.arguments[0].res= false;
    return true;
}


function scintillaBlur() {
    scin.focus = true;
    scin.caretPeriod = 0;
}


function scintillaFocus() {
    scin.caretPeriod = 500;
}


function InsertShortcut(shortcutWidget) {
    // Get the shortcut string from the menuitem widget and append it to
    // the current command string.
    scin.replaceSel(shortcutWidget.getAttribute("shortcut"));
    snippetvalue.focus();
    updateOK();
}


function OpenHelp(url /* = null */) {
    var windowMgr = Components.classes["@mozilla.org/rdf/datasource;1?name=window-mediator"].
                    getService(Components.interfaces.nsIWindowMediator);
    var helpWindow = windowMgr.getMostRecentWindow('komodo_help');
    if (helpWindow) {
        // XXX it'd be nice to have a way to change the help page of an already
        // open window instead of closing and restarting it.
        helpWindow.close();
    }

    func_helpHelp(url);
}



