/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

/*

Toolbox Manager 

Heavily modeled on the Project Manager.

The Toolbox manager reads items from a "per-user" project file called "toolbox.kpf" in the user's userDataDir.

*/

// Globals
var toolboxMgr = null;

// three XUL-related events: onload, can unload, and onunload.
function toolboxManagerOnLoad()
{
    toolboxMgr = new toolboxManager();
}

function toolboxManagerCanUnload()
{
    return true;
}


function toolboxManagerOnUnload()
{
    if (!toolboxManagerCanUnload()) return false;
    try {
        toolboxMgr.toolbox.save();
    } catch(ex) {
        var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
            getService(Components.interfaces.koILastErrorService);
        alertDialog('There was an error saving the toolbox: ' +
                    lastErrorSvc.getLastErrorMessage());
        return false;
    }
    return true;
}

function toolboxManager() {
    baseManager.apply(this, ["toolboxChanged"]);
    var userDataDir, koDirs;
    this._currentitem = null;
    this.viewMgr = document.getElementById('snippetview');
    this.viewMgr.onLoad(this,Components.interfaces.koIContainer);
    // our one and only datapoint for the toolbox
    this.viewMgr.view.datapoints['Name']='Name';
    
    gPartManagers.push(this);
    this.sorttype = 'Name';
    this.sortdir = 1;
    this.wrappedproject = null;
    
    this.toolbox = Components.classes["@activestate.com/koProject;1"]
                                    .createInstance(Components.interfaces.koIProject);
    koDirs = Components.classes["@activestate.com/koDirs;1"].
         getService(Components.interfaces.koIDirs);
    var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
        getService(Components.interfaces.koILastErrorService);
    userDataDir = koDirs.userDataDir;
    var fname = os.path.join(userDataDir, 'toolbox.kpf');
    this.toolboxURL = LocalFileToURL(fname);
    var shutil = Components.classes["@activestate.com/koShUtil;1"].getService(Components.interfaces.koIShUtil);
    var sampletoolbox = os.path.join(koDirs.installDir, 'samples');
    sampletoolbox = os.path.join(sampletoolbox, 'toolbox.kpf');
    if (!os.path.exists(fname)) {
        try {
            this.toolbox.create();
            this.toolbox.url = this.toolboxURL;
            this.toolbox.save()
        } catch(ex) {
            alertDialog('There was an error creating the default toolbox: ' +
                        lastErrorSvc.getLastErrorMessage());
            return;
        }
    }
    try {
        this.toolbox.load(this.toolboxURL);
    } catch(e) {
        var badfname = fname+".error.txt";
        logError("Unable to load "+fname+", "+
                 "a backup will be made as "+badfname+"\n "+e+"\n");
        alertDialog("There was an error loading your toolbox data.  "+
                 "A backup will be made at "+badfname+" and a new "+
                 "toolbox will be created.");
        try {
            shutil.copyfile(fname, badfname);
        } catch (e) {
            // Ignore it -- we can just go along w/ an empty toolbox (see below).
            alertDialog("Could not backup toolbox data file: " +
                        lastErrorSvc.getLastErrorMessage());
        }
    }

    // Install sample toolbox for a particular version
    var infoSvc = Components.classes["@activestate.com/koInfoService;1"].
        getService(Components.interfaces.koIInfoService);
    var prefName = "haveInstalledSampleToolbox-Komodo" + infoSvc.version;
    if (! globalPrefs.hasBooleanPref(prefName) || ! globalPrefs.getBooleanPref(prefName)) {
        this.installSamples(sampletoolbox, infoSvc.version);
        globalPrefs.setBooleanPref(prefName, "true");
    }
    this.loadFromProject(this.toolbox);
    this.languageRegistry = Components.classes["@activestate.com/koLanguageRegistryService;1"].getService(Components.interfaces.koILanguageRegistryService);
    // let the status service know it has work to do
    try {
        gObserverSvc.Notify(this, 'file_update_now', null);
    } catch(e) { /* exception if no listeners */ }
}


// The following two lines ensure proper inheritance (see Flanagan, p. 144).
toolboxManager.prototype = new baseManager();
toolboxManager.prototype.constructor = toolboxManager;

toolboxManager.prototype.loadFromProject = function(project) {
    // Trick to make our items have a level 0, as required by the partviiewer for close behavior (so far).
    this.wrappedproject = new partWrapper(project, -1);
    // additem checks if these are true before handling
    // creation of wrappers for children
    this.wrappedproject.isContainer = true;
    this.wrappedproject.isContainerOpen = true;
    this.sort(false);
    koRequestUpdateCommands(this._command_name);
}

toolboxManager.prototype.installSamples = function(sampleToolboxPath, version) {
    var sample_project = Components.classes["@activestate.com/koProject;1"]
                                .createInstance(Components.interfaces.koIProject);
    var folder = Components.classes["@activestate.com/koPart?type=folder;1"]
                                    .createInstance(Components.interfaces.koIPart_folder);

    sample_project.load(LocalFileToURL(sampleToolboxPath));
    folder.name = "Samples (" + version + ")";

    var i;
    var children = new Array();
    sample_project.getChildren(children, new Object());
    children = children.value;
    for (i = 0; i < children.length; i++) {
        folder.addChild(children[i]);
    }
    this.toolbox.addChild(folder);
}

toolboxManager.prototype.sort = function(sortdir) {
    if (!this.wrappedproject) {
        return;
    }
    // change sortdirection if we need to
    if (typeof(sortdir) != 'undefined' && sortdir) {
        this.viewMgr.view.updateSortIndicators(this.sorttype);
        this.sortdir = this.sortdir * -1;
    }
        
    gWrapperSortDir = this.sortdir;
    gWrapperComparison = this.sorttype;

    // clear the list and reinsert the projects
    this.viewMgr.view.clearList();
    // force fetching the projects children so that they
    // will get properly sorted when we open the toolbox
    // the first time
    if (this.wrappedproject.children) {
        this.wrappedproject.sortChildren();
        for (var child_index in this.wrappedproject.children) {
            this.viewMgr.view.addToplevelPart(this.wrappedproject.children[child_index]);
        }
    }
}

toolboxManager.prototype.renameItem = function(item)
{
    var ok = baseManager.prototype.renameItem.apply(this, [item]);
    if (ok) {
        try {
            this.toolbox.save();
        } catch(ex) {
            var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                getService(Components.interfaces.koILastErrorService);
            alertDialog('There was an error saving the toolbox: ' +
                        lastErrorSvc.getLastErrorMessage());
            return false;
        }
        this.sort(false);
    }
    return ok;
}

toolboxManager.prototype.removeItem = function(item, skipdialog)
{
    if (baseManager.prototype.removeItem.apply(this, [item,skipdialog])) {
        if (item.level == 0) {
            this.wrappedproject.removeChild(item);
        } 
        try {
            this.toolbox.save();
        } catch(ex) {
            var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                getService(Components.interfaces.koILastErrorService);
            alertDialog('There was an error saving the toolbox: ' +
                        lastErrorSvc.getLastErrorMessage());
        }
    }
}

toolboxManager.prototype.addItem = function(/* koIPart */ part, index) {
    var isproject = false;
    try {
        /* we don't allow projects to be added to the
           toolbox, but we do allow links to projects, which
           allows us to open projects from the toolbox */
        isproject = part.QueryInterface(Components.interfaces.koIProject) != null;
    } catch(e) {}
    
    if (isproject) {
        var p_part = Components.classes["@activestate.com/koPart?type=ProjectRef;1"]
                                        .createInstance(Components.interfaces.koIPart_ProjectRef);
        p_part.setStringAttribute('url', part.url);
        part = p_part;
    }
    
    try {
        var parent = this.wrappedproject;
        if (index >= 0 && this.viewMgr.view.entriesList[index].isContainer) {
            if (!this.viewMgr.view.entriesList[index].isContainer) {
                index = this.viewMgr.view.getParentIndex(index);
            }
            parent = this.viewMgr.view.entriesList[index];
        }
        if (part.hasAttribute('url')) {
            var url = part.getStringAttribute('url');
            if (parent.part.getChildWithTypeAndStringAttribute(part.type,'url',url,false)) 
                return false;
        }
        parent.part.addChild(part);
        var firstchild = parent.updateWrappedPart();
        this.sort(false);
        this.viewMgr.view.selectPart(firstchild);
        
        // this will prevent projects from being removed from project pane
        // if the project is dragged into the toolbox
        return !isproject;
    } catch(e) { logError('toolboxManager addItem exception: '+e+'\n'); }
    return false;
}

toolboxManager.prototype.getItemsByURL = function(url) {
    items = [];
    var item = this.findItemByURL(this.wrappedproject, null, url);
    if (item != null) items.push(item);
    return items;
}

toolboxManager.prototype.findItemByURL = function (parent, type, url) {
    var item;
    var child;
    var i;
    var j;
    if (!parent.hasWrappedChildren()) return null;
    for (i in parent.children) {
        item = parent.children[i];
        if (!type || item.type == type) {
            if (item.url == url || (item.part.hasAttribute('url')
                && item.part.getStringAttribute('url') == url)) {
                return item;
            }
        }
        if (item.type == 'folder') {
            for (j in parent.children) {
                child = this.findItemByURL(parent.children[j], type, url);
                if (child != null) {
                    return child;
                }
            }
        }
    }
    if (parent.url == url || (parent.part.hasAttribute('url')
            && parent.part.getStringAttribute('url') == url)) {
        return parent;
    }
    return null;
}

toolboxManager.prototype.findItemByAttributeValue= function (attribute, value) {
    return this.findChildByAttributeValue(this.wrappedproject, attribute, value);
}

toolboxManager.prototype.findPartByTypeAttributeValue= function (type, attribute, value) {
    return this.toolbox.getChildWithTypeAndStringAttribute(type, attribute, value,true);
}

toolboxManager.prototype.findPartByAttributeValue= function (attribute, value) {
    return this.toolbox.getChildByAttributeValue(attribute, value, true);
}

toolboxManager.prototype.applyPartKeybindings = function () {
    this.wrappedproject.applyPartKeybindings();
}

toolboxManager.prototype.getPartsByURL = function (url) {
    var part = this.toolbox.getChildByAttributeValue('url', url, true);
    if (part != null) return [part];
    return [];
}

toolboxManager.prototype.updateMenuForInterface = function(defaultMenuInterface,menuEl) {
    // we ignore the defaultMenuInterface, and just use the global
    // function that handles this.
    //extensionManager.updateMenuForInterface(defaultMenuInterface, menuEl);
    toolboxUpdateMainMenu(menuEl);
}

function toolboxUpdateMainMenu(menuEl)
{
    gFocusedProjectView = toolboxMgr.viewMgr.view;
    gFocusedProjectView.manager.viewMgr.focus();
    extensionManager.updateMenuForInterface(Components.interfaces.koIContainer, menuEl);
    extensionManager.updateMenuForInterface(Components.interfaces.koIToolbox, menuEl, true);
    return true;
}

function Toolbox_Export() {
    var mode = Components.interfaces.nsIFilePicker.modeSave
    var fp = Components.classes["@mozilla.org/filepicker;1"]
        .createInstance(Components.interfaces.nsIFilePicker);
    fp.init(window, "Export To...", mode);
    fp.appendFilter("Komodo Project/Toolbox files (*.kpf)", "*.kpf");
    fp.appendFilter("All files (*.*)", "*.*");
    var retval = fp.show();
    if (retval != Components.interfaces.nsIFilePicker.returnOK &&
        retval != Components.interfaces.nsIFilePicker.returnReplace)  {
        return;
    }
    var filename = fp.file.path;
    if (! filename) return;
    var extension = '.kpf';
    if (filename.slice(-extension.length) != extension) {
        filename += extension;
    }

    var koDirs = Components.classes["@activestate.com/koDirs;1"].
         getService(Components.interfaces.koIDirs);
    var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
        getService(Components.interfaces.koILastErrorService);
    var shutil = Components.classes["@activestate.com/koShUtil;1"].
        getService(Components.interfaces.koIShUtil);

    var userDataDir = koDirs.userDataDir;
    var toolboxfname = os.path.join(userDataDir, 'toolbox.kpf');
    try {
        shutil.copyfile(toolboxfname, filename);
    } catch (e) {
        // Ignore it -- we can just go along w/ an empty toolbox (see below).
        alertDialog("Could not make a copy of the toolbox data file: " +
                    lastErrorSvc.getLastErrorMessage());
    }
}

function Toolbox_ExportItems(items) {
    var mode = Components.interfaces.nsIFilePicker.modeSave
    var fp = Components.classes["@mozilla.org/filepicker;1"]
        .createInstance(Components.interfaces.nsIFilePicker);
    fp.init(window, "Export Toolbox To...", mode);
    fp.appendFilter("Komodo Project/Toolbox files (*.kpf)", "*.kpf");
    fp.appendFilter("All files (*.*)", "*.*");
    var retval = fp.show();
    if (retval != Components.interfaces.nsIFilePicker.returnOK &&
        retval != Components.interfaces.nsIFilePicker.returnReplace)  {
        return;
    }
    var filename = fp.file.path;
    if (! filename) return;
    var extension = '.kpf';
    if (filename.slice(-extension.length) != extension) {
        filename += extension;
    }
    var project = Components.classes["@activestate.com/koProject;1"]
                                     .createInstance(Components.interfaces.koIProject);
    try {
        var url = LocalFileToURL(filename);
        project.create();
        project.url = url;
        for (i = 0; i < items.length; i++) {
            project.addChild(items[i].part);
        }
        project.save();
    } catch(ex) {
        var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
            getService(Components.interfaces.koILastErrorService);
        alertDialog('There was an error saving project "' +
                    project.name + '": ' +
                    lastErrorSvc.getLastErrorMessage());
    }
}

function Toolbox_Import() {
    var filters = [['Komodo Project/Toolbox files',['*.kpf']],
                   ['All files',['*.*']]];
    var filename = BrowseForFile('', filters);
    if (! filename) return;
    // Create a new folder
    var folder = Components.classes["@activestate.com/koPart?type=folder;1"]
                                    .createInstance(Components.interfaces.koIPart_folder);
    folder.name = "Items from " + os.path.basename(filename);
    var tempProject = Components.classes["@activestate.com/koProject;1"]
                                         .createInstance(Components.interfaces.koIProject);

    var url = LocalFileToURL(filename);
    tempProject.load(url);
    var i;
    var children = new Array();
    tempProject.getChildren(children, new Object());
    children = children.value;
    for (i = 0; i < children.length; i++) {
        folder.addChild(children[i]);
    }
    var firstchild = toolboxMgr.wrappedproject.updateWrappedPart();
    //toolboxMgr.sort(false);
    //toolboxMgr.viewMgr.view.selectPart(firstchild);
    toolboxMgr.addItem(folder,-1);
}

