/* Copyright (c) 2000-2001 ActiveState SRL. 
   See the file LICENSE.txt for licensing information. */

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * String interpolation query dialog (see interpolation_functions.js)
 *
 * Used when a strings being interpolated include codes requiring that the user
 * be asked for information to place in the string.
 */

//---- globals

var gQueries = null;
var gPrefPrefix = "interpolationQueryDialog";



//---- internal support routines

// Restores the dialog's dimensions from prefs prefixed with the given
// prefix. If no preference is set then the default is used.
//
//    "prefix" is the pref prefix to use for restoring from prefs.
//    "default*" are the defaults to use. If null, then no default it set.
//
// Returns true if restored from prefs, false if any defaults were used.
//
function _RestoreWindowDimensions(prefix, defaultWidth, defaultHeight,
                                  defaultX, defaultY)
{
    var retval = true;
    var outerHeight=null, outerWidth=null; screenX=null, screenY=null;
    var prefSvc = Components.classes["@activestate.com/koPrefService;1"].getService();
    if (prefSvc.prefs.hasLongPref(prefix+"_outerWidth")) {
        outerWidth = prefSvc.prefs.getLongPref(prefix+"_outerWidth");
    } else {
        outerWidth = defaultWidth;
        retval = false;
    }
    if (prefSvc.prefs.hasLongPref(prefix+"_outerHeight")) {
        outerHeight = prefSvc.prefs.getLongPref(prefix+"_outerHeight");
    } else {
        outerHeight = defaultHeight;
        retval = false;
    }
    if (prefSvc.prefs.hasLongPref(prefix+"_screenX")) {
        screenX = prefSvc.prefs.getLongPref(prefix+"_screenX");
    } else {
        screenX = defaultX;
        retval = false;
    }
    if (prefSvc.prefs.hasLongPref(prefix+"_screenY")) {
        screenY = prefSvc.prefs.getLongPref(prefix+"_screenY");
    } else {
        screenY = defaultY;
        retval = false;
    }
    if (outerWidth != null) window.outerWidth = outerWidth;
    if (outerHeight != null) window.outerHeight = outerHeight;
    if (screenX != null) window.screenX = screenX;
    if (screenY != null) window.screenY = screenY;
    
    return retval;
}


function _SaveWindowsDimensions(prefix)
{
    // Save Window location.
    var prefSvc = Components.classes["@activestate.com/koPrefService;1"].getService();
    prefSvc.prefs.setLongPref(prefix+"_outerHeight", window.outerHeight);
    prefSvc.prefs.setLongPref(prefix+"_outerWidth", window.outerWidth);
    prefSvc.prefs.setLongPref(prefix+"_screenY", window.screenY);
    prefSvc.prefs.setLongPref(prefix+"_screenX", window.screenX);
}



//---- interface routines for interpolationquery.xul

function OnLoad()
{
    // Process input arguments.
    if (typeof window.arguments[0].title != "undefined" &&
        window.arguments[0].title != null) {
        window.title = window.arguments[0].title;
    } else {
        window.title = "Interpolation Query";
    }
    gQueries = window.arguments[0].queries;

    // Generate UI for the queries.
    var queryRows = document.getElementById("query-rows");

    // Special case the first one.
    // (XXX See "Gecko Sucks" comment in the XUL.)
    var q, label, textbox, row, hbox, i;
    q = gQueries[0];
    label = document.getElementById("query0-label");
    label.setAttribute("value", q.question+":");
    textbox = document.getElementById("query0-textbox");
    if (q.answer) {
        textbox.setAttribute("value", q.answer);
    } else {
        textbox.setAttribute("value", "");
    }
    textbox.setAttribute("komodo-preference-set", q.mruName+"Mru");
    // Need to delay this until "komodo-preference-set" is properly set
    // for the first (hardcoded) row.
    var dummy = new MRUAutoCompleteSession(textbox);
    
    for (i=1; i < gQueries.length; i++) {
        // Want the following XUL for each query:
        //    <row>
        //        <hbox>
        //            <label id="query0-label" value="QUESTION"
        //                   control="query0-textbox"/>
        //        </hbox>
        //        <textbox id="query0-textbox" flex="1" value="ANSWER"
        //                 onfocus="this.setSelectionRange(0,-1);"
        //                 style="min-height: 16px;"/>
        //                 type="autocomplete" maxrows="3"
        //                 komodo-preference-set="MRUNAMEMru"
        //                 oninit="new MRUAutoCompleteSession(this);"/>
        //    </row>
        q = gQueries[i];
        
        row = document.createElement("row");

        hbox = document.createElement("hbox");
        label = document.createElement("label");
        label.setAttribute("id", "query"+i+"-label");
        label.setAttribute("value", q.question+":");
        label.setAttribute("control", "query"+i+"-textbox");
        label.setAttribute("crop", "end");
        label.setAttribute("flex", "1");
        hbox.setAttribute("align", "center");
        hbox.appendChild(label);
        row.appendChild(hbox);
        
        textbox = document.createElement("textbox");
        textbox.setAttribute("id", "query"+i+"-textbox");
        textbox.setAttribute("flex", "1");
        if (q.answer) {
            textbox.setAttribute("value", q.answer);
        } else {
            textbox.setAttribute("value", "");
        }
        textbox.setAttribute("onfocus", "this.setSelectionRange(0,-1);");
        textbox.setAttribute("style", "min-height: 16px;");
        textbox.setAttribute("type", "autocomplete");
        textbox.setAttribute("maxrows", "3");
        textbox.setAttribute("komodo-preference-set", q.mruName+"Mru");
        textbox.setAttribute("oninit", "new MRUAutoCompleteSession(this);");
        textbox.setAttribute("align", "center");
        row.appendChild(textbox);

        queryRows.appendChild(row);
    }

    _RestoreWindowDimensions(gPrefPrefix, null, null, 100, 100);

    // Need to "sizeToContent" in a timeout because it seems ineffectual
    // when run here right away.
    // XXX Note that the sizeToContent and the .focus() sometimes fail to
    //     work. This seems to be a timing issue.
    setTimeout('window.sizeToContent(); document.getElementById("query0-textbox").focus();', 0);
}


function OnUnload()
{
    if (typeof window.arguments[0].retval == "undefined") {
        // This happens when "X" window close button is pressed.
        window.arguments[0].retval = "cancel";
    } else if (window.arguments[0].retval == "ok") {
        for (var i=0; i < gQueries.length; i++) {
            var q = gQueries[i];
            var id = "query"+i+"-textbox";
            var queryTextbox = document.getElementById(id);
            if (queryTextbox.value) {
                q.answer = queryTextbox.value;
                MRU_updateFromWidget(queryTextbox);
            } else {
                q.answer = "";
            }
        }
    }

    _SaveWindowsDimensions(gPrefPrefix);
    opener.focus();
}


function Ok()
{
    // Store users answers to query objects.
    window.arguments[0].retval = "ok";
    window.close();
}


function Cancel()
{
    window.arguments[0].retval = "cancel";
    window.close();
}

