/* Copyright (c) 2000-2001 ActiveState SRL. 
   See the file LICENSE.txt for licensing information. */

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * Komodo's Run dialog
 *
 * This dialog allows the user to run an arbitrary command string. Some
 * tweaks try to make it more useful to the user. E.g. the string is
 * interpolated to translate '%' to the full name of the file currently being
 * editted... etc. Basically the plan is to be able to do what can be done with
 * the various incarnations of ":!<shell-command>" in Vim.
 *
 * TODO:
 * - HTML documentation.
 * - "help" button in run dialog leading to this documentation
 */

//---- globals

// object to hold all XUL element references
var dialog;

var _gPrefSvc = null;


//---- interface routines for run.xul

function OnLoad()
{
    dialog = {};
    dialog.runButton = document.getElementById('ok');
    dialog.cancelButton = document.getElementById('cancel');
    dialog.moreOptionsBox = document.getElementById('more-options-box');
    dialog.commandTextbox = document.getElementById("command-textbox");
    dialog.opOnSelWidget = document.getElementById("operate-on-selection-checkbox");
    dialog.insertOutputWidget = document.getElementById("insert-output-checkbox");
    dialog.doNotOpenOutWinWidget = document.getElementById("do-not-open-output-window-checkbox");
    dialog.parseOutputCheckbox = document.getElementById("parse-output-checkbox");
    dialog.parseRegexTextbox = document.getElementById("parse-regex-textbox");
    dialog.showParsedOutputListCheckbox = document.getElementById("show-parsed-output-list-checkbox");
    dialog.runInMenulist = document.getElementById("run-in-menulist");
    dialog.cwdTextbox = document.getElementById("cwd-textbox");
    dialog.addToToolboxWidget = document.getElementById("add-to-toolbox-checkbox");
    dialog.saveOptsCheckbox = document.getElementById("save-options-checkbox");

    _gPrefSvc = Components.classes["@activestate.com/koPrefService;1"].
                getService(Components.interfaces.koIPrefService);

    // Determine if there is a current selection. If so then presume the user
    // wants to operate on that selection.
    var view = opener.gLastFocusedViewManager.getCurrentView();
    if (view) {
        var scimoz = view._scimoz;
        var selectedText = scimoz.selText;
        if (selectedText != "") {
            // If the selected text has newline characters in it or *is* an
            // entire line (without the end-of-line) then "operate on
            // selection".
            var curLineObj = new Object;
            scimoz.getCurLine(curLineObj);
            var curLine = curLineObj.value;
            if (selectedText.search(/\n/) != -1
                || selectedText == curLine.substring(0,curLine.length-1))
            {
                var opOnSelWidget = document.getElementById(
                    "operate-on-selection-checkbox");
                opOnSelWidget.checked = true;
                var insertOutputWidget = document.getElementById(
                    "insert-output-checkbox");
                insertOutputWidget.checked = true;
            }
        }
    }
    
    dialog.runButton.setAttribute('label', 'Run');
    dialog.runButton.setAttribute('accesskey', 'r');
    dialog.cancelButton.setAttribute('label', 'Cancel');
    dialog.cancelButton.setAttribute('accesskey', 'c');
    koDoSetOKCancel(RunCommandAndExit, Exit);

    RunEnv_OnLoad();
    RestoreUIPrefs();
    
    // Insert any text from the input buffer and give the command textbox the
    // initial focus.
    var inputBufferContents = opener.InputBuffer_Finish();
    if (inputBufferContents) {
        dialog.commandTextbox.setAttribute('value', inputBufferContents);
    }
    dialog.commandTextbox.focus();
    UpdateCommand();
    
    SetWindowLocation();
}


function OnUnload()
{
    SaveWindowLocation();
    SaveUIPrefs();
    opener.focus();
}


function ToggleCheck(name)
{
    var id = name + "-checkbox";
    var widget = document.getElementById(id);
    widget.checked = !widget.checked;

    if (name == "insert-output") {
        UpdateInsertOutput();
    }
}


function RestoreUIPrefs()
{
    // Restore run dialog UI preferences.
    var showMoreOptions = _gPrefSvc.prefs.getBooleanPref("runShowMoreOptions");
    if (showMoreOptions) {
        ToggleMoreOptions();
    }

    dialog.cwdTextbox.value = _gPrefSvc.prefs.getStringPref("runCwd");

    var runIn = _gPrefSvc.prefs.getStringPref("runRunIn");
    var menuitems = dialog.runInMenulist.firstChild.childNodes;
    for (var i=0; i < menuitems.length; i++) {
        var menuitem = menuitems[i];
        if (menuitem.getAttribute("value") == runIn) {
            dialog.runInMenulist.selectedIndex = i;
        }
    }
    dialog.doNotOpenOutWinWidget.checked =
        _gPrefSvc.prefs.getBooleanPref("runDoNotOpenOutWin");
    dialog.parseOutputCheckbox.checked =
        _gPrefSvc.prefs.getBooleanPref("runParseOutput");
    dialog.parseRegexTextbox.value =
        _gPrefSvc.prefs.getStringPref("runParseRegex");
    dialog.showParsedOutputListCheckbox.checked =
        _gPrefSvc.prefs.getBooleanPref("runShowParsedOutputList");
    UpdateRunIn();

    var env = _gPrefSvc.prefs.getStringPref("runEnv");
    gRunEnvView.SetEnvironmentStrings(env);
}


function SaveUIPrefs()
{
    // Save dialog UI preferences.
    if (dialog.moreOptionsBox.getAttribute("collapsed") == "false") {
        _gPrefSvc.prefs.setBooleanPref("runShowMoreOptions", true);
    } else {
        _gPrefSvc.prefs.setBooleanPref("runShowMoreOptions", false);
    }

    if (dialog.saveOptsCheckbox.checked) {
        _gPrefSvc.prefs.setStringPref("runCwd", dialog.cwdTextbox.value);
        _gPrefSvc.prefs.setStringPref("runRunIn", dialog.runInMenulist.value);
        _gPrefSvc.prefs.setBooleanPref("runDoNotOpenOutWin",
                                       dialog.doNotOpenOutWinWidget.checked);
        _gPrefSvc.prefs.setBooleanPref("runParseOutput",
                                       dialog.parseOutputCheckbox.checked);
        _gPrefSvc.prefs.setStringPref("runParseRegex",
                                      dialog.parseRegexTextbox.value);
        _gPrefSvc.prefs.setBooleanPref("runShowParsedOutputList",
                                       dialog.showParsedOutputListCheckbox.checked);
        _gPrefSvc.prefs.setStringPref("runEnv",
                                      gRunEnvView.GetEnvironmentStrings());
    }
}


function RunCommandAndExit()
{
    // Validate.
    if (! dialog.commandTextbox.value) {
        alertDialog("You must specify a command string.");
        dialog.commandTextbox.focus();
        return;
    }
    if (dialog.parseOutputCheckbox.checked && !dialog.parseRegexTextbox.value) {
        alertDialog("You have chosen to parse command output. You must "+
                    "specify a regular expression with which to parse.");
        dialog.parseRegexTextbox.focus();
        return;
    }

    // Get the information from the UI
    var cmd = dialog.commandTextbox.value;
    var cwd = dialog.cwdTextbox.value;
    var env = gRunEnvView.GetEnvironmentStrings();
    var insertOutput = dialog.insertOutputWidget.checked;
    var operateOnSelection = dialog.opOnSelWidget.checked;
    var doNotOpenOutputWindow = dialog.doNotOpenOutWinWidget.checked;
    var runIn = dialog.runInMenulist.value;
    var parseOutput = dialog.parseOutputCheckbox.checked;
    var parseRegex = dialog.parseRegexTextbox.value;
    var showParsedOutputList = dialog.showParsedOutputListCheckbox.checked;

    if (cmd == "") {
        alertDialog("You must specify a command.");
        return;
    }
    
    // Run the command.
    var launched = Run_RunCommand(opener, cmd, cwd, env, insertOutput,
                                  operateOnSelection, doNotOpenOutputWindow,
                                  runIn, parseOutput, parseRegex,
                                  showParsedOutputList);
    if (! launched) {
        // Don't close the window if there was an error launching.
        return;
    }

    // Add the command to the toolbox (need a better UI for this).
    if (dialog.addToToolboxWidget.checked) {
        opener.AddCommandToToolbox(cmd, cwd, env, insertOutput,
                                   operateOnSelection, doNotOpenOutputWindow,
                                   runIn, parseOutput, parseRegex,
                                   showParsedOutputList);
    }

    // Exit
    opener.focus(); // must explicitly ensure that opener comes to foreground
    window.close();
}


function Exit()  {
    window.close();
}


function _InsertShortcut(textbox, shortcut)
{
    var oldValue = textbox.value;
    var selStart = textbox.selectionStart;
    var selEnd = textbox.selectionEnd;
    
    var newValue = oldValue.slice(0, selStart) + shortcut
                   + oldValue.slice(selEnd, oldValue.length);
    textbox.setAttribute("value", newValue);
    textbox.selectionStart = selStart;
    textbox.selectionEnd = selStart + shortcut.length;
}

function InsertCommandShortcut(shortcutWidget)
{
    // Get the shortcut string from the menuitem widget and append it to
    // the current command string.
    var shortcut = shortcutWidget.getAttribute("shortcut");
    var textbox = document.getElementById("command-textbox");
    _InsertShortcut(textbox, shortcut);
}

function InsertCwdShortcut(shortcutWidget)
{
    var shortcut = shortcutWidget.getAttribute("shortcut");
    var textbox = document.getElementById("cwd-textbox");
    _InsertShortcut(textbox, shortcut);
}


function ToggleMoreOptions()
{
    var moreOptionsBox = document.getElementById("more-options-box");
    var moreOptionsButton = document.getElementById("more-options-button");
    var collapsed = moreOptionsBox.getAttribute("collapsed");

    if (collapsed == "true") {
        moreOptionsBox.setAttribute("collapsed", "false");
        moreOptionsButton.setAttribute("label", "Less");
        moreOptionsButton.setAttribute("image", "chrome://komodo/skin/images/less.gif");
    } else if (collapsed == "false") {
        moreOptionsBox.setAttribute("collapsed", "true");
        moreOptionsButton.setAttribute("label", "More");
        moreOptionsButton.setAttribute("image", "chrome://komodo/skin/images/more.gif");
    } else {
        throw("Unexpected 'collapsed' state for 'more-options-box': "
              + collapsed);
    }
    window.sizeToContent();
}


// Make changes necessary for a change in the "Insert Output" checkbox.
// - The "Run in:" section is immaterial if we are capturing the output.
function UpdateInsertOutput()
{
    if (dialog.insertOutputWidget.checked) {
        dialog.runInMenulist.setAttribute("disabled", "true");
        dialog.doNotOpenOutWinWidget.setAttribute("disabled", "true");
        dialog.parseOutputCheckbox.setAttribute("disabled", "true");
        dialog.parseRegexTextbox.setAttribute("disabled", "true");
        dialog.showParsedOutputListCheckbox.setAttribute("disabled", "true");
    } else {
        dialog.runInMenulist.removeAttribute("disabled");
        UpdateRunIn();
    }
}


function UpdateRunIn()
{
    // Make changes necessary for a change in the "Run In" menulist.
    if (dialog.runInMenulist.value == "command-output-window") {
        dialog.doNotOpenOutWinWidget.removeAttribute("disabled");
        dialog.parseOutputCheckbox.removeAttribute("disabled");
        UpdateParseOutput();
    } else {
        dialog.doNotOpenOutWinWidget.setAttribute("disabled", "true");
        dialog.parseOutputCheckbox.setAttribute("disabled", "true");
        dialog.parseRegexTextbox.setAttribute("disabled", "true");
        dialog.showParsedOutputListCheckbox.setAttribute("disabled", "true");
    }
}


function UpdateParseOutput()
{
    if (dialog.parseOutputCheckbox.checked) {
        dialog.parseRegexTextbox.removeAttribute("disabled");
        dialog.showParsedOutputListCheckbox.removeAttribute("disabled");
    } else {
        dialog.parseRegexTextbox.setAttribute("disabled", "true");
        dialog.showParsedOutputListCheckbox.setAttribute("disabled", "true");
    }
}


// Make changes necessary for a change in the command string.
function UpdateCommand()
{
    if (dialog.commandTextbox.value) {
        dialog.runButton.removeAttribute("disabled");
    } else {
        dialog.runButton.setAttribute("disabled", "true");
    }
}


function BrowseForCwd()
{
    // from window-functions.js
    var cwd = BrowseForDir(dialog.cwdTextbox.value);
    if (cwd) {
        dialog.cwdTextbox.value = cwd;
    }
}


function OpenHelp(url /* = null */) {
    var windowMgr = Components.classes["@mozilla.org/rdf/datasource;1?name=window-mediator"].
                    getService(Components.interfaces.nsIWindowMediator);
    var helpWindow = windowMgr.getMostRecentWindow('komodo_help');
    if (helpWindow) {
        // XXX it'd be nice to have a way to change the help page of an already
        // open window instead of closing and restarting it.
        helpWindow.close();
    }

    func_helpHelp(url);
}


//---- The methods for the run environment outliner area.

function _UpdateSortIndicators(sortId)
{
    var sortedColumn = null;

    // Set the sort indicator on the column we are sorted by.
    if (sortId) {
        sortedColumn = document.getElementById(sortId);
        if (sortedColumn) {
            var sortDirection = sortedColumn.getAttribute("sortDirection");
            if (sortDirection && sortDirection == "ascending") {
                sortedColumn.setAttribute("sortDirection", "descending");
            } else {
                sortedColumn.setAttribute("sortDirection", "ascending");
            }
        }
    }

    // Remove the sort indicator from all the columns except the one we are
    // sorted by.
    var currCol = document.getElementById("env").firstChild;
    while (currCol) {
        while (currCol && currCol.localName != "outlinercol") {
            currCol = currCol.nextSibling;
        }
        if (currCol && (currCol != sortedColumn)) {
            currCol.removeAttribute("sortDirection");
        }
        if (currCol) {
            currCol = currCol.nextSibling;
        }
    }
}


function RunEnv_OnLoad()
{
    // Should be called in the onload handler for the window containing the
    // find results tab XUL.
    gRunEnvView = Components.classes[
        '@activestate.com/koRunEnvView;1']
        .createInstance(Components.interfaces.koIRunEnvView);
    var envOutliner = document.getElementById("env");
    envOutliner.boxObject
               .QueryInterface(Components.interfaces.nsIOutlinerBoxObject)
               .view = gRunEnvView;
}


function RunEnv_OnClick(event)
{
    // c.f. mozilla/mailnews/base/resources/content/threadPane.js
    var t = event.originalTarget;
    var index = gRunEnvView.selection.currentIndex;
    var rowCount = gRunEnvView.rowCount;
    //dump("RunEnv_OnClick(): start, index="+index+", rowCount="+rowCount+"\n");

    // single-click on a column
    if (t.localName == "outlinercol") {
        //dump("RunEnv_OnClick: single-click on a column...");
        if (rowCount > 0) {
            //dump("run sort\n");
            RunEnv_Sort(t.id);
        } else {
            //dump("ignored\n");
        }
    }

    // double-click in the outliner body
    else if (event.detail == 2 && t.localName == "outlinerbody") {
        //dump("RunEnv_OnClick: double-click in the outliner body...");
        if (0 <= index && index < rowCount) {
            //dump("run edit\n");
            RunEnv_Edit();
        } else {
            //dump("ignored\n");
        }
    }

    // single-click in the outliner body
    else if (event.detail == 1 && t.localName == "outlinerbody") {
        //dump("RunEnv_OnClick: single-click in the outliner body...");
        if (0 <= index && index < rowCount) {
            //dump("enable buttons\n");
            document.getElementById("env-edit-button").disabled = false;
            document.getElementById("env-delete-button").disabled = false;
        } else {
            //dump("ignored\n");
        }
    }
}


function RunEnv_Sort(columnID)
{
    gRunEnvView.Sort(columnID);
    _UpdateSortIndicators(columnID);
}


function RunEnv_OnKeyPress(event)
{
    var index = gRunEnvView.selection.currentIndex;
    var rowCount = gRunEnvView.rowCount;
    //dump("RunEnv_OnKeyPress(): start, index="+index+", rowCount="+rowCount+
    //     ", keyCode="+event.keyCode+"\n");

    if (event.keyCode == 13 /* <Enter> */) {
        //dump("RunEnv_OnKeyPress(): <Enter> key...");
        if (0 <= index && index < rowCount) {
            //dump("run edit and cancel bubble\n");
            RunEnv_Edit();
            event.cancelBubble = true;
        } else {
            //dump("ignored\n");
        }
    }
    return true;
}


// Add a new environment variable.
function RunEnv_New()
{
    // Disable the other buttons and unselect any current variable while the
    // edit dialog is up.
    document.getElementById("env").currentIndex = -1;
    document.getElementById("env-edit-button").disabled = true;
    document.getElementById("env-delete-button").disabled = true;

    var obj = new Object();
    obj.name = "";
    obj.value = "";
    window.openDialog("chrome://komodo/content/pref/editenvironvar.xul",
                      "Environment Variable",
                      "chrome,dialog=yes,close=yes,dependent=yes,modal=yes",
                      obj);
    if (obj.res == "cancel") { // Cancelled.
        return;
    }
    var addIt = "Yes";
    if (gRunEnvView.Have(obj.name)) {
        addIt = yesNoDialog("There is already a value for '" +
                            obj.name + "'. Overwrite it?", "no");
    }
    if (addIt == "Yes") {
        gRunEnvView.Set(obj.name, obj.value);

        // Select the new variable.
        var index = gRunEnvView.Index(obj.name);
        gRunEnvView.selection.currentIndex = index;
        gRunEnvView.selection.select(index);
        document.getElementById("env-edit-button").disabled = false;
        document.getElementById("env-delete-button").disabled = false;
    }
}


// Edit the currently selected environment variable row.
function RunEnv_Edit()
{
    var index = document.getElementById("env").currentIndex;
    var rowCount = gRunEnvView.rowCount;
    if (index < 0 || rowCount <= index) {
        return;
    }

    var obj = new Object();
    obj.name = gRunEnvView.GetVariable(index);
    obj.value = gRunEnvView.GetValue(index);
    window.openDialog("chrome://komodo/content/pref/editenvironvar.xul",
                      "Environment Variable",
                      "chrome,dialog=yes,close=yes,dependent=yes,modal=yes",
                      obj);
    if (obj.res == "cancel") { // Cancelled.
        return;
    }
    gRunEnvView.Set(obj.name, obj.value);

    // Select the editted variable.
    index = gRunEnvView.Index(obj.name);
    gRunEnvView.selection.currentIndex = index;
    gRunEnvView.selection.select(index);
    document.getElementById("env-edit-button").disabled = false;
    document.getElementById("env-delete-button").disabled = false;
}


// Delete the currently selected environment variable row.
function RunEnv_Delete()
{
    var index = document.getElementById("env").currentIndex;
    var rowCount = gRunEnvView.rowCount;
    if (index < 0 || rowCount <= index) {
        return;
    }

    gRunEnvView.Delete(gRunEnvView.selection.currentIndex);

    // Select the previous variable.
    index = gRunEnvView.selection.currentIndex;
    rowCount = gRunEnvView.rowCount;
    if (rowCount > 0 && index >= rowCount) {
        index = rowCount - 1;
        gRunEnvView.selection.currentIndex = index;
        gRunEnvView.selection.select(index);
    }

    if (index >= 0 && index < rowCount) {
        document.getElementById("env-edit-button").disabled = false;
        document.getElementById("env-delete-button").disabled = false;
    } else {
        document.getElementById("env-edit-button").disabled = true;
        document.getElementById("env-delete-button").disabled = true;
    }
}

