/*** -*- Mode: Javascript; tab-width: 2;

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is Collabnet code.
The Initial Developer of the Original Code is Collabnet.

Portions created by Collabnet are
Copyright (C) 2000 Collabnet.  All
Rights Reserved.

Contributor(s): Pete Collins, Doug Turner, Brendan Eich, Warren Harris, Eric Plaster


JS Directory Class API

  dir.js

Function List

    exists(aPath);                   // does the dir exist?
    getPrefsDir();                   // returns the path to the prefs dir
    getMozUserHomeDir();             // returns the path to the mozilla user dir
    getMozHomeDir();                 // returns the path to the mozilla dir
    getChromeDir();                  // returns the path to the chrome dir
    mkdir(aPath, aPermissions);      // permissions are optional
    rmdir(aPath);                    // remove a directory
    readDir(aDirPath);               // returns an array listing of a dirs contents 
    permissions(aPath);              // returns the files permissions
    dateModified(aPath);             // returns the last modified date in locale string
    help();                          // currently dumps a list of available functions 

Instructions:

Warning: these API's are not for religious types

*/

/****************** Globals **********************/

const JSDIR_FILE                    = "dir.js";

const JSDIR_LOCAL_CID               = "@mozilla.org/file/local;1";
const JSDIR_LOCATOR_PROGID          = '@mozilla.org/filelocator;1';
const JSDIR_CID                     = "@mozilla.org/file/directory_service;1";

const JSDIR_I_LOCAL_FILE            = "nsILocalFile";
const JSDIR_I_PROPS                 = "nsIProperties";
const JSDIR_INIT_W_PATH             = "initWithPath";

const JSDIR_PREFS_DIR               = 65539;

const JSDIR_DIRECTORY               = 0x01;     // 1
const JSDIR_OK                      = true;

const JSDIR_CHROME_DIR              = "AChrom";
const JSDIR_USR_DEFAULT             = "DefProfRt";
const JSDIR_PREF_DIR                = "PrefD";

const JSDIR_FilePath = new Components.Constructor( JSDIR_LOCAL_CID, JSDIR_I_LOCAL_FILE, JSDIR_INIT_W_PATH);


/****************** Dir Object Class *********************/
function Dir() {
} // constructor

Dir.prototype  = {
  mode            : null,
  fileInst        : null,
  fileChannel     : null,
  inputStream     : null,

/********************* GETPREFSDIR ***************************/
getPrefsDir : function () 
{
  var path = null;

  try {
    var fileLocator = Components.classes[JSDIR_LOCATOR_PROGID].getService();
    if (fileLocator) {
      fileLocator = fileLocator.QueryInterface(Components.interfaces.nsIFileLocator);
      var spec = fileLocator.GetFileLocation(JSDIR_PREFS_DIR);
      if(spec) {
        spec = spec.QueryInterface(Components.interfaces.nsIFileSpec);
        dump(JSDIR_FILE+": "+spec.nativePath+"\n");
        path = spec.nativePath;
      }
    }
  } catch(e) {
    dump(JSDIR_FILE+":getPrefsDir: problem in getPrefsDir: "+e+"\n");
    throw(e);
  }
  return path;
},

/********************* GET CHROME DIR ***************************/
getChromeDir  : function (){

    var DIR     = new Components.Constructor(JSDIR_CID, JSDIR_I_PROPS);
    var cd      = new DIR();

    var retval;
    try
    {
        var prop = JSDIR_CHROME_DIR;
        var file = cd.get(prop, Components.interfaces.nsIFile);
        retval=file.path;
    }

    catch (e)
    {
      dump(JSDIR_FILE+"ERROR: "+e.name+" "+e.message+"\n");
      retval=null;
    }

    return retval;

},
/********************* GET CHROME DIR ***************************/

/********************* GET HOME DIR *****************************/
getMozHomeDir  : function (){

    var DIR     = new Components.Constructor(JSDIR_CID, JSDIR_I_PROPS);
    var usrHome = new DIR();

    var retval;

    try
    {
        var file = usrHome.get(JSDIR_USR_DEFAULT, Components.interfaces.nsIFile);
        retval=file.path;
    }

    catch(e)
    {
      dump(JSDIR_FILE+":getMozHomeDir:ERROR: "+e.name+ " "+e.message+"\n\n");
      retval=null;
    }

    return retval;
},
/********************* GET HOME DIR *****************************/

/********************* GET USER HOME DIR ************************/
getMozUserHomeDir  : function (){

    var DIR     = new Components.Constructor(JSDIR_CID, JSDIR_I_PROPS);
    var usrHome = new DIR();

    var retval;

    try
    {
        var file = usrHome.get(JSDIR_PREF_DIR, Components.interfaces.nsIFile);
        retval=file.path;
    }

    catch(e)
    {
      dump(JSDIR_FILE+":getMozUserHomeDir:ERROR: "+e.name+ " "+e.message+"\n\n");
      retval=null;
    }

    return retval;
},
/********************* GET USER HOME DIR ************************/

/********************* EXISTS ***************************/
exists : function (aPath) 
{
  if(!aPath)
  {
    dump("dir.js:exists:ERROR: Missing path argument\n");
    return null;
  }

  var retval;
  try
  { 
    var file                = new JSDIR_FilePath(aPath);
    retval=file.exists();
  }

  catch(e) { 
    dump(JSDIR_FILE+":exists:ERROR: "+e.name+"\n"+e.message+"\n"); 
    retval=null;
  }

  return retval;
},

/********************* MKDIR ****************************/
mkdir : function(aPath, aPermissions) 
{
  if(!aPath)
  {
    dump(JSDIR_FILE+":rm:ERROR: Missing path argument\n");
    return null;
  }

  var fileExists = this.exists(aPath);

  if(aPermissions){
    var checkPerms = this.validatePermissions(aPermissions);

    if(!checkPerms){
      dump(JSDIR_FILE+":mkdir:ERROR: Sorry invalid permissions set\n");
      return null;
    }               

    var baseTen = permissions.toString(10);

    if(baseTen.substring(0,1) != 0)
      aPermissions = 0+baseTen;
  }

  var retval;
  try
  { 
    this.fileInst  = new JSDIR_FilePath(aPath);

    if (!fileExists)
    {
      if(!aPermissions)
        aPermissions     = 0755;

      this.fileInst.create( JSDIR_DIRECTORY, parseInt(aPermissions) ); 
      retval=JSDIR_OK;
    } else {
      retval=false; 
    }
  }
  catch (e)
  { 
    dump(JSDIR_FILE+":mkdir:ERROR: "+e.name+"\n"+e.message+'\n');
    return null;
  }

  return retval;
},

/********************* RMDIR ****************************/
rmdir : function (aPath) 
{

  if(!aPath)
  {
    dump(JSDIR_FILE+":rmdir:ERROR: Missing path argument\n");
    return null;
  }

  if(!this.exists(aPath))
  {
    dump(JSDIR_FILE+":rmdir:ERROR: Sorry, file "+aPath+" doesn't exist\n");
    return null;
  }

  var retval;
  try
  { 
    this.fileInst            = new JSDIR_FilePath(aPath);
    this.fileInst['delete'](true);
    retval=JSDIR_OK;
  }

  catch (e)
  { 
    dump(JSDIR_FILE+":rmdir: ERROR: "+e.name+"\n"+e.message+'\n');
    retval=null;
  }

  return retval;
},

/********************* READDIR **************************/
readDir : function (aDirPath) 
{
  if(!aDirPath)
  {
    dump(JSDIR_FILE+":readDir:ERROR: Missing path argument\n");
    return null;
  }

  if(!this.exists(aDirPath))
  {
    dump(JSDIR_FILE+":readDir:ERROR: Sorry, directory "+dirPath+" doesn't exist\n");
    return null;
  }

  var retval;
  try
  {
    var file      = new JSDIR_FilePath(aDirPath);

    if(!file.isDirectory())
    {
      dump(JSDIR_FILE+":readDir:ERROR: Sorry, "+aDirPath+" is not a directory\n");
      return null;
    }

    var files     = file.directoryEntries;
    var listings  = new Array();

    while(files.hasMoreElements())
      listings.push(new File(files.getNext().QueryInterface(Components.interfaces.nsILocalFile).path));

    retval=listings;
  }

  catch(e)
  { 
    dump(JSDIR_FILE+":readDir:ERROR: "+e.name+"\n"+e.message+'\n');
    retval=null;
  }

  return retval;
},

/********************* VALIDATE PERMISSIONS *************/
validatePermissions : function(aNum) 
{
  if ( parseInt(aNum.toString(10).length) < 3 ) 
    return false;

  return true;
},

/********************* PERMISSIONS **********************/
permissions : function (aPath) 
{
  if(!aPath)
  {
    dump(JSDIR_FILE+":permissions:ERROR: Missing path argument\n");
    return null;
  }

  if(!this.exists(aPath))
  {
    dump(JSDIR_FILE+":permissions:ERROR: File doesn't exist\n");
    return null;
  }

  var retval;
  try
  { 
    var fileInst  = new JSDIR_FilePath(aPath); 
    retval=fileInst.permissions.toString(8);
  }

  catch(e)
  { 
    dump(JSDIR_FILE+":permissions:ERROR: "+e.name+"\n"+e.message+'\n');
    retval=null;
  }

  return retval;
},

/********************* MODIFIED *************************/
dateModified  : function (aPath) 
{
  if(!aPath)
  {
    dump(JSDIR_FILE+":dateModified:ERROR: Missing path argument\n");
    return null;
  }

  if(!this.exists(aPath))
  {
    dump(JSDIR_FILE+":dateModified:ERROR: File doesn't exist\n");
    return null;
  }

  var retval;
  try
  { 
    var fileInst  = new JSDIR_FilePath(aPath); 
    var date = new Date(fileInst.lastModificationDate).toLocaleString();
    retval=date;
  }

  catch(e)
  { 
    dump(JSDIR_FILE+":dateModified:ERROR: "+e.name+"\n"+e.message+'\n');
    retval=null;
  }

  return retval;
},

/********************* HELP *****************************/
help  : function()
{
  var help =

    "\n\nFunction List:\n"                 +
    "\n"                                   +
    "   exists(aPath);\n"                  +
    "   getPrefsDir();\n"                  +
    "   getChromeDir();\n"                 +
    "   getMozHomeDir();\n"                +
    "   getMozUserHomeDir();\n"            +
    "   mkdir(aPath, aPermissions);\n"     +
    "   rmdir(aPath);\n"                   +
    "   readDir(aDirPath);\n"              +
    "   permissions(aPath);\n"             +
    "   dateModified(aPath);\n"            +
    "   help();\n";

  dump(help+'\n');
  return "";
}

}

