/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * string_functions.js
 * Common functions for string processing
 */

const watchWindowType = "komodo_tail";
const remoteWatchWindowType = "komodo_rtail";
const editorWindowType = "komodo_text";
const selvarWindowType = "komodo_selvars";

const KOMBO_STATE_UNLOADED = 1;    // not in editor
const KOMBO_STATE_SELECTED = 2;    // current buffer in editor
const KOMBO_STATE_UNSELECTED = 3;  // in editor, but not the current buffer


var rgb = new Object ();
rgb.yellow = RGB (0xff, 0xff, 0x00);
rgb.red = RGB (0xff, 0x00, 0x00);
rgb.green = RGB (0x00, 0xff, 0x00);
rgb.blue = RGB (0x00, 0x00, 0xff);
rgb.black = RGB (0x00, 0x00, 0x00);


function RGB(r,g,b) {
  return r + g * 256 + b * 256 * 256;
}


function makeLeafName(url) {
  var path = displayNameFromURL(url);
  var a = path.match(/.*[\/\\]([^\/\\]*)$/);
  var urllib = Components.classes["@activestate.com/koUrllib;1"].
      getService(Components.interfaces.koIUrllib);
  if (a) {
      var leafName = a[a.length - 1];
      if (leafName.search(/\S+/) >= 0) {
	  return urllib.unquote(leafName);
      }
  }
  return urllib.unquote(path);
}

// Full path*with* filename
// This function is used for two purposes:
//   1. Creating pretty names for display to the user.
//   2. Producing local paths for URLs for use by the linters (see
//      _request in kombo.js).
function displayNameFromURL(url) {
  // don't even attempt the conversion if the URL lacks a scheme as 
  // otherwise Mozilla will assert
  if (url && url.match(/^\w+:\/\//)) {
      try {
	  return URLToLocalFile(url).path;
      } catch (e) {
	  return url;
      }
  }
  return url;
}

// Full path*without* filename
function displayPathFromURL(url) {
  url = url.substring(0, url.lastIndexOf('/') + 1);    
  return displayNameFromURL(url);
}

function fixupUrl(url) {
    // XXX:
    // Fix the URL using Mozilla's URI fixup service.
    // nsIURIFixup quotes spaces in the URI.
    //  -- Ken Simpson
    var fixup = Components.classes["@mozilla.org/docshell/urifixup;1"].
	getService(Components.interfaces.nsIURIFixup);
    try {
	var fixupURI = fixup.createFixupURI(url);
	if(fixupURI.spec) {
	    url = fixupURI.spec;
	    
	    // urifixup inserts a '|' after the drive letter; we use colons.  
	    // So, we have to fix up the fixed-up URL. :-(
	    if (url.match(/^file:\/\/\/.\|\//)) {
		url = url.slice(0, 9) + ":" + url.slice(10);
	    }
	    
	    //we also need to fixup UNC paths by hand
	    if (url.match(/^file:\/\/\/\/[^\/]/)) {
		url = "file://///" + url.slice(9);
	    }
	}
	
    } catch (e) {
        // logdebug because we expect this to happen, and it doesn't
        // need to be reported for release builds
	// logDebug("nsIURIFixup could not fixup '" + url + "' - " + e + "\n");
	// Leave url alone (presumably to fail below!)
    }
    return url;
}


// Utility functions to escape and unescape whitespace

function escapeWhitespace(text) {
    text = text.replace(/\\/g, '\\\\'); // escape backslashes
    text = text.replace(/\r\n/g, '\\n'); // convert all different ends of lines to literal \n
    text = text.replace(/\n/g, '\\n'); 
    text = text.replace(/\r/g, '\\n'); 
    text = text.replace(/\t/g, '\\t'); 
    return text;
}

function unescapeWhitespace(text, eol) {
    var i;
    var newtext = '';
    for (i = 0; i < text.length; i++) {
        switch (text[i]) {
            case '\\':
                i++;
                switch (text[i]) {
                    case 'n':
                        newtext += eol;
                        break;
                    case 't':
                        newtext += '\t';
                        break;
                    case '\\':
                        newtext += '\\';
                        break;
                    // For backward compatiblity for strings that were not
                    // escaped but are being unescaped to ensure that, e.g.:
                    //    C:\WINNT\System32
                    // ends up unchanged after unescaping.
                    default:
                        i--;
                        newtext += '\\';
                }
                break;
            default:
                newtext += text[i];
        }
    }
    return newtext;
}
