/* Copyright (c) 2000-2001 ActiveState SRL. 
   See the file LICENSE.txt for licensing information. */

/* Web Services Setup Wizard - panel */

//---- globals

var _gPanelName = "wsSetup_ChooseModuleDir";



//---- internal support routines

function _CreateTreeItem(label)
{
    const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
    var treecell = document.createElementNS(XUL_NS, 'treecell');
    treecell.setAttribute('label', label);
    var treerow = document.createElementNS(XUL_NS, 'treerow');
    treerow.appendChild(treecell);
    var treeitem = document.createElementNS(XUL_NS, 'treeitem');
    treeitem.appendChild(treerow);
    return treeitem;
}


function _UpdateModuleDir(moduleDir)
{
    // Persist.
    parent.wd_moduleDir = moduleDir;

    // Display the moduleDir selection.
    var moduleDirWidget = document.getElementById("module-dir");
    if (moduleDir == null) {
        moduleDirWidget.value = "";
    } else {
        moduleDirWidget.value = moduleDir;
    }

    // Pass to the setup service.
    var wsSetupSvc = parent.wsSetupSvcs[parent.wd_language];
    wsSetupSvc.SetModuleDir(moduleDir);
}



//---- Wizard Infrastructure interface routines

function validate()
{
    // Validate this page.
    // - Ask the setup service if the setup will work in this
    //   installation.
    var wsSetupSvc = parent.wsSetupSvcs[parent.wd_language];
    var problem = wsSetupSvc.ReasonCannotSetup_GivenModuleDir();
    if (problem) {
        alertDialog(problem,    // Message to display
                    true,   // Okay button is the default
                    true,   // hide the cancel button
                    "OK",   // Okay button label
                    "");   // Cancel button label
        return false;
    } else {
        return true;
    }
}


//---- interface for wsSetup_ChooseInstallation.xul

function OnLoad()
{
    // Give a nice header name for this pane.
    var headerWidget = parent.document.getElementById("header");
    headerWidget.setAttribute("title", "Choose Module Directory");
    headerWidget.setAttribute("description",
        "Select directory in which to install Web Service modules.");

    // Fill in the set of current include path directories for the
    // chosen PHP installation.
    var listWidget = document.getElementById("include-path-list");
    // - remove any existing items
    while (listWidget.hasChildNodes()) {
        listWidget.removeChild(listWidget.childNodes[0]);
    }
    // - get a list of include paths
    var phpInfoEx = Components.classes['@activestate.com/koPHPInfoEx;1'].
                    getService(Components.interfaces.koIPHPInfoEx);
    phpInfoEx.installationPath = parent.wd_installation;
    var includePath = new Array();
    includePath = phpInfoEx.GetIncludePathArray(new Object());
    // - populate the tree listing them
    if (includePath.length != 0) {
        for (var i = 0; i < includePath.length; i++) {
            listWidget.appendChild(_CreateTreeItem(includePath[i]));
        }
    }
    
    // Fill in the module dir with a reasonable value if possible.
    if (parent.wd_moduleDir) {
        _UpdateModuleDir(parent.wd_moduleDir);
    }
    
    parent.wizardManager.onPageLoad(_gPanelName);
}


function UpdateModuleDir_OnSelection()
{
    // User selected one of the exist include_path dirs.
    // Load the default installation textfield with the selection
    var tree = document.getElementById("include-path-tree");
    if (tree.selectedItems.length > 0) {
        var selection = tree.selectedItems[0];
        //          treeitem . treerow  . treecell 
        var value = selection.firstChild.firstChild
                    .getAttribute("label");
        _UpdateModuleDir(value);
    }
}


function UpdateModuleDir_OnKey()
{
    // User typed into the installation field.
    // - Unselect any tree item.
    var tree = document.getElementById("include-path-tree");
    tree.clearItemSelection();
    
    // - Persist the value.
    var moduleDirWidget = document.getElementById("module-dir");
    _UpdateModuleDir(moduleDirWidget.value);
}


function UpdateModuleDir_OnBrowse()
{
    var moduleDir = BrowseForDir();
    if (moduleDir != null) {
        _UpdateModuleDir(moduleDir);
    }
}

