/* Copyright (c) 2000-2001 ActiveState SRL. 
   See the file LICENSE.txt for licensing information. */

/* Web Services Setup Wizard - panel */

//---- globals

var _gPanelName = "chooseInstallation";
var _gFindingInterps = false;


//---- internal support routines

function _CreateTreeItem(label)
{
    const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
    var treecell = document.createElementNS(XUL_NS, 'treecell');
    treecell.setAttribute('label', label);
    var treerow = document.createElementNS(XUL_NS, 'treerow');
    treerow.appendChild(treecell);
    var treeitem = document.createElementNS(XUL_NS, 'treeitem');
    treeitem.appendChild(treerow);
    return treeitem;
}


function _InterpPathToInstallPath(interpreterPath)
{
    var installationPath = null;
    if (interpreterPath) {
        if (parent._gInfoSvc.platform.substring(0,3) == "win") {
            installationPath = parent._gOsPath.dirname(interpreterPath);
        } else {
            installationPath = parent._gOsPath.dirname(parent._gOsPath.dirname(
                interpreterPath));
        }
    }
    return installationPath;
}


function _UpdateInstallation(installation)
{
    // Persist.
    parent.wd_info.interpreter = installation;

    // Display the installation selection.
    var installWidget = document.getElementById("installation");
    if (installation == null) {
        installWidget.value = "";
    } else {
        installWidget.value = installation;
    }
}



//---- Wizard Infrastructure interface routines

function validate()
{
    if (parent.wd_info.interpreter &&
        parent._gOsPath.exists(parent.wd_info.interpreter)) {
        parent._gPhpInfo.executablePath = parent.wd_info.interpreter;
        if (parent._gPhpInfo.valid_version) return true;
        var version = parent._gPhpInfo.version;
        alertDialog("The version of the selected PHP executable "+
                    "("+version+") is not supported by Komodo\n");
    }
    return false;
}


//---- interface for wsSetup_ChooseInstallation.xul

function OnLoad()
{
    // Give a nice header name for this pane.
    var headerWidget = parent.document.getElementById("header");
    headerWidget.setAttribute("title", "Choose Installation");
    headerWidget.setAttribute("description",
        "Select which installation to setup.");

    UpdateInterpreterList()
    _UpdateInstallation(parent.wd_info.interpreter);
    parent.wizardManager.onPageLoad(_gPanelName);
}

function onInit()
{
}

function UpdateInterpreterList()
{
    // setup "avail-install-list"
    var listWidget = document.getElementById("avail-install-list");
    // - remove any existing items and add a "finding..." one
    _gFindingInterps = true;
    while (listWidget.hasChildNodes()) {
        listWidget.removeChild(listWidget.childNodes[0]);
    }
    listWidget.appendChild(_CreateTreeItem(
        "Finding available PHP installation..."));
    // - get a list of installations
    var sysUtils = Components.classes['@activestate.com/koSysUtils;1'].
                   getService(Components.interfaces.koISysUtils);

    // get a list of installed PHP interpreters
    var numFound = new Object();
    var availInterps = parent._gPhpInfo.FindInstallationExecutables(numFound);

    // - populate the tree listing them
    listWidget.removeChild(listWidget.childNodes[0]); // "Finding..."
    if (availInterps.length != 0) {
        for (var i = 0; i < availInterps.length; i++) {
            listWidget.appendChild(_CreateTreeItem(availInterps[i]));
        }
    }
    _gFindingInterps = false;

    // If there is no installation setting or a new language was
    // selected then pick a reasonable default.
    if (!parent.wd_info.interpreter) {
        parent.wd_info.interpreter = parent._gPhpInfo.executablePath;
    }
    _UpdateInstallation(parent.wd_info.interpreter);
}


function UpdateInstallation_OnSelection()
{
    // User selected one of the "found installations".
    if (! _gFindingInterps) {
        // Load the default installation textfield with the selection
        var tree = document.getElementById("avail-install-tree");
        if (tree.selectedItems.length > 0) {
            var selection = tree.selectedItems[0];
            //          treeitem . treerow  . treecell 
            var value = selection.firstChild.firstChild
                        .getAttribute("label");
            _UpdateInstallation(value);
        }
    }
}


function UpdateInstallation_OnKey()
{
    // User typed into the installation field.
    // - Unselect any tree item.
    var tree = document.getElementById("avail-install-tree");
    tree.clearItemSelection();
    
    // - Persist the value.
    var installWidget = document.getElementById("installation");
    _UpdateInstallation(installWidget.value);
}


function UpdateInstallation_OnBrowse()
{
    if (isLinux) {
        filters = [['All Files',['*']]];
    } else {
        filters = [['Executables',['*.exe']]];
    }    
    var php = BrowseForFile(document.getElementById('installation').value, filters)
    if (php != null) {
        _UpdateInstallation(php);
    }
}
