#!python
# Copyright (c) 2000-2001 ActiveState Tool Corporation.
# See the file LICENSE.txt for licensing information.

from xpcom import components
from koLogger import logError
from xmlrpclibBasicAuth import Server
import xmlrpclib

class koASPNService:
    _com_interfaces_ = [components.interfaces.koIASPNService]
    _reg_clsid_ = "{b69dcf44-21e3-4f7a-a63c-06fb6194e141}"
    _reg_contractid_ = "@activestate.com/koASPNService;1"
    _reg_desc_ = "Komodo ASPN"

    baseURL = 'http://aspn.activestate.com/ASPN/Cookbook'
    testURL = 'http://stapler.activestate.com:8080/ASPN/Cookbook'
    authenticated = 0
    whyFailure = ''

    def login(self, username, password):
        try:
            retval = xmlrpclib.Server(self.baseURL).ASPN.login(username, password)
        except xmlrpclib.Fault, e:
            self.coookie = ''
            self.whyFailure = e.faultString
            return 0

        if retval.find('ASPNauth="' + username) != -1:
            self.authenticated = 1
            self.cookie = retval
        else:
            self.authenticated = 0
            self.whyFailure = 'UNKNOWN'
        return self.authenticated

    def logout(self):
        self.authenticated = 0
        self.cookie = ''

    def authenticate_with_cookie(self, cookie):
        self._server = Server(self.baseURL, None, None, cookie)
        if self._server.can_add():
            self.authenticated = 1
            self.whyFailure = ''
            self.cookie = cookie
        else:
            logError(str(retval))
            self.whyFailure = retval[0]
            self.authenticated = 0
        return self.authenticated

class koCookbookService(koASPNService):
    _com_interfaces_ = [components.interfaces.koICookbookService]
    _reg_clsid_ = "{5F30FF99-697A-49c3-8051-804B74399D96}"
    _reg_contractid_ = "@activestate.com/koCookbookService;1"
    _reg_desc_ = "Komodo ASPN Cookbook Service"

    _cookbookNames = None
    _cookbookURLs = {}
        
    def getCookbookNames(self):
        self._cookbookNames= self._server.list_cookbooks()
        _cookbookInfo = self._server.get_cookbooks()
        for info in _cookbookInfo:
            self._cookbookURLs[info['type']] = info['url']
        return self._cookbookNames

    def getCategories(self, cookbookName):
        if self._cookbookNames is None:
            self.getCookbookNames()
        assert cookbookName in self._cookbookNames, "Unknown cookbook: %s' % cookbookName"
        
        return self._server.list_categories(cookbookName)

    def submitRecipe(self, cookbookName, recipe):
        if self._cookbookNames is None:
            self.getCookbookNames()
        assert cookbookName in self._cookbookNames, "Unknown cookbook: %s' % cookbookName"

        r = {'data_type':'Article',
             'COOKBOOK_TYPE': cookbookName,
             'subtitle': recipe.title,
             'description': recipe.summary,
             'formatted_content' : recipe.code,
             'discussion': recipe.discussion,
             'keyword': recipe.category}
        retval = self._server.add_recipe(r)
        if isinstance(retval, type(0)):
            return self.baseURL + self._cookbookURLs[cookbookName] + '/Recipe/' + str(retval)
        else:
            raise ServerException

class koRecipe:
    _com_interfaces_ = [components.interfaces.koIRecipe]
    _reg_clsid_ = "{94744BF7-794A-4e1c-97B5-0F1EA8689935}"
    _reg_contractid_ = "@activestate.com/koRecipe;1"
    _reg_desc_ = "Komodo ASPN Recipe"
    
if __name__ == "__main__":
	username='davida@activestate.com'
	password='bar'
        try:
             print xmlrpclib.Server(testURL).ASPN.login(username, password)
        except xmlrpclib.Fault, e:
             print e.faultString
	password='foo'
        cookie = xmlrpclib.Server(testURL).ASPN.login(username, password)
        print '1'*10 + ' ' + cookie
        retval = Server(testURL, '', '', cookie)
        print retval.can_add()
        x = koASPNService()
        print x.login(username, password)
        print x.authenticate_with_cookie(x.cookie)
