"""Language package for HTML"""

from xpcom import components, ServerException

from koLanguageServiceBase import *

class koHTMLLanguage(KoLanguageBase):
    name = "HTML"
    _reg_desc_ = "%s Language" % name
    _reg_contractid_ = "@activestate.com/koLanguage?language=%s;1" \
                       % (name)
    _reg_clsid_ = "{7FDF82EB-7E8E-40f4-BD39-563FD2B7E94A}"

    defaultExtension = ".html" # XXX how do we deal w/ .htm vs. html?
    #XXX Boy it would be nice to have this be 6 so we could get green
    #    squigglies for linting.
    styleBits = 7
    commentDelimiterInfo = { "block": [ ("<!--", "-->") ]  }
    indicatorBits = 1

    def get_lexer(self):
        if self._lexer is None:
            self._lexer = KoLexerLanguageService()
            self._lexer.setLexer(components.interfaces.ISciMoz.SCLEX_HTML)
            # XXX config options needed?
            self._lexer.setProperty('fold.html','0')
            #self._lexer.setProperty('fold.compact','1')
            self._lexer.supportsFolding = 0
            # keyword sets
            # 0 = html # 1 = JavaScript # 2 = VBScript # 3 = PythonScript # 4 = PHP
            self._lexer.setKeywords(1, self._js_keywords)
            self._lexer.setKeywords(2, self._vb_keywords)
            self._lexer.setKeywords(4, self._php_keywords)
        return self._lexer


    def get_linter(self):
        if self._linter is None:
            self._linter = KoLinterLanguageService( 
                "@activestate.com/koHTMLLinter;1",
                None)
        return self._linter

    def get_commenter(self):
        if self._commenter is None:
            self._commenter = KoCommenterLanguageService(
                self.commentDelimiterInfo)
        return self._commenter

    # XXX later these need to be centralized. koPHPLanguage.py has the same
    _php_keywords = ["for", "break", "continue", "if", "else",
                 "elseif", "endif", "do", "while", "return",
                 "switch", "case", "function", "class",
                 "new", "static", "define", "__LINE__",
                 "__FILE__", "PHP_VERSION", "PHP_OS", "TRUE",
                 "FALSE", "E_ERROR", "E_WARNING", "E_PARSE",
                 "E_NOTICE", "int", "integer", "real", "double",
                 "float", "string", "array", "object", "var",
                 "extends", "cfunction", "and", "default",
                 "empty", "endfor", "endswitch", "endwhile",
                 "E_ALL", "foreach", "include", "include_once",
                 "require", "require_once", "global", "list",
                 "or", "this", "xor", "virtual", "echo"]

    _js_keywords = ["abstract", "boolean", "break", "byte", "case", "catch",
                 "char", "class", "const", "continue", "debugger", "default",
                 "delete", "do", "double", "else", "enum", "export",
                 "extends", "final", "finally", "float", "for", "function",
                 "goto", "if", "implements", "import", "in", "instanceof",
                 "int", "interface", "long", "native", "new", "package",
                 "private", "protected", "public", "return", "short",
                 "static", "super", "switch", "synchronized", "this", "throw",
                 "throws", "transient", "try", "typeof", "var", "void",
                 "volatile", "while", "with"]

    _vb_keywords = ["and", "as", "attribute", "begin", 
                    "byref", "byval", "case", "call", "const",
                    "continue", "dim", "do", "each", "else",
                    "elseif", "end", "error", "exit", "false",
                    "for", "function", "global", "goto", "if",
                    "in", "loop", "me", "new", "next", "not",
                    "nothing", "on", "optional", "or", "private",
                    "public", "redim", "rem", "resume", "select",
                    "set", "sub", "then", "to", "true", "type",
                    "while", "with", "boolean", "byte", "currency",
                    "date", "double", "integer", "long", "object",
                    "option", "single", "string", "type", "variant"]
