import fnmatch
from types import *


from testable import MultiTestable
from xpcom import components, nsError, ServerException
import xpcom.server
from xpcom.server.enumerator import SimpleEnumerator
from koBasicTypes import koString
import time, string


class koLanguageItem:
    _com_interfaces_ = [components.interfaces.koIHierarchyItem]
    _reg_contractid_ = "@activestate.com/koLanguageItem;1"
    _reg_clsid_ = "{33532bc1-302a-4b2d-ad14-13c5eadf4d93}"
    
    def __init__(self, language):
        self.language = language
        self.name = language
        
    def get_available_types(self):
        return components.interfaces.koIHierarchyItem.ITEM_STRING

    def get_item_string(self):
        return self.language

    def get_container(self):
        return 0

class koLanguageContainer:
    _com_interfaces_ = [components.interfaces.koIHierarchyItem]
    _reg_contractid_ = "@activestate.com/koLanguageContainer;1"
    _reg_clsid_ = "{878ed885-6274-4c07-9668-a9a01a0ae09c}"
    
    def __init__(self, label, languages):
        self.name = label
        self.languages = languages
    
    def getChildren(self):
        return self.languages

    def get_available_types(self):
        return 0
        return components.interfaces.koIHierarchyItem.ITEM_STRING

    def get_item_string(self):
        return 0
        return self.label

    def get_container(self):
        return 1
    
# The LanguageRegistryService keeps track of which languages/services
# are available.  It is used by packages of services to inform the
# system of their presence, and is used internally by the other
# language services classes to get information on the services
# available.

class koLanguageRegistryService:
    _com_interfaces_ = [components.interfaces.koILanguageRegistryService,
                        components.interfaces.nsIObserver]
    _reg_contractid_ = "@activestate.com/koLanguageRegistryService;1"
    _reg_clsid_ = "{4E76795E-CC92-47c6-8801-C9ACFC1B02E3}"

    def __init__(self):
        self.__languages = {}
        self.__accesskeys = {}
        self.__filters = None
        self.__filtersByLanguage = None
        self.__associations = None
        self.__absolutefilters = {}
        self.__initPrefs()
        self._resetFileAssociations()

    def Observe(self, aSubject, aTopic, someData):
        if someData == "fileAssociations":
            self.__filters = None
            self.__filtersByLanguage = None
            self.__associations = []
            self._resetFileAssociations()
            
    def _resetFileAssociations(self):
        associations = self.__prefs.getPref("fileAssociations") \
                       .QueryInterface(components.interfaces.koIOrderedPreference)
        self.__filters = []
        self.__filtersByLanguage = {}
        self.__associations = []
        self.__absolutefilters = {}
        for i in range(0, associations.length):
            association = associations.getPref(i) \
                          .QueryInterface(components.interfaces.koIOrderedPreference)
            filterMatch = association.getStringPref(0)
            filterLanguage = association.getStringPref(1)

            # set filters
            self.__filters.append(filterMatch)

            # set filters by language
            if not self.__filtersByLanguage.has_key(filterLanguage):
                self.__filtersByLanguage[filterLanguage] = []
            self.__filtersByLanguage[filterLanguage].append(filterMatch)

            # get the file extensions only
            if filterMatch[:2] == "*.":
                self.__absolutefilters[filterMatch[2:]] = filterLanguage
            self.__absolutefilters[filterMatch] = filterLanguage
            
            # set associations
            self.__associations.append((filterMatch,filterLanguage))

    def getLanguageHierarchy(self):
        # We could determine primary languages programmatically but seeing which
        # features each language offers, but that would involve creating an
        # XPCOM object for each language.

        primary = ['HTML', 'JavaScript', 'PHP', 'Perl', 'Python', 'Tcl', 'Text', 'XML', 'XSLT']
        secondary = [i for i in self.__languages.keys() if i not in primary]
        secondary.sort()

        otherContainer = koLanguageContainer('Other', [koLanguageItem(i) for i in secondary])
        return koLanguageContainer('', [koLanguageItem(i) for i in primary] + [otherContainer])
                
    def getLanguages(self):

        languages = self.__languages.keys()
        languages.sort()
        return languages
    
    def getFilters(self):
        if self.__filters is None:
            self._resetFileAssociations()

        return self.__filters

    def getFiltersForLanguage(self, language):
        if self.__filtersByLanguage is None:
            self._resetFileAssociations()

        if self.__filtersByLanguage.has_key(language):
            filters = self.__filtersByLanguage[language]
        else:
            filters = []

        return filters

    def registerLanguage(self, language, accesskey):
        assert not self.__languages.has_key(language), \
               "Language '%s' already registered" % (language)
        
        self.__languages[language] = None
        self.__accesskeys[language] = accesskey

    def getAccessKeys(self):
        languages = self.__languages.keys()
        languages.sort()
        return [self.__accesskeys[language] for language in languages]
    
    def getLanguage(self, language):
        # return a koILanguage for language.  Create it if it does not
        # exist yet.
        if not language: language="Text"
        
        assert self.__languages.has_key(language), \
               "Attempt to get service for unregistered language '%s'" \
               % (language)

        if self.__languages[language] is None:
            contractid = "@activestate.com/koLanguage?language=%s;1" \
                         % (language)
            self.__languages[language] = components.classes[contractid] \
                    .createInstance(components.interfaces.koILanguage)

        return self.__languages[language]

    def suggestLanguageForFile(self, filename):
        if self.__associations is None:
            self._resetFileAssociations()

        language = ''  # default

        # get the file extension
        parts = string.split(filename,".")

        if len(parts) > 0:
            ext = parts[len(parts)-1];
            if self.__absolutefilters.has_key(ext):
                return self.__absolutefilters[ext]

        if self.__absolutefilters.has_key(filename):
            return self.__absolutefilters[filename]
        
        # check for file pattern matches        
        for (pat, lang) in self.__associations:
            if fnmatch.fnmatch(filename, pat):
                language = lang
                break
            
        return language
        
    def __initPrefs(self):
        self.__prefs = components \
                       .classes["@activestate.com/koPrefService;1"] \
                       .getService(components.interfaces.koIPrefService)\
                       .prefs
        # Observers will be QI'd for a weak-reference, so we must keep the
        # observer alive ourself, and must keep the COM object alive,
        # _not_ just the Python instance!!!
        # XXX - this is a BUG in the weak-reference support.
        # It should NOT be necessary to do this, as the COM object is
        # kept alive by the service manager.  I suspect that this bug
        # happens due to the weak-reference being made during
        # __init__.  FIXME!
        self._observer = xpcom.server.WrapObject(self,
                                      components.interfaces.nsIObserver)
        self.__prefs.AddObserver(self._observer)



class koSourceFile:
    _com_interfaces_ = [components.interfaces.koISourceFile]
    _reg_contractid_ = "@activestate.com/koSourceFile;1"
    _reg_clsid_ = "{D579D901-7DBF-4fa6-AE7A-1A85DAC9CA83}" 

    _registry = None
    def __init__(self):
        if koSourceFile._registry is None:
            koSourceFile._registry = components.classes["@activestate.com/koLanguageRegistryService;1"].getService(components.interfaces.koILanguageRegistryService) 
        
    def init(self, scintilla):
        self._scin = scintilla
        self.suggestLanguage("Text")

    def getLanguageBlocks(self):
        #return SimpleEnumerator(self.__languageBlocks)
        return self._languageBlocks

    def getLanguageBlockAtPosition(self, position):
        # for now, we only allow one language block per file
        assert 0 <= position <= self._scin.length, "position out of range"
        assert len(self._languageBlocks) == 1, "internal inconsistency"
        
        return self._languageBlocks[0]

    def suggestLanguage(self, language):
        self._languageBlocks = [
            self._createLanguageBlock(language, 0, self._scin.length)
            ]
        
    def suggestFilename(self, basename):
        # make the association and create a language block appropriately
        language = koSourceFile._registry.suggestLanguageForFile(basename)
        self.suggestLanguage(language or 'Text')

    def _createLanguageBlock(self, language, offset, length):
        return koLanguageBlock(self._scin, language, offset, length, koSourceFile._registry)


class koLanguageBlock:
    _com_interfaces_ = [components.interfaces.koILanguageBlock]

    def __init__(self, scintilla, language, offset, length, registry):
        # for now, we only allow one language block per file
        assert offset == 0
        assert length == scintilla.length
        
        self.__scin = scintilla
        self.offset = offset
        self.length = length
        self.__language = language
        self.__registry = registry


    def get_language(self):
        return self.__registry.getLanguage(self.__language)
    
    def get_text(self):
        assert self.offset+self.length <= self.__scin.length
        
        return self.__scin.text[self.offset:self.offset+self.length]


    def getRange(self, start, end):
        assert self.offset+start <= self.__scin.length
        assert self.offset+end <= self.__scin.length
        assert start <= end
        
        return self.get_text()[start:end]



# Unit Tests

# Doesn't do much of anything.  Used just to test if we can get
# to a language service.
class koTestLanguage:
    _com_interfaces_ = [components.interfaces.koILanguage]
    name = "Test"
    _reg_contractid_ = "@activestate.com/koLanguage?language=%s;1" % (name)
    _reg_clsid_ = "{EF2B949C-ECEC-4338-8960-CE2CB2822017}"

    def getLanguageService(self, iid):
        if iid == components.interfaces.koITestLanguageService:
            return koTestLanguageService()
        else:
            return None


class koTestLanguageService:
    _com_interfaces_ = [components.interfaces.koITestLanguageService]

    def getTestString(self):
        return "All systems go!"


# Used so we can construct tests that do not depend on having a real
# Scintilla available.  Only contains text.
class FakeScintilla:
    _com_interfaces_ = [components.interfaces.ISciMoz]

    def __init__(self, text):
        self.text = text
        self.length = len(text)


class TestKoLanguageRegistryService(MultiTestable):
    _com_interfaces_ = [components.interfaces.koITestable,
                        components.interfaces.koIMultiTestable]
    unitName = "koLanguageRegistryService"
    _reg_contractid_ = "@activestate.com/koTestable-%s;1" % (unitName)
    _reg_clsid_ = "{567462DF-BAE1-4c9d-9CE1-7D4A3EBA2C36}"

    def test_registerOneLanguage(self):
        testName = "registerOneLanguage"

        registry_contractid = "@activestate.com/koLanguageRegistryService;1"
        registry_iid = components.interfaces.koILanguageRegistryService
        registry = components.classes[registry_contractid] \
                   .createInstance(registry_iid)

        registry.registerLanguage("Test","t")

        languages=registry.getLanguages()

        if languages == ["Test"]:
            return "PASS: %s" % (testName)
        else:
            return "FAIL: %s" % (testName)

    def test_registerManyLanguages(self):
        testName = "registerManyLanguages"

        registry_contractid = "@activestate.com/koLanguageRegistryService;1"
        registry_iid = components.interfaces.koILanguageRegistryService
        registry = components.classes[registry_contractid] \
                   .createInstance(registry_iid)

        registry.registerLanguage("Test","t")
        registry.registerLanguage("Test2","t")

        languages=registry.getLanguages()

        # to pass the test, there must be exactly one instance of
        # "Test", and exactly one instance of "Test2", no repetitions
        # or additional entries are permitted

        found_items = []
        for language in languages:
            if language in found_items:
                return "FAIL: %s, duplicate entries" % (testName)
            elif language not in ["Test", "Test2"]:
                return "FAIL: %s, extra entries" % (testName)
            else:
                found_items.append(language)

        if len(found_items) == 2:
            return "PASS: %s" % (testName)
        else:
            return "FAIL: %s, missing entries" % (testName)


    def test_getLanguageOnce(self):
        testName = "getLanguageOnce"

        registry_contractid = "@activestate.com/koLanguageRegistryService;1"
        registry_iid = components.interfaces.koILanguageRegistryService
        registry = components.classes[registry_contractid] \
                   .createInstance(registry_iid)

        registry.registerLanguage("Test","t")

        language = registry.getLanguage("Test")

        return "PASS: %s" % (testName)

    def test_getLanguageTwice(self):
        testName = "getLanguageTwice"

        registry_contractid = "@activestate.com/koLanguageRegistryService;1"
        registry_iid = components.interfaces.koILanguageRegistryService
        registry = components.classes[registry_contractid] \
                   .createInstance(registry_iid)

        registry.registerLanguage("Test","t")

        # I'd like to test for identity here, but if you wrap the same
        # Python object twice, you get different XPCOM objects.  An
        # equality test is the closest we can come.
        
        if registry.getLanguage("Test") == registry.getLanguage("Test"):
            return "PASS: %s" % (testName)
        else:
            return "FAIL: %s" % (testName)

    def test_getLanguageService(self):
        testName = "getLanguageService"

        registry_contractid = "@activestate.com/koLanguageRegistryService;1"
        registry_iid = components.interfaces.koILanguageRegistryService
        registry = components.classes[registry_contractid] \
                   .createInstance(registry_iid)

        registry.registerLanguage("Test","t")

        language = registry.getLanguage("Test")
        service = language.getLanguageService(components.interfaces.koITestLanguageService)

        if service and service.getTestString() == "All systems go!":
            return "PASS: %s" % (testName)
        else:
            return "FAIL: %s" % (testName)
  


class TestKoLanguage(MultiTestable):
    _com_interfaces_ = [components.interfaces.koITestable,
                        components.interfaces.koIMultiTestable]
    unitName = "koLanguage"
    _reg_contractid_ = "@activestate.com/koTestable-" + unitName + ";1"
    _reg_clsid_ = "{55BEC619-7468-4647-8B57-92DCDDD92A4E}"

    def __init__(self):
        self.__registry = components.classes \
                   ["@activestate.com/koLanguageRegistryService;1"] \
                   .getService(components.interfaces.koILanguageRegistryService)

        languages=self.__registry.getLanguages()
            
        if "Test" not in languages:
            self.__registry.registerLanguage("Test","t")

    def test_retrieveTextFromLanguageBlock(self):
        text = "import callkomodo;callkomdo.brk()"
        scin = FakeScintilla(text)
        sourceFile = components.classes["@activestate.com/koSourceFile;1"] \
                     .createInstance(components.interfaces.koISourceFile)
        sourceFile.init(scin)
        sourceFile.suggestLanguage("Test")

        languageBlocks = sourceFile.getLanguageBlocks()        
        languageBlock = languageBlocks[0].QueryInterface(components.interfaces.koILanguageBlock)
        if languageBlock.text == text:
            return "PASS: retrieveTextFromLanguageBlock"
        else:
            return "FAIL: retrieveTextFromLanguageBlock"
        

    def test_languageBlockGetRange(self):
        text = "import callkomodo;callkomdo.brk()"
        scin = FakeScintilla(text)
        sourceFile = components.classes["@activestate.com/koSourceFile;1"] \
                     .createInstance(components.interfaces.koISourceFile)
        sourceFile.init(scin)
        sourceFile.suggestLanguage("Test")
        languageBlocks = sourceFile.getLanguageBlocks()        
        languageBlock = languageBlocks[0].QueryInterface(components.interfaces.koILanguageBlock)
        if languageBlock.getRange(7, 17) == text[7:17]:
            return "PASS: languageBlockGetRange"
        else:
            return "FAIL: languageBlockGetRange"

    def test_languageBlocksCanAccessLanguageServices(self):
        text = "import callkomodo;callkomdo.brk()" 
        scin = FakeScintilla(text)
        sourceFile = components.classes["@activestate.com/koSourceFile;1"] \
                     .createInstance(components.interfaces.koISourceFile)
        sourceFile.init(scin)
        sourceFile.suggestLanguage("Test")
        languageBlocks = sourceFile.getLanguageBlocks()        
        languageBlock = languageBlocks[0].QueryInterface(components.interfaces.koILanguageBlock)
        myLanguageService = languageBlock.language.getLanguageService(
            components.interfaces.koITestLanguageService)

        expected_result = "All systems go!"
        actual_result = myLanguageService.getTestString()
        if actual_result == expected_result:
            return "PASS: languageBlocksCanAccessLanguageServices"
        else:
            return "FAIL: languageBlocksCanAccessLanguageServices" \
                   + "(expected '%s', got '%s')" \
                   % (expected_result, actual_result)





