#!python
# Copyright (c) 2000-2001 ActiveState Tool Corporation.
# See the file LICENSE.txt for licensing information.

from koLogger import logWarning, logError, logDebug
from xpcom import components

def RGB(r,g,b): return r+g*256+b*256*256

class KoLintDisplayer:
    _com_interfaces_ = [components.interfaces.koILintDisplayer]
    _reg_desc_ = "Komodo Lint Displayer"
    _reg_clsid_ = "{21648855-492F-11d4-AC24-0090273E6A60}"
    _reg_contractid_ = "@activestate.com/koLintDisplayer;1"

    def __init__(self):
        self._in_display = 0 # recursion protection.

    def displayClear(self, scin, numBits):
#        print "Clearing old results"
        # clear the old results
        if numBits == 2:
            mask = scin.INDIC1_MASK | scin.INDIC2_MASK
        else:
            mask = scin.INDIC2_MASK
        self._in_display = 1
        try:
            prevEndStyled = scin.endStyled
            scin.startStyling(0, mask)
            scin.setStyling(scin.length, 0)

            # ensure scin.endStyled = prevEndStyled so the lexer does not get
            # confused.
            scin.startStyling(prevEndStyled, mask)
            scin.setStyling(0,0)
            assert prevEndStyled == scin.endStyled, "unexpected end styled %d/%d!!" % (prevEndStyled, scin.endStyled)

            # This code is called by a timer, not as the text is styled for painting
            # so if we restyled anything currently on screen, we need to repaint.
            # no "clean" way to do this!
            scin.tabWidth = scin.tabWidth # this forces a repaint.

        finally:
            self._in_display = 0
    
    def display(self, scin, lintResults, numBits, startPos, styleLen):
        if self._in_display:
            return
        self._in_display = 1
        try:
            return self._display(scin, lintResults, numBits, startPos, styleLen);
        finally:
            self._in_display = 0

    def _display(self, scin, lintResults, numBits, startPos, styleLen):
        assert lintResults is not None, "Should always have results to display"
        if scin.length == 0:
            return
        offsetsAndValues = []
#        print "_display from", startPos, "length =", scin.length, "endStyled =", scin.endStyled, "results=", lintResults

        if numBits == 2:
            mask = scin.INDIC1_MASK | scin.INDIC2_MASK
        else:
            mask = scin.INDIC2_MASK

        # stash these for efficiency
        firstLine = scin.lineFromPosition(startPos)
        endLine = scin.lineFromPosition(startPos+styleLen)

        displayableResults = lintResults.getResultsInLineRange(firstLine + 1, endLine + 1)

        for r in displayableResults:
            #XXX Why do I have to QI this?  It seems PyXPCOM's not doing it's job
            r = r.QueryInterface(components.interfaces.koILintResult)
            # sanity check lint results
            if (r.columnStart >= 1 and r.columnEnd >= 1 and r.lineStart >= 1 and r.lineEnd >= 1) \
                and (r.lineEnd > r.lineStart or (r.lineEnd == r.lineStart and r.columnEnd >= r.columnStart)):
            
                offsetStart = scin.positionFromLine(r.lineStart - 1) + r.columnStart - 1
                offsetEnd = scin.positionFromLine(r.lineEnd-1) + r.columnEnd - 1

                if offsetEnd <= scin.length and offsetEnd >= offsetStart:
                    if r.lineEnd > scin.lineCount:
                        offsetEnd = scin.length
                    length = offsetEnd-offsetStart
                    if numBits == 2:
                        if r.severity == r.SEV_ERROR:
                            value = scin.INDIC2_MASK
                        else:
                            value = scin.INDIC1_MASK
                    else:
                        value = scin.INDIC2_MASK

                    offsetsAndValues.append((offsetStart, length, value))
                else:
                    logError("Suspicious lint result discarded (offsetEnd > scin.length or offsetStart > offsetEnd): lineStart = %d, columnStart = %d, lineEnd = %d, columnEnd = %d" % (r.lineStart, r.columnStart, r.lineEnd, r.columnEnd))
            else:
                logError("Suspicious lint result discarded (sanity check failed): lineStart = %d, columnStart = %d, lineEnd = %d, columnEnd = %d"% (r.lineStart, r.columnStart, r.lineEnd, r.columnEnd))

        if not offsetsAndValues:
            return

        prevEndStyled = scin.endStyled

        for start, length, value in offsetsAndValues:
            #XXX we may want to detect and ignore
            #XXX repeated ranges to boost performance
            #XXX Perl, in particular, seems to give
            #XXX many results on the same line, sometimes
            if length > 0: # one last sanity check, as if that's not true it can cause a lockup
                scin.startStyling(start, mask)
                scin.setStyling(length, value)

        # ensure scin.endStyled = prevEndStyled so the lexer does not get
        # confused.
        scin.startStyling(prevEndStyled, mask)
        scin.setStyling(0,0)

        # This code is called by a timer, not as the text is styled for painting
        # so if we restyled anything currently on screen, we need to repaint.
        # no "clean" way to do this!
        scin.tabWidth = scin.tabWidth # this forces a repaint.
        assert prevEndStyled == scin.endStyled, "unexpected end styled %d/%d!!" % (prevEndStyled, scin.endStyled)

