#!/usr/bin/env python
# Copyright (c) 2000-2002 ActiveState SRL.
# See the file LICENSE.txt for licensing information.

"""
Komodo Logging system service
"""

import logging
from xpcom import components, nsError, ServerException, COMException, server

class koLoggingService:
    _com_interfaces_ = [components.interfaces.koILoggingService]
    _reg_clsid_ = "{5DBBF27D-F084-432B-9422-9C9B4707452D}"
    _reg_contractid_ = "@activestate.com/koLoggingService;1"
    _reg_desc_ = "Komodo Logging system service"

    def __init__(self):
        hdlr = logging.StreamHandler()
        fmt = logging.Formatter("%(levelname)s: %(name)s: %(message)s")
        hdlr.setFormatter(fmt)
        logging.root.addHandler(hdlr)
        
    def getLogger(self, logger_name):
        logger = koLogger(logger_name)
        logger = server.WrapObject(logger, components.interfaces.koILogger)
        return logger
    
    def getLoggerNames(self):
        return logging.Logger.manager.getLoggerNames()
    
class koLogger:
    _com_interfaces_ = [components.interfaces.koILogger]
    _reg_clsid_ = "{EE4DC45E-1BE7-4C3C-B521-1434FA1CF054}"
    _reg_contractid_ = "@activestate.com/koLogger;1"
    _reg_desc_ = "Komodo Logger"

    def __init__(self, name):
        self._logger = logging.Logger.manager.getLogger(name)

    def setLevel(self, level):
        if not level:
            raise ServerException, "Invalid level"
        self._logger.setLevel(level)
        
    def getEffectiveLevel(self):
        return self._logger.getEffectiveLevel()

    def isEnabledFor(self, level):
        return self._logger.isEnabledFor(level)
    
    def debug(self, msg):
        return self._logger.debug(msg)

    def info(self, msg):
        return self._logger.info(msg)

    def warn(self, msg):
        return self._logger.warn(msg)

    def error(self, msg):
        return self._logger.error(msg)

    def critical(self, msg):
        return self._logger.critical(msg)
