#!python
# Copyright (c) 2000-2002 ActiveState SRL.
# See the file LICENSE.txt for licensing information.

# The implementation of the Komodo Run Output Window Terminal

import os
import sys
import re
import types
import threading
import process
import time

from koSciMozEvents import SciMozEvents
from koEditorController import CommandHandler
from xpcom import components, nsError, ServerException, COMException
from xpcom._xpcom import PROXY_SYNC, PROXY_ALWAYS, PROXY_ASYNC, getProxyForObject
from koLogger import logWarning, logError, logDebug


#---- globals



#---- internal support classes and routines

class _OutlinerView:
    """A base implementation of the nsIOutlinerView. This
    should be subclassed to get any useful behaviour."""
    _com_interfaces_ = [components.interfaces.nsIOutlinerView]
        
    def __init__(self, debug=0):
        self._debug = debug
        self.selection = None   # nsIOutlinerSelection

    def isSeparator(self, index):
        return 0
    
    def get_rowCount( self ):
        # Result: int32
        if self._debug:
            logDebug("get_rowCount()")
        return 0
    def getRowProperties( self, index, properties ):
        # Result: void - None
        # In: param0: int32
        # In: param1: nsISupportsArray
        if self._debug:
            logDebug("getRowProperties(%s, %s)" % (index, properties))
        pass
    def getCellProperties(self, row, colID, properties):
        # Result: void - None
        # In: param0: int32
        # In: param1: wstring
        # In: param2: nsISupportsArray
        if self._debug:
            logDebug("getCellProperties()")
        pass
    def getColumnProperties(self, colID, colElt, properties):
        # Result: void - None
        # In: param0: wstring
        # In: param1: nsIDOMElement
        # In: param2: nsISupportsArray
        if self._debug:
            logDebug("getColumnProperties(colID=%s, colElt=%s, props=%s)"\
                     % (colID, colElt, properties))
        pass
    def isContainer(self, index):
        # Result: boolean
        # In: param0: int32
        if self._debug:
            logDebug("isContainer()")
        return 0
    def isContainerOpen(self, index):
        # Result: boolean
        # In: param0: int32
        if self._debug:
            logDebug("isContainerOpen()")
        return 0
    def isContainerEmpty( self, param0 ):
        # Result: boolean
        # In: param0: int32
        if self._debug:
            logDebug("isContainerEmpty()")
        pass
    def isSorted( self ):
        # Result: boolean
        if self._debug:
            logDebug("isSorted()")
        return 0
    def canDropOn( self, param0 ):
        # Result: boolean
        # In: param0: int32
        if self._debug:
            logDebug("canDropOn()")
        return 0
    def canDropBeforeAfter( self, param0, param1 ):
        # Result: boolean
        # In: param0: int32
        # In: param1: boolean
        if self._debug:
            logDebug("canDropBeforeAfter()")
        return 0
    def drop( self, param0, param1 ):
        # Result: void - None
        # In: param0: int32
        # In: param1: int32
        if self._debug:
            logDebug("drop()")
        pass
    def getParentIndex( self, param0 ):
        # Result: int32
        # In: param0: int32
        if self._debug:
            logDebug("getParentIndex()")
        #XXX huh?
        pass
    def hasNextSibling( self, param0, param1 ):
        # Result: boolean
        # In: param0: int32
        # In: param1: int32
        if self._debug:
            logDebug("hasNextSibling()")
        return 0
    def getLevel(self, index):
        # Result: int32
        # In: param0: int32
        if self._debug:
            logDebug("getLevel()")
        return 0
    def getCellText(self, row, colID):
        # Result: wstring
        # In: param0: int32
        # In: param1: wstring
        if self._debug:
            logDebug("getCellText()")
            return "row: %s, col: %s" % (row, colID)
        else:
            return ""
    def setOutliner(self, outliner):
        # Result: void - None
        # In: param0: nsIOutlinerBoxObject
        #XXX may move to be an attribute.
        if self._debug:
            logDebug("setOutliner()")
        if outliner is None:
            self._outliner = outliner
        else:
            outlinerProxy = getProxyForObject(0,
                components.interfaces.nsIOutlinerBoxObject,
                outliner,
                PROXY_ALWAYS | PROXY_SYNC)
            self._outliner = outlinerProxy
    def toggleOpenState(self, index):
        # Result: void - None
        # In: param0: int32
        if self._debug:
            logDebug("toggleOpenState()")
        pass
    def cycleHeader(self, colID, elt):
        # Result: void - None
        # In: param0: wstring
        # In: param1: nsIDOMElement
        if self._debug:
            logDebug("cycleHeader()")
        pass
    def selectionChanged(self):
        # Result: void - None
        if self._debug:
            logDebug("selectionChanged()")
        pass
    def cycleCell(self, row, colID):
        # Result: void - None
        # In: param0: int32
        # In: param1: wstring
        if self._debug:
            logDebug("cycleCell()")
        pass
    def isEditable(self, row, colID):
        # Result: boolean
        # In: param0: int32
        # In: param1: wstring
        if self._debug:
            logDebug("isEditable()")
        return 0
    def setCellText( self, param0, param1, param2 ):
        # Result: void - None
        # In: param0: int32
        # In: param1: wstring
        # In: param2: wstring
        if self._debug:
            logDebug("setCellText()")
        pass
    def performAction(self, action):
        # Result: void - None
        # In: param0: wstring
        if self._debug:
            logDebug("performAction(%s)" % action)
        pass
    def performActionOnRow(self, action, row):
        # Result: void - None
        # In: param0: wstring
        # In: param1: int32
        if self._debug:
            logDebug("performActionOnRow(%s, %s)" % (action, row))
        pass
    def performActionOnCell(self, action, row, colID):
        # Result: void - None
        # In: param0: wstring
        # In: param1: int32
        # In: param2: wstring
        if self._debug:
            logDebug("performActionOnCell(%s, %s, %s)" % (action, row, colID))
        pass

def _isurl(path):
    urlpattern = re.compile("^\w+://")
    if urlpattern.match(path):
        return 1
    else:
        return 0



#---- implementation

class _KoRunTerminalFile(process.IOBuffer):
    """A std handler for the Run output window.
    I.e. one each for <stdin>, <stdout>, and <stderr>.
    """
    _com_interfaces_ = [components.interfaces.koIFile]

    def __init__(self, name, terminal, scintilla, passBackLines=0):
        """Create a terminal file with the given name.
        "passBackLines" indicates whether this stream should use the
            terminal's .parseAndAddLine() method to have output lines parsed
            into its outliner.
        """
        #print "[%s] _KoRunTerminalFile.__init__(name=%r)" % (name, name)
        self._terminal = terminal
        self._scintilla = scintilla
        self._passBackLines = passBackLines
        if self._passBackLines:
            self._pbbuf = ""  # pass-back buffer
        process.IOBuffer.__init__(self, self._terminal.mutex,
                                  self._terminal.stateChange, name=name)

    def _doWrite(self, s):
        #print "[%s] _KoRunTerminalFile._doWrite: write in: %r"\
        #      % (self._name, s)
        self._scintilla.readOnly = 0
        if self._name == "<stderr>":
            start = self._scintilla.length
            self._scintilla.addText(len(s), s)
            # we want to set all of the non-indicator bits.  This means 5 LSBs, which appears to be the default
            self._scintilla.startStyling(start, 0x1f);  
            self._scintilla.setStyling(len(s), self._scintilla.SCE_C_COMMENT);
        else:
            self._scintilla.addText(len(s), s)
        self._scintilla.gotoPos(self._scintilla.length)
        self._scintilla.readOnly = 1
        
        if self._passBackLines:
            self._pbbuf += s
            lines = self._pbbuf.splitlines(1)
            for line in lines:
                if line.endswith("\r\n") or line.endswith("\n") or line.endswith("\r"):
                    self._terminal.parseAndAddLine(line)
                else:
                    # This must be the last (incomplete line).
                    self._pbbuf = line
                    break
            else:
                self._pbbuf = ""

    def _doRead(self, n):
        """Pop 'n' bytes from the terminal in-buffer and return them."""
        #print "[%s] _KoRunTerminalFile._doRead(n=%r)" % (self._name, n)
        if n < 0:
            idx = len(self._terminal.inBuffer)
        else:
            idx = min(n, len(self._terminal.inBuffer))
        retval = self._terminal.inBuffer[:idx]
        self._terminal.inBuffer = self._terminal.inBuffer[idx:]
        return retval

    def _doReadline(self, n):
        """Pop the front line (or n bytes of it, whichever is less) from
        the internal buffer and return it.
        """
        #print "[%s] _KoRunTerminalFile._doReadline(n=%r)" % (self._name, n)
        idx = self._terminal.inBuffer.find('\n')
        if idx == -1:
            idx = len(self._terminal.inBuffer)
        else:
            idx += 1 # include the '\n'
        if n is not None:
            idx = min(idx, n) 
        retval = self._terminal.inBuffer[:idx]
        self._terminal.inBuffer = self._terminal.inBuffer[idx:]
        return retval

    def _haveLine(self):
        #print "[%s] _KoRunTerminalFile._haveLine()" % self._name
        return len(self._terminal.inBuffer) > 0

    def _haveNumBytes(self, n):
        #print "[%s] _KoRunTerminalFile._haveNumBytes(n=%r)"\
        #      % (self._name, n)
        # Note: Checking for >0 rather than >=n because don't want to
        #       do any buffering.
        return len(self._terminal.inBuffer) > 0

    def _doClose(self):
        #print "[%s] _KoRunTerminalFile._doClose" % (self._name)
        if self._passBackLines:
            if self._pbbuf:
                self._terminal.parseAndAddLine(self._pbbuf)
                self._pbbuf = ""
            self._terminal.parsedLastLine()


class _KoRunTerminalCommandHandler(CommandHandler):
    _com_interfaces_ = components.interfaces.nsIController

    def __init__(self, scintilla, terminal):
        self.wrapper = None  #XXX make CommandHandler happy
        self._terminal = terminal  #XXX creates a cycle!
        self._scintilla = scintilla

    def _do_cmd_newline(self):
        #print "XXX [%s] _KoRunTerminalCommandHandler._do_cmd_newline()" % id(self)
        if self._terminal.active:
            self._terminal.mutex.acquire()
            # .newLine() will cause cause the terminal's onCharAdded
            # handler to be called, adding the EOL marker to lineBuffer,
            # so copy lineBuffer beforehand, and clear it afterward.
            #XXX Do we have to explicitly use '\n' and not just take the
            #    native newline terminator added by scintilla.newLine()?
            self._terminal.inBuffer = self._terminal.lineBuffer + '\n'
            #print "inBuffer = ", self._terminal.lineBuffer
            self._scintilla.readOnly = 0
            self._scintilla.newLine()
            self._scintilla.readOnly = 1
            self._terminal.lineBuffer = ''
            self._terminal.mutex.release()
            # Notify the terminal in-file of the new input.
            self._terminal.stateChange.acquire()
            self._terminal.stateChange.notify()
            self._terminal.stateChange.release()

    def _do_cmd_backSmart(self):
        #print "XXX [%s] _KoRunTerminalCommandHandler._do_cmd_backSmart()" % id(self)
        if self._terminal.active and self._terminal.lineBuffer:
            self._terminal.mutex.acquire()
            self._scintilla.readOnly = 0
            self._scintilla.selectionStart = self._scintilla.length - 1
            self._scintilla.selectionEnd = self._scintilla.length
            self._scintilla.replaceSel('')
            self._scintilla.readOnly = 1
            self._terminal.lineBuffer = self._terminal.lineBuffer[:-1]
            self._terminal.mutex.release()



class KoRunTerminal(SciMozEvents, _OutlinerView):
    """This is the interface between run sub-processes and Komodo's command
    output window (or _one_ of the command output windows if there are ever
    more than one). This acts as the handler for events on the output
    window's <scintilla> widget. It acts as the outliner view for the output
    window's <outliner> widget.

    The embedded Scintilla is kept in a 'readOnly' state unless
    currently interacting with the child process (self.active is True).
    XXX Perhaps there is a better API for that (perhaps call it
    'interactive')?

    Input is buffered in line chunks, i.e. user input is forwarded to
    the child a line at a time when <Enter> is pressed. This means that
    one can use backspace on the current line only.

    If enabled, output lines are parsed with a given regular expression
    as results into its outliner.
    """
    _com_interfaces_ = [components.interfaces.koIRunTerminal,
                        components.interfaces.ISciMozEvents,
                        components.interfaces.nsIOutlinerView]
    _reg_clsid_ = "{A077FDDF-DBAD-4e49-9090-4390C084893B}"
    _reg_contractid_ = "@activestate.com/koRunTerminal;1"
    _reg_desc_ = "Run Output Window Terminal Handler"

    def __init__(self):
        #print "XXX [%s] KoRunTerminal.__init__()" % id(self)
        self._lastErrorSvc = components.classes["@activestate.com/koLastErrorService;1"]\
            .getService(components.interfaces.koILastErrorService)

        self.active = 0 # Initially NOT interacting with a child process.
        self.stdin = None
        self.stdout = None
        self.stderr = None

        # Initialization of outliner and parsing attributes.
        _OutlinerView.__init__(self, debug=0)
        self._data = []
        self._sortedBy = None
        self._parseRegex = None
        self._cwd = None
        self._currentFile = None
        # These two vars are used to group update calls to the outliner,
        # .rowCountChanged(), .invalidate(), for performance reasons.
        self._updateGroupSize = 20
        self._updateGroupCounter = 0
        
    #---- koIRunTerminal methods
    def setScintilla(self, scintilla):
        #print "XXX [%s] KoRunTerminal.setScintilla(...)" % id(self)
        self._unproxiedScintilla = scintilla
        self._scintilla = getProxyForObject(None, components.interfaces.ISciMoz,
                                            self._unproxiedScintilla,
                                            PROXY_ALWAYS | PROXY_SYNC)
        self._scintilla.styleSetFore(self._scintilla.SCE_C_COMMENT, 0x0000bb);
        self.controller = _KoRunTerminalCommandHandler(scintilla, self)

        # Synchronization between the terminal and its _KoRunTerminalFile's.
        self.mutex = threading.RLock()
        self.stateChange = threading.Condition()

        self.lineBuffer = ''    # Current line input.
        self.inBuffer = ''      # All unread input except the current line.

    def get_controller(self): #XXX necessary?
        return self.controller

    def startSession(self):
        # Move insertion point to the end of the output window (this is
        # necessary if we are not clearing the window's contents and the
        # cursor has been moved).
        self._scintilla.selectionStart = self._scintilla.length-1;
        self._scintilla.selectionEnd = self._scintilla.length-1;

        self.stdin = _KoRunTerminalFile("<stdin>", self, self._scintilla)
        self.stdout = _KoRunTerminalFile("<stdout>", self, self._scintilla, 1)
        self.stderr = _KoRunTerminalFile("<stderr>", self, self._scintilla, 1)
        self.active = 1

    def endSession(self):
        self.active = 0
        self.stdin = None
        self.stdout = None
        self.stderr = None

    def setParseRegex(self, parseRegex):
        try:
            self._parseRegex = re.compile(parseRegex)
        except re.error, ex:
            self._lastErrorSvc.setLastError(0, str(ex))
            raise ServerException(nsError.NS_ERROR_INVALID_ARG, str(ex))

    def setCwd(self, cwd):
        if cwd:
            self._cwd = cwd
        else:
            self._cwd = os.getcwd()

    def setCurrentFile(self, filename):
        self._currentFile = filename

    def clear(self):
        #print "XXX KoRunTerminal.clear()"
        self.mutex.acquire()
        self._scintilla.readOnly = 0
        self._scintilla.text = ''
        self.lineBuffer = ''    # Current line input.
        self.inBuffer = ''      # All unread input except the current line.
        self._scintilla.gotoPos(self._scintilla.length)
        self._scintilla.readOnly = 1
        self.mutex.release()

        self._parseRegex = None
        self._cwd = None
        self._currentFile = None
        
        self._data = []
        self._sortedBy = None
        self._outliner.rowCountChanged(0, 0)
        self._outliner.invalidate()

    def getFile(self, index):
        return self._data[index]["file"]
    def getLine(self, index):
        return self._data[index]["line"]
    def getColumn(self, index):
        return self._data[index]["column"]

    def parseLine(self, line):
        """Parse the given line with the current regex and return:
            (file, line, column)

        Raise an exception and set an error on the last error service if
        there is no current regex or the line does not match.
        
        (This is used to handle double-clicks in the command output window's
        scintilla.)
        """
        if not self._parseRegex:
            errmsg = "There was no parse regex specified for this command.";
            self._lastErrorSvc.setLastError(0, errmsg);
            raise ServerException(nsError.NS_ERROR_NOT_AVAILABLE, errmsg)

        match = self._parseRegex.search(line)
        if not match:
            errmsg = "This line does not match the current pattern: '%s'"\
                     % self._parseRegex.pattern
            self._lastErrorSvc.setLastError(0, errmsg);
            raise ServerException(nsError.NS_ERROR_NOT_AVAILABLE, errmsg)

        datum = match.groupdict("")
        if "file" not in datum:
            if self._currentFile:
                datum["file"] = self._currentFile
            else:
                datum["file"] = ""
        elif not _isurl(datum["file"]) \
                and not os.path.isabs(datum["file"]) \
                and self._cwd:
            # If this is a relative path name then prepend the cwd.
            datum["file"] = os.path.join(self._cwd, datum["file"])
        if "line" not in datum:
            datum["line"] = ""
        if "column" not in datum:
            datum["column"] = ""
        return datum["file"], datum["line"], datum["column"]

    def sort(self, sortBy):
        """Sort the current data by the given key. If already sorted by this
        key then reverse the sorting order."""
        if self._sortedBy == sortBy:
            self._data.reverse()
        else:
            try:
                if sortBy in ('runoutput-outliner-line',
                              'runoutput-outliner-column'):
                    # compare the integer values (line and column string
                    # can possibly be ranges, so pick first part of range)
                    try:
                        self._data.sort(lambda dict1,dict2,sortBy=sortBy:
                                            cmp(int( dict1[sortBy].split('-')[0] ),
                                                int( dict2[sortBy].split('-')[0] ))
                                       )
                    except ValueError:
                        self._data.sort(lambda dict1,dict2,sortBy=sortBy:
                                            cmp(dict1[sortBy], dict2[sortBy])
                                       )
                elif sortBy == 'runoutput-outliner-content':
                    # strip leading whitespace for content sort order
                    self._data.sort(lambda dict1,dict2,sortBy=sortBy:
                                        cmp(dict1[sortBy].lstrip(), dict2[sortBy].lstrip())
                                   )
                else:
                    self._data.sort(lambda dict1,dict2,sortBy=sortBy:
                                        cmp(dict1[sortBy], dict2[sortBy])
                                   )
            except KeyError:
                logError("Cannot sort find results by: '%s'" % sortBy)
                raise
        self._sortedBy = sortBy
        self._outliner.invalidate()

    #---- ISciMozEvents methods
    def onCharAdded(self, char):
        #print "XXX [%s] KoRunTerminal.onCharAdded(char=%r/%r)"\
        #      % (id(self), char, chr(char)) 
        if self.active and char not in (10, 13):
            # Only add characters if the terminal is active.  Ignore
            # CR/LF because they are being added in _do_cmd_newline()
            # above.
            # XXX: This is relying, is seems, on the questionable
            #      behaviour of scintilla sending onCharAdded on an
            #      attempt to add chars to a read-only buffer. As well,
            #      using onCharAdded does not get text drops and pastes
            #      so we can be missing entered text here. The right
            #      answer is separate shell XBL that handles all these
            #      things as appropriate for the shell.
            self.mutex.acquire()
            self._scintilla.readOnly = 0
            self._scintilla.insertText(self._scintilla.length, chr(char))
            self._scintilla.gotoPos(self._scintilla.length)
            self._scintilla.readOnly = 1
            self.lineBuffer += chr(char)
            self.mutex.release()

    def onZoom(self):
        s = self._scintilla
        if s.getMarginWidthN(0):
            s.setMarginWidthN(0, s.textWidth(0, str(s.lineCount))+5)

    #---- nsIOutliner methods
    def get_rowCount(self):
        return len(self._data)

    def getCellText(self, row, col):
        try:
            datum = self._data[row][col]
        except IndexError:
            logError("no %sth result" % row)
            return ""
        except KeyError:
            logError("unknown result column id: '%s'" % col)
            return ""
        #if type(datum) not in (types.StringType, types.UnicodeType):
        #    datum = str(datum)
        return datum

    #---- non-XPCOM methods
    def parseAndAddLine(self, line):
        #print "XXX KoRunTerminal.parseAndAddLine(line=%r)" % line
        if not self._parseRegex:
            return
        try:
            match = self._parseRegex.search(line)
        except RuntimeError:
            # Can get an error like the following:
            #   RuntimeError: maximum recursion limit exceeded
            # from the regex engine for rediculously long lines with
            # potential null byte content. Just silently drop those.
            return
        if match:
            datum = match.groupdict("")
            # Ensure that there are reasonable defaults for the std
            # outliner column names.
            if "file" not in datum:
                if self._currentFile:
                    datum["file"] = self._currentFile
                else:
                    datum["file"] = ""
            elif not _isurl(datum["file"]) \
                 and not os.path.isabs(datum["file"]) \
                 and self._cwd:
                # If this is a relative path name then prepend the cwd.
                datum["file"] = os.path.join(self._cwd, datum["file"])
            if "line" not in datum:
                datum["line"] = ""
            if "column" not in datum:
                datum["column"] = ""
            if "content" not in datum:
                datum["content"] = line # default "content" to the whole line
            if "runoutput-outliner-file" not in datum:
                datum["runoutput-outliner-file"] = datum["file"]
            if "runoutput-outliner-line" not in datum:
                datum["runoutput-outliner-line"] = datum["line"]
            if "runoutput-outliner-column" not in datum:
                datum["runoutput-outliner-column"] = datum["column"]
            if "runoutput-outliner-content" not in datum:
                datum["runoutput-outliner-content"] = datum["content"]

            self._data.append(datum)
            self._sortedBy = None

            # Group update calls to the outliner for performance.
            self._updateGroupCounter += 1
            if self._updateGroupCounter == self._updateGroupSize:
                self._outliner.rowCountChanged(
                    max(0, len(self._data) - 1 - self._updateGroupCounter),
                    self._updateGroupCounter)
                self._updateGroupCounter = 0

    def parsedLastLine(self):
        """Indicates that the last line of data to parse has been send.
        
        This lets the terminal do any necessary finalization.
        """
        if self._updateGroupCounter:
            self._outliner.rowCountChanged(
                max(0, len(self._data) - 1 - self._updateGroupCounter),
                self._updateGroupCounter)
            self._updateGroupCounter = 0


