#!/usr/bin/env python
# Copyright (c) 2000-2001 ActiveState SRL.
# See the file LICENSE.txt for licensing information.

import sys
import os

from xpcom import components, nsError, ServerException, COMException
from koLogger import logError, logWarning, logDebug


#---- component implementation

class KoSystemDataService:
    # This is just a very light wrapper around the sysdata.py module.
    _com_interfaces_ = [components.interfaces.koISystemDataService]
    _reg_clsid_ = "{6BFADA7E-7032-4128-842C-E49C1F8808DA}"
    _reg_contractid_ = "@activestate.com/koSystemDataService;1"
    _reg_desc_ = "Service for managing named, cache-able system data"

    def __init__(self):
        self._lastErrorSvc = components.classes["@activestate.com/koLastErrorService;1"]\
                             .getService(components.interfaces.koILastErrorService)
    
    def _get(self, name):
        import sysdata
        try:
            return sysdata.get(name)
        except sysdata.SysDataError, ex:
            errmsg = str(ex)
            logError(errmsg)
            self.lastErrorSvc.setLastError(0, errmsg)
            raise ServerException(nsError.NS_ERROR_UNEXPECTED, errmsg)

    getBoolean = _get
    getLong = _get
    getDouble = _get
    getString = _get
    getStringList = _get

    def flush(self, name):
        import sysdata
        try:
            return sysdata.flush(name)
        except sysdata.SysDataError, ex:
            errmsg = str(ex)
            logError(errmsg)
            self.lastErrorSvc.setLastError(0, errmsg)
            raise ServerException(nsError.NS_ERROR_UNEXPECTED, errmsg)

