"""Language package for Tcl"""

from xpcom import components, ServerException
from koLanguageServiceBase import *
import os

import autoinsert

class koTclLanguage(KoLanguageBase):
    name = "Tcl"
    _reg_desc_ = "%s Language" % name
    _reg_contractid_ = "@activestate.com/koLanguage?language=%s;1" \
                       % (name)
    _reg_clsid_ = "{491A0CE5-7180-425b-A27A-9EA36BCBA50F}"

    namedBlockRE = "^[ \t;]*(proc|class|method)\s+[\w:]+(.*\{\n)?"
    namedBlockDescription = 'Tcl procedures and [incr Tcl] classes'
    defaultExtension = ".tcl"

    # XXX read from config somewhere, download from ActiveState?
    downloadURL = 'http://www.ActiveState.com/Products/ActiveTcl'
    remoteDebuggerSupport = 1
    commentDelimiterInfo = { "line": [ "#" ]  }
    cookbookName = "Tcl"
    cookbookUrl = "http://aspn.activestate.com/ASPN/Cookbook/Tcl"
    variableIndicators = '$'

    def get_debugger(self):
        if self._debugger is None:
            self._debugger = KoDebuggerLanguageService(self.name)
        return self._debugger

    def get_lexer(self):
        if self._lexer is None:
            self._lexer = KoLexerLanguageService()
            self._lexer.setLexer(components.interfaces.ISciMoz.SCLEX_TCL)
            self._lexer.setKeywords(0, self._tcl_keywords)
            self._lexer.supportsFolding = 1
        return self._lexer


    def get_linter(self):
        if self._linter is None:
            self._linter = KoLinterLanguageService(
                "@activestate.com/koTclLinter;1", None)
        return self._linter

    def get_methodcompletion(self):
        if self._methodcompletion is None:
            self._methodcompletion = components.classes["@activestate.com/koTclCompletionLanguageService;1"].getService(components.interfaces.koICompletionLanguageService)
        return self._methodcompletion

    def get_commenter(self):
        if self._commenter is None:
            self._commenter = KoCommenterLanguageService(
                self.commentDelimiterInfo)
        return self._commenter

    def get_cookbook(self):
        if self._cookbook is None:
            self._cookbook = KoCookbookLanguageService(
                self.cookbookName, self.cookbookUrl)
        return self._cookbook

#XXX temporarily disabled until whoever wrote this writes
#    autoinsert.TclAutoInsert()
#    def get_inserter(self):
#        if self._inserter is None:
#            self._inserter = KoWSInsertLanguageService( \
#                autoinsert.TclAutoInsert(),
#                "%s = webservice.WebService(\"%s\")",
#                "webservice")
#        return self._inserter
    
    def get_interpreter(self):
        if self._interpreter is None:
            self._interpreter = components.classes["@activestate.com/koTclInfoEx;1"].getService()
        return self._interpreter
    


    # XXX later these need to be centralized.

    _tcl_keywords = ["after", "append", "array", "auto_execok",
                     "auto_import", "auto_load", "auto_load_index",
                     "auto_qualify", "bgerror", "binary", "break", "case",
                     "catch", "cd", "clear", "clock", "close", "concat",
                     "continue", "else", "then", "elseif", "encoding",
                     "eof", "error", "eval", "exec", "exit", "expr",
                     "fblocked", "fconfigure", "fcopy", "file",
                     "fileevent", "flush", "for", "foreach", "format",
                     "gets", "glob", "global", "history", "if", "incr",
                     "info", "interp", "join", "lappend", "lindex",
                     "linsert", "list", "llength", "load", "lrange",
                     "lreplace", "lsearch", "lsort", "namespace", "open",
                     "package", "pid", "pkg_compareExtension",
                     "pkg_mkIndex", "proc", "puts", "pwd", "read",
                     "regexp", "regsub", "rename", "return", "scan",
                     "seek", "set", "socket", "source", "split", "string",
                     "subst", "switch", "tcl_unknown", "tell", "time",
                     "trace", "unknown", "unset", "update", "uplevel",
                     "upvar", "variable", "vwait", "while",
                     "bell", "bind", "bindtags", "button", "canvas",
                     "checkbutton", "clipboard", "destroy", "entry",
                     "event", "focus", "font", "frame", "grab", "grid",
                     "image", "label", "listbox", "lower", "menu",
                     "menubutton", "message", "option", "pack", "place",
                     "radiobutton", "raise", "registry", "scale",
                     "scrollbar", "selection", "spinbox",
                     "tcl_findLibrary", "text", "tk", "tk_chooseColor",
                     "tk_chooseDirectory", "tk_getOpenFile",
                     "tk_getSaveFile", "tk_menuSetFocus", "tk_messageBox",
                     "tk_popup", "tk_textCopy", "tk_textCut",
                     "tk_textPaste", "tkwait", "toplevel", "winfo", "wm"]
