#!python
# Copyright (c) 2000-2001 ActiveState Tool Corporation.
# See the file LICENSE.txt for licensing information.

from xpcom import components, ServerException, nsError
import sys, os, process

def _escapeArg(arg):
    """Escape the given command line argument for the shell."""
    #XXX I suspect that there is a more that we should escape here.
    if not arg: return arg
    arg.replace('"', r'\"')
    if (' ' in arg or ';' in arg) and not (arg.startswith('"') and arg.endswith('"')):
        arg = '"' + arg + '"'
    return arg

def _joinArgv(argv):
    r"""Join an arglist to a string appropriate for running.

        >>> import os
        >>> _joinArgv(['foo', 'bar "baz'])
        'foo "bar \\"baz"'
    """
    cmdstr = ""
    for arg in argv:
        cmdstr += _escapeArg(arg) + ' '
    if cmdstr.endswith(' '): cmdstr = cmdstr[:-1]  # strip trailing space
    return cmdstr

class KoWSProxyGenerator:
    """XSL Proxy Generator for Web Services."""
    _com_interfaces_ = [components.interfaces.koIWSProxyGenerator]
    _reg_clsid_ = "{189FA1DE-9E3D-4337-B340-EAD0CB7768F0}"
    _reg_contractid_ = "@activestate.com/KoWSProxyGenerator;1"
    _reg_desc_ = "XSL Proxy Generator for Web Services"

    def __init__(self):
        self.koDirs = components.classes["@activestate.com/koDirs;1"].\
                getService(components.interfaces.koIDirs)
        self.tmpFileSvc = components.classes["@activestate.com/koTempFileFactory;1"].\
                getService(components.interfaces.koITempFileFactory)
        # get some paths we need
        self.dir = os.path.join( self.koDirs.installDir, 'xslt')
        self.pass1 = os.path.join( self.koDirs.installDir, 'xslt', 'pass1.xsl')
        self.lastErrorSvc = components.classes["@activestate.com/koLastErrorService;1"]\
                       .getService(components.interfaces.koILastErrorService)

        if sys.platform.startswith('win'): 
            self.xslt =  os.path.join( self.koDirs.installDir, 'xslt', 'ActiveXSLT.exe' )
        else:
            self.xslt = os.path.join( self.koDirs.installDir, 'xslt', 'XalanDebugger' )
    
    def generate(self, language, wsdl):
        proxy = os.path.join( self.koDirs.installDir, 'xslt', language+'Proxy.xsl')
        out1 = self.tmpFileSvc.MakeTempName('wsdlgen')
        
        pass1cmd = _joinArgv([self.xslt,'-XSL',self.pass1,'-IN',wsdl,'-OUT',out1])
        proxycmd = _joinArgv([self.xslt,'-XSL',proxy,'-IN',out1])
        
        # do the first pass
        # our intermediary file is in a temp file
        p = process.ProcessOpen(cmd=pass1cmd,cwd=self.dir)
        try:
            p.stdin.close()
            output = p.stdout.read()
            error = p.stderr.read()
            
            self.lastErrorSvc.setLastError(0, str(error))
            if p.wait() != 0:
                raise ServerException(nsError.NS_ERROR_FAILURE, 'Pass1 Proxy Generation failed')
        finally:
            p.close()
        
        # our generated proxy is read for stdout
        # our input is the temp file generated by pass1
        p = process.ProcessOpen(cmd=proxycmd,cwd=self.dir)
        try:
            p.stdin.close()
            output = p.stdout.read()
            error = p.stderr.read()
            self.lastErrorSvc.setLastError(0, str(error))
            if p.wait() != 0:
                raise ServerException(nsError.NS_ERROR_FAILURE, 'Proxy Generation failed')
        finally:
            p.close()
        
        # cleanup
        os.unlink(out1)
        return output
        
        
        
if __name__ == '__main__':
    proxygen = components.classes["@activestate.com/KoWSProxyGenerator;1"].\
                getService(components.interfaces.koIWSProxyGenerator)
    
    print proxygen.generate('perl','http://www.epionet.com/webservices/epiowebservices/livecitycams.wsdl');
    
        