<?php
require_once("soapx4/class.soap_client.php");

DEFINE("WSDL_CACHE_MAX_AGE",12*60*60);
DEFINE("WSDL_CACHE_USE",1); // set to zero to turn off caching

class WebService {
    
    /**
     * _cacheDir
     * return the path to the cache, if it doesn't exist, make it
     */
    function _cacheDir() {
        $dir = getenv("WSDLCACHE");
        if (!$dir) $dir = "./wsdlcache";
        @mkdir($dir, 0700);
        return $dir;
    }
    
    /**
     * _getCacheFile
     * retreives file from cache if it exists, otherwise retreive from net,
     * add to cache, and return from cache
     */
    function _getCacheFile($wsdl_fname, $cache) {
        if (!WSDL_CACHE_USE) return $wsdl_fname;
        $cachename = WebService::_cacheDir() . "/" . md5($wsdl_fname);
        $cacheinfo = $cachename.".info";
        $cachename .= ".wsdl";
        $md5_wsdl = "";
        if (file_exists($cachename)) {
            $wf = fopen($cachename,"rb");
            if ($wf) {
                $fc = fread($wf, filesize($cachename));
                $md5_wsdl = md5($fc);
                fclose($wf);
            }
            if ($cache) {
                if ($cache != $md5_wsdl) {
                    echo "WSDL Checksum error!<br>\n";
                    return NULL;
                }
            } else {
                $fi = stat($cachename);
                $cache_mtime = $fi[8];
                #print cache_mtime, time()
                if ($cache_mtime + WSDL_CACHE_MAX_AGE < time()) {
                    # expired
                    $md5_wsdl = ""; # refetch
                }
            }
        }
        if (!$md5_wsdl) {
            $fp = fopen ($wsdl_fname, "r");
            if (!$fp) {
                echo "Unable to retreive $wsdl_fname";
                return NULL;
            }
            
            $d = "";
            while (!feof($fp)) {
                $d .= fread($fp,8192);
            }
            fclose ($fp);

            if (!d) {
                echo "Unable to retrieve WSDL<br>\n";
                return NULL;
            }
            
            $md5_wsdl = md5($d);
            
            $fp = fopen($cachename, "wb");
            fwrite($fp, $d);
            fclose($fp);
        }
        if ($cache && $cache != $md5_wsdl) {
            echo "WSDL Checksum error!<br>\n";
            return NULL;
        }
        return $cachename;
    }
    
    /**
     * generateWSDLCode
     * pass in a wsdl class to generate code
     */
    function _generateWSDLCode($wsdl) {
        if (!$port) $port = $wsdl->currentPort;
        // XXX currentPort is BAD
        $clienturl = $wsdl->ports[$port]["location"];
        $classname = $wsdl->serviceName."_webservice";
        $class = "class $classname extends soapclient {\n".
        "    function $classname() {\n".
        "        \$this->soapclient(\"$clienturl\", 0);\n".
        "    }\n";

        // get the binding, from that get the port type
        $primaryBinding = $wsdl->ports[$port]["binding"];
        $primaryBinding = preg_replace("/^(.*:)/","",$primaryBinding);
        $portType = $wsdl->bindings[$primaryBinding]["type"];
        $portType = preg_replace("/^(.*:)/","",$portType);
        
        // XXX currentPortType is BAD
        foreach ($wsdl->portTypes[$portType] as $opname => $operation) {
            $args = "";
            $argarray = "";
            foreach ($operation["input"] as $argname => $argtype) {
                if ($argname == "message") {
                    $_msgtype = preg_replace("/^(.*:)/","",$argtype);
                    foreach ($wsdl->messages[$_msgtype] as $_argname => $_argtype) {
                        if ($args) $args .= ", ";
                        $args .= "\$".$_argname;
                        if ($argarray) $argarray .= ", ";
                        $argarray .= "\"$_argname\"=>\$".$_argname;
                    }
                }
            }
            //$binding = $operation["binding"]["name"];
            $soapaction = $wsdl->bindings[$primaryBinding]["operations"][$opname]["soapAction"];
            $namespace = $wsdl->bindings[$primaryBinding]["operations"][$opname]["input"]["namespace"];
            $class .= "    function $opname($args) {\n".
            "        return \$this->call(\"$opname\", array($argarray), \"$namespace\", \"$soapaction\"); \n".
            "    }\n";
        }    
        $class .= "}\n";
        return $class;
    }
    
    /**
     * generateWSDLCode
     * can be used externally to get a string containing the generated code
     */
    function generateWSDLCode($wsdl_fname, $cache="") {
        $wsdl_fname = WebService::_getCacheFile($wsdl_fname, $cache);
        if (!$wsdl_fname) {
            echo "Cache file inconsistent!<br>\n";
            return NULL;
        }
        
        $wsdl = new wsdl($wsdl_fname);
        $class = WebService::_generateWSDLCode($wsdl);
        return $class;
    }
    
    /**
     * ServiceProxy
     * parses the wsdl and returns a new instance of a class for that wsdl.
     */
    function &ServiceProxy($wsdl_fname, $cache="") {
        $wsdl_fname = WebService::_getCacheFile($wsdl_fname, $cache);
        if (!$wsdl_fname) {
            echo "Cache file inconsistent!<br>\n";
            return NULL;
        }
        
        $wsdl = new wsdl($wsdl_fname);
        if ($wsdl->fault) {
            echo ("Error handling WSDL file.<br>\n");
            return NULL;
        }
        $classname = $wsdl->serviceName."_webservice";
        // if the class is already eval'd, dont eval again
        if (!class_exists($classname)) {
            $class = WebService::_generateWSDLCode($wsdl);
            eval($class);
        }
        return new $classname;
    }

}

## test web service
## display generated code
#echo WebService::generateWSDLCode("http://www.xmethods.net/sd/BabelFishService.wsdl");
## use babel fish
#$ws = WebService::ServiceProxy("http://www.xmethods.net/sd/BabelFishService.wsdl");
## do a translation
#$res = $ws->BabelFish("en_es", "Hello, my name is Shane.");
#echo $res."\n";
?>
