# Copyright (c) 2000-2001 ActiveState Tool Corporation.
# See the file LICENSE.txt for licensing information.

from xpcom import components
from xpcom import Exception
import thread, urlparse, os

mutex = None
localFile = None
standard_url = None

def _initComponents():
    global mutex
    global localFile
    global standard_url
    mutex = thread.allocate_lock()
    localFile = components.classes["@mozilla.org/file/local;1"]\
          .createInstance(components.interfaces.nsILocalFile)
    standard_url = components.classes["@mozilla.org/network/standard-url;1"]\
          .createInstance(components.interfaces.nsIFileURL)

# returns a string
def LocalFileToURL(localFileName):
    if not mutex: _initComponents()
    # moz crashes if spec is set to an empty string, so prevent it
    if not localFileName: return None
    
    mutex.acquire()
    localFile.initWithPath(localFileName)

    standard_url.file = localFile
    # Meaning we can reference the "spec" attribute holding the URL
    spec = standard_url.spec
    mutex.release()
    return spec

# returns nsIFile Object, so must create the instance!
def URLToLocalFile(url_string):
    if not mutex: _initComponents()
    # moz crashes if spec is set to an empty string, so prevent it
    if not url_string: return None
    
    # Create a file URL.
    standard_url = components.classes["@mozilla.org/network/standard-url;1"]\
         .createInstance(components.interfaces.nsIFileURL)
    # Setting the spec means we can access the path!
    if url_string.find(':') == -1:
        return None
    standard_url.spec = url_string
    try:
        return standard_url.file
    except Exception: 
        return None

# returns a string    
def LocalPathFromURL(url_string):
    if not mutex: _initComponents()
    # moz crashes if spec is set to an empty string, so prevent it
    if not url_string: return None
    
    mutex.acquire()
    path = None
    if url_string.find(':') != -1:
        standard_url.spec = url_string
        try:
            path = standard_url.file.path
        except Exception: 
            path = None
    mutex.release()
    return path

def _cleanfileURL(url):
    if url.startswith('file:/'):
        if not url.startswith('file:///'):
            url = 'file:///' + url[len('file:/'):]
    return url
    
def RelativizeURL(origbaseurl, origurl):
    "Take a URL, and return a relative path to _our_ URL if possible, and return it unchanged otherwise"
    # XXXXXXXXXXXXXXX This algorithm is not smart enough.  If you change this algorithm, also change the "joining"
    # done in koProject.py (see XXX)
    baseurl = _cleanfileURL(origbaseurl)
    url = _cleanfileURL(origurl)
    (base_addressing_scheme, base_network_location, base_path, base_parameters, base_query, base_fragment_identifier) = urlparse.urlparse(url)
    (addressing_scheme, network_location, path, parameters, query, fragment_identifier) = urlparse.urlparse(url)
    if addressing_scheme not in ('ftp', 'file'):
        # we only relativize FTP and local files
        #print "NOT THE RIGHT addressing_scheme", addressing_scheme
        return url
    if addressing_scheme != base_addressing_scheme:
        #print "NOT THE SAME addressing_scheme", addressing_scheme, base_addressing_scheme
        return url
    if network_location != base_network_location:
        #print "NOT THE SAME network_location", network_location, base_network_location
        return url
    if addressing_scheme == 'file':
        # we're dealing with files on _this_ computer, so we should use the platform-native file comparisons methods
        base_file = LocalPathFromURL(baseurl)
        other_file = LocalPathFromURL(url)
        norm_base_file = os.path.normcase(base_file)
        norm_other_file = os.path.normcase(other_file)
        common_prefix = os.path.commonprefix((norm_base_file, norm_other_file))
        if common_prefix == norm_base_file:
            # the other is a child of the base
            return url[len(baseurl):]
        # if I don't know how to at this point, I give up and return it as is (absolute)
        return url

