# Copyright (c) 2000-2001 ActiveState Tool Corporation.
# See the file LICENSE.txt for licensing information.

# A simple broadcaster-observer system for Python.
# To use:
#
# 1. Derive your class from koBroadcaster.
# 2. Register observers with an instance of your class
#    using registerObserver.
# 3. When you need to call a method on all of the observers,
#    call self.broadcast(methodName, *args).
#
# Yes, the observers usually all support some sort of uniform
# interface!
#
# See the example at the bottom of this file.

class koBroadcaster:
    def __init__(self):
        self._observers = []

    def registerObserver(self, observer):
        self._observers.append(observer)

    def deRegisterObserver(self, observer):
        if self.hasObserver(observer):
            self._observers.remove(observer)
        else:
            raise "Trying to de-register a non-existent observer."

    def deRegisterAllObservers(self):
        for observer in self._observers:
            self.deRegisterObserver(observer)
    
    def hasObserver(self, observer):
        return observer in self._observers

    def broadcast(self, methodName, *args):
        for obs in self._observers:
            func = getattr(obs, methodName)
            func(*args)

class Foo:
    def bar(self, a, b):
        print "Foo foo bar bar"
        print a
        print b

class Twinkle:
    def bar(self, a, b):
        print "Twinkle twinkle little star"
        print a
        print b

if __name__ == '__main__':
    b = koBroadcaster()
    f = Foo()
    b.registerObserver(f)
    b.registerObserver(Twinkle())
    b.broadcast("bar", "abc", "123")

    print b.hasObserver(f)
    b.deRegisterObserver(f)
    print b.hasObserver(f)    

    
